import unittest
from unittest import TestCase, mock

import pandas as pd

from FedPy.Payment_Systems import util
from FedPy.Payment_Systems.ACH import ACH
from FedPy.Payment_Systems.Check_Services import Check_Services
from FedPy.Payment_Systems.Currency import Currency
from FedPy.Payment_Systems.FedWire import FedWire
from FedPy.Payment_Systems.National_Settlement_Service import NSS

demo_list = [
    ["blah1", "blah2", "blah3", "blahN"],
    ["2020", "2019", "2018"],
    [
        ["x", "y", "z"],
        ["x", "y", "z"],
        ["x", "y", "z"],
    ],
]


class Test_Util(TestCase):
    def test_to_DataFrame(self) -> None:
        scenario = demo_list
        result = util.to_DataFrame(scenario)
        self.assertIsInstance(result, pd.DataFrame)


class Test_FedWire(TestCase):
    def test_get_data(self) -> None:
        scenario = "https://www.frbservices.org/resources/financial-services/securities/volume-value-stats/annual-stats.html"
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\n\n<!doctype html>\n<html lang="en">\n<head>\n<meta charset="UTF-8">\n<meta name="viewport" content="width=device-width,initial-scale=1">\n<meta http-equiv="X-UA-Compatible" content="IE=edge">\n<link rel="icon" href="/webfiles/1626607380014/images/global/favicon.ico" type="image/x-icon">\n<link rel="shortcut icon" href="/webfiles/1626607380014/images/global/favicon.ico" type="image/x-icon">\n<!--[if lt IE 9]>\n    <script src="/js/html5shiv.js"></script>\n<![endif]-->\n<!-- Google Tag Manager -->\n<script>(function(w,d,s,l,i){w[l]=w[l]||[];w[l].push({\'gtm.start\': new Date().getTime(),event:\'gtm.js\'});var f=d.getElementsByTagName(s)[0], j=d.createElement(s),dl=l!=\'dataLayer\'?\'&l=\'+l:\'\';j.async=true;j.src=\'https://www.googletagmanager.com/gtm.js?id=\'+i+dl;f.parentNode.insertBefore(j,f);})(window,document,\'script\',\'dataLayer\',\'GTM-KXGSJGD\');</script>\n<!-- End Google Tag Manager -->\n<link href="https://fonts.googleapis.com/css?family=Open+Sans:300,400,700" rel="stylesheet">\n<link rel="stylesheet" type="text/css" href="/webfiles/1626607380014/css/bootstrap.css">\n<link rel="stylesheet" type="text/css" href="/webfiles/1626607380014/css/default.css">\n<style>\n        .pagesettings {\n            width: 100%;\n            position: relative;\n            background-color: silver;\n            color: white;\n            font-size: 18px;\n            font-weight: bold;\n            font-style: italic;\n            text-align: center;\n            padding: 1em 0;\n        }\n    </style>\n<title>Fedwire Securities Service - Annual Statistics</title>\n<meta name="description" content="Annual Fedwire Securities Service statistics."/>\n<meta property="og:description" content="Annual Fedwire Securities Service statistics."/>\n<link rel="canonical" href="https://www.frbservices.org/resources/financial-services/securities/volume-value-stats/annual-stats.html"/>\n</head>\n<body>\n<!-- Google Tag Manager (noscript) -->\n<noscript><iframe src="https://www.googletagmanager.com/ns.html?id=GTM-KXGSJGD" height="0" width="0" style="display:none;visibility:hidden"></iframe></noscript>\n<!-- End Google Tag Manager (noscript) -->\n<a href="#content" class="sr-only sr-only-focusable">Skip to main content</a>\n<div class="container">\n    <div class="component-container">\n\n\n\n\n\n\n\n\n\n\n    </div>\n\n\n\n\n\n<nav class="navbar-static-top navbar" id="navbar-frbs">\n    <div class="navbar-header"> <a href="https://www.frbservices.org/"><img src="https://www.frbservices.org/webfiles/1626607380014/images/federal-reserve-logo-frbservices.svg" alt="The Federal Reserve" class="logo"></a>\n        <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar" aria-expanded="false" aria-controls="navbar"> <span class="sr-only">Toggle navigation</span> <span class="icon-bar"></span> <span class="icon-bar"></span> <span class="icon-bar"></span></button>\n    </div>\n    <div class="navbar__utility hidden-xs" style="z-index: 2000;">\n        <form role="search" class="search" id="search-level-one" action="/search/">\n            <label for="search-query" class="item-invisible">Search</label>\n            <input id="search-query" name="q" type="text" class="form-control" autocomplete="off" placeholder="Begin your search...">\n            <button class="btn btn-default btn-search" type="submit" aria-label="Search"><span class="glyphicon glyphicon-search" aria-hidden="true"></span></button>\n        </form>\n\n\n\n        <div class="btn-group-sm btn-group--app">\n                    <a class="btn btn-default btn-sm dropdown-toggle" data-toggle="dropdown" href="">\n                        Application Sign In\n                        <span class="caret"></span>\n                    </a>\n                        <ul class="dropdown-menu">\n                                <li><a href="/Electronic-Access/AccessFedLine.html">FedLine Web</a></li>\n                                <li><a href="/sign-in/reporting-central.html">Reporting Central</a></li>\n                                <li><a href="/sign-in/reserves-central.html">Reserves Central</a></li>\n                                <li><a href="/sign-in/survey-central.html">Survey Central</a></li>\n                                <li><a href="/sign-in/term-deposit-facility.html">Term Deposit Facility</a></li>\n                        </ul>\n                    <a class="btn btn-default btn-service-status" href="/app/status/serviceStatus.do" style="background-image: url(/app/status/serviceStatusLink)">\n                        <span class="glyphicon glyphicon-check" aria-hidden="true"></span> Service Status\n                    </a>\n                    <a class="btn btn-default "  href="/contacts/index.jsp">\n                        Contact\n                        \n                    </a>\n        </div>\n    </div>\n</nav>\n\n\n\n<div id="navbar" class="navbar-collapse collapse collapse-blue">\n    <ul class="nav navbar-nav" data-smartmenus-id="159500933331223">\n        <li class="visible-xs-block">\n            <form role="search" class="form-search" id="search-level-two" action="/search/">\n                <label for="search-query-two" class="item-invisible">Search</label>\n                <input id="search-query-two" name="q" type="text" class="form-control form-search__input" autocomplete="off" placeholder="Begin your search...">\n                <button class="btn btn-default btn-search" type="submit" aria-label="Search"><span class="glyphicon glyphicon-search" aria-hidden="true"></span></button>\n            </form>\n        </li>\n\n\n\n                <li class="visible-xs-block">\n                    <a class="btn-sm dropdown-toggle" href="">\n                        Application Sign In\n                    </a>\n                        <ul class="dropdown-menu">\n                                <li><a href="/Electronic-Access/AccessFedLine.html">FedLine Web</a></li>\n                                <li><a href="/sign-in/reporting-central.html">Reporting Central</a></li>\n                                <li><a href="/sign-in/reserves-central.html">Reserves Central</a></li>\n                                <li><a href="/sign-in/survey-central.html">Survey Central</a></li>\n                                <li><a href="/sign-in/term-deposit-facility.html">Term Deposit Facility</a></li>\n                        </ul>\n                </li>\n                <li class="visible-xs-block">\n                    <a class="btn-service-status" href="/app/status/serviceStatus.do" style="background-image: url(/app/status/serviceStatusLink)">\n                        Service Status\n                    </a>\n                </li>\n                <li class="visible-xs-block">\n                    <a class="" href="/contacts/index.jsp">\n                        Contact\n                    </a>\n                </li>\n                     \n                        <li class="dropdown" id="Financial Services"><a href="https://www.frbservices.org/financial-services" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Financial Services<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Financial Services Home</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/accounting" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Accounting<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting">Accounting Services Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/account-management-info.html">Account Management Information</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/daily-statement.html">Daily Statement of Account</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/daylight-overdraft-report.html">Daylight Overdraft Reports</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/ami-premium.html">Premium Accounting Information Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/statement-service-charges.html">Statement of Service Charges</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/accounting-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/ami-business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/accounting">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/accounting">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/service-setup">Service Setup</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/ach" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">ACH<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach">FedACH Products & Services Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/exception-resolution.html">FedACH Exception Resolution Service</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/file-services.html">FedACH Information File Service</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/origination-receipt.html">FedACH Origination & Receipt</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/risk">FedACH Risk Management Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/same-day-service.html">FedACH SameDay Service</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/fedglobal">FedGlobal ACH Payments</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/fedpayments-reporter">FedPayments Reporter Service for FedACH Services</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/ach-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/ach">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/ach">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/service-setup">Service Setup</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/testing-opportunities.html">Testing Opportunities</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/bundled-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Bundled Solutions<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/bundled-solutions">Bundled Solutions Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/bundled-solutions/fedcomplete">FedComplete</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/bundled-solutions/fedtransaction-analyzer.html">FedTransaction Analyzer</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/fedcomplete-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/bundled-solutions">Resources</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/cash" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Cash<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash">FedCash Services Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/coin.html">Coin</a></li>\n                                                            <li><a href="/financial-services/cash/coin-allocation">Coin Allocation</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/coin-terminal.html">Coin Terminal Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/currency.html">Currency</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/fedline-web">FedCash Services via the FedLine Web Solution</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/cash-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/business-continuity">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/cash">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/cash">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/service-setup">Service Setup</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/check" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Check<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/check">Check Products & Services Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/check21.html">Check 21-Enabled Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/check-adjustments.html">Check Adjustments Services</a></li>\n                                                            <li><a href="/financial-services/check/feddetect">FedDetect Duplicate Treasury Check Notifier Service</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/fedforward-image-deposit.html">FedForward Image Deposit Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/fedimage-electronic-check">FedImage and Electronic Check Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/fedpayments-reporter">FedPayments Reporter Service for Check Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/fedreceipt">FedReceipt Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/fedreturn.html">FedReturn Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/foreign-canadian.html">Foreign and Canadian Check Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/image-enabled-savings-bond.html">Image-Enabled Savings Bond Processing</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/paper-clearing">Paper Check Clearing Services</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/check-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/check">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/check">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/service-setup">Service Setup</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/testing-opportunities.html">Testing Opportunities</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/fednow" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedNow Service<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow">FedNow Service Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/about.html">About FedNow</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/blog">Blog Articles</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/community">FedNow Community</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/in-the-news.html">In the News</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/instant-payments-education">Instant Payments Education</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/prepare-for-fednow.html">Prepare for FedNow</a></li>\n                                                            <li><a href="/financial-services/fednow/research">Research</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/news/email-notifications.html">Sign Up for FedNow Emails</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/national-settlement-service" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">National Settlement Service<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/national-settlement-service">National Settlement Service Home</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/national-settlement-service-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/national-settlement-service/business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/national-settlement-service">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/national-settlement-service">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/national-settlement-service/service-setup">Service Setup</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/securities" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Securities<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/securities">Fedwire Securities Service Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/securities/joint-custody.html">Joint Custody Service</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/securities-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/securities/business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/securities">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/securities">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/securities/service-setup.html">Service Setup</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/wires/testing">Testing Opportunities</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/wires" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Wires<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/wires">Fedwire Funds Service Home</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/wires-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/wires/business-continuity">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/wires">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/wires">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/wires/service-setup">Service Setup</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/wires/testing">Testing Opportunities</a></li>\n                                                </ul>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="Central Bank"><a href="https://www.frbservices.org/central-bank" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Central Bank<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/central-bank" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Central Bank Home</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbdiscountwindow.org/" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Lending Central (Off-site)</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/central-bank/reporting-central" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Reporting Central<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/central-bank/reporting-central">Reporting Central Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reporting-central/user-guides.html">Reporting Central User Guides</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reporting-central/stats-reporting-disruption.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/central-bank">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/central-bank">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reporting-central/service-setup">Service Setup</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/central-bank/reserves-central" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Reserves Central<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/central-bank/reserves-central">Reserves Central Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reserves-central/excess-balance-account.html">Excess Balance Account</a></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reserves-central/term-deposit-facility">Term Deposit Facility</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reserves-central/reserve-admin-business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/central-bank">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/central-bank">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reserves-central/service-setup">Service Setup</a></li>\n                                                </ul>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="Treasury Services"><a href="https://www.frbservices.org/treasury-services" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Treasury Services<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury Services Home</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/collateral-services.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Collateral Services</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/savings-bonds-financial-institutions.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Savings Bonds For Financial Institutions</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/savings-bond-news.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Savings Bonds News You Can Use</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/ach-reclamation.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury ACH Reclamation</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/auctions.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury Auctions</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/check-reclamation.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury Check Reclamation</a>\n                                        </li>\n                                        <li role="separator" class="divider"></li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/business-continuity.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Business Continuity</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/treasury" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Forms</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/treasury" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Resources</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/service-setup" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Service Setup</a>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="Fedline Solutions"><a href="https://www.frbservices.org/fedline-solutions" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Fedline Solutions<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Solutions Home</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/fedline-advantage.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Advantage</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/fedline-command.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Command</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/fedline-direct.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Direct</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/fedline-web.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Web</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/fedmail.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedMail</a>\n                                        </li>\n                                        <li role="separator" class="divider"></li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/fees/fedline-solutions-2021.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">2021 Fees</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/fedline-solutions/application-connectivity-testing.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Application and Connectivity Testing</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/business-continuity" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Business Continuity</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/fedline-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Forms</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/fedline-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Resources</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/service-setup" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Service Setup</a>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="Forms"><a href="https://www.frbservices.org/forms" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Forms<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Forms Home</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/accounting" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Accounting</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/ach" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">ACH</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/cash" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Cash</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/central-bank" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Central Bank</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/check" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Check</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/district-information" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">District Information</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/fedline-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Solutions</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/national-settlement-service" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">National Settlement Service</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/securities" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Securities</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/treasury" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury Services</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/wires" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Wires</a>\n                                        </li>\n                            </ul>\n                        </li>\n                     \n                        <li class="dropdown" id="Resources"><a href="https://www.frbservices.org/resources" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Resources<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/central-bank" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Central Bank<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/central-bank">Central Bank Resources Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/central-bank/faq">Frequently Asked Questions</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/central-bank/reporting-central.html">Reporting Central Resources</a></li>\n                                                            <li><a href="/resources/central-bank/survey-central.html">Survey Central Resources</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="/resources/covid-19-resources/" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">COVID-19</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/routing-number-directory" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">E-Payments Routing Directory</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/fedline-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Solutions<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions">FedLine Solutions Resources Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions/euac.html">End User Authorization Contact (EUAC) Support</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions/command-matrix.html">FedLine Command Environment and Configuration Change Matrix</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions/direct-file-matrix.html">FedLine Direct File Environment and Configuration Change Matrix</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions/direct-message-matrix.html">FedLine Direct Message Environment and Configuration Change Matrix</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions/faq">Frequently Asked Questions</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/fees" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Fees</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/financial-services" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Financial Services<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/accounting">Accounting</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/ach">ACH</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/bundled-solutions">Bundled Solutions</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/cash">Cash</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/check">Check</a></li>\n                                                            <li><a href="/app/fedachfx/home.html">Foreign Exchange Rates</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/national-settlement-service">National Settlement Service</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/securities">Securities</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/wires">Wires</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/industry-links" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Industry Links</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/resource-centers" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Resource Centers<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers">Resource Centers Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers/business-banking-toolbox">Business Banking Toolbox</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers/business-continuity">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers/international-payments">International Payments</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers/risk-mgmt-toolbox">Risk Management Toolbox</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers/same-day-ach">Same Day ACH</a></li>\n                                                            <li><a href="/resources/resource-centers/security-resiliency-assurance-program">Security and Resiliency Assurance Program</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/rules-regulations" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Rules & Regulations<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/rules-regulations">Rules and Regulations Resources Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/rules-regulations/operating-circulars.html">Operating Circulars</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/rules-regulations/regulations.html">Regulations</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/service-setup" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Service & Access Setup<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/service-setup">Service and Access Setup Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/service-setup/current-customer.html">Current Financial Services Customer</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/service-setup/merger.html">Financial Institution Merger</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/service-setup/new-customer.html">New Financial Services Customer</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/treasury" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury Services<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/treasury">Treasury Services Resources Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/treasury/collateral-services.html">Collateral Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/treasury/faq">Frequently Asked Questions</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/treasury/savings-bonds-financial-institutions.html">Savings Bonds for Financial Institutions</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/treasury/treasury-auctions.html">Treasury Auctions</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/treasury/where-to-send-security-deposits.html">Where to Send Security Deposits</a></li>\n                                                </ul>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="Education"><a href="https://www.frbservices.org/education" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Education<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/education/webinars" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Federal Reserve Bank Webinars</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/education/industry-events.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Industry Events</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/education/products-services-education" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Products & Services Education</a>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="News"><a href="https://www.frbservices.org/news" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">News<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/news/communications" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Communications</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/news/email-notifications.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Email Notifications</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/news/fed360" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Fed360</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/news/press-releases" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Press Releases</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/news/research.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Research Studies</a>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="About"><a href="https://www.frbservices.org/about" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">About<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/about/frb-bank-offices.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Bank Offices</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/about/holiday-schedules" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Holiday Schedules</a>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n    </ul>\n</div>\n<!--HstSiteMapItemService{canonicalPath=\'/hst:crsocms/hst:configurations/crsocms/hst:workspace/hst:sitemap/resources/financial-services/securities/volume-value-stats/annual-stats.html\', site=HstSiteService{name=\'crsocms\',configurationPath=\'/hst:crsocms/hst:configurations/crsocms\'}}-->\n<!---->\n    <nav class="navbar navbar-static-top navbar__margin-bottom" id="subnav">\n        <div class="navbar-header navbar-header-inner">\n                <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar-inner" aria-expanded="false" aria-controls="navbar">Resources<span class="caret"></span> </button>\n        </div>\n        <div id="navbar-inner" class="navbar-collapse collapse">\n            <ul class="nav navbar-nav" data-smartmenus-id="16098773335059512">\n                            <li><a href="https://www.frbservices.org/resources/central-bank" aria-haspopup="true" aria-expanded="false" class="has-submenu">Central Bank<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/central-bank">Central Bank Resources Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/central-bank/faq">Frequently Asked Questions</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/central-bank/reporting-central.html">Reporting Central Resources</a></li>\n                                                <li><a href="/resources/central-bank/survey-central.html">Survey Central Resources</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="/resources/covid-19-resources/" aria-haspopup="true" aria-expanded="false" >COVID-19</a>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/routing-number-directory" aria-haspopup="true" aria-expanded="false" >E-Payments Routing Directory</a>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/fedline-solutions" aria-haspopup="true" aria-expanded="false" class="has-submenu">FedLine Solutions<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions">FedLine Solutions Resources Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions/euac.html">End User Authorization Contact (EUAC) Support</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions/command-matrix.html">FedLine Command Environment and Configuration Change Matrix</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions/direct-file-matrix.html">FedLine Direct File Environment and Configuration Change Matrix</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions/direct-message-matrix.html">FedLine Direct Message Environment and Configuration Change Matrix</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions/faq">Frequently Asked Questions</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/fees" aria-haspopup="true" aria-expanded="false" >Fees</a>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/financial-services" aria-haspopup="true" aria-expanded="false" class="has-submenu">Financial Services<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/accounting">Accounting</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/ach">ACH</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/bundled-solutions">Bundled Solutions</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/cash">Cash</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/check">Check</a></li>\n                                                <li><a href="/app/fedachfx/home.html">Foreign Exchange Rates</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/national-settlement-service">National Settlement Service</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/securities">Securities</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/wires">Wires</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/industry-links" aria-haspopup="true" aria-expanded="false" >Industry Links</a>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/resource-centers" aria-haspopup="true" aria-expanded="false" class="has-submenu">Resource Centers<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers">Resource Centers Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers/business-banking-toolbox">Business Banking Toolbox</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers/business-continuity">Business Continuity</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers/international-payments">International Payments</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers/risk-mgmt-toolbox">Risk Management Toolbox</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers/same-day-ach">Same Day ACH</a></li>\n                                                <li><a href="/resources/resource-centers/security-resiliency-assurance-program">Security and Resiliency Assurance Program</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/rules-regulations" aria-haspopup="true" aria-expanded="false" class="has-submenu">Rules & Regulations<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/rules-regulations">Rules and Regulations Resources Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/rules-regulations/operating-circulars.html">Operating Circulars</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/rules-regulations/regulations.html">Regulations</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/service-setup" aria-haspopup="true" aria-expanded="false" class="has-submenu">Service & Access Setup<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/service-setup">Service and Access Setup Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/service-setup/current-customer.html">Current Financial Services Customer</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/service-setup/merger.html">Financial Institution Merger</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/service-setup/new-customer.html">New Financial Services Customer</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/treasury" aria-haspopup="true" aria-expanded="false" class="has-submenu">Treasury Services<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/treasury">Treasury Services Resources Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/treasury/collateral-services.html">Collateral Services</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/treasury/faq">Frequently Asked Questions</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/treasury/savings-bonds-financial-institutions.html">Savings Bonds for Financial Institutions</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/treasury/treasury-auctions.html">Treasury Auctions</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/treasury/where-to-send-security-deposits.html">Where to Send Security Deposits</a></li>\n                                    </ul>\n                            </li>\n            </ul>\n        </div>\n    </nav>\n    <div id="content" role="main">\n\n\n\n\n\n\n<div class="row hidden-print" id="content">\n\t<div class="col-sm-12">\n\t\t<ol class="breadcrumb">\n\t\t\t<li><a href="/">Home</a></li>\n\t\t\t\t\t<li><a href="/resources">Resources</a></li>\n\t\t\t\t\t<li><a href="/resources/financial-services">Financial Services</a></li>\n\t\t\t\t\t<li><a href="/resources/financial-services/securities">Securities</a></li>\n\t\t\t\t\t<li><a href="/resources/financial-services/securities/volume-value-stats">Fedwire<sup>&reg;</sup> Securities Service Volume and Value Statistics</a></li>\n  \t\t\t\t\t<li>Fedwire<sup>&reg;</sup> Securities Service - Annual Statistics</li>\n\t\t</ol>\n\t</div>\n</div>    <div class="row">\n        <div class="col-sm-8 col-md-9 col-sm-push-4 col-md-push-3">\n\n    <div>\n                <div>\n                    \n\n\n    <h1>Fedwire<sup>&reg;</sup> Securities Service - Annual Statistics</h1>\r\n      <table class="table table-striped table-responsive table-hover">\r\n        <caption>\r\n        Annual Fedwire Securities Service Statistics\r\n        </caption>\r\n        <thead>\r\n          <tr>\r\n            <th scope="col">Year</th>\r\n            <th scope="col">Transfers originated<br/>\r\n              (number)</th>\r\n            <th scope="col">Annual volume growth<br/>\r\n              (percent)</th>\r\n            <th scope="col">Value of transfers originated<br/>\r\n              ($millions)</th>\r\n            <th scope="col">Annual value growth<br/>\r\n              (percent)</th>\r\n            <th scope="col">Average value per transfer<br/>\r\n              ($millions)</th>\r\n            <th scope="col">Average daily volume of transfers<a href="#1ftn"><sup>1</sup></a><br/>\r\n              (number)</th>\r\n            <th scope="col">Average daily value of transfers<a href="#1ftn"><sup>1</sup></a><br/>\r\n              ($millions)</th>\r\n\t\t\t<th scope="col">Securities held in custody at year end<a href="#2ftn"><sup>2</sup></a><br/>\r\n              ($millions)</th>\r\n          </tr>\r\n        </thead>\r\n        <tbody>\r\n<tr>\r\n\t\t     <th scope="row">2020</th>\r\n\t\t     <td>20,824,673</td>\r\n\t\t     <td>8.2</td>\r\n\t\t     <td>361,728,932</td>\r\n\t\t     <td>4.6</td>\r\n\t\t     <td>17.37</td>\r\n\t\t     <td>82,311</td>\r\n\t\t     <td>1,429,759</td>\r\n\t\t\t <td>96,952,666</td>\r\n\t       </tr>\r\n\t\t   <tr>\r\n\t\t     <th scope="row">2019</th>\r\n\t\t     <td>19,247,530</td>\r\n\t\t     <td>12.6</td>\r\n\t\t     <td>345,813,248</td>\r\n\t\t     <td>16.7</td>\r\n\t\t     <td>17.97</td>\r\n\t\t     <td>76,683</td>\r\n\t\t     <td>1,377,742</td>\r\n\t\t\t <td>88,271,327</td>\r\n\t       </tr>\r\n\t\t   <tr>\r\n\t\t\t<th scope="row">2018</th>\r\n\t\t\t<td>17,094,201</td>\r\n\t\t\t<td>4.4</td>\r\n\t\t\t<td>296,335,209</td>\r\n\t\t\t<td>(1.0)</td>\r\n\t\t\t<td>17.34</td>\r\n\t\t\t<td>68,104</td>\r\n\t\t\t<td>1,180,618</td>\r\n\t\t\t<td>85,141,500</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2017</th>\r\n\t\t\t<td>16,368,207</td>\r\n\t\t\t<td>(1.2)</td>\r\n\t\t\t<td>299,334,719</td>\r\n\t\t\t<td>4.4</td>\r\n\t\t\t<td>18.29</td>\r\n\t\t\t<td>65,212</td>\r\n\t\t\t<td>1,192,569</td>\r\n\t\t\t<td>76,440,851</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2016</th>\r\n\t\t\t<td>16,565,473</td>\r\n\t\t\t<td>(5.1)</td>\r\n\t\t\t<td>286,671,689</td>\r\n\t\t\t<td>(3.1)</td>\r\n\t\t\t<td>17.31</td>\r\n\t\t\t<td>65,998</td>\r\n\t\t\t<td>1,142,118</td>\r\n\t\t\t<td>74,039,804</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2015</th>\r\n\t\t\t<td>17,463,476</td>\r\n\t\t\t<td>2.5</td>\r\n\t\t\t<td>295,755,612</td>\r\n\t\t\t<td>3.0</td>\r\n\t\t\t<td>16.94</td>\r\n\t\t\t<td>69,300</td>\r\n\t\t\t<td>1,173,633</td>\r\n\t\t\t<td>70,489,690</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2014</th>\r\n\t\t\t<td>17,040,141</td>\r\n\t\t\t<td>(10.5)</td>\r\n\t\t\t<td>287,104,205</td>\r\n\t\t\t<td>(2.7)</td>\r\n\t\t\t<td>16.85</td>\r\n\t\t\t<td>67,889</td>\r\n\t\t\t<td>1,143,841</td>\r\n\t\t\t<td>67,914,939</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2013</th>\r\n\t\t\t<td>19,037,148</td>\r\n\t\t\t<td>4.4</td>\r\n\t\t\t<td>295,186,170</td>\r\n\t\t\t<td>3.8</td>\r\n\t\t\t<td>15.51</td>\r\n\t\t\t<td>75,845</td>\r\n\t\t\t<td>1,176,041</td>\r\n\t\t\t<td>65,725,983</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2012</th>\r\n\t\t\t<td>18,231,152</td>\r\n\t\t\t<td>(2.1)</td>\r\n\t\t\t<td>284,401,670</td>\r\n\t\t\t<td>(2.5)</td>\r\n\t\t\t<td>15.60</td>\r\n\t\t\t<td>72,634</td>\r\n\t\t\t<td>1,133,074</td>\r\n\t\t\t<td>63,196,046</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2011</th>\r\n\t\t\t<td>18,613,058</td>\r\n\t\t\t<td>(5.9)</td>\r\n\t\t\t<td>291,823,993</td>\r\n\t\t\t<td>(8.8)</td>\r\n\t\t\t<td>15.68</td>\r\n\t\t\t<td>74,156</td>\r\n\t\t\t<td>1,162,645</td>\r\n\t\t\t<td>60,295,945</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2010</th>\r\n\t\t\t<td>19,779,423</td>\r\n\t\t\t<td>(6.2)</td>\r\n\t\t\t<td>320,123,901</td>\r\n\t\t\t<td>8.2</td>\r\n\t\t\t<td>16.18</td>\r\n\t\t\t<td>78,490</td>\r\n\t\t\t<td>1,270,333</td>\r\n\t\t\t<td>56,722,615<a href="#3ftn"><sup>3</sup></a></td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2009</th>\r\n\t\t\t<td>21,081,242</td>\r\n\t\t\t<td>(15.7)</td>\r\n\t\t\t<td>295,741,666</td>\r\n\t\t\t<td>(29.5)</td>\r\n\t\t\t<td>14.03</td>\r\n\t\t\t<td>83,656</td>\r\n\t\t\t<td>1,173,578</td>\r\n\t\t\t<td>52,662,266</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2008</th>\r\n\t\t\t<td>25,007,806</td>\r\n\t\t\t<td>3.3</td>\r\n\t\t\t<td>419,347,256</td>\r\n\t\t\t<td>(3.7)</td>\r\n\t\t\t<td>16.77</td>\r\n\t\t\t<td>99,237</td>\r\n\t\t\t<td>1,664,076</td>\r\n\t\t\t<td>47,427,140</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2007</th>\r\n\t\t\t<td>24,216,823</td>\r\n\t\t\t<td>8.7</td>\r\n\t\t\t<td>435,577,505</td>\r\n\t\t\t<td>15.5</td>\r\n\t\t\t<td>17.99</td>\r\n\t\t\t<td>96,481</td>\r\n\t\t\t<td>1,735,369</td>\r\n\t\t\t<td>43,358,966</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2006</th>\r\n\t\t\t<td>22,288,805</td>\r\n\t\t\t<td>(0.3)</td>\r\n\t\t\t<td>377,258,592</td>\r\n\t\t\t<td>2.3</td>\r\n\t\t\t<td>16.93</td>\r\n\t\t\t<td>88,800</td>\r\n\t\t\t<td>1,503,022</td>\r\n\t\t\t<td>35,902,630</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2005</th>\r\n\t\t\t<td>22,359,335</td>\r\n\t\t\t<td>10.9</td>\r\n\t\t\t<td>368,896,819</td>\r\n\t\t\t<td>17.7</td>\r\n\t\t\t<td>16.50</td>\r\n\t\t\t<td>89,081</td>\r\n\t\t\t<td>1,469,708</td>\r\n\t\t\t<td>31,945,464</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2004</th>\r\n\t\t\t<td>20,164,811</td>\r\n\t\t\t<td>(1.0)</td>\r\n\t\t\t<td>313,425,252</td>\r\n\t\t\t<td>17.1</td>\r\n\t\t\t<td>15.54</td>\r\n\t\t\t<td>79,703</td>\r\n\t\t\t<td>1,238,835</td>\r\n\t\t\t<td>29,370,123</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2003</th>\r\n\t\t\t<td>20,371,395</td>\r\n\t\t\t<td>16.8</td>\r\n\t\t\t<td>267,644,194</td>\r\n\t\t\t<td>16.9</td>\r\n\t\t\t<td>13.14</td>\r\n\t\t\t<td>81,161</td>\r\n\t\t\t<td>1,066,312</td>\r\n\t\t\t<td>27,042,565</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2002</th>\r\n\t\t\t<td>17,436,524</td>\r\n\t\t\t<td>16.1</td>\r\n\t\t\t<td>228,907,121</td>\r\n\t\t\t<td>7.8</td>\r\n\t\t\t<td>13.13</td>\r\n\t\t\t<td>69,468</td>\r\n\t\t\t<td>911,981</td>\r\n\t\t\t<td>23,050,957</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2001</th>\r\n\t\t\t<td>15,013,873</td>\r\n\t\t\t<td>10.4</td>\r\n\t\t\t<td>212,343,034</td>\r\n\t\t\t<td>12.9</td>\r\n\t\t\t<td>14.14</td>\r\n\t\t\t<td>59,816</td>\r\n\t\t\t<td>845,988</td>\r\n\t\t\t<td>17,327,032</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2000</th>\r\n\t\t\t<td>13,595,988</td>\r\n\t\t\t<td>1.5</td>\r\n\t\t\t<td>188,133,178</td>\r\n\t\t\t<td>4.8</td>\r\n\t\t\t<td>13.84</td>\r\n\t\t\t<td>53,952</td>\r\n\t\t\t<td>746,560</td>\r\n\t\t\t<td>14,943,324</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1999</th>\r\n\t\t\t<td>13,397,547</td>\r\n\t\t\t<td>(6.7)</td>\r\n\t\t\t<td>179,486,282</td>\r\n\t\t\t<td>(9.3)</td>\r\n\t\t\t<td>13.40</td>\r\n\t\t\t<td>53,165</td>\r\n\t\t\t<td>712,247</td>\r\n\t\t\t<td>14,261,967</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1998</th>\r\n\t\t\t<td>14,365,609</td>\r\n\t\t\t<td>11.0</td>\r\n\t\t\t<td>197,781,609</td>\r\n\t\t\t<td>13.1</td>\r\n\t\t\t<td>13.77</td>\r\n\t\t\t<td>57,006</td>\r\n\t\t\t<td>784,848</td>\r\n\t\t\t<td>&nbsp;</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1997</th>\r\n\t\t\t<td>12,944,447</td>\r\n\t\t\t<td>(1.2)</td>\r\n\t\t\t<td>174,949,330</td>\r\n\t\t\t<td>8.9</td>\r\n\t\t\t<td>13.52</td>\r\n\t\t\t<td>51,572</td>\r\n\t\t\t<td>697,009</td>\r\n\t\t\t<td>&nbsp;</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1996</th>\r\n\t\t\t<td>13,098,856</td>\r\n\t\t\t<td>2.2</td>\r\n\t\t\t<td>160,637,460</td>\r\n\t\t\t<td>7.3</td>\r\n\t\t\t<td>12.26</td>\r\n\t\t\t<td>51,980</td>\r\n\t\t\t<td>637,450</td>\r\n\t\t\t<td>&nbsp;</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1995</th>\r\n\t\t\t<td>12,810,706</td>\r\n\t\t\t<td>1.8</td>\r\n\t\t\t<td>149,764,431</td>\r\n\t\t\t<td>3.5</td>\r\n\t\t\t<td>11.69</td>\r\n\t\t\t<td>51,039</td>\r\n\t\t\t<td>596,671</td>\r\n\t\t\t<td>&nbsp;</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1994</th>\r\n\t\t\t<td>12,590,196</td>\r\n\t\t\t<td>5.3</td>\r\n\t\t\t<td>144,702,226</td>\r\n\t\t\t<td>(1.0)</td>\r\n\t\t\t<td>11.49</td>\r\n\t\t\t<td>50,160</td>\r\n\t\t\t<td>576,503</td>\r\n\t\t\t<td>&nbsp;</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1993</th>\r\n\t\t\t<td>11,957,053</td>\r\n\t\t\t<td>1.7</td>\r\n\t\t\t<td>146,220,304</td>\r\n\t\t\t<td>4.7</td>\r\n\t\t\t<td>12.23</td>\r\n\t\t\t<td>47,449</td>\r\n\t\t\t<td>580,239</td>\r\n\t\t\t<td>&nbsp;</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1992</th>\r\n\t\t\t<td>11,753,217</td>\r\n\t\t\t<td>6.3</td>\r\n\t\t\t<td>139,675,710</td>\r\n\t\t\t<td>20.1</td>\r\n\t\t\t<td>11.88</td>\r\n\t\t\t<td>46,455</td>\r\n\t\t\t<td>552,078</td>\r\n\t\t\t<td>&nbsp;</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1991</th>\r\n\t\t\t<td>11,061,621</td>\r\n\t\t\t<td>1.7</td>\r\n\t\t\t<td>116,315,973</td>\r\n\t\t\t<td>16.5</td>\r\n\t\t\t<td>10.52</td>\r\n\t\t\t<td>44,070</td>\r\n\t\t\t<td>463,410</td>\r\n\t\t\t<td>&nbsp;</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1990</th>\r\n\t\t\t<td>10,877,413</td>\r\n\t\t\t<td>(0.1)</td>\r\n\t\t\t<td>99,861,205</td>\r\n\t\t\t<td>4.4</td>\r\n\t\t\t<td>9.18</td>\r\n\t\t\t<td>43,336</td>\r\n\t\t\t<td>397,853</td>\r\n\t\t\t<td>&nbsp;</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1989</th>\r\n\t\t\t<td>10,884,155</td>\r\n\t\t\t<td>---</td>\r\n\t\t\t<td>95,654,471</td>\r\n\t\t\t<td>---</td>\r\n\t\t\t<td>8.79</td>\r\n\t\t\t<td>43,363</td>\r\n\t\t\t<td>381,094</td>\r\n\t\t\t<td>&nbsp;</td>\r\n\t\t</tr>\r\n        </tbody>\r\n      </table>\r\n      <h2>Notes</h2>\r\n      <p><sup id="1ftn">1</sup> Based on the number of business days per period.</p>\r\n\t  <p><sup id="2ftn">2</sup> Began reporting this statistic in 1999.</p>\r\n\t  <p><sup id="3ftn">3</sup> Amount restated in August 2011.</p>\r\n      <p> <a href="#content">Top of Page</a> </p>\n\n                </div>\n    </div>\n        </div>\n        <div class="col-sm-4 col-md-3 col-sm-pull-8 col-md-pull-9 hidden-print">\n\n    <div>\n                <div>\n                    \n\n\n\t<div class="spotlight spotlight--securities">\n\t<h2>Securities Service Resources</h2>\n\t\n\n\t<ul class="list--hide">\n\t\t\t\t\t\t<li><a href="/resources/financial-services/securities" target="">Securities Service Resources Home</a></li>\n\t\t\t\t\t\t<li><a href="/binaries/content/assets/crsocms/financial-services/securities/fedwirebook-entrysecuritiesservice.pdf" target="">Automated Claims Adjustment Process (PDF)</a></li>\n\t\t\t\t\t\t<li><a href="/app/callnotices/callNoticesHome.html" target="">Call Notices</a></li>\n\t\t\t\t\t\t<li><a href="/resources/financial-services/securities/clearing-memos.html" target="">Clearing Memos</a></li>\n\t\t\t\t\t\t<li><a href="/resources/routing-number-directory" target="">E-Payments Routing Directory</a></li>\n\t\t\t\t\t\t<li><a href="/resources/financial-services/securities/user-guide.html" target="">Fedwire<sup>&reg;</sup> Securities Service Issuer Guide</a></li>\n\t\t\t\t\t\t<li><a href="/resources/financial-services/wires/extension-guidelines.html" target="">Fedwire<sup>&reg;</sup> Services Extension Guidelines</a></li>\n\t\t\t\t\t<li><a data-toggle="collapse" data-parent="#accordion" aria-expanded="false" href="#spotlight1" target="">Fedwire<sup>&reg;</sup> Services Frequently Asked Questions <span class="caret"></span></a></li>\n\t\t\t\t\n\t\t\t\t<div id="spotlight1" class="spotlight collapse">\n\t\t\t\t\t\n\t\t\t\t\t<ul class="hide-list-type">\n\t\t\t\t\t\t\t<li><a href="/resources/financial-services/wires/faq" target="">Frequently Asked Questions Home</a></li>\n\t\t\t\t\t\t\t<li><a href="/resources/routing-number-directory/faqs.html" target="">E-Payments Routing Directory</a></li>\n\t\t\t\t\t\t\t<li><a href="/resources/financial-services/wires/faq/testing-opportunities.html" target="">Testing Opportunities</a></li>\n\t\t\t\t\t</ul>\n\t\t\t\t</div>\n\t\t\t\t\t\t<li><a href="/resources/financial-services/wires/operating-hours.html" target="">Fedwire<sup>&reg;</sup> Services Operating Hours</a></li>\n\t\t\t\t\t<li><a data-toggle="collapse" data-parent="#accordion" aria-expanded="false" href="#spotlight2" target="">Fedwire<sup>&reg;</sup> Services Volume and Value Statistics <span class="caret"></span></a></li>\n\t\t\t\t\n\t\t\t\t<div id="spotlight2" class="spotlight collapse">\n\t\t\t\t\t\n\t\t\t\t\t<ul class="hide-list-type">\n\t\t\t\t\t\t\t<li><a href="/resources/financial-services/securities/volume-value-stats" target="">Fedwire<sup>&reg;</sup> Securities Volume and Value Statistics Home</a></li>\n\t\t\t\t\t\t\t<li><a href="/resources/financial-services/securities/volume-value-stats/monthly-stats.html" target="">Fedwire<sup>&reg;</sup> Securities Service \xe2\x80\x93 Monthly Statistics</a></li>\n\t\t\t\t\t\t\t<li><a href="/resources/financial-services/securities/volume-value-stats/quarterly-stats.html" target="">Fedwire<sup>&reg;</sup> Securities Service \xe2\x80\x93 Quarterly Statistics</a></li>\n\t\t\t\t\t\t\t<li><a href="/resources/financial-services/securities/volume-value-stats/annual-stats.html" target="">Fedwire<sup>&reg;</sup> Securities Service \xe2\x80\x93 Annual Statistics</a></li>\n\t\t\t\t\t</ul>\n\t\t\t\t</div>\n\t\t\t\t\t\t<li><a href="/resources/financial-services/securities/glossary.html" target="">Glossary</a></li>\n\t\t\t\t\t\t<li><a href="/resources/rules-regulations/operating-circulars.html#7" target="">Operating Circular 7</a></li>\n\t</ul>\n</div>\n\n                </div>\n    </div>\n        </div>\n    </div>\n    </div>\n\n\n\n\n<footer class="hidden-print">\n    <div class="row row--no-margin">\n        <div class="twitter col-sm-12 text-right">\n            FOLLOW US\n            <a class="twitter" href="https://twitter.com/frbservices">\n                <img src="https://www.frbservices.org/webfiles/1626607380014/images/twitter-logo-blue.svg" alt="Follow us on Twitter" height="42" width="36">\n            </a>\n            <a class="twitter" href="https://www.linkedin.com/company/federal-reserve-financial-services/ ">\n                <img src="https://www.frbservices.org/webfiles/1626607380014/images/lI-In-bug.png" alt="Go to Federal Reserve Financial Services on LinkedIn" height="30" width="38">\n            </a>\n        </div>\n    </div>\n    <div class="row row--no-margin">\n        <div class="col-xs-12">\n            <hr>\n        </div>\n    </div>\n    <div class="row row--no-margin" style="position: relative">\n\n\n\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/financial-services" class="footer__list-header">Financial Services</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/accounting">Accounting</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/ach">ACH</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/bundled-solutions">Bundled Solutions</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/cash">Cash</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/check">Check</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/fednow">FedNow Service</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/national-settlement-service">National Settlement Service</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/securities">Securities</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/wires">Wires</a></li>\n                        <li><a href="https://www.frbservices.org/central-bank" class="footer__list-header">Central Bank</a></li>\n                            <li><a href="https://www.frbdiscountwindow.org/">Lending Central (Off-Site)</a></li>\n                            <li><a href="https://www.frbservices.org/central-bank/reporting-central">Reporting Central</a></li>\n                            <li><a href="https://www.frbservices.org/central-bank/reserves-central">Reserves Central</a></li>\n                </ul>\n            </div>\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/treasury-services" class="footer__list-header">Treasury Services</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/ach-reclamation.html">ACH Reclamation</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/auctions.html">Auctions</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/check-reclamation.html">Check Reclamation</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/collateral-services.html">Collateral Services</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/savings-bonds-financial-institutions.html">Savings Bonds For Financial Institutions</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/savings-bond-news.html">Savings Bonds News You Can Use</a></li>\n                        <li><a href="https://www.frbservices.org/fedline-solutions" class="footer__list-header">Fedline<sup>\xc2\xae</sup> Solutions</a></li>\n                            <li><a href="https://www.frbservices.org/fedline-solutions/fedline-advantage.html">FedLine Advantage<sup>\xc2\xae</sup></a></li>\n                            <li><a href="https://www.frbservices.org/fedline-solutions/fedline-command.html">FedLine Command<sup>\xc2\xae</sup></a></li>\n                            <li><a href="https://www.frbservices.org/fedline-solutions/fedline-direct.html">FedLine Direct<sup>\xc2\xae</sup></a></li>\n                            <li><a href="https://www.frbservices.org/fedline-solutions/fedline-web.html">FedLine Web<sup>\xc2\xae</sup></a></li>\n                            <li><a href="https://www.frbservices.org/fedline-solutions/fedmail.html">FedMail<sup>\xc2\xae</sup></a></li>\n                </ul>\n            </div>\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/forms" class="footer__list-header">Forms</a></li>\n                            <li><a href="https://www.frbservices.org/forms/accounting">Accounting</a></li>\n                            <li><a href="https://www.frbservices.org/forms/ach">ACH</a></li>\n                            <li><a href="https://www.frbservices.org/forms/cash">Cash</a></li>\n                            <li><a href="https://www.frbservices.org/forms/central-bank">Central Bank</a></li>\n                            <li><a href="https://www.frbservices.org/forms/check">Check</a></li>\n                            <li><a href="https://www.frbservices.org/forms/district-information">District Information</a></li>\n                            <li><a href="https://www.frbservices.org/forms/fedline-solutions">FedLine<sup>\xc2\xae</sup> Solutions</a></li>\n                            <li><a href="https://www.frbservices.org/forms/national-settlement-service">National Settlement Service</a></li>\n                            <li><a href="https://www.frbservices.org/forms/securities">Securities</a></li>\n                            <li><a href="https://www.frbservices.org/forms/treasury">Treasury Services</a></li>\n                            <li><a href="https://www.frbservices.org/forms/wires">Wires</a></li>\n                </ul>\n            </div>\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/resources" class="footer__list-header">Resources</a></li>\n                            <li><a href="https://www.frbservices.org/resources/central-bank">Central Bank</a></li>\n                            <li><a href="https://www.frbservices.org/resources/fedline-solutions">FedLine<sup>\xc2\xae</sup> Solutions</a></li>\n                            <li><a href="https://www.frbservices.org/resources/fees">Fees</a></li>\n                            <li><a href="https://www.frbservices.org/resources/financial-services">Financial Services</a></li>\n                            <li><a href="https://www.frbservices.org/resources/industry-links">Industry Links</a></li>\n                            <li><a href="https://www.frbservices.org/resources/resource-centers">Resource Centers</a></li>\n                            <li><a href="https://www.frbservices.org/resources/routing-number-directory">E-Payments Routing Directory</a></li>\n                            <li><a href="https://www.frbservices.org/resources/rules-regulations">Rules & Regulations</a></li>\n                            <li><a href="https://www.frbservices.org/resources/service-setup">Service & Access Setup</a></li>\n                            <li><a href="https://www.frbservices.org/resources/treasury">Treasury Services</a></li>\n                </ul>\n            </div>\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/education" class="footer__list-header">Education</a></li>\n                            <li><a href="https://www.frbservices.org/education/industry-events.html">Events</a></li>\n                            <li><a href="https://www.frbservices.org/education/products-services-education">Products & Services Education</a></li>\n                            <li><a href="https://www.frbservices.org/education/webinars">Webinars</a></li>\n                </ul>\n            </div>\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/news" class="footer__list-header">News</a></li>\n                            <li><a href="https://www.frbservices.org/news/communications">Communications</a></li>\n                            <li><a href="https://www.frbservices.org/news/email-notifications.html">Email Notifications</a></li>\n                            <li><a href="https://www.frbservices.org/news/fed360">Fed360<sup>\xc2\xae</sup></a></li>\n                            <li><a href="https://www.frbservices.org/news/press-releases">Press Releases</a></li>\n                            <li><a href="https://www.frbservices.org/news/research.html">Research Studies</a></li>\n                        <li><a href="https://www.frbservices.org/about" class="footer__list-header">About Us</a></li>\n                            <li><a href="https://www.frbservices.org/about/frb-bank-offices.html">Bank Offices</a></li>\n                            <li><a href="https://www.frbservices.org/about/holiday-schedules">Holiday Schedules</a></li>\n                        <li><a href="/contacts/index.jsp" class="footer__list-header">Contact</a></li>\n                        <li><a href="https://www.frbservices.org/sitemap" class="footer__list-header">Sitemap</a></li>\n                </ul>\n            </div>\n    </div>\n    <div class="row row--no-margin">\n        <div class="col-xs-12">\n            <hr>\n        </div>\n    </div>\n    <div class="row row--no-margin">\n        <div class="footnote col-sm-6 col-xs-12">\n            <p>\n                This site is a product of the Federal Reserve Banks.<br>\n\n\n\n                <a href="https://www.frbservices.org/terms">Terms of Use</a> | \n                <a href="https://www.frbservices.org/terms/supported-browsers.html">Supported Browsers</a> | \n                <a href="https://www.frbservices.org/terms/privacy.html">Privacy Policy</a>\n            <br>\n                <a href="https://www.frbservices.org/terms/data-privacy.html">General Reserve Bank Data Privacy Notice</a> | \n                <a href="https://www.frbservices.org/terms/reprinting.html">Reprinting Articles</a>\n            <br>\n            </p>\n        </div>\n        <div class="footnote text-right col-sm-6 col-xs-12">\n            <p>Address comments and questions to the<br>\n                <a href="mailto:siteinfo@frbservices.org">FRBservices.org Webmaster</a><br>\n                \xc2\xa92021 Federal Reserve Banks</p>\n        </div>\n    </div>\n</footer>    <script src="/webfiles/1626607380014/js/jquery.min.js"></script>\n    <script src="/webfiles/1626607380014/js/jquery.frbservices.min.js"></script>\n    <script src="/webfiles/1626607380014/js/bootstrap.min.js"></script>\n    <script src="/webfiles/1626607380014/js/html5shiv.js"></script>\n    <script src="/webfiles/1626607380014/js/jquery.smartmenus.min.js"></script>\n    <script src="/webfiles/1626607380014/js/jquery.smartmenus.bootstrap.min.js"></script>\n    <script src="/webfiles/1626607380014/js/abasearch.js"></script>\n</div>\n</body>\n</html>'  # noqa: E501
            result = FedWire().get_data(scenario)
            self.assertIsInstance(result, list)
            self.assertEqual(len(result), 3)


class Test_ACH(TestCase):
    def test_get_data(self) -> None:
        scenario = "https://www.federalreserve.gov/paymentsystems/fedach_yearlygovt.htm"
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\xef\xbb\xbf<!doctype html>\n<html lang="en" class="no-js">\n    <head>\n        <meta charset="utf-8">\n        <meta http-equiv="X-UA-Compatible" content="IE=edge"/>\n        <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0 maximum-scale=1.6, user-scalable=1"/>\n        <meta name="keywords" content="Board of Governors of the Federal Reserve System, Federal Reserve Board of Governors, Federal Reserve Board, Federal Reserve" />\n        <meta name="description" content="The Federal Reserve Board of Governors in Washington DC." />\n        <meta property="og:site_name" content="Board of Governors of the Federal Reserve System"/>\n        <meta property="og:type" content="article" /> \n        <meta property="og:image"  content="" /> \n        <meta name="twitter:card" content="summary" />\n        <meta name="twitter:image" content="" />\n        \n\n\n \n        <title>Federal Reserve Board - Government Automated Clearinghouse Transactions Processed by the Federal Reserve--Annual Data</title>\n        \n    <link href="/css/bootstrap.css" rel="stylesheet" type="text/css">\n    <link href="/css/bluesteel-theme.css" rel="stylesheet" type="text/css">\n    <script src="/js/modernizr-latest.js" type="text/javascript"></script>\n    <script src="/js/grunticon.loader.js"></script>\n    <script>\n        grunticon(["/css/icons.data.svg.css", "/css/icons.data.png.css", "/css/icons.fallback.css"]);\n    </script>\n    <noscript>\n        <link href="/css/icons.fallback.css" rel="stylesheet">\n    </noscript>                \n    <script src="/js/angular.min.js" type="text/javascript"></script>\n    <script src="/js/app.js" type="text/javascript"></script>\n        \n    <!-- For Linear Gradients in IE9 or greater -->\n    <!--[if gte IE 9]>\n        <style type="text/css">\n          .gradient {\n             filter: none;\n          }\n        </style>\n      <![endif]-->\n\n\n \n    <script type="text/javascript" >\n  (function(i,s,o,g,r,a,m){i[\'GoogleAnalyticsObject\']=r;i[r]=i[r]||function(){\n  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),\n  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)\n  })(window,document,\'script\',\'//www.google-analytics.com/analytics.js\',\'ga\');\n  ga(\'create\', \'UA-35121701-1\', \'federalreserve.gov\', {\'cookieExpires\': 0});\n  ga(\'set\', \'anonymizeIp\', true);\n  ga(\'send\', \'pageview\');\n</script></head>\n    <body data-ng-app="pubwebApp">\n        <a href="#content" class="globalskip">Skip to main content</a>\n<nav class="nav-primary-mobile" id="nav-primary-mobile">\n</nav>\n<div class="t1_nav navbar navbar-default navbar-fixed-top" role="navigation">\n    <div class="t1_container clearfix">\n        <a id="logo" class="btn btn-default icon-FRB_logo-bw visible-xs-table-cell t1__btnlogo" href="/default.htm" role="button"><span class="sr-only">Back to Home</span>\n        </a>\n        <a href="/default.htm" id="banner" class="visible-xs-table-cell t1__banner" title="Link to Home Page" role="button">Board of Governors of the Federal Reserve System</a>\n        <ul class="nav navbar-nav visible-md visible-lg t1_toplinks">\n            <li class="navbar-text navbar-text__social">\n                <span class="navbar-link" tabindex="0">Stay Connected</span>\n                <ul class="nav navbar-nav t1_social">\n                    \n\n\n    \n                <li class="social__item"><a data-icon="facebook" href="https://www.facebook.com/federalreserve" class="noIcon"><div class="icon-facebook-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="twitter" href="https://twitter.com/federalreserve" class="noIcon"><div class="icon-twitter-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="youtube" href="https://www.youtube.com/federalreserve" class="noIcon"><div class="icon-youtube-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="flickr" href="https://www.flickr.com/photos/federalreserve/" class="noIcon"><div class="icon-flickr-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="linkedin" href="https://www.linkedin.com/company/federal-reserve-board" class="noIcon"><div class="icon-linkedin-color icon icon__sm"></div><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="rss" href="/feeds/feeds.htm" class="noIcon"><div class="icon-rss-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to RSS</span></a></li>\n            \n                <li class="social__item"><a data-icon="email" href="/subscribe.htm" class="noIcon"><div class="icon-email-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to Email</span></a></li>\n                    \n    \n \n \n                </ul>\n            </li>\n           \r\n                                <li class="navbar-text"><a class="navbar-link" href="/recentpostings.htm">Recent Postings</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/newsevents/calendar.htm">Calendar</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/publications.htm">Publications</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/sitemap.htm">Site Map</a></li>\r\n                \r\n                                <li class="navbar-text"><a class="navbar-link" href="/azindex.htm">A-Z index</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/careers.htm">Careers</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/faqs.htm">FAQs</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/videos.htm">Videos</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/aboutthefed/contact-us-topics.htm">Contact</a></li>\r\n                \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \n        </ul>\n        <form class="nav navbar-form hidden-xs nav__search" role="search" action="//www.fedsearch.org/board_public/search" method="get">\n            <div class="form-group input-group input-group-sm">\n                <label for="t1search" class=\'sr-only\'>Search</label>\n                <input id="t1search" class="nav__input form-control" name="text" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                    <button type="submit" class="nav__button btn btn-default" id="headerTopLinksSearchFormSubmit" name="Search">\n                        <span class="sr-only">Submit Search Button</span>\n                        <span class="icon-magnifying-glass icon icon__xs icon--right"></span>\n                    </button>\n                </span>\n            </div>\n            <div class=\'nav__advanced\'><a class="noIcon" href="//www.fedsearch.org/board_public">Advanced</a></div>\n        </form>        \n        <div class="btn-group visible-xs-table-cell visible-sm-table-cell t1__dropdown">\n            <span class="icon icon__md icon--right dropdown-toggle icon-options-nav" data-toggle="dropdown">\n                <span class="sr-only">Toggle Dropdown Menu</span>\n            </span>\n        </div>\n    </div>\n</div>\n<header class="jumbotron hidden-xs">\n    <a class="jumbotron__link" href="/default.htm" title="Link to Home Page">\n        <div class="container-fluid">\n            <h1 class="jumbotron__heading">Board of Governors of the Federal Reserve System</h1>\n            <p class="jumbotron__content"><em>The Federal Reserve, the central bank of the United States, provides the nation with a safe, flexible, and stable monetary and financial system.</em></p>\n        </div>\n    </a>\n</header>\n<div class="t2__offcanvas visible-xs-block">\n    <div class="navbar-header">\n        <button type="button" class="offcanvas__toggle icon-offcanvas-nav" data-toggle="offcanvas" data-target="#offcanvasT2Menu" data-canvas="body">\n            <span class="sr-only">Main Menu Toggle Button</span>\n        </button>\n        <span class="navbar-brand offcanvas__title">Sections</span>\n        <button type="button" class="offcanvas__search icon-search-nav" data-toggle="collapse" data-target="#navbar-collapse">\n            <span class="sr-only">Search Toggle Button</span>\n        </button>\n    </div>\n    <div class="navbar-collapse collapse mobile-form" id="navbar-collapse">\n        <form method="get" class="navbar-form" role="search" action="//www.fedsearch.org/board_public/search">\n            <div class="form-group input-group mobileSearch">\n                <label class="sr-only" for="t2search">Search</label>\n                <input id="t2search" name="text" class="form-control" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                <span class="sr-only">Search Submit Button</span>\n                <button type="submit" class="btn btn-default">Submit</button>\n                </span>\n            </div>\n        </form>\n    </div>\n</div>\n<nav class="nav-primary navbar hidden-xs" id="nav-primary" role="navigation">\n    <ul class="nav navbar-nav" role="menubar">\n        \n \n<li role="menuitem" class="dropdown nav-about  dropdown--3Col">\n    <a href="/aboutthefed.htm" class="sr-only-focusable" aria-haspopup="true" id="aboutMenu">About<br />the Fed</a>\n    <ul aria-labelledby="aboutMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-4 list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/structure-federal-reserve-system.htm">Structure of the Federal Reserve System</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/pf.htm">Purposes &amp; Functions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/bios/board/default.htm">Board Members</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/advisorydefault.htm">Advisory Councils</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/federal-reserve-system.htm">Federal Reserve Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fract.htm">Federal Reserve Act</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/currency.htm">Currency</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardmeetings/meetingdates.htm">Board Meetings</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardvotes.htm">Board Votes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/diversityinclusion.htm">Diversity & Inclusion</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/careers.htm">Careers</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/procurement/about.htm">Do Business with the Board</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/k8.htm">Holidays Observed - K.8</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/contact-us-topics.htm">Contact </a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/foia/about_foia.htm">Requesting Information (FOIA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/faqs.htm">FAQs</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/educational-tools/default.htm">Economic Education</a>\n</li>\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fed-financial-statements.htm">Fed Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/innovation.htm">Innovation</a>\n</li>\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-news  dropdown--1Col">\n    <a href="/newsevents.htm" class="sr-only-focusable" aria-haspopup="true" id="newsMenu">News<br />&amp; Events</a>\n    <ul aria-labelledby="newsMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        \n<li>\n    <a class="sr-only-focusable" href="/newsevents/pressreleases.htm">Press Releases</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/speeches.htm">Speeches</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/testimony.htm">Testimony</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/calendar.htm">Calendar</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/videos.htm">Videos\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/photogallery.htm">Photo Gallery\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-monetary  dropdown--2Col">\n    <a href="/monetarypolicy.htm" class="sr-only-focusable" aria-haspopup="true" id="monetaryMenu">Monetary<br />Policy</a>\n    <ul aria-labelledby="monetaryMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Federal Open Market Committee</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc.htm">About the FOMC</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomccalendars.htm">Meeting calendars and information</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_historical.htm">Transcripts and other historical materials</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_projectionsfaqs.htm">FAQs</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Monetary Policy Principles and Practice</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/monetary-policy-principles-and-practice.htm">Notes</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Policy Implementation</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policy-normalization.htm">Policy Normalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policytools.htm">Policy Tools</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/mpr_default.htm">Monetary Policy Report</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/beige-book-default.htm">Beige Book</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/quarterly-balance-sheet-developments-report.htm">Quarterly Report on Federal Reserve Balance Sheet Developments</a>\n</li>\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Review of Monetary Policy Strategy, Tools, and Communications</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/review-of-monetary-policy-strategy-tools-and-communications.htm">Overview</a>\n</li>\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-supervision  dropdown--fw">\n    <a href="/supervisionreg.htm"  class="sr-only-focusable" id="supervisionMenu" aria-haspopup="true">Supervision<br />&amp; Regulation</a>\n    <ul aria-labelledby="supervisionMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Institution Supervision</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/community-and-regional-financial-institutions.htm">Community & Regional Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/large-financial-institutions.htm">Large Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/foreign-banking-organizations.htm">Foreign Banking Organizations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/financial-market-utility-supervision.htm">Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/consumer-compliance.htm">Consumer Compliance</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supervision-and-regulation-report.htm">Federal Reserve Supervision and Regulation Report</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reporting Forms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/default.aspx">All Reporting Forms</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reporting-forms-whats-new.htm">Recent Updates</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/review.aspx">Information collections under review</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?xTdTa+FaSlI=">Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?+PhkJwiBjbA=">Applications/structure change</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?1/RIXO/UduI=">Federal Financial Institutions Examination Council (FFIEC)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?L877VwEj9Ss=">Municipal &amp; Government Securities</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Supervision &amp; Regulation Letters</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/srletters/srletters.htm">By Year</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/topics/topics.htm">By Topic</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Banking Applications &amp; Legal Developments</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/application-process.htm">Application Process</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/board-and-reserve-bank-actions.htm">Board & Reserve Bank Action</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/legal-developments.htm">Enforcement Actions & Legal Developments</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Regulatory Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">Regulations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supmanual.htm">Manuals</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/basel/basel-default.htm">Basel Regulatory Framework</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/volcker-rule.htm">Volcker Rule</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/training-and-assistance-landing.htm">Education, Training, and Assistance</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Banking &amp; Data Structure</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/insider.aspx">Beneficial Ownership reports</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/">U.S. Offices of Foreign Entities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/fhc.htm">Financial Holding Companies</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/isb-default.htm">Interstate Branching</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/suds.htm">Securities Underwriting & Dealing Subsidiaries</a>\n</li>\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-payment  dropdown--fw">\n    <a href="/paymentsystems.htm"  class="sr-only-focusable" id="paymentMenu" aria-haspopup="true">Payment<br />Systems</a>\n    <ul aria-labelledby="paymentMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations &amp; Statutes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regcc-about.htm">Regulation CC (Availability of Funds and Collection of Checks)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regii-about.htm">Regulation II (Debit Card Interchange Fees and Routing)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/reghh-about.htm">Regulation HH (Financial Market Utilities)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/other-regulations.htm">Other Regulations and Statutes</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Payment Policies</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pfs_about.htm">Federal Reserve\'s Key Policies for the Provision of Financial Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/oo_about.htm">Overnight Overdrafts</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_about.htm">Payment System Risk</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/telecomm.htm">Sponsorship for Priority Telecommunication Services</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reserve Bank Payment Services &amp; Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/check_about.htm">Check Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_data.htm">Daylight Overdrafts and Fees</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fednow_about.htm">FedNow\xe2\x84\xa0 Service</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedfunds_about.htm">Fedwire Funds Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedsecs_about.htm">Fedwire Securities Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fisagy_about.htm">Fiscal Agency Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/natl_about.htm">National Settlement Service</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Financial Market Utilities &amp; Infrastructures</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/over_about.htm">Supervision & Oversight of Financial Market Infrastructures</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/designated_fmu_about.htm">Designated Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/int_standards.htm">International Standards for Financial Market Infrastructures</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Research, Committees, and Forums</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/payres_about.htm">Payment Research</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fr-payments-study.htm">Federal Reserve Payments Study (FRPS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pspa_committee.htm">Payments System Policy Advisory Committee</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/forums.htm">Forums</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-econ dropdown--right dropdown--2Col">\n    <a href="/econres.htm" class="sr-only-focusable" aria-haspopup="true" id="econMenu">Economic<br />Research</a>\n    <ul aria-labelledby="econMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Working Papers and Notes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/feds/index.htm">Finance and Economics Discussion Series (FEDS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/notes/feds-notes/default.htm">FEDS Notes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/ifdp/index.htm">International Finance Discussion Papers (IFDP)</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/econres/theeconomists.htm">Meet the Economists</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Models and Tools</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/us-models-about.htm">FRB/US Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/edo-models-about.htm">Estimated Dynamic Optimization (EDO) Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n\n\n\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-data dropdown--right dropdown--fw">\n    <a href="/data.htm"  class="sr-only-focusable" id="dataMenu" aria-haspopup="true">Data</a>\n    <ul aria-labelledby="dataMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/datadownload"><span class="icon icon-ddp icon__sm"></span> Data Download Program</a>\n</li>\n\n\n<li class="nav__header">\n    <p>\n        <strong>Bank Assets and Liabilities</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h3/current/default.htm">Aggregate Reserves of Depository Institutions and the Monetary Base - H.3</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h8/current/default.htm">Assets and Liabilities of Commercial Banks in the U.S. - H.8</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/assetliab/current.htm">Assets and Liabilities of U.S. Branches and Agencies of Foreign Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/chargeoff/">Charge-Off and Delinquency Rates on Loans and Leases at Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sfos/sfos.htm">Senior Financial Officer Survey</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sloos.htm">Senior Loan Officer Opinion Survey on Bank Lending Practices</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Bank Structure Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for the U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Business Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/cp/">Commercial Paper</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g20/current/g20.htm">Finance Companies - G.20</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/govsecure/current.htm">New Security Issues, State and Local Governments</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/corpsecure/current.htm">New Security Issues, U.S. Corporations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Dealer Financing Terms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/scoos.htm">Senior Credit Officer Opinion Survey on Dealer Financing Terms</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Exchange Rates and International Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h10/current/">Foreign Exchange Rates - H.10/G.5</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/intlsumm/current.htm">International Summary Statistics</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/secholdtrans/current.htm">Securities Holdings and Transactions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/statbanksus/current.htm">Statistics Reported by Banks and Other Financial Firms in the United States</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Financial Accounts</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/z1/default.htm">Financial Accounts of the United States - Z.1</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Household Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g19/current/default.htm">Consumer Credit - G.19</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/housedebt/default.htm">Household Debt Service and Financial Obligations Ratios</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mortoutstand/current.htm">Mortgage Debt Outstanding</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Industrial Activity</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g17/current/default.htm">Industrial Production and Capacity Utilization - G.17</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Interest Rates</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h15/">Selected Interest Rates - H.15</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Micro Data Reference Manual (MDRM)</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mdrm.htm">Micro and Macro Data Collections</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Money Stock and Reserve Balances</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h41/">Factors Affecting Reserve Balances - H.4.1</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h6/current/default.htm">Money Stock Measures - H.6</a>\n</li>\n\n\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Other</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/yield-curve-models.htm">Yield Curve Models and Data</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-consumer dropdown--right dropdown--4Col">\n    <a href="/consumerscommunities.htm" class="sr-only-focusable" aria-haspopup="true" id="consumerMenu">Consumers<br />&amp; Communities</a>\n    <ul aria-labelledby="consumerMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cra_about.htm">Community Reinvestment Act (CRA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">All Regulations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Supervision&nbsp;&amp; Enforcement</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/caletters/caletters.htm">CA Letters</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/enforcementactions/default.aspx">Enforcement Actions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/independent-foreclosure-review.htm">Independent Foreclosure Review</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Community Development</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/neighborhood-revitalization.htm">Housing and Neighborhood Revitalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/small-business-and-entrepreneurship.htm">Small Business and Entrepreneurship</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/workforce.htm">Employment and Workforce Development</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cdf.htm">Community Development Finance</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/rural-community-economic-development.htm">Rural Community and Economic Development</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Research&nbsp;&amp; Analysis</strong>\n    </p>\n</li>\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/community-development-publications.htm">Research Publications & Data Analysis</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/cac.htm">Community Advisory Council</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Consumer Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/foreclosure.htm">Mortgage and Foreclosure Resources</a>\n</li>\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/comm-dev-system-map.htm">Federal Reserve Community Development Resources</a>\n</li>\n\n\n\n                </ul>\n            </div>\n        </li>             \n    </ul>\n</li>\n \n \n \n \n \n \n \n \n \n \n \n \n    </ul>\n</nav>\n        <!-- BODY CONTENT -->\n        <div id="content" class="container container__main" role="main">\n            <div class="row">\n                <div class="page-header">\n                    <ol class="breadcrumb">\n                            <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/default.htm">Home</a></li>\n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems.htm">Payment Systems</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/fedach_data.htm">Data</a></li>\n                                          \n \n                                          \n \n                                          \n \n                                          \n \n \n                    </ol>\n                    <div class="header-group">\n                        <div id="page-title" class="page-title"> \n                        \n                                \t\t<h2>\n                                \t\t    Automated Clearinghouse Services\n                                \t\t</h2>\n                            \t\t\n                        </div> \n                        \n                  </div>\n                </div><!-- / .page-header -->\n            \n            </div><!-- / .row -->\n  \n\n\n        <div class=\'row\'>\n        \n                                <div class="col-xs-12 col-sm-4 col-md-3">\n                                  <div id="t4_nav" class="t4_nav list-group sticky">\n                                    \n                                    \r\n\r\n\r\n\r\n<!-- <h3 class="text-uppercase">Chapters</h3> -->\r\n\r\n\r\n\r\n\r\n<a href="/paymentsystems/fedach_data.htm" class="list-group-item" title="">Data</a>\n                                  </div>\n                                </div>\n                        \n                                                    <div id="article" class="col-xs-12 col-sm-8 col-md-9">\n                                                    <h3>Government Automated Clearinghouse Transactions Processed by the Federal Reserve--Annual Data</h3>\n                                                \n                                <!-- Article Area -->\n                                    \n                                    <div id="floatRightRail" class="col-xs-12 col-sm-12 col-md-5 pull-right">\n                                        \n                                        \n\n<!-- Text Box Begins -->\n\n<div class="panel panel-related">\n    <div class="panel-heading">\n                   \n                <h5 class="panel-title text-capitalize">\n                    Other Formats  \n                </h5>\n                \n    </div>\n    <div class="panel-body">\n        <ul>\n\t<li><a href="/paymentsystems/files/fedach_yearlygovt.txt">ASCII</a></li>\n\t<li><a href="/paymentsystems/files/fedach_yearlygovt.pdf">PDF (97&nbsp;KB)</a></li>\n</ul>\n    </div>\n</div>\n\n\n\n\n            \n                                    </div>     \n                                    \n                                    \n                                    <p>The daily average volume and average daily value of items are based on the number of business days in each year.<a id="ft1" name="ft1"></a><a href="#fn1" title="footnote 1"><sup>1</sup></a></p>\n\n<div class="data-table" data-table-popout="" id="data-table-0">\n<table class="pubtables" data-other-sticky-ids="navbar-mobile,nav-primary" data-sticky-columns="1" data-sticky-rows="1">\n\t<thead>\n\t\t<tr class="header">\n\t\t\t<th align="center" scope="col" valign="bottom">Year</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Volume<br />\n\t\t\t(millions<br />\n\t\t\tof items)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Volume<br />\n\t\t\tpercent<br />\n\t\t\tchange</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Value<br />\n\t\t\t(billions<br />\n\t\t\tof dollars)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Value<br />\n\t\t\tpercent<br />\n\t\t\tchange</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average daily<br />\n\t\t\tvolume<br />\n\t\t\t(millions<br />\n\t\t\tof items)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average daily<br />\n\t\t\tvalue<br />\n\t\t\t(billions<br />\n\t\t\tof dollars)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average value<br />\n\t\t\tper<br />\n\t\t\ttransaction<br />\n\t\t\t(dollars)</th>\n\t\t</tr>\n\t</thead>\n\t<tbody>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2020</strong></td>\n\t\t\t<td class="left2">1,878</td>\n\t\t\t<td class="left2">10.2</td>\n\t\t\t<td class="left2">6,853</td>\n\t\t\t<td class="left2">18.4</td>\n\t\t\t<td class="left2">7.3</td>\n\t\t\t<td class="left2">26.8</td>\n\t\t\t<td class="left2">3,648</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2019</strong></td>\n\t\t\t<td class="left2">1,704</td>\n\t\t\t<td class="left2">2.2</td>\n\t\t\t<td class="left2">5,787</td>\n\t\t\t<td class="left2">4.9</td>\n\t\t\t<td class="left2">6.8</td>\n\t\t\t<td class="left2">23.1</td>\n\t\t\t<td class="left2">3,396</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2018</strong></td>\n\t\t\t<td class="left2">1,668</td>\n\t\t\t<td class="left2">2.4</td>\n\t\t\t<td class="left2">5,515</td>\n\t\t\t<td class="left2">2.7</td>\n\t\t\t<td class="left2">6.6</td>\n\t\t\t<td class="left2">22.0</td>\n\t\t\t<td class="left2">3,306</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2017</strong></td>\n\t\t\t<td class="left2">1,629</td>\n\t\t\t<td class="left2">2.2</td>\n\t\t\t<td class="left2">5,371</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">6.5</td>\n\t\t\t<td class="left2">21.5</td>\n\t\t\t<td class="left2">3,296</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2016</strong></td>\n\t\t\t<td class="left2">1,594</td>\n\t\t\t<td class="left2">2.4</td>\n\t\t\t<td class="left2">5,193</td>\n\t\t\t<td class="left2">2.7</td>\n\t\t\t<td class="left2">6.4</td>\n\t\t\t<td class="left2">20.7</td>\n\t\t\t<td class="left2">3,257</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2015</strong></td>\n\t\t\t<td class="left2">1,558</td>\n\t\t\t<td class="left2">2.7</td>\n\t\t\t<td class="left2">5,054</td>\n\t\t\t<td class="left2">3.7</td>\n\t\t\t<td class="left2">6.2</td>\n\t\t\t<td class="left2">20.1</td>\n\t\t\t<td class="left2">3,245</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2014</strong></td>\n\t\t\t<td class="left2">1,516</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">4,873</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">6.0</td>\n\t\t\t<td class="left2">19.4</td>\n\t\t\t<td class="left2">3,214</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2013</strong></td>\n\t\t\t<td class="left2">1,467</td>\n\t\t\t<td class="left2">6.2</td>\n\t\t\t<td class="left2">4,714</td>\n\t\t\t<td class="left2">2.3</td>\n\t\t\t<td class="left2">5.8</td>\n\t\t\t<td class="left2">18.8</td>\n\t\t\t<td class="left2">3,214</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2012</strong></td>\n\t\t\t<td class="left2">1,382</td>\n\t\t\t<td class="left2">5.9</td>\n\t\t\t<td class="left2">4,610</td>\n\t\t\t<td class="left2">1.7</td>\n\t\t\t<td class="left2">5.5</td>\n\t\t\t<td class="left2">18.4</td>\n\t\t\t<td class="left2">3,336</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2011</strong></td>\n\t\t\t<td class="left2">1,305</td>\n\t\t\t<td class="left2">6.8</td>\n\t\t\t<td class="left2">4,535</td>\n\t\t\t<td class="left2">2.4</td>\n\t\t\t<td class="left2">5.2</td>\n\t\t\t<td class="left2">18.1</td>\n\t\t\t<td class="left2">3,476</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2010</strong></td>\n\t\t\t<td class="left2">1,222</td>\n\t\t\t<td class="left2">2.3</td>\n\t\t\t<td class="left2">4,427</td>\n\t\t\t<td class="left2">3.0</td>\n\t\t\t<td class="left2">4.8</td>\n\t\t\t<td class="left2">17.6</td>\n\t\t\t<td class="left2">3,623</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2009</strong></td>\n\t\t\t<td class="left2">1,195</td>\n\t\t\t<td class="left2">5.5</td>\n\t\t\t<td class="left2">4,297</td>\n\t\t\t<td class="left2">7.2</td>\n\t\t\t<td class="left2">4.7</td>\n\t\t\t<td class="left2">17.1</td>\n\t\t\t<td class="left2">3,597</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2008</strong></td>\n\t\t\t<td class="left2">1,132</td>\n\t\t\t<td class="left2">10.2</td>\n\t\t\t<td class="left2">4,008</td>\n\t\t\t<td class="left2">7.8</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">15.9</td>\n\t\t\t<td class="left2">3,539</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2007</strong></td>\n\t\t\t<td class="left2">1,027</td>\n\t\t\t<td class="left2">3.6</td>\n\t\t\t<td class="left2">3,717</td>\n\t\t\t<td class="left2">7.0</td>\n\t\t\t<td class="left2">4.1</td>\n\t\t\t<td class="left2">14.8</td>\n\t\t\t<td class="left2">3,618</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2006</strong></td>\n\t\t\t<td class="left2">992</td>\n\t\t\t<td class="left2">2.8</td>\n\t\t\t<td class="left2">3,474</td>\n\t\t\t<td class="left2">10.1</td>\n\t\t\t<td class="left2">4.0</td>\n\t\t\t<td class="left2">13.8</td>\n\t\t\t<td class="left2">3,504</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2005</strong></td>\n\t\t\t<td class="left2">964</td>\n\t\t\t<td class="left2">2.5</td>\n\t\t\t<td class="left2">3,157</td>\n\t\t\t<td class="left2">8.4</td>\n\t\t\t<td class="left2">3.8</td>\n\t\t\t<td class="left2">12.6</td>\n\t\t\t<td class="left2">3,273</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2004</strong></td>\n\t\t\t<td class="left2">941</td>\n\t\t\t<td class="left2">3.0</td>\n\t\t\t<td class="left2">2,913</td>\n\t\t\t<td class="left2">3.7</td>\n\t\t\t<td class="left2">3.7</td>\n\t\t\t<td class="left2">11.5</td>\n\t\t\t<td class="left2">3,096</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2003</strong></td>\n\t\t\t<td class="left2">914</td>\n\t\t\t<td class="left2">3.5</td>\n\t\t\t<td class="left2">2,810</td>\n\t\t\t<td class="left2">3.6</td>\n\t\t\t<td class="left2">3.6</td>\n\t\t\t<td class="left2">11.2</td>\n\t\t\t<td class="left2">3,075</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2002</strong></td>\n\t\t\t<td class="left2">883</td>\n\t\t\t<td class="left2">-1.9</td>\n\t\t\t<td class="left2">2,711</td>\n\t\t\t<td class="left2">7.2</td>\n\t\t\t<td class="left2">3.5</td>\n\t\t\t<td class="left2">10.8</td>\n\t\t\t<td class="left2">3,070</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2001</strong></td>\n\t\t\t<td class="left2">900</td>\n\t\t\t<td class="left2">7.3</td>\n\t\t\t<td class="left2">2,529</td>\n\t\t\t<td class="left2">5.2</td>\n\t\t\t<td class="left2">3.6</td>\n\t\t\t<td class="left2">10.1</td>\n\t\t\t<td class="left2">2,808</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2000</strong></td>\n\t\t\t<td class="left2">839</td>\n\t\t\t<td class="left2">3.5</td>\n\t\t\t<td class="left2">2,404</td>\n\t\t\t<td class="left2">7.7</td>\n\t\t\t<td class="left2">3.3</td>\n\t\t\t<td class="left2">9.5</td>\n\t\t\t<td class="left2">2,869</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1999</strong></td>\n\t\t\t<td class="left2">809</td>\n\t\t\t<td class="left2">7.4</td>\n\t\t\t<td class="left2">2,233</td>\n\t\t\t<td class="left2">12.3</td>\n\t\t\t<td class="left2">3.2</td>\n\t\t\t<td class="left2">8.9</td>\n\t\t\t<td class="left2">2,760</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1998</strong></td>\n\t\t\t<td class="left2">753</td>\n\t\t\t<td class="left2">11.2</td>\n\t\t\t<td class="left2">1,988</td>\n\t\t\t<td class="left2">25.7</td>\n\t\t\t<td class="left2">3.0</td>\n\t\t\t<td class="left2">7.9</td>\n\t\t\t<td class="left2">2,640</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1997</strong></td>\n\t\t\t<td class="left2">677</td>\n\t\t\t<td class="left2">8.3</td>\n\t\t\t<td class="left2">1,581</td>\n\t\t\t<td class="left2">26.5</td>\n\t\t\t<td class="left2">2.7</td>\n\t\t\t<td class="left2">6.3</td>\n\t\t\t<td class="left2">2,335</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1996</strong></td>\n\t\t\t<td class="left2">625</td>\n\t\t\t<td class="left2">4.3</td>\n\t\t\t<td class="left2">1,250</td>\n\t\t\t<td class="left2">11.9</td>\n\t\t\t<td class="left2">2.5</td>\n\t\t\t<td class="left2">5.0</td>\n\t\t\t<td class="left2">2,000</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1995</strong></td>\n\t\t\t<td class="left2">599</td>\n\t\t\t<td class="left2">4.4</td>\n\t\t\t<td class="left2">1,117</td>\n\t\t\t<td class="left2">17.7</td>\n\t\t\t<td class="left2">2.4</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">1,865</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1994</strong></td>\n\t\t\t<td class="left2">574</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">949</td>\n\t\t\t<td class="left2">7.2</td>\n\t\t\t<td class="left2">2.3</td>\n\t\t\t<td class="left2">3.8</td>\n\t\t\t<td class="left2">1,653</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1993</strong></td>\n\t\t\t<td class="left2">555</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">885</td>\n\t\t\t<td class="left2">2.9</td>\n\t\t\t<td class="left2">2.2</td>\n\t\t\t<td class="left2">3.5</td>\n\t\t\t<td class="left2">1,595</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1992</strong></td>\n\t\t\t<td class="left2">531</td>\n\t\t\t<td class="left2">1.9</td>\n\t\t\t<td class="left2">860</td>\n\t\t\t<td class="left2">18.9</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">1,620</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1991</strong></td>\n\t\t\t<td class="left2">521</td>\n\t\t\t<td class="left2">0.2</td>\n\t\t\t<td class="left2">723</td>\n\t\t\t<td class="left2">48.5</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">2.9</td>\n\t\t\t<td class="left2">1,388</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1990</strong></td>\n\t\t\t<td class="left2">520</td>\n\t\t\t<td class="left2">17.9</td>\n\t\t\t<td class="left2">487</td>\n\t\t\t<td class="left2">24.6</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">1.9</td>\n\t\t\t<td class="left2">937</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1989</strong></td>\n\t\t\t<td class="left2">441</td>\n\t\t\t<td class="left2">8.4</td>\n\t\t\t<td class="left2">391</td>\n\t\t\t<td class="left2">n.a.</td>\n\t\t\t<td class="left2">1.8</td>\n\t\t\t<td class="left2">1.6</td>\n\t\t\t<td class="left2">887</td>\n\t\t</tr>\n\t</tbody>\n</table>\n</div>\n\n<h5>Footnotes</h5>\n\n<div class="footnotes">\n<p><a id="fn1" name="fn1">1.</a> Values are subject to minor changes based on revisions to underlying data. <a href="#ft1">Return to text</a></p>\n\n<p>n.a. Not available.</p>\n</div>\n\n                                    \n                                    \n                                </div>\n                                \n        </div>            \n        <a id="back-top" class="icon__backTop icon-backtop" href="#" title="Scroll To Top"><span class="sr-only">Back to Top</span></a>\n\n        \n        <div class="row">\n            <div class=\'lastUpdate\' id="lastUpdate">Last Update:\n              \n                            March 09, 2021\n                          \n            </div>\n        </div>\n        </div><!-- / #content .container -->\n        <footer class="container footer">\n    <div class="row footer__content">\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="footer__heading"><span class="text-uppercase">Board of Governors</span> <em>of the</em> <span class="text-uppercase">Federal Reserve System</span></h6>\n            <ul class="list-unstyled">\n                <li class=\'footer__listItem\'><a class="footer__link" href="/aboutthefed.htm">About the Fed</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/newsevents.htm">News & Events</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/monetarypolicy.htm">Monetary Policy</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/supervisionreg.htm">Supervision & Regulation</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/paymentsystems.htm">Payment Systems</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/econres.htm">Economic Research</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/data.htm">Data</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/consumerscommunities.htm">Consumers & Communities</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/financial-stability.htm">Financial Stability</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/economic-disparities-work.htm">Federal Reserve\'s Work Related to Economic Disparities</a></li> \n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="text-uppercase footer__heading">Tools and Information</h6>\n            <ul class="list-unstyled">\n                 <li class=\'footer__listItem\'><a href="/aboutthefed/contact-us-topics.htm" class="footer__link">Contact</a></li>\n                <li class=\'footer__listItem\'><a href="/publications.htm" class="footer__link">Publications</a></li>\n                <li class=\'footer__listItem\'><a href="/foia/about_foia.htm" class="footer__link">Freedom of Information (FOIA)</a></li>\n                <li class=\'footer__listItem\'><a href="https://oig.federalreserve.gov/" class="footer__link">Office of Inspector General</a></li>\n                <li class=\'footer__listItem\'><a href="/publications/annual-report.htm" class="footer__link">Budget &amp; Performance</a> | <a href="/regreform/audit.htm" class="footer__link">Audit</a></li>\n                <li class=\'footer__listItem\'><a href="/eeo.htm" class="footer__link">No FEAR Act</a></li>\n                <li class=\'footer__listItem\'><a href="/espanol.htm" class="footer__link">Espa\xc3\xb1ol</a></li>\n                <li class=\'footer__listItem\'><a href="/website-linking-policies.htm" class="footer__link">Website Policies</a>  | <a href="/privacy.htm" class="footer__link">Privacy Program</a></li>\n                <li class=\'footer__listItem\'><a href="/accessibility.htm" class="footer__link">Accessibility</a></li>\n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <div class="footer__social">\n                <h6 class="text-uppercase footer__heading footer__heading--social">Stay Connected</h6>\n                \n\n\n    \n                <a class="icon__md icon footer__btn icon-facebook-color" href="https://www.facebook.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-twitter-color" href="https://twitter.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-youtube-color" href="https://www.youtube.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-flickr-color" href="https://www.flickr.com/photos/federalreserve/"><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-linkedin-color" href="https://www.linkedin.com/company/federal-reserve-board"><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-rss-color" href="/feeds/feeds.htm"><span class=\'sr-only\'>Subscribe to RSS</span></a>\n            \n                <a class="icon__md icon footer__btn icon-email-color" href="/subscribe.htm"><span class=\'sr-only\'>Subscribe to Email</span></a>\n                    \n    \n \n \n            </div>\n            <a href="https://www.usa.gov/" target="_blank" title="Link to USA.gov">\n                <img class=\'footer__image\' src="/images/USAGov%402x.png" alt="Link to USA.gov" />\n            </a>\n            <a href="/open/open.htm" target="_blank" title="Link to Open.gov">\n                <img class=\'footer__image\' src="/images/OpenGov%402x.png" alt="Link to Open.gov" />\n            </a>\n        </div>\n    </div>\n    <div class="row">\n        <div class="col-xs-12 footer__footer">\n            <p class="text-uppercase footer__text footer__text--left">Board of Governors <em class="text-lowercase">of the</em> Federal Reserve System</p>\n            <p class="footer__text footer__text--right">20th Street and Constitution Avenue N.W., Washington, DC 20551</p>\n        </div>\n    </div>\n</footer>\n\n\n                <script src="/js/jquery1.11.1.min.js" type="text/javascript"></script>\n                <script src="/js/bootstrap.min.js" type="text/javascript"></script>\n                <script src="/js/scripts.js" type="text/javascript"></script>\n                <script src="/js/modal.js" type="text/javascript"></script>\n                <script src="/js/sticky-tables.js" type="text/javascript"></script>\n                <script type="text/javascript" src="/js/slides/ekko-lightbox.min.js"></script>\n                <script src="/js/t4-nav-controller.js" type="text/javascript"></script>\n              \n<script type="text/javascript">\n    $(document).ready(function($) {\n        // delegate calls to data-toggle="lightbox"\n        $(document).delegate(\'*[data-toggle="lightbox"]:not([data-gallery="navigateTo"])\', \'click\', function(event) {\n            event.preventDefault();\n            $(this).ekkoLightbox();\n        });\n        $(\'a[data-click="false"]\').click(function(e) {\n            e.preventDefault();\n            var target = $(this).data(\'target\');\n            $(target).click();\n        });\n    });\n</script>          \n             \n\n\n        \n        <!--<ul></ul> -->\n    <script type="text/javascript" src="/resources/track_downloads.js"></script>\n</body>\n</html>\n '  # noqa: E501
            result = ACH().get_data(scenario)
            self.assertIsInstance(result, list)
            self.assertEqual(len(result), 3)
            self.assertIsInstance(result, list)
            self.assertEqual(len(result), 3)

    def test_commercial(self) -> None:
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\xef\xbb\xbf<!doctype html>\n<html lang="en" class="no-js">\n    <head>\n        <meta charset="utf-8">\n        <meta http-equiv="X-UA-Compatible" content="IE=edge"/>\n        <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0 maximum-scale=1.6, user-scalable=1"/>\n        <meta name="keywords" content="Board of Governors of the Federal Reserve System, Federal Reserve Board of Governors, Federal Reserve Board, Federal Reserve" />\n        <meta name="description" content="The Federal Reserve Board of Governors in Washington DC." />\n        <meta property="og:site_name" content="Board of Governors of the Federal Reserve System"/>\n        <meta property="og:type" content="article" /> \n        <meta property="og:image"  content="" /> \n        <meta name="twitter:card" content="summary" />\n        <meta name="twitter:image" content="" />\n        \n\n\n \n        <title>Federal Reserve Board - Commercial Automated Clearinghouse Transactions Processed by the Federal Reserve--Annual Data</title>\n        \n    <link href="/css/bootstrap.css" rel="stylesheet" type="text/css">\n    <link href="/css/bluesteel-theme.css" rel="stylesheet" type="text/css">\n    <script src="/js/modernizr-latest.js" type="text/javascript"></script>\n    <script src="/js/grunticon.loader.js"></script>\n    <script>\n        grunticon(["/css/icons.data.svg.css", "/css/icons.data.png.css", "/css/icons.fallback.css"]);\n    </script>\n    <noscript>\n        <link href="/css/icons.fallback.css" rel="stylesheet">\n    </noscript>                \n    <script src="/js/angular.min.js" type="text/javascript"></script>\n    <script src="/js/app.js" type="text/javascript"></script>\n        \n    <!-- For Linear Gradients in IE9 or greater -->\n    <!--[if gte IE 9]>\n        <style type="text/css">\n          .gradient {\n             filter: none;\n          }\n        </style>\n      <![endif]-->\n\n\n \n    <script type="text/javascript" >\n  (function(i,s,o,g,r,a,m){i[\'GoogleAnalyticsObject\']=r;i[r]=i[r]||function(){\n  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),\n  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)\n  })(window,document,\'script\',\'//www.google-analytics.com/analytics.js\',\'ga\');\n  ga(\'create\', \'UA-35121701-1\', \'federalreserve.gov\', {\'cookieExpires\': 0});\n  ga(\'set\', \'anonymizeIp\', true);\n  ga(\'send\', \'pageview\');\n</script></head>\n    <body data-ng-app="pubwebApp">\n        <a href="#content" class="globalskip">Skip to main content</a>\n<nav class="nav-primary-mobile" id="nav-primary-mobile">\n</nav>\n<div class="t1_nav navbar navbar-default navbar-fixed-top" role="navigation">\n    <div class="t1_container clearfix">\n        <a id="logo" class="btn btn-default icon-FRB_logo-bw visible-xs-table-cell t1__btnlogo" href="/default.htm" role="button"><span class="sr-only">Back to Home</span>\n        </a>\n        <a href="/default.htm" id="banner" class="visible-xs-table-cell t1__banner" title="Link to Home Page" role="button">Board of Governors of the Federal Reserve System</a>\n        <ul class="nav navbar-nav visible-md visible-lg t1_toplinks">\n            <li class="navbar-text navbar-text__social">\n                <span class="navbar-link" tabindex="0">Stay Connected</span>\n                <ul class="nav navbar-nav t1_social">\n                    \n\n\n    \n                <li class="social__item"><a data-icon="facebook" href="https://www.facebook.com/federalreserve" class="noIcon"><div class="icon-facebook-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="twitter" href="https://twitter.com/federalreserve" class="noIcon"><div class="icon-twitter-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="youtube" href="https://www.youtube.com/federalreserve" class="noIcon"><div class="icon-youtube-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="flickr" href="https://www.flickr.com/photos/federalreserve/" class="noIcon"><div class="icon-flickr-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="linkedin" href="https://www.linkedin.com/company/federal-reserve-board" class="noIcon"><div class="icon-linkedin-color icon icon__sm"></div><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="rss" href="/feeds/feeds.htm" class="noIcon"><div class="icon-rss-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to RSS</span></a></li>\n            \n                <li class="social__item"><a data-icon="email" href="/subscribe.htm" class="noIcon"><div class="icon-email-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to Email</span></a></li>\n                    \n    \n \n \n                </ul>\n            </li>\n           \r\n                                <li class="navbar-text"><a class="navbar-link" href="/recentpostings.htm">Recent Postings</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/newsevents/calendar.htm">Calendar</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/publications.htm">Publications</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/sitemap.htm">Site Map</a></li>\r\n                \r\n                                <li class="navbar-text"><a class="navbar-link" href="/azindex.htm">A-Z index</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/careers.htm">Careers</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/faqs.htm">FAQs</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/videos.htm">Videos</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/aboutthefed/contact-us-topics.htm">Contact</a></li>\r\n                \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \n        </ul>\n        <form class="nav navbar-form hidden-xs nav__search" role="search" action="//www.fedsearch.org/board_public/search" method="get">\n            <div class="form-group input-group input-group-sm">\n                <label for="t1search" class=\'sr-only\'>Search</label>\n                <input id="t1search" class="nav__input form-control" name="text" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                    <button type="submit" class="nav__button btn btn-default" id="headerTopLinksSearchFormSubmit" name="Search">\n                        <span class="sr-only">Submit Search Button</span>\n                        <span class="icon-magnifying-glass icon icon__xs icon--right"></span>\n                    </button>\n                </span>\n            </div>\n            <div class=\'nav__advanced\'><a class="noIcon" href="//www.fedsearch.org/board_public">Advanced</a></div>\n        </form>        \n        <div class="btn-group visible-xs-table-cell visible-sm-table-cell t1__dropdown">\n            <span class="icon icon__md icon--right dropdown-toggle icon-options-nav" data-toggle="dropdown">\n                <span class="sr-only">Toggle Dropdown Menu</span>\n            </span>\n        </div>\n    </div>\n</div>\n<header class="jumbotron hidden-xs">\n    <a class="jumbotron__link" href="/default.htm" title="Link to Home Page">\n        <div class="container-fluid">\n            <h1 class="jumbotron__heading">Board of Governors of the Federal Reserve System</h1>\n            <p class="jumbotron__content"><em>The Federal Reserve, the central bank of the United States, provides the nation with a safe, flexible, and stable monetary and financial system.</em></p>\n        </div>\n    </a>\n</header>\n<div class="t2__offcanvas visible-xs-block">\n    <div class="navbar-header">\n        <button type="button" class="offcanvas__toggle icon-offcanvas-nav" data-toggle="offcanvas" data-target="#offcanvasT2Menu" data-canvas="body">\n            <span class="sr-only">Main Menu Toggle Button</span>\n        </button>\n        <span class="navbar-brand offcanvas__title">Sections</span>\n        <button type="button" class="offcanvas__search icon-search-nav" data-toggle="collapse" data-target="#navbar-collapse">\n            <span class="sr-only">Search Toggle Button</span>\n        </button>\n    </div>\n    <div class="navbar-collapse collapse mobile-form" id="navbar-collapse">\n        <form method="get" class="navbar-form" role="search" action="//www.fedsearch.org/board_public/search">\n            <div class="form-group input-group mobileSearch">\n                <label class="sr-only" for="t2search">Search</label>\n                <input id="t2search" name="text" class="form-control" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                <span class="sr-only">Search Submit Button</span>\n                <button type="submit" class="btn btn-default">Submit</button>\n                </span>\n            </div>\n        </form>\n    </div>\n</div>\n<nav class="nav-primary navbar hidden-xs" id="nav-primary" role="navigation">\n    <ul class="nav navbar-nav" role="menubar">\n        \n \n<li role="menuitem" class="dropdown nav-about  dropdown--3Col">\n    <a href="/aboutthefed.htm" class="sr-only-focusable" aria-haspopup="true" id="aboutMenu">About<br />the Fed</a>\n    <ul aria-labelledby="aboutMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-4 list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/structure-federal-reserve-system.htm">Structure of the Federal Reserve System</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/pf.htm">Purposes &amp; Functions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/bios/board/default.htm">Board Members</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/advisorydefault.htm">Advisory Councils</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/federal-reserve-system.htm">Federal Reserve Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fract.htm">Federal Reserve Act</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/currency.htm">Currency</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardmeetings/meetingdates.htm">Board Meetings</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardvotes.htm">Board Votes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/diversityinclusion.htm">Diversity & Inclusion</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/careers.htm">Careers</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/procurement/about.htm">Do Business with the Board</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/k8.htm">Holidays Observed - K.8</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/contact-us-topics.htm">Contact </a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/foia/about_foia.htm">Requesting Information (FOIA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/faqs.htm">FAQs</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/educational-tools/default.htm">Economic Education</a>\n</li>\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fed-financial-statements.htm">Fed Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/innovation.htm">Innovation</a>\n</li>\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-news  dropdown--1Col">\n    <a href="/newsevents.htm" class="sr-only-focusable" aria-haspopup="true" id="newsMenu">News<br />&amp; Events</a>\n    <ul aria-labelledby="newsMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        \n<li>\n    <a class="sr-only-focusable" href="/newsevents/pressreleases.htm">Press Releases</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/speeches.htm">Speeches</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/testimony.htm">Testimony</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/calendar.htm">Calendar</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/videos.htm">Videos\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/photogallery.htm">Photo Gallery\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-monetary  dropdown--2Col">\n    <a href="/monetarypolicy.htm" class="sr-only-focusable" aria-haspopup="true" id="monetaryMenu">Monetary<br />Policy</a>\n    <ul aria-labelledby="monetaryMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Federal Open Market Committee</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc.htm">About the FOMC</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomccalendars.htm">Meeting calendars and information</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_historical.htm">Transcripts and other historical materials</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_projectionsfaqs.htm">FAQs</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Monetary Policy Principles and Practice</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/monetary-policy-principles-and-practice.htm">Notes</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Policy Implementation</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policy-normalization.htm">Policy Normalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policytools.htm">Policy Tools</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/mpr_default.htm">Monetary Policy Report</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/beige-book-default.htm">Beige Book</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/quarterly-balance-sheet-developments-report.htm">Quarterly Report on Federal Reserve Balance Sheet Developments</a>\n</li>\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Review of Monetary Policy Strategy, Tools, and Communications</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/review-of-monetary-policy-strategy-tools-and-communications.htm">Overview</a>\n</li>\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-supervision  dropdown--fw">\n    <a href="/supervisionreg.htm"  class="sr-only-focusable" id="supervisionMenu" aria-haspopup="true">Supervision<br />&amp; Regulation</a>\n    <ul aria-labelledby="supervisionMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Institution Supervision</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/community-and-regional-financial-institutions.htm">Community & Regional Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/large-financial-institutions.htm">Large Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/foreign-banking-organizations.htm">Foreign Banking Organizations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/financial-market-utility-supervision.htm">Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/consumer-compliance.htm">Consumer Compliance</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supervision-and-regulation-report.htm">Federal Reserve Supervision and Regulation Report</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reporting Forms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/default.aspx">All Reporting Forms</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reporting-forms-whats-new.htm">Recent Updates</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/review.aspx">Information collections under review</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?xTdTa+FaSlI=">Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?+PhkJwiBjbA=">Applications/structure change</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?1/RIXO/UduI=">Federal Financial Institutions Examination Council (FFIEC)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?L877VwEj9Ss=">Municipal &amp; Government Securities</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Supervision &amp; Regulation Letters</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/srletters/srletters.htm">By Year</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/topics/topics.htm">By Topic</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Banking Applications &amp; Legal Developments</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/application-process.htm">Application Process</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/board-and-reserve-bank-actions.htm">Board & Reserve Bank Action</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/legal-developments.htm">Enforcement Actions & Legal Developments</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Regulatory Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">Regulations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supmanual.htm">Manuals</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/basel/basel-default.htm">Basel Regulatory Framework</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/volcker-rule.htm">Volcker Rule</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/training-and-assistance-landing.htm">Education, Training, and Assistance</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Banking &amp; Data Structure</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/insider.aspx">Beneficial Ownership reports</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/">U.S. Offices of Foreign Entities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/fhc.htm">Financial Holding Companies</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/isb-default.htm">Interstate Branching</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/suds.htm">Securities Underwriting & Dealing Subsidiaries</a>\n</li>\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-payment  dropdown--fw">\n    <a href="/paymentsystems.htm"  class="sr-only-focusable" id="paymentMenu" aria-haspopup="true">Payment<br />Systems</a>\n    <ul aria-labelledby="paymentMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations &amp; Statutes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regcc-about.htm">Regulation CC (Availability of Funds and Collection of Checks)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regii-about.htm">Regulation II (Debit Card Interchange Fees and Routing)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/reghh-about.htm">Regulation HH (Financial Market Utilities)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/other-regulations.htm">Other Regulations and Statutes</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Payment Policies</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pfs_about.htm">Federal Reserve\'s Key Policies for the Provision of Financial Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/oo_about.htm">Overnight Overdrafts</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_about.htm">Payment System Risk</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/telecomm.htm">Sponsorship for Priority Telecommunication Services</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reserve Bank Payment Services &amp; Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/check_about.htm">Check Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_data.htm">Daylight Overdrafts and Fees</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fednow_about.htm">FedNow\xe2\x84\xa0 Service</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedfunds_about.htm">Fedwire Funds Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedsecs_about.htm">Fedwire Securities Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fisagy_about.htm">Fiscal Agency Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/natl_about.htm">National Settlement Service</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Financial Market Utilities &amp; Infrastructures</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/over_about.htm">Supervision & Oversight of Financial Market Infrastructures</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/designated_fmu_about.htm">Designated Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/int_standards.htm">International Standards for Financial Market Infrastructures</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Research, Committees, and Forums</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/payres_about.htm">Payment Research</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fr-payments-study.htm">Federal Reserve Payments Study (FRPS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pspa_committee.htm">Payments System Policy Advisory Committee</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/forums.htm">Forums</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-econ dropdown--right dropdown--2Col">\n    <a href="/econres.htm" class="sr-only-focusable" aria-haspopup="true" id="econMenu">Economic<br />Research</a>\n    <ul aria-labelledby="econMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Working Papers and Notes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/feds/index.htm">Finance and Economics Discussion Series (FEDS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/notes/feds-notes/default.htm">FEDS Notes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/ifdp/index.htm">International Finance Discussion Papers (IFDP)</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/econres/theeconomists.htm">Meet the Economists</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Models and Tools</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/us-models-about.htm">FRB/US Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/edo-models-about.htm">Estimated Dynamic Optimization (EDO) Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n\n\n\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-data dropdown--right dropdown--fw">\n    <a href="/data.htm"  class="sr-only-focusable" id="dataMenu" aria-haspopup="true">Data</a>\n    <ul aria-labelledby="dataMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/datadownload"><span class="icon icon-ddp icon__sm"></span> Data Download Program</a>\n</li>\n\n\n<li class="nav__header">\n    <p>\n        <strong>Bank Assets and Liabilities</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h3/current/default.htm">Aggregate Reserves of Depository Institutions and the Monetary Base - H.3</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h8/current/default.htm">Assets and Liabilities of Commercial Banks in the U.S. - H.8</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/assetliab/current.htm">Assets and Liabilities of U.S. Branches and Agencies of Foreign Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/chargeoff/">Charge-Off and Delinquency Rates on Loans and Leases at Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sfos/sfos.htm">Senior Financial Officer Survey</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sloos.htm">Senior Loan Officer Opinion Survey on Bank Lending Practices</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Bank Structure Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for the U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Business Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/cp/">Commercial Paper</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g20/current/g20.htm">Finance Companies - G.20</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/govsecure/current.htm">New Security Issues, State and Local Governments</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/corpsecure/current.htm">New Security Issues, U.S. Corporations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Dealer Financing Terms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/scoos.htm">Senior Credit Officer Opinion Survey on Dealer Financing Terms</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Exchange Rates and International Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h10/current/">Foreign Exchange Rates - H.10/G.5</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/intlsumm/current.htm">International Summary Statistics</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/secholdtrans/current.htm">Securities Holdings and Transactions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/statbanksus/current.htm">Statistics Reported by Banks and Other Financial Firms in the United States</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Financial Accounts</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/z1/default.htm">Financial Accounts of the United States - Z.1</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Household Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g19/current/default.htm">Consumer Credit - G.19</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/housedebt/default.htm">Household Debt Service and Financial Obligations Ratios</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mortoutstand/current.htm">Mortgage Debt Outstanding</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Industrial Activity</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g17/current/default.htm">Industrial Production and Capacity Utilization - G.17</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Interest Rates</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h15/">Selected Interest Rates - H.15</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Micro Data Reference Manual (MDRM)</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mdrm.htm">Micro and Macro Data Collections</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Money Stock and Reserve Balances</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h41/">Factors Affecting Reserve Balances - H.4.1</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h6/current/default.htm">Money Stock Measures - H.6</a>\n</li>\n\n\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Other</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/yield-curve-models.htm">Yield Curve Models and Data</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-consumer dropdown--right dropdown--4Col">\n    <a href="/consumerscommunities.htm" class="sr-only-focusable" aria-haspopup="true" id="consumerMenu">Consumers<br />&amp; Communities</a>\n    <ul aria-labelledby="consumerMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cra_about.htm">Community Reinvestment Act (CRA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">All Regulations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Supervision&nbsp;&amp; Enforcement</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/caletters/caletters.htm">CA Letters</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/enforcementactions/default.aspx">Enforcement Actions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/independent-foreclosure-review.htm">Independent Foreclosure Review</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Community Development</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/neighborhood-revitalization.htm">Housing and Neighborhood Revitalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/small-business-and-entrepreneurship.htm">Small Business and Entrepreneurship</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/workforce.htm">Employment and Workforce Development</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cdf.htm">Community Development Finance</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/rural-community-economic-development.htm">Rural Community and Economic Development</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Research&nbsp;&amp; Analysis</strong>\n    </p>\n</li>\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/community-development-publications.htm">Research Publications & Data Analysis</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/cac.htm">Community Advisory Council</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Consumer Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/foreclosure.htm">Mortgage and Foreclosure Resources</a>\n</li>\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/comm-dev-system-map.htm">Federal Reserve Community Development Resources</a>\n</li>\n\n\n\n                </ul>\n            </div>\n        </li>             \n    </ul>\n</li>\n \n \n \n \n \n \n \n \n \n \n \n \n    </ul>\n</nav>\n        <!-- BODY CONTENT -->\n        <div id="content" class="container container__main" role="main">\n            <div class="row">\n                <div class="page-header">\n                    <ol class="breadcrumb">\n                            <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/default.htm">Home</a></li>\n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems.htm">Payment Systems</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/fedach_data.htm">Data</a></li>\n                                          \n \n                                          \n \n                                          \n \n                                          \n \n \n                    </ol>\n                    <div class="header-group">\n                        <div id="page-title" class="page-title"> \n                        \n                                \t\t<h2>\n                                \t\t    Automated Clearinghouse Services\n                                \t\t</h2>\n                            \t\t\n                        </div> \n                        \n                  </div>\n                </div><!-- / .page-header -->\n            \n            </div><!-- / .row -->\n  \n\n\n        <div class=\'row\'>\n        \n                                <div class="col-xs-12 col-sm-4 col-md-3">\n                                  <div id="t4_nav" class="t4_nav list-group sticky">\n                                    \n                                    \r\n\r\n\r\n\r\n<!-- <h3 class="text-uppercase">Chapters</h3> -->\r\n\r\n\r\n\r\n\r\n<a href="/paymentsystems/fedach_data.htm" class="list-group-item" title="">Data</a>\n                                  </div>\n                                </div>\n                        \n                                                    <div id="article" class="col-xs-12 col-sm-8 col-md-9">\n                                                    <h3>Commercial Automated Clearinghouse Transactions Processed by the Federal Reserve--Annual Data</h3>\n                                                \n                                <!-- Article Area -->\n                                    \n                                    <div id="floatRightRail" class="col-xs-12 col-sm-12 col-md-5 pull-right">\n                                        \n                                        \n\n<!-- Text Box Begins -->\n\n<div class="panel panel-related">\n    <div class="panel-heading">\n                   \n                <h5 class="panel-title text-capitalize">\n                    Other Formats  \n                </h5>\n                \n    </div>\n    <div class="panel-body">\n        <ul>\n\t<li><a href="/paymentsystems/files/fedach_yearlycomm.txt">ASCII</a></li>\n\t<li><a href="/paymentsystems/files/fedach_yearlycomm.pdf">PDF (102&nbsp;KB)</a></li>\n</ul>\n    </div>\n</div>\n\n\n\n\n            \n                                    </div>     \n                                    \n                                    \n                                    <p>The daily average volume and average daily value of items are based on the number of business days in each year.<a id="ft1" name="ft1"></a><a href="#fn1" title="footnote 1"><sup>1</sup></a></p>\n\n<div class="data-table" data-table-popout="" id="data-table-0">\n<table class="pubtables" data-other-sticky-ids="navbar-mobile,nav-primary" data-sticky-columns="1" data-sticky-rows="1">\n\t<thead>\n\t\t<tr class="header">\n\t\t\t<th align="center" scope="col" valign="bottom">Year</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Volume<br />\n\t\t\t(millions<br />\n\t\t\tof items)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Volume<br />\n\t\t\tpercent<br />\n\t\t\tchange</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Value<br />\n\t\t\t(billions<br />\n\t\t\tof dollars)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Value<br />\n\t\t\tpercent<br />\n\t\t\tchange</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average daily<br />\n\t\t\tvolume<br />\n\t\t\t(millions<br />\n\t\t\tof items)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average daily<br />\n\t\t\tvalue<br />\n\t\t\t(billions<br />\n\t\t\tof dollars)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average value<br />\n\t\t\tper<br />\n\t\t\ttransaction<br />\n\t\t\t(dollars)</th>\n\t\t</tr>\n\t</thead>\n\t<tbody>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2020</strong></td>\n\t\t\t<td class="left2">16,549</td>\n\t\t\t<td class="left2">6.2</td>\n\t\t\t<td class="left2">31,446</td>\n\t\t\t<td class="left2">12.0</td>\n\t\t\t<td class="left2">64.6</td>\n\t\t\t<td class="left2">122.8</td>\n\t\t\t<td class="left2">1,900</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2019</strong></td>\n\t\t\t<td class="left2">15,584</td>\n\t\t\t<td class="left2">6.1</td>\n\t\t\t<td class="left2">28,082</td>\n\t\t\t<td class="left2">8.6</td>\n\t\t\t<td class="left2">62.1</td>\n\t\t\t<td class="left2">111.9</td>\n\t\t\t<td class="left2">1,802</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2018</strong></td>\n\t\t\t<td class="left2">14,692</td>\n\t\t\t<td class="left2">6.9</td>\n\t\t\t<td class="left2">25,860</td>\n\t\t\t<td class="left2">10.5</td>\n\t\t\t<td class="left2">58.5</td>\n\t\t\t<td class="left2">103.0</td>\n\t\t\t<td class="left2">1,760</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2017</strong></td>\n\t\t\t<td class="left2">13,749</td>\n\t\t\t<td class="left2">6.1</td>\n\t\t\t<td class="left2">23,399</td>\n\t\t\t<td class="left2">7.5</td>\n\t\t\t<td class="left2">55.0</td>\n\t\t\t<td class="left2">93.6</td>\n\t\t\t<td class="left2">1,702</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2016</strong></td>\n\t\t\t<td class="left2">12,960</td>\n\t\t\t<td class="left2">5.4</td>\n\t\t\t<td class="left2">21,772</td>\n\t\t\t<td class="left2">5.9</td>\n\t\t\t<td class="left2">51.6</td>\n\t\t\t<td class="left2">86.7</td>\n\t\t\t<td class="left2">1,680</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2015</strong></td>\n\t\t\t<td class="left2">12,298</td>\n\t\t\t<td class="left2">5.8</td>\n\t\t\t<td class="left2">20,565</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">49.0</td>\n\t\t\t<td class="left2">81.9</td>\n\t\t\t<td class="left2">1,672</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2014</strong></td>\n\t\t\t<td class="left2">11,620</td>\n\t\t\t<td class="left2">4.3</td>\n\t\t\t<td class="left2">19,891</td>\n\t\t\t<td class="left2">1.0</td>\n\t\t\t<td class="left2">46.3</td>\n\t\t\t<td class="left2">79.2</td>\n\t\t\t<td class="left2">1,712</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2013</strong></td>\n\t\t\t<td class="left2">11,143</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">19,689</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">44.4</td>\n\t\t\t<td class="left2">78.4</td>\n\t\t\t<td class="left2">1,767</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2012</strong></td>\n\t\t\t<td class="left2">10,665</td>\n\t\t\t<td class="left2">3.1</td>\n\t\t\t<td class="left2">19,294</td>\n\t\t\t<td class="left2">8.4</td>\n\t\t\t<td class="left2">42.5</td>\n\t\t\t<td class="left2">76.9</td>\n\t\t\t<td class="left2">1,809</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2011</strong></td>\n\t\t\t<td class="left2">10,349</td>\n\t\t\t<td class="left2">1.1</td>\n\t\t\t<td class="left2">17,802</td>\n\t\t\t<td class="left2">5.1</td>\n\t\t\t<td class="left2">41.2</td>\n\t\t\t<td class="left2">70.9</td>\n\t\t\t<td class="left2">1,720</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2010</strong></td>\n\t\t\t<td class="left2">10,233</td>\n\t\t\t<td class="left2">2.7</td>\n\t\t\t<td class="left2">16,941</td>\n\t\t\t<td class="left2">9.9</td>\n\t\t\t<td class="left2">40.6</td>\n\t\t\t<td class="left2">67.2</td>\n\t\t\t<td class="left2">1,656</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2009</strong></td>\n\t\t\t<td class="left2">9,966</td>\n\t\t\t<td class="left2">-0.7</td>\n\t\t\t<td class="left2">15,419</td>\n\t\t\t<td class="left2">-1.6</td>\n\t\t\t<td class="left2">39.5</td>\n\t\t\t<td class="left2">61.2</td>\n\t\t\t<td class="left2">1,547</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2008</strong></td>\n\t\t\t<td class="left2">10,040</td>\n\t\t\t<td class="left2">7.2</td>\n\t\t\t<td class="left2">15,663</td>\n\t\t\t<td class="left2">7.7</td>\n\t\t\t<td class="left2">39.8</td>\n\t\t\t<td class="left2">62.2</td>\n\t\t\t<td class="left2">1,560</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2007</strong></td>\n\t\t\t<td class="left2">9,363</td>\n\t\t\t<td class="left2">13.8</td>\n\t\t\t<td class="left2">14,547</td>\n\t\t\t<td class="left2">10.8</td>\n\t\t\t<td class="left2">37.3</td>\n\t\t\t<td class="left2">58.0</td>\n\t\t\t<td class="left2">1,554</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2006</strong></td>\n\t\t\t<td class="left2">8,231</td>\n\t\t\t<td class="left2">12.2</td>\n\t\t\t<td class="left2">13,124</td>\n\t\t\t<td class="left2">2.5</td>\n\t\t\t<td class="left2">32.8</td>\n\t\t\t<td class="left2">52.3</td>\n\t\t\t<td class="left2">1,595</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2005</strong></td>\n\t\t\t<td class="left2">7,339</td>\n\t\t\t<td class="left2">13.1</td>\n\t\t\t<td class="left2">12,802</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">29.2</td>\n\t\t\t<td class="left2">51.0</td>\n\t\t\t<td class="left2">1,744</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2004</strong></td>\n\t\t\t<td class="left2">6,486</td>\n\t\t\t<td class="left2">16.1</td>\n\t\t\t<td class="left2">12,544</td>\n\t\t\t<td class="left2">-10.1</td>\n\t\t\t<td class="left2">25.6</td>\n\t\t\t<td class="left2">49.6</td>\n\t\t\t<td class="left2">1,934</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2003</strong></td>\n\t\t\t<td class="left2">5,588</td>\n\t\t\t<td class="left2">12.1</td>\n\t\t\t<td class="left2">13,952</td>\n\t\t\t<td class="left2">6.2</td>\n\t\t\t<td class="left2">22.3</td>\n\t\t\t<td class="left2">55.6</td>\n\t\t\t<td class="left2">2,497</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2002</strong></td>\n\t\t\t<td class="left2">4,986</td>\n\t\t\t<td class="left2">12.1</td>\n\t\t\t<td class="left2">13,135</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">19.9</td>\n\t\t\t<td class="left2">52.3</td>\n\t\t\t<td class="left2">2,634</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2001</strong></td>\n\t\t\t<td class="left2">4,448</td>\n\t\t\t<td class="left2">16.7</td>\n\t\t\t<td class="left2">12,707</td>\n\t\t\t<td class="left2">9.4</td>\n\t\t\t<td class="left2">17.7</td>\n\t\t\t<td class="left2">50.6</td>\n\t\t\t<td class="left2">2,857</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2000</strong></td>\n\t\t\t<td class="left2">3,812</td>\n\t\t\t<td class="left2">14.0</td>\n\t\t\t<td class="left2">11,620</td>\n\t\t\t<td class="left2">7.0</td>\n\t\t\t<td class="left2">15.1</td>\n\t\t\t<td class="left2">46.1</td>\n\t\t\t<td class="left2">3,048</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1999</strong></td>\n\t\t\t<td class="left2">3,344</td>\n\t\t\t<td class="left2">12.7</td>\n\t\t\t<td class="left2">10,862</td>\n\t\t\t<td class="left2">5.1</td>\n\t\t\t<td class="left2">13.3</td>\n\t\t\t<td class="left2">43.1</td>\n\t\t\t<td class="left2">3,248</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1998</strong></td>\n\t\t\t<td class="left2">2,966</td>\n\t\t\t<td class="left2">13.9</td>\n\t\t\t<td class="left2">10,338</td>\n\t\t\t<td class="left2">13.2</td>\n\t\t\t<td class="left2">11.8</td>\n\t\t\t<td class="left2">41.0</td>\n\t\t\t<td class="left2">3,486</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><a id="ft2" name="ft2"> </a><strong>1997<a href="#fn2" title="footnote 2"><sup>2</sup></a></strong></td>\n\t\t\t<td class="left2">2,603</td>\n\t\t\t<td class="left2">9.7</td>\n\t\t\t<td class="left2">9,129</td>\n\t\t\t<td class="left2">10.1</td>\n\t\t\t<td class="left2">10.4</td>\n\t\t\t<td class="left2">36.4</td>\n\t\t\t<td class="left2">3,507</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1996</strong></td>\n\t\t\t<td class="left2">2,372</td>\n\t\t\t<td class="left2">15.9</td>\n\t\t\t<td class="left2">8,288</td>\n\t\t\t<td class="left2">6.0</td>\n\t\t\t<td class="left2">9.4</td>\n\t\t\t<td class="left2">32.9</td>\n\t\t\t<td class="left2">3,494</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1995</strong></td>\n\t\t\t<td class="left2">2,046</td>\n\t\t\t<td class="left2">17.8</td>\n\t\t\t<td class="left2">7,817</td>\n\t\t\t<td class="left2">10.2</td>\n\t\t\t<td class="left2">8.2</td>\n\t\t\t<td class="left2">31.1</td>\n\t\t\t<td class="left2">3,821</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1994</strong></td>\n\t\t\t<td class="left2">1,737</td>\n\t\t\t<td class="left2">16.9</td>\n\t\t\t<td class="left2">7,094</td>\n\t\t\t<td class="left2">9.9</td>\n\t\t\t<td class="left2">6.9</td>\n\t\t\t<td class="left2">28.3</td>\n\t\t\t<td class="left2">4,084</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1993</strong></td>\n\t\t\t<td class="left2">1,486</td>\n\t\t\t<td class="left2">12.0</td>\n\t\t\t<td class="left2">6,455</td>\n\t\t\t<td class="left2">-1.2</td>\n\t\t\t<td class="left2">5.9</td>\n\t\t\t<td class="left2">25.6</td>\n\t\t\t<td class="left2">4,344</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1992</strong></td>\n\t\t\t<td class="left2">1,327</td>\n\t\t\t<td class="left2">18.6</td>\n\t\t\t<td class="left2">6,531</td>\n\t\t\t<td class="left2">17.7</td>\n\t\t\t<td class="left2">5.2</td>\n\t\t\t<td class="left2">25.8</td>\n\t\t\t<td class="left2">4,922</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1991</strong></td>\n\t\t\t<td class="left2">1,119</td>\n\t\t\t<td class="left2">22.3</td>\n\t\t\t<td class="left2">5,549</td>\n\t\t\t<td class="left2">32.9</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">22.1</td>\n\t\t\t<td class="left2">4,959</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><a id="ft3" name="ft3"> </a><strong>1990<a href="#fn3" title="footnote 3"><sup>3</sup></a></strong></td>\n\t\t\t<td class="left2">915</td>\n\t\t\t<td class="left2">23.6</td>\n\t\t\t<td class="left2">4,174</td>\n\t\t\t<td class="left2">8.7</td>\n\t\t\t<td class="left2">3.6</td>\n\t\t\t<td class="left2">16.6</td>\n\t\t\t<td class="left2">4,562</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1989</strong></td>\n\t\t\t<td class="left2">741</td>\n\t\t\t<td class="left2">22.9</td>\n\t\t\t<td class="left2">3,840</td>\n\t\t\t<td class="left2">13.8</td>\n\t\t\t<td class="left2">3.0</td>\n\t\t\t<td class="left2">15.5</td>\n\t\t\t<td class="left2">5,182</td>\n\t\t</tr>\n\t</tbody>\n</table>\n</div>\n\n<h5>Footnotes</h5>\n\n<div class="footnotes">\n<p><a id="fn1" name="fn1">1.</a> Values are subject to minor changes based on revisions to underlying data. <a href="#ft1">Return to text</a></p>\n\n<p><a id="fn2" name="fn2">2.</a> Beginning in 1997, the reported ACH volumes no longer include non-value items. <a href="#ft2">Return to text</a></p>\n\n<p><a id="fn3" name="fn3">3.</a> Data for years preceding 1991 do not include items sent to the Reserve Banks by the New York Automated Clearinghouse. <a href="#ft3">Return to text</a></p>\n</div>\n\n                                    \n                                    \n                                </div>\n                                \n        </div>            \n        <a id="back-top" class="icon__backTop icon-backtop" href="#" title="Scroll To Top"><span class="sr-only">Back to Top</span></a>\n\n        \n        <div class="row">\n            <div class=\'lastUpdate\' id="lastUpdate">Last Update:\n              \n                            March 09, 2021\n                          \n            </div>\n        </div>\n        </div><!-- / #content .container -->\n        <footer class="container footer">\n    <div class="row footer__content">\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="footer__heading"><span class="text-uppercase">Board of Governors</span> <em>of the</em> <span class="text-uppercase">Federal Reserve System</span></h6>\n            <ul class="list-unstyled">\n                <li class=\'footer__listItem\'><a class="footer__link" href="/aboutthefed.htm">About the Fed</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/newsevents.htm">News & Events</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/monetarypolicy.htm">Monetary Policy</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/supervisionreg.htm">Supervision & Regulation</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/paymentsystems.htm">Payment Systems</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/econres.htm">Economic Research</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/data.htm">Data</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/consumerscommunities.htm">Consumers & Communities</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/financial-stability.htm">Financial Stability</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/economic-disparities-work.htm">Federal Reserve\'s Work Related to Economic Disparities</a></li> \n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="text-uppercase footer__heading">Tools and Information</h6>\n            <ul class="list-unstyled">\n                 <li class=\'footer__listItem\'><a href="/aboutthefed/contact-us-topics.htm" class="footer__link">Contact</a></li>\n                <li class=\'footer__listItem\'><a href="/publications.htm" class="footer__link">Publications</a></li>\n                <li class=\'footer__listItem\'><a href="/foia/about_foia.htm" class="footer__link">Freedom of Information (FOIA)</a></li>\n                <li class=\'footer__listItem\'><a href="https://oig.federalreserve.gov/" class="footer__link">Office of Inspector General</a></li>\n                <li class=\'footer__listItem\'><a href="/publications/annual-report.htm" class="footer__link">Budget &amp; Performance</a> | <a href="/regreform/audit.htm" class="footer__link">Audit</a></li>\n                <li class=\'footer__listItem\'><a href="/eeo.htm" class="footer__link">No FEAR Act</a></li>\n                <li class=\'footer__listItem\'><a href="/espanol.htm" class="footer__link">Espa\xc3\xb1ol</a></li>\n                <li class=\'footer__listItem\'><a href="/website-linking-policies.htm" class="footer__link">Website Policies</a>  | <a href="/privacy.htm" class="footer__link">Privacy Program</a></li>\n                <li class=\'footer__listItem\'><a href="/accessibility.htm" class="footer__link">Accessibility</a></li>\n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <div class="footer__social">\n                <h6 class="text-uppercase footer__heading footer__heading--social">Stay Connected</h6>\n                \n\n\n    \n                <a class="icon__md icon footer__btn icon-facebook-color" href="https://www.facebook.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-twitter-color" href="https://twitter.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-youtube-color" href="https://www.youtube.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-flickr-color" href="https://www.flickr.com/photos/federalreserve/"><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-linkedin-color" href="https://www.linkedin.com/company/federal-reserve-board"><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-rss-color" href="/feeds/feeds.htm"><span class=\'sr-only\'>Subscribe to RSS</span></a>\n            \n                <a class="icon__md icon footer__btn icon-email-color" href="/subscribe.htm"><span class=\'sr-only\'>Subscribe to Email</span></a>\n                    \n    \n \n \n            </div>\n            <a href="https://www.usa.gov/" target="_blank" title="Link to USA.gov">\n                <img class=\'footer__image\' src="/images/USAGov%402x.png" alt="Link to USA.gov" />\n            </a>\n            <a href="/open/open.htm" target="_blank" title="Link to Open.gov">\n                <img class=\'footer__image\' src="/images/OpenGov%402x.png" alt="Link to Open.gov" />\n            </a>\n        </div>\n    </div>\n    <div class="row">\n        <div class="col-xs-12 footer__footer">\n            <p class="text-uppercase footer__text footer__text--left">Board of Governors <em class="text-lowercase">of the</em> Federal Reserve System</p>\n            <p class="footer__text footer__text--right">20th Street and Constitution Avenue N.W., Washington, DC 20551</p>\n        </div>\n    </div>\n</footer>\n\n\n                <script src="/js/jquery1.11.1.min.js" type="text/javascript"></script>\n                <script src="/js/bootstrap.min.js" type="text/javascript"></script>\n                <script src="/js/scripts.js" type="text/javascript"></script>\n                <script src="/js/modal.js" type="text/javascript"></script>\n                <script src="/js/sticky-tables.js" type="text/javascript"></script>\n                <script type="text/javascript" src="/js/slides/ekko-lightbox.min.js"></script>\n                <script src="/js/t4-nav-controller.js" type="text/javascript"></script>\n              \n<script type="text/javascript">\n    $(document).ready(function($) {\n        // delegate calls to data-toggle="lightbox"\n        $(document).delegate(\'*[data-toggle="lightbox"]:not([data-gallery="navigateTo"])\', \'click\', function(event) {\n            event.preventDefault();\n            $(this).ekkoLightbox();\n        });\n        $(\'a[data-click="false"]\').click(function(e) {\n            e.preventDefault();\n            var target = $(this).data(\'target\');\n            $(target).click();\n        });\n    });\n</script>          \n             \n\n\n        \n        <!--<ul></ul> -->\n    <script type="text/javascript" src="/resources/track_downloads.js"></script>\n</body>\n</html>\n '  # noqa: E501
            result = ACH().commercial()
            self.assertIsInstance(result, pd.DataFrame)

    def test_government(self) -> None:
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\xef\xbb\xbf<!doctype html>\n<html lang="en" class="no-js">\n    <head>\n        <meta charset="utf-8">\n        <meta http-equiv="X-UA-Compatible" content="IE=edge"/>\n        <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0 maximum-scale=1.6, user-scalable=1"/>\n        <meta name="keywords" content="Board of Governors of the Federal Reserve System, Federal Reserve Board of Governors, Federal Reserve Board, Federal Reserve" />\n        <meta name="description" content="The Federal Reserve Board of Governors in Washington DC." />\n        <meta property="og:site_name" content="Board of Governors of the Federal Reserve System"/>\n        <meta property="og:type" content="article" /> \n        <meta property="og:image"  content="" /> \n        <meta name="twitter:card" content="summary" />\n        <meta name="twitter:image" content="" />\n        \n\n\n \n        <title>Federal Reserve Board - Government Automated Clearinghouse Transactions Processed by the Federal Reserve--Annual Data</title>\n        \n    <link href="/css/bootstrap.css" rel="stylesheet" type="text/css">\n    <link href="/css/bluesteel-theme.css" rel="stylesheet" type="text/css">\n    <script src="/js/modernizr-latest.js" type="text/javascript"></script>\n    <script src="/js/grunticon.loader.js"></script>\n    <script>\n        grunticon(["/css/icons.data.svg.css", "/css/icons.data.png.css", "/css/icons.fallback.css"]);\n    </script>\n    <noscript>\n        <link href="/css/icons.fallback.css" rel="stylesheet">\n    </noscript>                \n    <script src="/js/angular.min.js" type="text/javascript"></script>\n    <script src="/js/app.js" type="text/javascript"></script>\n        \n    <!-- For Linear Gradients in IE9 or greater -->\n    <!--[if gte IE 9]>\n        <style type="text/css">\n          .gradient {\n             filter: none;\n          }\n        </style>\n      <![endif]-->\n\n\n \n    <script type="text/javascript" >\n  (function(i,s,o,g,r,a,m){i[\'GoogleAnalyticsObject\']=r;i[r]=i[r]||function(){\n  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),\n  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)\n  })(window,document,\'script\',\'//www.google-analytics.com/analytics.js\',\'ga\');\n  ga(\'create\', \'UA-35121701-1\', \'federalreserve.gov\', {\'cookieExpires\': 0});\n  ga(\'set\', \'anonymizeIp\', true);\n  ga(\'send\', \'pageview\');\n</script></head>\n    <body data-ng-app="pubwebApp">\n        <a href="#content" class="globalskip">Skip to main content</a>\n<nav class="nav-primary-mobile" id="nav-primary-mobile">\n</nav>\n<div class="t1_nav navbar navbar-default navbar-fixed-top" role="navigation">\n    <div class="t1_container clearfix">\n        <a id="logo" class="btn btn-default icon-FRB_logo-bw visible-xs-table-cell t1__btnlogo" href="/default.htm" role="button"><span class="sr-only">Back to Home</span>\n        </a>\n        <a href="/default.htm" id="banner" class="visible-xs-table-cell t1__banner" title="Link to Home Page" role="button">Board of Governors of the Federal Reserve System</a>\n        <ul class="nav navbar-nav visible-md visible-lg t1_toplinks">\n            <li class="navbar-text navbar-text__social">\n                <span class="navbar-link" tabindex="0">Stay Connected</span>\n                <ul class="nav navbar-nav t1_social">\n                    \n\n\n    \n                <li class="social__item"><a data-icon="facebook" href="https://www.facebook.com/federalreserve" class="noIcon"><div class="icon-facebook-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="twitter" href="https://twitter.com/federalreserve" class="noIcon"><div class="icon-twitter-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="youtube" href="https://www.youtube.com/federalreserve" class="noIcon"><div class="icon-youtube-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="flickr" href="https://www.flickr.com/photos/federalreserve/" class="noIcon"><div class="icon-flickr-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="linkedin" href="https://www.linkedin.com/company/federal-reserve-board" class="noIcon"><div class="icon-linkedin-color icon icon__sm"></div><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="rss" href="/feeds/feeds.htm" class="noIcon"><div class="icon-rss-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to RSS</span></a></li>\n            \n                <li class="social__item"><a data-icon="email" href="/subscribe.htm" class="noIcon"><div class="icon-email-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to Email</span></a></li>\n                    \n    \n \n \n                </ul>\n            </li>\n           \r\n                                <li class="navbar-text"><a class="navbar-link" href="/recentpostings.htm">Recent Postings</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/newsevents/calendar.htm">Calendar</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/publications.htm">Publications</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/sitemap.htm">Site Map</a></li>\r\n                \r\n                                <li class="navbar-text"><a class="navbar-link" href="/azindex.htm">A-Z index</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/careers.htm">Careers</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/faqs.htm">FAQs</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/videos.htm">Videos</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/aboutthefed/contact-us-topics.htm">Contact</a></li>\r\n                \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \n        </ul>\n        <form class="nav navbar-form hidden-xs nav__search" role="search" action="//www.fedsearch.org/board_public/search" method="get">\n            <div class="form-group input-group input-group-sm">\n                <label for="t1search" class=\'sr-only\'>Search</label>\n                <input id="t1search" class="nav__input form-control" name="text" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                    <button type="submit" class="nav__button btn btn-default" id="headerTopLinksSearchFormSubmit" name="Search">\n                        <span class="sr-only">Submit Search Button</span>\n                        <span class="icon-magnifying-glass icon icon__xs icon--right"></span>\n                    </button>\n                </span>\n            </div>\n            <div class=\'nav__advanced\'><a class="noIcon" href="//www.fedsearch.org/board_public">Advanced</a></div>\n        </form>        \n        <div class="btn-group visible-xs-table-cell visible-sm-table-cell t1__dropdown">\n            <span class="icon icon__md icon--right dropdown-toggle icon-options-nav" data-toggle="dropdown">\n                <span class="sr-only">Toggle Dropdown Menu</span>\n            </span>\n        </div>\n    </div>\n</div>\n<header class="jumbotron hidden-xs">\n    <a class="jumbotron__link" href="/default.htm" title="Link to Home Page">\n        <div class="container-fluid">\n            <h1 class="jumbotron__heading">Board of Governors of the Federal Reserve System</h1>\n            <p class="jumbotron__content"><em>The Federal Reserve, the central bank of the United States, provides the nation with a safe, flexible, and stable monetary and financial system.</em></p>\n        </div>\n    </a>\n</header>\n<div class="t2__offcanvas visible-xs-block">\n    <div class="navbar-header">\n        <button type="button" class="offcanvas__toggle icon-offcanvas-nav" data-toggle="offcanvas" data-target="#offcanvasT2Menu" data-canvas="body">\n            <span class="sr-only">Main Menu Toggle Button</span>\n        </button>\n        <span class="navbar-brand offcanvas__title">Sections</span>\n        <button type="button" class="offcanvas__search icon-search-nav" data-toggle="collapse" data-target="#navbar-collapse">\n            <span class="sr-only">Search Toggle Button</span>\n        </button>\n    </div>\n    <div class="navbar-collapse collapse mobile-form" id="navbar-collapse">\n        <form method="get" class="navbar-form" role="search" action="//www.fedsearch.org/board_public/search">\n            <div class="form-group input-group mobileSearch">\n                <label class="sr-only" for="t2search">Search</label>\n                <input id="t2search" name="text" class="form-control" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                <span class="sr-only">Search Submit Button</span>\n                <button type="submit" class="btn btn-default">Submit</button>\n                </span>\n            </div>\n        </form>\n    </div>\n</div>\n<nav class="nav-primary navbar hidden-xs" id="nav-primary" role="navigation">\n    <ul class="nav navbar-nav" role="menubar">\n        \n \n<li role="menuitem" class="dropdown nav-about  dropdown--3Col">\n    <a href="/aboutthefed.htm" class="sr-only-focusable" aria-haspopup="true" id="aboutMenu">About<br />the Fed</a>\n    <ul aria-labelledby="aboutMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-4 list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/structure-federal-reserve-system.htm">Structure of the Federal Reserve System</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/pf.htm">Purposes &amp; Functions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/bios/board/default.htm">Board Members</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/advisorydefault.htm">Advisory Councils</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/federal-reserve-system.htm">Federal Reserve Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fract.htm">Federal Reserve Act</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/currency.htm">Currency</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardmeetings/meetingdates.htm">Board Meetings</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardvotes.htm">Board Votes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/diversityinclusion.htm">Diversity & Inclusion</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/careers.htm">Careers</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/procurement/about.htm">Do Business with the Board</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/k8.htm">Holidays Observed - K.8</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/contact-us-topics.htm">Contact </a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/foia/about_foia.htm">Requesting Information (FOIA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/faqs.htm">FAQs</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/educational-tools/default.htm">Economic Education</a>\n</li>\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fed-financial-statements.htm">Fed Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/innovation.htm">Innovation</a>\n</li>\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-news  dropdown--1Col">\n    <a href="/newsevents.htm" class="sr-only-focusable" aria-haspopup="true" id="newsMenu">News<br />&amp; Events</a>\n    <ul aria-labelledby="newsMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        \n<li>\n    <a class="sr-only-focusable" href="/newsevents/pressreleases.htm">Press Releases</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/speeches.htm">Speeches</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/testimony.htm">Testimony</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/calendar.htm">Calendar</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/videos.htm">Videos\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/photogallery.htm">Photo Gallery\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-monetary  dropdown--2Col">\n    <a href="/monetarypolicy.htm" class="sr-only-focusable" aria-haspopup="true" id="monetaryMenu">Monetary<br />Policy</a>\n    <ul aria-labelledby="monetaryMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Federal Open Market Committee</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc.htm">About the FOMC</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomccalendars.htm">Meeting calendars and information</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_historical.htm">Transcripts and other historical materials</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_projectionsfaqs.htm">FAQs</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Monetary Policy Principles and Practice</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/monetary-policy-principles-and-practice.htm">Notes</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Policy Implementation</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policy-normalization.htm">Policy Normalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policytools.htm">Policy Tools</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/mpr_default.htm">Monetary Policy Report</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/beige-book-default.htm">Beige Book</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/quarterly-balance-sheet-developments-report.htm">Quarterly Report on Federal Reserve Balance Sheet Developments</a>\n</li>\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Review of Monetary Policy Strategy, Tools, and Communications</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/review-of-monetary-policy-strategy-tools-and-communications.htm">Overview</a>\n</li>\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-supervision  dropdown--fw">\n    <a href="/supervisionreg.htm"  class="sr-only-focusable" id="supervisionMenu" aria-haspopup="true">Supervision<br />&amp; Regulation</a>\n    <ul aria-labelledby="supervisionMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Institution Supervision</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/community-and-regional-financial-institutions.htm">Community & Regional Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/large-financial-institutions.htm">Large Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/foreign-banking-organizations.htm">Foreign Banking Organizations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/financial-market-utility-supervision.htm">Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/consumer-compliance.htm">Consumer Compliance</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supervision-and-regulation-report.htm">Federal Reserve Supervision and Regulation Report</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reporting Forms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/default.aspx">All Reporting Forms</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reporting-forms-whats-new.htm">Recent Updates</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/review.aspx">Information collections under review</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?xTdTa+FaSlI=">Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?+PhkJwiBjbA=">Applications/structure change</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?1/RIXO/UduI=">Federal Financial Institutions Examination Council (FFIEC)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?L877VwEj9Ss=">Municipal &amp; Government Securities</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Supervision &amp; Regulation Letters</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/srletters/srletters.htm">By Year</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/topics/topics.htm">By Topic</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Banking Applications &amp; Legal Developments</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/application-process.htm">Application Process</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/board-and-reserve-bank-actions.htm">Board & Reserve Bank Action</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/legal-developments.htm">Enforcement Actions & Legal Developments</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Regulatory Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">Regulations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supmanual.htm">Manuals</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/basel/basel-default.htm">Basel Regulatory Framework</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/volcker-rule.htm">Volcker Rule</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/training-and-assistance-landing.htm">Education, Training, and Assistance</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Banking &amp; Data Structure</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/insider.aspx">Beneficial Ownership reports</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/">U.S. Offices of Foreign Entities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/fhc.htm">Financial Holding Companies</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/isb-default.htm">Interstate Branching</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/suds.htm">Securities Underwriting & Dealing Subsidiaries</a>\n</li>\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-payment  dropdown--fw">\n    <a href="/paymentsystems.htm"  class="sr-only-focusable" id="paymentMenu" aria-haspopup="true">Payment<br />Systems</a>\n    <ul aria-labelledby="paymentMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations &amp; Statutes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regcc-about.htm">Regulation CC (Availability of Funds and Collection of Checks)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regii-about.htm">Regulation II (Debit Card Interchange Fees and Routing)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/reghh-about.htm">Regulation HH (Financial Market Utilities)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/other-regulations.htm">Other Regulations and Statutes</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Payment Policies</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pfs_about.htm">Federal Reserve\'s Key Policies for the Provision of Financial Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/oo_about.htm">Overnight Overdrafts</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_about.htm">Payment System Risk</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/telecomm.htm">Sponsorship for Priority Telecommunication Services</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reserve Bank Payment Services &amp; Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/check_about.htm">Check Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_data.htm">Daylight Overdrafts and Fees</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fednow_about.htm">FedNow\xe2\x84\xa0 Service</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedfunds_about.htm">Fedwire Funds Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedsecs_about.htm">Fedwire Securities Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fisagy_about.htm">Fiscal Agency Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/natl_about.htm">National Settlement Service</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Financial Market Utilities &amp; Infrastructures</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/over_about.htm">Supervision & Oversight of Financial Market Infrastructures</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/designated_fmu_about.htm">Designated Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/int_standards.htm">International Standards for Financial Market Infrastructures</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Research, Committees, and Forums</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/payres_about.htm">Payment Research</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fr-payments-study.htm">Federal Reserve Payments Study (FRPS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pspa_committee.htm">Payments System Policy Advisory Committee</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/forums.htm">Forums</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-econ dropdown--right dropdown--2Col">\n    <a href="/econres.htm" class="sr-only-focusable" aria-haspopup="true" id="econMenu">Economic<br />Research</a>\n    <ul aria-labelledby="econMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Working Papers and Notes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/feds/index.htm">Finance and Economics Discussion Series (FEDS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/notes/feds-notes/default.htm">FEDS Notes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/ifdp/index.htm">International Finance Discussion Papers (IFDP)</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/econres/theeconomists.htm">Meet the Economists</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Models and Tools</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/us-models-about.htm">FRB/US Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/edo-models-about.htm">Estimated Dynamic Optimization (EDO) Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n\n\n\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-data dropdown--right dropdown--fw">\n    <a href="/data.htm"  class="sr-only-focusable" id="dataMenu" aria-haspopup="true">Data</a>\n    <ul aria-labelledby="dataMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/datadownload"><span class="icon icon-ddp icon__sm"></span> Data Download Program</a>\n</li>\n\n\n<li class="nav__header">\n    <p>\n        <strong>Bank Assets and Liabilities</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h3/current/default.htm">Aggregate Reserves of Depository Institutions and the Monetary Base - H.3</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h8/current/default.htm">Assets and Liabilities of Commercial Banks in the U.S. - H.8</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/assetliab/current.htm">Assets and Liabilities of U.S. Branches and Agencies of Foreign Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/chargeoff/">Charge-Off and Delinquency Rates on Loans and Leases at Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sfos/sfos.htm">Senior Financial Officer Survey</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sloos.htm">Senior Loan Officer Opinion Survey on Bank Lending Practices</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Bank Structure Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for the U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Business Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/cp/">Commercial Paper</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g20/current/g20.htm">Finance Companies - G.20</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/govsecure/current.htm">New Security Issues, State and Local Governments</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/corpsecure/current.htm">New Security Issues, U.S. Corporations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Dealer Financing Terms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/scoos.htm">Senior Credit Officer Opinion Survey on Dealer Financing Terms</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Exchange Rates and International Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h10/current/">Foreign Exchange Rates - H.10/G.5</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/intlsumm/current.htm">International Summary Statistics</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/secholdtrans/current.htm">Securities Holdings and Transactions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/statbanksus/current.htm">Statistics Reported by Banks and Other Financial Firms in the United States</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Financial Accounts</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/z1/default.htm">Financial Accounts of the United States - Z.1</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Household Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g19/current/default.htm">Consumer Credit - G.19</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/housedebt/default.htm">Household Debt Service and Financial Obligations Ratios</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mortoutstand/current.htm">Mortgage Debt Outstanding</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Industrial Activity</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g17/current/default.htm">Industrial Production and Capacity Utilization - G.17</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Interest Rates</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h15/">Selected Interest Rates - H.15</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Micro Data Reference Manual (MDRM)</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mdrm.htm">Micro and Macro Data Collections</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Money Stock and Reserve Balances</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h41/">Factors Affecting Reserve Balances - H.4.1</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h6/current/default.htm">Money Stock Measures - H.6</a>\n</li>\n\n\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Other</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/yield-curve-models.htm">Yield Curve Models and Data</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-consumer dropdown--right dropdown--4Col">\n    <a href="/consumerscommunities.htm" class="sr-only-focusable" aria-haspopup="true" id="consumerMenu">Consumers<br />&amp; Communities</a>\n    <ul aria-labelledby="consumerMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cra_about.htm">Community Reinvestment Act (CRA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">All Regulations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Supervision&nbsp;&amp; Enforcement</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/caletters/caletters.htm">CA Letters</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/enforcementactions/default.aspx">Enforcement Actions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/independent-foreclosure-review.htm">Independent Foreclosure Review</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Community Development</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/neighborhood-revitalization.htm">Housing and Neighborhood Revitalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/small-business-and-entrepreneurship.htm">Small Business and Entrepreneurship</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/workforce.htm">Employment and Workforce Development</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cdf.htm">Community Development Finance</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/rural-community-economic-development.htm">Rural Community and Economic Development</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Research&nbsp;&amp; Analysis</strong>\n    </p>\n</li>\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/community-development-publications.htm">Research Publications & Data Analysis</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/cac.htm">Community Advisory Council</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Consumer Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/foreclosure.htm">Mortgage and Foreclosure Resources</a>\n</li>\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/comm-dev-system-map.htm">Federal Reserve Community Development Resources</a>\n</li>\n\n\n\n                </ul>\n            </div>\n        </li>             \n    </ul>\n</li>\n \n \n \n \n \n \n \n \n \n \n \n \n    </ul>\n</nav>\n        <!-- BODY CONTENT -->\n        <div id="content" class="container container__main" role="main">\n            <div class="row">\n                <div class="page-header">\n                    <ol class="breadcrumb">\n                            <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/default.htm">Home</a></li>\n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems.htm">Payment Systems</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/fedach_data.htm">Data</a></li>\n                                          \n \n                                          \n \n                                          \n \n                                          \n \n \n                    </ol>\n                    <div class="header-group">\n                        <div id="page-title" class="page-title"> \n                        \n                                \t\t<h2>\n                                \t\t    Automated Clearinghouse Services\n                                \t\t</h2>\n                            \t\t\n                        </div> \n                        \n                  </div>\n                </div><!-- / .page-header -->\n            \n            </div><!-- / .row -->\n  \n\n\n        <div class=\'row\'>\n        \n                                <div class="col-xs-12 col-sm-4 col-md-3">\n                                  <div id="t4_nav" class="t4_nav list-group sticky">\n                                    \n                                    \r\n\r\n\r\n\r\n<!-- <h3 class="text-uppercase">Chapters</h3> -->\r\n\r\n\r\n\r\n\r\n<a href="/paymentsystems/fedach_data.htm" class="list-group-item" title="">Data</a>\n                                  </div>\n                                </div>\n                        \n                                                    <div id="article" class="col-xs-12 col-sm-8 col-md-9">\n                                                    <h3>Government Automated Clearinghouse Transactions Processed by the Federal Reserve--Annual Data</h3>\n                                                \n                                <!-- Article Area -->\n                                    \n                                    <div id="floatRightRail" class="col-xs-12 col-sm-12 col-md-5 pull-right">\n                                        \n                                        \n\n<!-- Text Box Begins -->\n\n<div class="panel panel-related">\n    <div class="panel-heading">\n                   \n                <h5 class="panel-title text-capitalize">\n                    Other Formats  \n                </h5>\n                \n    </div>\n    <div class="panel-body">\n        <ul>\n\t<li><a href="/paymentsystems/files/fedach_yearlygovt.txt">ASCII</a></li>\n\t<li><a href="/paymentsystems/files/fedach_yearlygovt.pdf">PDF (97&nbsp;KB)</a></li>\n</ul>\n    </div>\n</div>\n\n\n\n\n            \n                                    </div>     \n                                    \n                                    \n                                    <p>The daily average volume and average daily value of items are based on the number of business days in each year.<a id="ft1" name="ft1"></a><a href="#fn1" title="footnote 1"><sup>1</sup></a></p>\n\n<div class="data-table" data-table-popout="" id="data-table-0">\n<table class="pubtables" data-other-sticky-ids="navbar-mobile,nav-primary" data-sticky-columns="1" data-sticky-rows="1">\n\t<thead>\n\t\t<tr class="header">\n\t\t\t<th align="center" scope="col" valign="bottom">Year</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Volume<br />\n\t\t\t(millions<br />\n\t\t\tof items)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Volume<br />\n\t\t\tpercent<br />\n\t\t\tchange</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Value<br />\n\t\t\t(billions<br />\n\t\t\tof dollars)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Value<br />\n\t\t\tpercent<br />\n\t\t\tchange</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average daily<br />\n\t\t\tvolume<br />\n\t\t\t(millions<br />\n\t\t\tof items)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average daily<br />\n\t\t\tvalue<br />\n\t\t\t(billions<br />\n\t\t\tof dollars)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average value<br />\n\t\t\tper<br />\n\t\t\ttransaction<br />\n\t\t\t(dollars)</th>\n\t\t</tr>\n\t</thead>\n\t<tbody>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2020</strong></td>\n\t\t\t<td class="left2">1,878</td>\n\t\t\t<td class="left2">10.2</td>\n\t\t\t<td class="left2">6,853</td>\n\t\t\t<td class="left2">18.4</td>\n\t\t\t<td class="left2">7.3</td>\n\t\t\t<td class="left2">26.8</td>\n\t\t\t<td class="left2">3,648</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2019</strong></td>\n\t\t\t<td class="left2">1,704</td>\n\t\t\t<td class="left2">2.2</td>\n\t\t\t<td class="left2">5,787</td>\n\t\t\t<td class="left2">4.9</td>\n\t\t\t<td class="left2">6.8</td>\n\t\t\t<td class="left2">23.1</td>\n\t\t\t<td class="left2">3,396</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2018</strong></td>\n\t\t\t<td class="left2">1,668</td>\n\t\t\t<td class="left2">2.4</td>\n\t\t\t<td class="left2">5,515</td>\n\t\t\t<td class="left2">2.7</td>\n\t\t\t<td class="left2">6.6</td>\n\t\t\t<td class="left2">22.0</td>\n\t\t\t<td class="left2">3,306</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2017</strong></td>\n\t\t\t<td class="left2">1,629</td>\n\t\t\t<td class="left2">2.2</td>\n\t\t\t<td class="left2">5,371</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">6.5</td>\n\t\t\t<td class="left2">21.5</td>\n\t\t\t<td class="left2">3,296</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2016</strong></td>\n\t\t\t<td class="left2">1,594</td>\n\t\t\t<td class="left2">2.4</td>\n\t\t\t<td class="left2">5,193</td>\n\t\t\t<td class="left2">2.7</td>\n\t\t\t<td class="left2">6.4</td>\n\t\t\t<td class="left2">20.7</td>\n\t\t\t<td class="left2">3,257</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2015</strong></td>\n\t\t\t<td class="left2">1,558</td>\n\t\t\t<td class="left2">2.7</td>\n\t\t\t<td class="left2">5,054</td>\n\t\t\t<td class="left2">3.7</td>\n\t\t\t<td class="left2">6.2</td>\n\t\t\t<td class="left2">20.1</td>\n\t\t\t<td class="left2">3,245</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2014</strong></td>\n\t\t\t<td class="left2">1,516</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">4,873</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">6.0</td>\n\t\t\t<td class="left2">19.4</td>\n\t\t\t<td class="left2">3,214</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2013</strong></td>\n\t\t\t<td class="left2">1,467</td>\n\t\t\t<td class="left2">6.2</td>\n\t\t\t<td class="left2">4,714</td>\n\t\t\t<td class="left2">2.3</td>\n\t\t\t<td class="left2">5.8</td>\n\t\t\t<td class="left2">18.8</td>\n\t\t\t<td class="left2">3,214</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2012</strong></td>\n\t\t\t<td class="left2">1,382</td>\n\t\t\t<td class="left2">5.9</td>\n\t\t\t<td class="left2">4,610</td>\n\t\t\t<td class="left2">1.7</td>\n\t\t\t<td class="left2">5.5</td>\n\t\t\t<td class="left2">18.4</td>\n\t\t\t<td class="left2">3,336</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2011</strong></td>\n\t\t\t<td class="left2">1,305</td>\n\t\t\t<td class="left2">6.8</td>\n\t\t\t<td class="left2">4,535</td>\n\t\t\t<td class="left2">2.4</td>\n\t\t\t<td class="left2">5.2</td>\n\t\t\t<td class="left2">18.1</td>\n\t\t\t<td class="left2">3,476</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2010</strong></td>\n\t\t\t<td class="left2">1,222</td>\n\t\t\t<td class="left2">2.3</td>\n\t\t\t<td class="left2">4,427</td>\n\t\t\t<td class="left2">3.0</td>\n\t\t\t<td class="left2">4.8</td>\n\t\t\t<td class="left2">17.6</td>\n\t\t\t<td class="left2">3,623</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2009</strong></td>\n\t\t\t<td class="left2">1,195</td>\n\t\t\t<td class="left2">5.5</td>\n\t\t\t<td class="left2">4,297</td>\n\t\t\t<td class="left2">7.2</td>\n\t\t\t<td class="left2">4.7</td>\n\t\t\t<td class="left2">17.1</td>\n\t\t\t<td class="left2">3,597</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2008</strong></td>\n\t\t\t<td class="left2">1,132</td>\n\t\t\t<td class="left2">10.2</td>\n\t\t\t<td class="left2">4,008</td>\n\t\t\t<td class="left2">7.8</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">15.9</td>\n\t\t\t<td class="left2">3,539</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2007</strong></td>\n\t\t\t<td class="left2">1,027</td>\n\t\t\t<td class="left2">3.6</td>\n\t\t\t<td class="left2">3,717</td>\n\t\t\t<td class="left2">7.0</td>\n\t\t\t<td class="left2">4.1</td>\n\t\t\t<td class="left2">14.8</td>\n\t\t\t<td class="left2">3,618</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2006</strong></td>\n\t\t\t<td class="left2">992</td>\n\t\t\t<td class="left2">2.8</td>\n\t\t\t<td class="left2">3,474</td>\n\t\t\t<td class="left2">10.1</td>\n\t\t\t<td class="left2">4.0</td>\n\t\t\t<td class="left2">13.8</td>\n\t\t\t<td class="left2">3,504</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2005</strong></td>\n\t\t\t<td class="left2">964</td>\n\t\t\t<td class="left2">2.5</td>\n\t\t\t<td class="left2">3,157</td>\n\t\t\t<td class="left2">8.4</td>\n\t\t\t<td class="left2">3.8</td>\n\t\t\t<td class="left2">12.6</td>\n\t\t\t<td class="left2">3,273</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2004</strong></td>\n\t\t\t<td class="left2">941</td>\n\t\t\t<td class="left2">3.0</td>\n\t\t\t<td class="left2">2,913</td>\n\t\t\t<td class="left2">3.7</td>\n\t\t\t<td class="left2">3.7</td>\n\t\t\t<td class="left2">11.5</td>\n\t\t\t<td class="left2">3,096</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2003</strong></td>\n\t\t\t<td class="left2">914</td>\n\t\t\t<td class="left2">3.5</td>\n\t\t\t<td class="left2">2,810</td>\n\t\t\t<td class="left2">3.6</td>\n\t\t\t<td class="left2">3.6</td>\n\t\t\t<td class="left2">11.2</td>\n\t\t\t<td class="left2">3,075</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2002</strong></td>\n\t\t\t<td class="left2">883</td>\n\t\t\t<td class="left2">-1.9</td>\n\t\t\t<td class="left2">2,711</td>\n\t\t\t<td class="left2">7.2</td>\n\t\t\t<td class="left2">3.5</td>\n\t\t\t<td class="left2">10.8</td>\n\t\t\t<td class="left2">3,070</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2001</strong></td>\n\t\t\t<td class="left2">900</td>\n\t\t\t<td class="left2">7.3</td>\n\t\t\t<td class="left2">2,529</td>\n\t\t\t<td class="left2">5.2</td>\n\t\t\t<td class="left2">3.6</td>\n\t\t\t<td class="left2">10.1</td>\n\t\t\t<td class="left2">2,808</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2000</strong></td>\n\t\t\t<td class="left2">839</td>\n\t\t\t<td class="left2">3.5</td>\n\t\t\t<td class="left2">2,404</td>\n\t\t\t<td class="left2">7.7</td>\n\t\t\t<td class="left2">3.3</td>\n\t\t\t<td class="left2">9.5</td>\n\t\t\t<td class="left2">2,869</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1999</strong></td>\n\t\t\t<td class="left2">809</td>\n\t\t\t<td class="left2">7.4</td>\n\t\t\t<td class="left2">2,233</td>\n\t\t\t<td class="left2">12.3</td>\n\t\t\t<td class="left2">3.2</td>\n\t\t\t<td class="left2">8.9</td>\n\t\t\t<td class="left2">2,760</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1998</strong></td>\n\t\t\t<td class="left2">753</td>\n\t\t\t<td class="left2">11.2</td>\n\t\t\t<td class="left2">1,988</td>\n\t\t\t<td class="left2">25.7</td>\n\t\t\t<td class="left2">3.0</td>\n\t\t\t<td class="left2">7.9</td>\n\t\t\t<td class="left2">2,640</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1997</strong></td>\n\t\t\t<td class="left2">677</td>\n\t\t\t<td class="left2">8.3</td>\n\t\t\t<td class="left2">1,581</td>\n\t\t\t<td class="left2">26.5</td>\n\t\t\t<td class="left2">2.7</td>\n\t\t\t<td class="left2">6.3</td>\n\t\t\t<td class="left2">2,335</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1996</strong></td>\n\t\t\t<td class="left2">625</td>\n\t\t\t<td class="left2">4.3</td>\n\t\t\t<td class="left2">1,250</td>\n\t\t\t<td class="left2">11.9</td>\n\t\t\t<td class="left2">2.5</td>\n\t\t\t<td class="left2">5.0</td>\n\t\t\t<td class="left2">2,000</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1995</strong></td>\n\t\t\t<td class="left2">599</td>\n\t\t\t<td class="left2">4.4</td>\n\t\t\t<td class="left2">1,117</td>\n\t\t\t<td class="left2">17.7</td>\n\t\t\t<td class="left2">2.4</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">1,865</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1994</strong></td>\n\t\t\t<td class="left2">574</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">949</td>\n\t\t\t<td class="left2">7.2</td>\n\t\t\t<td class="left2">2.3</td>\n\t\t\t<td class="left2">3.8</td>\n\t\t\t<td class="left2">1,653</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1993</strong></td>\n\t\t\t<td class="left2">555</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">885</td>\n\t\t\t<td class="left2">2.9</td>\n\t\t\t<td class="left2">2.2</td>\n\t\t\t<td class="left2">3.5</td>\n\t\t\t<td class="left2">1,595</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1992</strong></td>\n\t\t\t<td class="left2">531</td>\n\t\t\t<td class="left2">1.9</td>\n\t\t\t<td class="left2">860</td>\n\t\t\t<td class="left2">18.9</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">1,620</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1991</strong></td>\n\t\t\t<td class="left2">521</td>\n\t\t\t<td class="left2">0.2</td>\n\t\t\t<td class="left2">723</td>\n\t\t\t<td class="left2">48.5</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">2.9</td>\n\t\t\t<td class="left2">1,388</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1990</strong></td>\n\t\t\t<td class="left2">520</td>\n\t\t\t<td class="left2">17.9</td>\n\t\t\t<td class="left2">487</td>\n\t\t\t<td class="left2">24.6</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">1.9</td>\n\t\t\t<td class="left2">937</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1989</strong></td>\n\t\t\t<td class="left2">441</td>\n\t\t\t<td class="left2">8.4</td>\n\t\t\t<td class="left2">391</td>\n\t\t\t<td class="left2">n.a.</td>\n\t\t\t<td class="left2">1.8</td>\n\t\t\t<td class="left2">1.6</td>\n\t\t\t<td class="left2">887</td>\n\t\t</tr>\n\t</tbody>\n</table>\n</div>\n\n<h5>Footnotes</h5>\n\n<div class="footnotes">\n<p><a id="fn1" name="fn1">1.</a> Values are subject to minor changes based on revisions to underlying data. <a href="#ft1">Return to text</a></p>\n\n<p>n.a. Not available.</p>\n</div>\n\n                                    \n                                    \n                                </div>\n                                \n        </div>            \n        <a id="back-top" class="icon__backTop icon-backtop" href="#" title="Scroll To Top"><span class="sr-only">Back to Top</span></a>\n\n        \n        <div class="row">\n            <div class=\'lastUpdate\' id="lastUpdate">Last Update:\n              \n                            March 09, 2021\n                          \n            </div>\n        </div>\n        </div><!-- / #content .container -->\n        <footer class="container footer">\n    <div class="row footer__content">\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="footer__heading"><span class="text-uppercase">Board of Governors</span> <em>of the</em> <span class="text-uppercase">Federal Reserve System</span></h6>\n            <ul class="list-unstyled">\n                <li class=\'footer__listItem\'><a class="footer__link" href="/aboutthefed.htm">About the Fed</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/newsevents.htm">News & Events</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/monetarypolicy.htm">Monetary Policy</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/supervisionreg.htm">Supervision & Regulation</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/paymentsystems.htm">Payment Systems</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/econres.htm">Economic Research</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/data.htm">Data</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/consumerscommunities.htm">Consumers & Communities</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/financial-stability.htm">Financial Stability</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/economic-disparities-work.htm">Federal Reserve\'s Work Related to Economic Disparities</a></li> \n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="text-uppercase footer__heading">Tools and Information</h6>\n            <ul class="list-unstyled">\n                 <li class=\'footer__listItem\'><a href="/aboutthefed/contact-us-topics.htm" class="footer__link">Contact</a></li>\n                <li class=\'footer__listItem\'><a href="/publications.htm" class="footer__link">Publications</a></li>\n                <li class=\'footer__listItem\'><a href="/foia/about_foia.htm" class="footer__link">Freedom of Information (FOIA)</a></li>\n                <li class=\'footer__listItem\'><a href="https://oig.federalreserve.gov/" class="footer__link">Office of Inspector General</a></li>\n                <li class=\'footer__listItem\'><a href="/publications/annual-report.htm" class="footer__link">Budget &amp; Performance</a> | <a href="/regreform/audit.htm" class="footer__link">Audit</a></li>\n                <li class=\'footer__listItem\'><a href="/eeo.htm" class="footer__link">No FEAR Act</a></li>\n                <li class=\'footer__listItem\'><a href="/espanol.htm" class="footer__link">Espa\xc3\xb1ol</a></li>\n                <li class=\'footer__listItem\'><a href="/website-linking-policies.htm" class="footer__link">Website Policies</a>  | <a href="/privacy.htm" class="footer__link">Privacy Program</a></li>\n                <li class=\'footer__listItem\'><a href="/accessibility.htm" class="footer__link">Accessibility</a></li>\n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <div class="footer__social">\n                <h6 class="text-uppercase footer__heading footer__heading--social">Stay Connected</h6>\n                \n\n\n    \n                <a class="icon__md icon footer__btn icon-facebook-color" href="https://www.facebook.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-twitter-color" href="https://twitter.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-youtube-color" href="https://www.youtube.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-flickr-color" href="https://www.flickr.com/photos/federalreserve/"><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-linkedin-color" href="https://www.linkedin.com/company/federal-reserve-board"><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-rss-color" href="/feeds/feeds.htm"><span class=\'sr-only\'>Subscribe to RSS</span></a>\n            \n                <a class="icon__md icon footer__btn icon-email-color" href="/subscribe.htm"><span class=\'sr-only\'>Subscribe to Email</span></a>\n                    \n    \n \n \n            </div>\n            <a href="https://www.usa.gov/" target="_blank" title="Link to USA.gov">\n                <img class=\'footer__image\' src="/images/USAGov%402x.png" alt="Link to USA.gov" />\n            </a>\n            <a href="/open/open.htm" target="_blank" title="Link to Open.gov">\n                <img class=\'footer__image\' src="/images/OpenGov%402x.png" alt="Link to Open.gov" />\n            </a>\n        </div>\n    </div>\n    <div class="row">\n        <div class="col-xs-12 footer__footer">\n            <p class="text-uppercase footer__text footer__text--left">Board of Governors <em class="text-lowercase">of the</em> Federal Reserve System</p>\n            <p class="footer__text footer__text--right">20th Street and Constitution Avenue N.W., Washington, DC 20551</p>\n        </div>\n    </div>\n</footer>\n\n\n                <script src="/js/jquery1.11.1.min.js" type="text/javascript"></script>\n                <script src="/js/bootstrap.min.js" type="text/javascript"></script>\n                <script src="/js/scripts.js" type="text/javascript"></script>\n                <script src="/js/modal.js" type="text/javascript"></script>\n                <script src="/js/sticky-tables.js" type="text/javascript"></script>\n                <script type="text/javascript" src="/js/slides/ekko-lightbox.min.js"></script>\n                <script src="/js/t4-nav-controller.js" type="text/javascript"></script>\n              \n<script type="text/javascript">\n    $(document).ready(function($) {\n        // delegate calls to data-toggle="lightbox"\n        $(document).delegate(\'*[data-toggle="lightbox"]:not([data-gallery="navigateTo"])\', \'click\', function(event) {\n            event.preventDefault();\n            $(this).ekkoLightbox();\n        });\n        $(\'a[data-click="false"]\').click(function(e) {\n            e.preventDefault();\n            var target = $(this).data(\'target\');\n            $(target).click();\n        });\n    });\n</script>          \n             \n\n\n        \n        <!--<ul></ul> -->\n    <script type="text/javascript" src="/resources/track_downloads.js"></script>\n</body>\n</html>\n '  # noqa: E501
            result = ACH().government()
            self.assertIsInstance(result, pd.DataFrame)

    def test_all(self) -> None:
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\xef\xbb\xbf<!doctype html>\n<html lang="en" class="no-js">\n    <head>\n        <meta charset="utf-8">\n        <meta http-equiv="X-UA-Compatible" content="IE=edge"/>\n        <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0 maximum-scale=1.6, user-scalable=1"/>\n        <meta name="keywords" content="Board of Governors of the Federal Reserve System, Federal Reserve Board of Governors, Federal Reserve Board, Federal Reserve" />\n        <meta name="description" content="The Federal Reserve Board of Governors in Washington DC." />\n        <meta property="og:site_name" content="Board of Governors of the Federal Reserve System"/>\n        <meta property="og:type" content="article" /> \n        <meta property="og:image"  content="" /> \n        <meta name="twitter:card" content="summary" />\n        <meta name="twitter:image" content="" />\n        \n\n\n \n        <title>Federal Reserve Board - Commercial Automated Clearinghouse Transactions Processed by the Federal Reserve--Annual Data</title>\n        \n    <link href="/css/bootstrap.css" rel="stylesheet" type="text/css">\n    <link href="/css/bluesteel-theme.css" rel="stylesheet" type="text/css">\n    <script src="/js/modernizr-latest.js" type="text/javascript"></script>\n    <script src="/js/grunticon.loader.js"></script>\n    <script>\n        grunticon(["/css/icons.data.svg.css", "/css/icons.data.png.css", "/css/icons.fallback.css"]);\n    </script>\n    <noscript>\n        <link href="/css/icons.fallback.css" rel="stylesheet">\n    </noscript>                \n    <script src="/js/angular.min.js" type="text/javascript"></script>\n    <script src="/js/app.js" type="text/javascript"></script>\n        \n    <!-- For Linear Gradients in IE9 or greater -->\n    <!--[if gte IE 9]>\n        <style type="text/css">\n          .gradient {\n             filter: none;\n          }\n        </style>\n      <![endif]-->\n\n\n \n    <script type="text/javascript" >\n  (function(i,s,o,g,r,a,m){i[\'GoogleAnalyticsObject\']=r;i[r]=i[r]||function(){\n  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),\n  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)\n  })(window,document,\'script\',\'//www.google-analytics.com/analytics.js\',\'ga\');\n  ga(\'create\', \'UA-35121701-1\', \'federalreserve.gov\', {\'cookieExpires\': 0});\n  ga(\'set\', \'anonymizeIp\', true);\n  ga(\'send\', \'pageview\');\n</script></head>\n    <body data-ng-app="pubwebApp">\n        <a href="#content" class="globalskip">Skip to main content</a>\n<nav class="nav-primary-mobile" id="nav-primary-mobile">\n</nav>\n<div class="t1_nav navbar navbar-default navbar-fixed-top" role="navigation">\n    <div class="t1_container clearfix">\n        <a id="logo" class="btn btn-default icon-FRB_logo-bw visible-xs-table-cell t1__btnlogo" href="/default.htm" role="button"><span class="sr-only">Back to Home</span>\n        </a>\n        <a href="/default.htm" id="banner" class="visible-xs-table-cell t1__banner" title="Link to Home Page" role="button">Board of Governors of the Federal Reserve System</a>\n        <ul class="nav navbar-nav visible-md visible-lg t1_toplinks">\n            <li class="navbar-text navbar-text__social">\n                <span class="navbar-link" tabindex="0">Stay Connected</span>\n                <ul class="nav navbar-nav t1_social">\n                    \n\n\n    \n                <li class="social__item"><a data-icon="facebook" href="https://www.facebook.com/federalreserve" class="noIcon"><div class="icon-facebook-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="twitter" href="https://twitter.com/federalreserve" class="noIcon"><div class="icon-twitter-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="youtube" href="https://www.youtube.com/federalreserve" class="noIcon"><div class="icon-youtube-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="flickr" href="https://www.flickr.com/photos/federalreserve/" class="noIcon"><div class="icon-flickr-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="linkedin" href="https://www.linkedin.com/company/federal-reserve-board" class="noIcon"><div class="icon-linkedin-color icon icon__sm"></div><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="rss" href="/feeds/feeds.htm" class="noIcon"><div class="icon-rss-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to RSS</span></a></li>\n            \n                <li class="social__item"><a data-icon="email" href="/subscribe.htm" class="noIcon"><div class="icon-email-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to Email</span></a></li>\n                    \n    \n \n \n                </ul>\n            </li>\n           \r\n                                <li class="navbar-text"><a class="navbar-link" href="/recentpostings.htm">Recent Postings</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/newsevents/calendar.htm">Calendar</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/publications.htm">Publications</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/sitemap.htm">Site Map</a></li>\r\n                \r\n                                <li class="navbar-text"><a class="navbar-link" href="/azindex.htm">A-Z index</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/careers.htm">Careers</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/faqs.htm">FAQs</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/videos.htm">Videos</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/aboutthefed/contact-us-topics.htm">Contact</a></li>\r\n                \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \n        </ul>\n        <form class="nav navbar-form hidden-xs nav__search" role="search" action="//www.fedsearch.org/board_public/search" method="get">\n            <div class="form-group input-group input-group-sm">\n                <label for="t1search" class=\'sr-only\'>Search</label>\n                <input id="t1search" class="nav__input form-control" name="text" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                    <button type="submit" class="nav__button btn btn-default" id="headerTopLinksSearchFormSubmit" name="Search">\n                        <span class="sr-only">Submit Search Button</span>\n                        <span class="icon-magnifying-glass icon icon__xs icon--right"></span>\n                    </button>\n                </span>\n            </div>\n            <div class=\'nav__advanced\'><a class="noIcon" href="//www.fedsearch.org/board_public">Advanced</a></div>\n        </form>        \n        <div class="btn-group visible-xs-table-cell visible-sm-table-cell t1__dropdown">\n            <span class="icon icon__md icon--right dropdown-toggle icon-options-nav" data-toggle="dropdown">\n                <span class="sr-only">Toggle Dropdown Menu</span>\n            </span>\n        </div>\n    </div>\n</div>\n<header class="jumbotron hidden-xs">\n    <a class="jumbotron__link" href="/default.htm" title="Link to Home Page">\n        <div class="container-fluid">\n            <h1 class="jumbotron__heading">Board of Governors of the Federal Reserve System</h1>\n            <p class="jumbotron__content"><em>The Federal Reserve, the central bank of the United States, provides the nation with a safe, flexible, and stable monetary and financial system.</em></p>\n        </div>\n    </a>\n</header>\n<div class="t2__offcanvas visible-xs-block">\n    <div class="navbar-header">\n        <button type="button" class="offcanvas__toggle icon-offcanvas-nav" data-toggle="offcanvas" data-target="#offcanvasT2Menu" data-canvas="body">\n            <span class="sr-only">Main Menu Toggle Button</span>\n        </button>\n        <span class="navbar-brand offcanvas__title">Sections</span>\n        <button type="button" class="offcanvas__search icon-search-nav" data-toggle="collapse" data-target="#navbar-collapse">\n            <span class="sr-only">Search Toggle Button</span>\n        </button>\n    </div>\n    <div class="navbar-collapse collapse mobile-form" id="navbar-collapse">\n        <form method="get" class="navbar-form" role="search" action="//www.fedsearch.org/board_public/search">\n            <div class="form-group input-group mobileSearch">\n                <label class="sr-only" for="t2search">Search</label>\n                <input id="t2search" name="text" class="form-control" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                <span class="sr-only">Search Submit Button</span>\n                <button type="submit" class="btn btn-default">Submit</button>\n                </span>\n            </div>\n        </form>\n    </div>\n</div>\n<nav class="nav-primary navbar hidden-xs" id="nav-primary" role="navigation">\n    <ul class="nav navbar-nav" role="menubar">\n        \n \n<li role="menuitem" class="dropdown nav-about  dropdown--3Col">\n    <a href="/aboutthefed.htm" class="sr-only-focusable" aria-haspopup="true" id="aboutMenu">About<br />the Fed</a>\n    <ul aria-labelledby="aboutMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-4 list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/structure-federal-reserve-system.htm">Structure of the Federal Reserve System</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/pf.htm">Purposes &amp; Functions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/bios/board/default.htm">Board Members</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/advisorydefault.htm">Advisory Councils</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/federal-reserve-system.htm">Federal Reserve Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fract.htm">Federal Reserve Act</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/currency.htm">Currency</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardmeetings/meetingdates.htm">Board Meetings</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardvotes.htm">Board Votes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/diversityinclusion.htm">Diversity & Inclusion</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/careers.htm">Careers</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/procurement/about.htm">Do Business with the Board</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/k8.htm">Holidays Observed - K.8</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/contact-us-topics.htm">Contact </a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/foia/about_foia.htm">Requesting Information (FOIA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/faqs.htm">FAQs</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/educational-tools/default.htm">Economic Education</a>\n</li>\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fed-financial-statements.htm">Fed Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/innovation.htm">Innovation</a>\n</li>\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-news  dropdown--1Col">\n    <a href="/newsevents.htm" class="sr-only-focusable" aria-haspopup="true" id="newsMenu">News<br />&amp; Events</a>\n    <ul aria-labelledby="newsMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        \n<li>\n    <a class="sr-only-focusable" href="/newsevents/pressreleases.htm">Press Releases</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/speeches.htm">Speeches</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/testimony.htm">Testimony</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/calendar.htm">Calendar</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/videos.htm">Videos\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/photogallery.htm">Photo Gallery\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-monetary  dropdown--2Col">\n    <a href="/monetarypolicy.htm" class="sr-only-focusable" aria-haspopup="true" id="monetaryMenu">Monetary<br />Policy</a>\n    <ul aria-labelledby="monetaryMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Federal Open Market Committee</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc.htm">About the FOMC</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomccalendars.htm">Meeting calendars and information</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_historical.htm">Transcripts and other historical materials</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_projectionsfaqs.htm">FAQs</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Monetary Policy Principles and Practice</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/monetary-policy-principles-and-practice.htm">Notes</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Policy Implementation</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policy-normalization.htm">Policy Normalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policytools.htm">Policy Tools</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/mpr_default.htm">Monetary Policy Report</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/beige-book-default.htm">Beige Book</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/quarterly-balance-sheet-developments-report.htm">Quarterly Report on Federal Reserve Balance Sheet Developments</a>\n</li>\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Review of Monetary Policy Strategy, Tools, and Communications</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/review-of-monetary-policy-strategy-tools-and-communications.htm">Overview</a>\n</li>\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-supervision  dropdown--fw">\n    <a href="/supervisionreg.htm"  class="sr-only-focusable" id="supervisionMenu" aria-haspopup="true">Supervision<br />&amp; Regulation</a>\n    <ul aria-labelledby="supervisionMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Institution Supervision</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/community-and-regional-financial-institutions.htm">Community & Regional Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/large-financial-institutions.htm">Large Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/foreign-banking-organizations.htm">Foreign Banking Organizations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/financial-market-utility-supervision.htm">Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/consumer-compliance.htm">Consumer Compliance</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supervision-and-regulation-report.htm">Federal Reserve Supervision and Regulation Report</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reporting Forms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/default.aspx">All Reporting Forms</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reporting-forms-whats-new.htm">Recent Updates</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/review.aspx">Information collections under review</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?xTdTa+FaSlI=">Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?+PhkJwiBjbA=">Applications/structure change</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?1/RIXO/UduI=">Federal Financial Institutions Examination Council (FFIEC)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?L877VwEj9Ss=">Municipal &amp; Government Securities</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Supervision &amp; Regulation Letters</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/srletters/srletters.htm">By Year</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/topics/topics.htm">By Topic</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Banking Applications &amp; Legal Developments</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/application-process.htm">Application Process</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/board-and-reserve-bank-actions.htm">Board & Reserve Bank Action</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/legal-developments.htm">Enforcement Actions & Legal Developments</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Regulatory Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">Regulations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supmanual.htm">Manuals</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/basel/basel-default.htm">Basel Regulatory Framework</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/volcker-rule.htm">Volcker Rule</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/training-and-assistance-landing.htm">Education, Training, and Assistance</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Banking &amp; Data Structure</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/insider.aspx">Beneficial Ownership reports</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/">U.S. Offices of Foreign Entities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/fhc.htm">Financial Holding Companies</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/isb-default.htm">Interstate Branching</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/suds.htm">Securities Underwriting & Dealing Subsidiaries</a>\n</li>\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-payment  dropdown--fw">\n    <a href="/paymentsystems.htm"  class="sr-only-focusable" id="paymentMenu" aria-haspopup="true">Payment<br />Systems</a>\n    <ul aria-labelledby="paymentMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations &amp; Statutes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regcc-about.htm">Regulation CC (Availability of Funds and Collection of Checks)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regii-about.htm">Regulation II (Debit Card Interchange Fees and Routing)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/reghh-about.htm">Regulation HH (Financial Market Utilities)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/other-regulations.htm">Other Regulations and Statutes</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Payment Policies</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pfs_about.htm">Federal Reserve\'s Key Policies for the Provision of Financial Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/oo_about.htm">Overnight Overdrafts</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_about.htm">Payment System Risk</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/telecomm.htm">Sponsorship for Priority Telecommunication Services</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reserve Bank Payment Services &amp; Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/check_about.htm">Check Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_data.htm">Daylight Overdrafts and Fees</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fednow_about.htm">FedNow\xe2\x84\xa0 Service</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedfunds_about.htm">Fedwire Funds Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedsecs_about.htm">Fedwire Securities Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fisagy_about.htm">Fiscal Agency Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/natl_about.htm">National Settlement Service</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Financial Market Utilities &amp; Infrastructures</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/over_about.htm">Supervision & Oversight of Financial Market Infrastructures</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/designated_fmu_about.htm">Designated Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/int_standards.htm">International Standards for Financial Market Infrastructures</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Research, Committees, and Forums</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/payres_about.htm">Payment Research</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fr-payments-study.htm">Federal Reserve Payments Study (FRPS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pspa_committee.htm">Payments System Policy Advisory Committee</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/forums.htm">Forums</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-econ dropdown--right dropdown--2Col">\n    <a href="/econres.htm" class="sr-only-focusable" aria-haspopup="true" id="econMenu">Economic<br />Research</a>\n    <ul aria-labelledby="econMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Working Papers and Notes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/feds/index.htm">Finance and Economics Discussion Series (FEDS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/notes/feds-notes/default.htm">FEDS Notes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/ifdp/index.htm">International Finance Discussion Papers (IFDP)</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/econres/theeconomists.htm">Meet the Economists</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Models and Tools</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/us-models-about.htm">FRB/US Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/edo-models-about.htm">Estimated Dynamic Optimization (EDO) Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n\n\n\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-data dropdown--right dropdown--fw">\n    <a href="/data.htm"  class="sr-only-focusable" id="dataMenu" aria-haspopup="true">Data</a>\n    <ul aria-labelledby="dataMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/datadownload"><span class="icon icon-ddp icon__sm"></span> Data Download Program</a>\n</li>\n\n\n<li class="nav__header">\n    <p>\n        <strong>Bank Assets and Liabilities</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h3/current/default.htm">Aggregate Reserves of Depository Institutions and the Monetary Base - H.3</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h8/current/default.htm">Assets and Liabilities of Commercial Banks in the U.S. - H.8</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/assetliab/current.htm">Assets and Liabilities of U.S. Branches and Agencies of Foreign Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/chargeoff/">Charge-Off and Delinquency Rates on Loans and Leases at Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sfos/sfos.htm">Senior Financial Officer Survey</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sloos.htm">Senior Loan Officer Opinion Survey on Bank Lending Practices</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Bank Structure Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for the U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Business Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/cp/">Commercial Paper</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g20/current/g20.htm">Finance Companies - G.20</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/govsecure/current.htm">New Security Issues, State and Local Governments</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/corpsecure/current.htm">New Security Issues, U.S. Corporations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Dealer Financing Terms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/scoos.htm">Senior Credit Officer Opinion Survey on Dealer Financing Terms</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Exchange Rates and International Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h10/current/">Foreign Exchange Rates - H.10/G.5</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/intlsumm/current.htm">International Summary Statistics</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/secholdtrans/current.htm">Securities Holdings and Transactions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/statbanksus/current.htm">Statistics Reported by Banks and Other Financial Firms in the United States</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Financial Accounts</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/z1/default.htm">Financial Accounts of the United States - Z.1</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Household Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g19/current/default.htm">Consumer Credit - G.19</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/housedebt/default.htm">Household Debt Service and Financial Obligations Ratios</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mortoutstand/current.htm">Mortgage Debt Outstanding</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Industrial Activity</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g17/current/default.htm">Industrial Production and Capacity Utilization - G.17</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Interest Rates</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h15/">Selected Interest Rates - H.15</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Micro Data Reference Manual (MDRM)</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mdrm.htm">Micro and Macro Data Collections</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Money Stock and Reserve Balances</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h41/">Factors Affecting Reserve Balances - H.4.1</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h6/current/default.htm">Money Stock Measures - H.6</a>\n</li>\n\n\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Other</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/yield-curve-models.htm">Yield Curve Models and Data</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-consumer dropdown--right dropdown--4Col">\n    <a href="/consumerscommunities.htm" class="sr-only-focusable" aria-haspopup="true" id="consumerMenu">Consumers<br />&amp; Communities</a>\n    <ul aria-labelledby="consumerMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cra_about.htm">Community Reinvestment Act (CRA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">All Regulations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Supervision&nbsp;&amp; Enforcement</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/caletters/caletters.htm">CA Letters</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/enforcementactions/default.aspx">Enforcement Actions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/independent-foreclosure-review.htm">Independent Foreclosure Review</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Community Development</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/neighborhood-revitalization.htm">Housing and Neighborhood Revitalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/small-business-and-entrepreneurship.htm">Small Business and Entrepreneurship</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/workforce.htm">Employment and Workforce Development</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cdf.htm">Community Development Finance</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/rural-community-economic-development.htm">Rural Community and Economic Development</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Research&nbsp;&amp; Analysis</strong>\n    </p>\n</li>\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/community-development-publications.htm">Research Publications & Data Analysis</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/cac.htm">Community Advisory Council</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Consumer Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/foreclosure.htm">Mortgage and Foreclosure Resources</a>\n</li>\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/comm-dev-system-map.htm">Federal Reserve Community Development Resources</a>\n</li>\n\n\n\n                </ul>\n            </div>\n        </li>             \n    </ul>\n</li>\n \n \n \n \n \n \n \n \n \n \n \n \n    </ul>\n</nav>\n        <!-- BODY CONTENT -->\n        <div id="content" class="container container__main" role="main">\n            <div class="row">\n                <div class="page-header">\n                    <ol class="breadcrumb">\n                            <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/default.htm">Home</a></li>\n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems.htm">Payment Systems</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/fedach_data.htm">Data</a></li>\n                                          \n \n                                          \n \n                                          \n \n                                          \n \n \n                    </ol>\n                    <div class="header-group">\n                        <div id="page-title" class="page-title"> \n                        \n                                \t\t<h2>\n                                \t\t    Automated Clearinghouse Services\n                                \t\t</h2>\n                            \t\t\n                        </div> \n                        \n                  </div>\n                </div><!-- / .page-header -->\n            \n            </div><!-- / .row -->\n  \n\n\n        <div class=\'row\'>\n        \n                                <div class="col-xs-12 col-sm-4 col-md-3">\n                                  <div id="t4_nav" class="t4_nav list-group sticky">\n                                    \n                                    \r\n\r\n\r\n\r\n<!-- <h3 class="text-uppercase">Chapters</h3> -->\r\n\r\n\r\n\r\n\r\n<a href="/paymentsystems/fedach_data.htm" class="list-group-item" title="">Data</a>\n                                  </div>\n                                </div>\n                        \n                                                    <div id="article" class="col-xs-12 col-sm-8 col-md-9">\n                                                    <h3>Commercial Automated Clearinghouse Transactions Processed by the Federal Reserve--Annual Data</h3>\n                                                \n                                <!-- Article Area -->\n                                    \n                                    <div id="floatRightRail" class="col-xs-12 col-sm-12 col-md-5 pull-right">\n                                        \n                                        \n\n<!-- Text Box Begins -->\n\n<div class="panel panel-related">\n    <div class="panel-heading">\n                   \n                <h5 class="panel-title text-capitalize">\n                    Other Formats  \n                </h5>\n                \n    </div>\n    <div class="panel-body">\n        <ul>\n\t<li><a href="/paymentsystems/files/fedach_yearlycomm.txt">ASCII</a></li>\n\t<li><a href="/paymentsystems/files/fedach_yearlycomm.pdf">PDF (102&nbsp;KB)</a></li>\n</ul>\n    </div>\n</div>\n\n\n\n\n            \n                                    </div>     \n                                    \n                                    \n                                    <p>The daily average volume and average daily value of items are based on the number of business days in each year.<a id="ft1" name="ft1"></a><a href="#fn1" title="footnote 1"><sup>1</sup></a></p>\n\n<div class="data-table" data-table-popout="" id="data-table-0">\n<table class="pubtables" data-other-sticky-ids="navbar-mobile,nav-primary" data-sticky-columns="1" data-sticky-rows="1">\n\t<thead>\n\t\t<tr class="header">\n\t\t\t<th align="center" scope="col" valign="bottom">Year</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Volume<br />\n\t\t\t(millions<br />\n\t\t\tof items)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Volume<br />\n\t\t\tpercent<br />\n\t\t\tchange</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Value<br />\n\t\t\t(billions<br />\n\t\t\tof dollars)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Value<br />\n\t\t\tpercent<br />\n\t\t\tchange</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average daily<br />\n\t\t\tvolume<br />\n\t\t\t(millions<br />\n\t\t\tof items)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average daily<br />\n\t\t\tvalue<br />\n\t\t\t(billions<br />\n\t\t\tof dollars)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average value<br />\n\t\t\tper<br />\n\t\t\ttransaction<br />\n\t\t\t(dollars)</th>\n\t\t</tr>\n\t</thead>\n\t<tbody>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2020</strong></td>\n\t\t\t<td class="left2">16,549</td>\n\t\t\t<td class="left2">6.2</td>\n\t\t\t<td class="left2">31,446</td>\n\t\t\t<td class="left2">12.0</td>\n\t\t\t<td class="left2">64.6</td>\n\t\t\t<td class="left2">122.8</td>\n\t\t\t<td class="left2">1,900</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2019</strong></td>\n\t\t\t<td class="left2">15,584</td>\n\t\t\t<td class="left2">6.1</td>\n\t\t\t<td class="left2">28,082</td>\n\t\t\t<td class="left2">8.6</td>\n\t\t\t<td class="left2">62.1</td>\n\t\t\t<td class="left2">111.9</td>\n\t\t\t<td class="left2">1,802</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2018</strong></td>\n\t\t\t<td class="left2">14,692</td>\n\t\t\t<td class="left2">6.9</td>\n\t\t\t<td class="left2">25,860</td>\n\t\t\t<td class="left2">10.5</td>\n\t\t\t<td class="left2">58.5</td>\n\t\t\t<td class="left2">103.0</td>\n\t\t\t<td class="left2">1,760</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2017</strong></td>\n\t\t\t<td class="left2">13,749</td>\n\t\t\t<td class="left2">6.1</td>\n\t\t\t<td class="left2">23,399</td>\n\t\t\t<td class="left2">7.5</td>\n\t\t\t<td class="left2">55.0</td>\n\t\t\t<td class="left2">93.6</td>\n\t\t\t<td class="left2">1,702</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2016</strong></td>\n\t\t\t<td class="left2">12,960</td>\n\t\t\t<td class="left2">5.4</td>\n\t\t\t<td class="left2">21,772</td>\n\t\t\t<td class="left2">5.9</td>\n\t\t\t<td class="left2">51.6</td>\n\t\t\t<td class="left2">86.7</td>\n\t\t\t<td class="left2">1,680</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2015</strong></td>\n\t\t\t<td class="left2">12,298</td>\n\t\t\t<td class="left2">5.8</td>\n\t\t\t<td class="left2">20,565</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">49.0</td>\n\t\t\t<td class="left2">81.9</td>\n\t\t\t<td class="left2">1,672</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2014</strong></td>\n\t\t\t<td class="left2">11,620</td>\n\t\t\t<td class="left2">4.3</td>\n\t\t\t<td class="left2">19,891</td>\n\t\t\t<td class="left2">1.0</td>\n\t\t\t<td class="left2">46.3</td>\n\t\t\t<td class="left2">79.2</td>\n\t\t\t<td class="left2">1,712</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2013</strong></td>\n\t\t\t<td class="left2">11,143</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">19,689</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">44.4</td>\n\t\t\t<td class="left2">78.4</td>\n\t\t\t<td class="left2">1,767</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2012</strong></td>\n\t\t\t<td class="left2">10,665</td>\n\t\t\t<td class="left2">3.1</td>\n\t\t\t<td class="left2">19,294</td>\n\t\t\t<td class="left2">8.4</td>\n\t\t\t<td class="left2">42.5</td>\n\t\t\t<td class="left2">76.9</td>\n\t\t\t<td class="left2">1,809</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2011</strong></td>\n\t\t\t<td class="left2">10,349</td>\n\t\t\t<td class="left2">1.1</td>\n\t\t\t<td class="left2">17,802</td>\n\t\t\t<td class="left2">5.1</td>\n\t\t\t<td class="left2">41.2</td>\n\t\t\t<td class="left2">70.9</td>\n\t\t\t<td class="left2">1,720</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2010</strong></td>\n\t\t\t<td class="left2">10,233</td>\n\t\t\t<td class="left2">2.7</td>\n\t\t\t<td class="left2">16,941</td>\n\t\t\t<td class="left2">9.9</td>\n\t\t\t<td class="left2">40.6</td>\n\t\t\t<td class="left2">67.2</td>\n\t\t\t<td class="left2">1,656</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2009</strong></td>\n\t\t\t<td class="left2">9,966</td>\n\t\t\t<td class="left2">-0.7</td>\n\t\t\t<td class="left2">15,419</td>\n\t\t\t<td class="left2">-1.6</td>\n\t\t\t<td class="left2">39.5</td>\n\t\t\t<td class="left2">61.2</td>\n\t\t\t<td class="left2">1,547</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2008</strong></td>\n\t\t\t<td class="left2">10,040</td>\n\t\t\t<td class="left2">7.2</td>\n\t\t\t<td class="left2">15,663</td>\n\t\t\t<td class="left2">7.7</td>\n\t\t\t<td class="left2">39.8</td>\n\t\t\t<td class="left2">62.2</td>\n\t\t\t<td class="left2">1,560</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2007</strong></td>\n\t\t\t<td class="left2">9,363</td>\n\t\t\t<td class="left2">13.8</td>\n\t\t\t<td class="left2">14,547</td>\n\t\t\t<td class="left2">10.8</td>\n\t\t\t<td class="left2">37.3</td>\n\t\t\t<td class="left2">58.0</td>\n\t\t\t<td class="left2">1,554</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2006</strong></td>\n\t\t\t<td class="left2">8,231</td>\n\t\t\t<td class="left2">12.2</td>\n\t\t\t<td class="left2">13,124</td>\n\t\t\t<td class="left2">2.5</td>\n\t\t\t<td class="left2">32.8</td>\n\t\t\t<td class="left2">52.3</td>\n\t\t\t<td class="left2">1,595</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2005</strong></td>\n\t\t\t<td class="left2">7,339</td>\n\t\t\t<td class="left2">13.1</td>\n\t\t\t<td class="left2">12,802</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">29.2</td>\n\t\t\t<td class="left2">51.0</td>\n\t\t\t<td class="left2">1,744</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2004</strong></td>\n\t\t\t<td class="left2">6,486</td>\n\t\t\t<td class="left2">16.1</td>\n\t\t\t<td class="left2">12,544</td>\n\t\t\t<td class="left2">-10.1</td>\n\t\t\t<td class="left2">25.6</td>\n\t\t\t<td class="left2">49.6</td>\n\t\t\t<td class="left2">1,934</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2003</strong></td>\n\t\t\t<td class="left2">5,588</td>\n\t\t\t<td class="left2">12.1</td>\n\t\t\t<td class="left2">13,952</td>\n\t\t\t<td class="left2">6.2</td>\n\t\t\t<td class="left2">22.3</td>\n\t\t\t<td class="left2">55.6</td>\n\t\t\t<td class="left2">2,497</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2002</strong></td>\n\t\t\t<td class="left2">4,986</td>\n\t\t\t<td class="left2">12.1</td>\n\t\t\t<td class="left2">13,135</td>\n\t\t\t<td class="left2">3.4</td>\n\t\t\t<td class="left2">19.9</td>\n\t\t\t<td class="left2">52.3</td>\n\t\t\t<td class="left2">2,634</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2001</strong></td>\n\t\t\t<td class="left2">4,448</td>\n\t\t\t<td class="left2">16.7</td>\n\t\t\t<td class="left2">12,707</td>\n\t\t\t<td class="left2">9.4</td>\n\t\t\t<td class="left2">17.7</td>\n\t\t\t<td class="left2">50.6</td>\n\t\t\t<td class="left2">2,857</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2000</strong></td>\n\t\t\t<td class="left2">3,812</td>\n\t\t\t<td class="left2">14.0</td>\n\t\t\t<td class="left2">11,620</td>\n\t\t\t<td class="left2">7.0</td>\n\t\t\t<td class="left2">15.1</td>\n\t\t\t<td class="left2">46.1</td>\n\t\t\t<td class="left2">3,048</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1999</strong></td>\n\t\t\t<td class="left2">3,344</td>\n\t\t\t<td class="left2">12.7</td>\n\t\t\t<td class="left2">10,862</td>\n\t\t\t<td class="left2">5.1</td>\n\t\t\t<td class="left2">13.3</td>\n\t\t\t<td class="left2">43.1</td>\n\t\t\t<td class="left2">3,248</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1998</strong></td>\n\t\t\t<td class="left2">2,966</td>\n\t\t\t<td class="left2">13.9</td>\n\t\t\t<td class="left2">10,338</td>\n\t\t\t<td class="left2">13.2</td>\n\t\t\t<td class="left2">11.8</td>\n\t\t\t<td class="left2">41.0</td>\n\t\t\t<td class="left2">3,486</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><a id="ft2" name="ft2"> </a><strong>1997<a href="#fn2" title="footnote 2"><sup>2</sup></a></strong></td>\n\t\t\t<td class="left2">2,603</td>\n\t\t\t<td class="left2">9.7</td>\n\t\t\t<td class="left2">9,129</td>\n\t\t\t<td class="left2">10.1</td>\n\t\t\t<td class="left2">10.4</td>\n\t\t\t<td class="left2">36.4</td>\n\t\t\t<td class="left2">3,507</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1996</strong></td>\n\t\t\t<td class="left2">2,372</td>\n\t\t\t<td class="left2">15.9</td>\n\t\t\t<td class="left2">8,288</td>\n\t\t\t<td class="left2">6.0</td>\n\t\t\t<td class="left2">9.4</td>\n\t\t\t<td class="left2">32.9</td>\n\t\t\t<td class="left2">3,494</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1995</strong></td>\n\t\t\t<td class="left2">2,046</td>\n\t\t\t<td class="left2">17.8</td>\n\t\t\t<td class="left2">7,817</td>\n\t\t\t<td class="left2">10.2</td>\n\t\t\t<td class="left2">8.2</td>\n\t\t\t<td class="left2">31.1</td>\n\t\t\t<td class="left2">3,821</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1994</strong></td>\n\t\t\t<td class="left2">1,737</td>\n\t\t\t<td class="left2">16.9</td>\n\t\t\t<td class="left2">7,094</td>\n\t\t\t<td class="left2">9.9</td>\n\t\t\t<td class="left2">6.9</td>\n\t\t\t<td class="left2">28.3</td>\n\t\t\t<td class="left2">4,084</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1993</strong></td>\n\t\t\t<td class="left2">1,486</td>\n\t\t\t<td class="left2">12.0</td>\n\t\t\t<td class="left2">6,455</td>\n\t\t\t<td class="left2">-1.2</td>\n\t\t\t<td class="left2">5.9</td>\n\t\t\t<td class="left2">25.6</td>\n\t\t\t<td class="left2">4,344</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1992</strong></td>\n\t\t\t<td class="left2">1,327</td>\n\t\t\t<td class="left2">18.6</td>\n\t\t\t<td class="left2">6,531</td>\n\t\t\t<td class="left2">17.7</td>\n\t\t\t<td class="left2">5.2</td>\n\t\t\t<td class="left2">25.8</td>\n\t\t\t<td class="left2">4,922</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1991</strong></td>\n\t\t\t<td class="left2">1,119</td>\n\t\t\t<td class="left2">22.3</td>\n\t\t\t<td class="left2">5,549</td>\n\t\t\t<td class="left2">32.9</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">22.1</td>\n\t\t\t<td class="left2">4,959</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><a id="ft3" name="ft3"> </a><strong>1990<a href="#fn3" title="footnote 3"><sup>3</sup></a></strong></td>\n\t\t\t<td class="left2">915</td>\n\t\t\t<td class="left2">23.6</td>\n\t\t\t<td class="left2">4,174</td>\n\t\t\t<td class="left2">8.7</td>\n\t\t\t<td class="left2">3.6</td>\n\t\t\t<td class="left2">16.6</td>\n\t\t\t<td class="left2">4,562</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1989</strong></td>\n\t\t\t<td class="left2">741</td>\n\t\t\t<td class="left2">22.9</td>\n\t\t\t<td class="left2">3,840</td>\n\t\t\t<td class="left2">13.8</td>\n\t\t\t<td class="left2">3.0</td>\n\t\t\t<td class="left2">15.5</td>\n\t\t\t<td class="left2">5,182</td>\n\t\t</tr>\n\t</tbody>\n</table>\n</div>\n\n<h5>Footnotes</h5>\n\n<div class="footnotes">\n<p><a id="fn1" name="fn1">1.</a> Values are subject to minor changes based on revisions to underlying data. <a href="#ft1">Return to text</a></p>\n\n<p><a id="fn2" name="fn2">2.</a> Beginning in 1997, the reported ACH volumes no longer include non-value items. <a href="#ft2">Return to text</a></p>\n\n<p><a id="fn3" name="fn3">3.</a> Data for years preceding 1991 do not include items sent to the Reserve Banks by the New York Automated Clearinghouse. <a href="#ft3">Return to text</a></p>\n</div>\n\n                                    \n                                    \n                                </div>\n                                \n        </div>            \n        <a id="back-top" class="icon__backTop icon-backtop" href="#" title="Scroll To Top"><span class="sr-only">Back to Top</span></a>\n\n        \n        <div class="row">\n            <div class=\'lastUpdate\' id="lastUpdate">Last Update:\n              \n                            March 09, 2021\n                          \n            </div>\n        </div>\n        </div><!-- / #content .container -->\n        <footer class="container footer">\n    <div class="row footer__content">\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="footer__heading"><span class="text-uppercase">Board of Governors</span> <em>of the</em> <span class="text-uppercase">Federal Reserve System</span></h6>\n            <ul class="list-unstyled">\n                <li class=\'footer__listItem\'><a class="footer__link" href="/aboutthefed.htm">About the Fed</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/newsevents.htm">News & Events</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/monetarypolicy.htm">Monetary Policy</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/supervisionreg.htm">Supervision & Regulation</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/paymentsystems.htm">Payment Systems</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/econres.htm">Economic Research</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/data.htm">Data</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/consumerscommunities.htm">Consumers & Communities</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/financial-stability.htm">Financial Stability</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/economic-disparities-work.htm">Federal Reserve\'s Work Related to Economic Disparities</a></li> \n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="text-uppercase footer__heading">Tools and Information</h6>\n            <ul class="list-unstyled">\n                 <li class=\'footer__listItem\'><a href="/aboutthefed/contact-us-topics.htm" class="footer__link">Contact</a></li>\n                <li class=\'footer__listItem\'><a href="/publications.htm" class="footer__link">Publications</a></li>\n                <li class=\'footer__listItem\'><a href="/foia/about_foia.htm" class="footer__link">Freedom of Information (FOIA)</a></li>\n                <li class=\'footer__listItem\'><a href="https://oig.federalreserve.gov/" class="footer__link">Office of Inspector General</a></li>\n                <li class=\'footer__listItem\'><a href="/publications/annual-report.htm" class="footer__link">Budget &amp; Performance</a> | <a href="/regreform/audit.htm" class="footer__link">Audit</a></li>\n                <li class=\'footer__listItem\'><a href="/eeo.htm" class="footer__link">No FEAR Act</a></li>\n                <li class=\'footer__listItem\'><a href="/espanol.htm" class="footer__link">Espa\xc3\xb1ol</a></li>\n                <li class=\'footer__listItem\'><a href="/website-linking-policies.htm" class="footer__link">Website Policies</a>  | <a href="/privacy.htm" class="footer__link">Privacy Program</a></li>\n                <li class=\'footer__listItem\'><a href="/accessibility.htm" class="footer__link">Accessibility</a></li>\n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <div class="footer__social">\n                <h6 class="text-uppercase footer__heading footer__heading--social">Stay Connected</h6>\n                \n\n\n    \n                <a class="icon__md icon footer__btn icon-facebook-color" href="https://www.facebook.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-twitter-color" href="https://twitter.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-youtube-color" href="https://www.youtube.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-flickr-color" href="https://www.flickr.com/photos/federalreserve/"><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-linkedin-color" href="https://www.linkedin.com/company/federal-reserve-board"><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-rss-color" href="/feeds/feeds.htm"><span class=\'sr-only\'>Subscribe to RSS</span></a>\n            \n                <a class="icon__md icon footer__btn icon-email-color" href="/subscribe.htm"><span class=\'sr-only\'>Subscribe to Email</span></a>\n                    \n    \n \n \n            </div>\n            <a href="https://www.usa.gov/" target="_blank" title="Link to USA.gov">\n                <img class=\'footer__image\' src="/images/USAGov%402x.png" alt="Link to USA.gov" />\n            </a>\n            <a href="/open/open.htm" target="_blank" title="Link to Open.gov">\n                <img class=\'footer__image\' src="/images/OpenGov%402x.png" alt="Link to Open.gov" />\n            </a>\n        </div>\n    </div>\n    <div class="row">\n        <div class="col-xs-12 footer__footer">\n            <p class="text-uppercase footer__text footer__text--left">Board of Governors <em class="text-lowercase">of the</em> Federal Reserve System</p>\n            <p class="footer__text footer__text--right">20th Street and Constitution Avenue N.W., Washington, DC 20551</p>\n        </div>\n    </div>\n</footer>\n\n\n                <script src="/js/jquery1.11.1.min.js" type="text/javascript"></script>\n                <script src="/js/bootstrap.min.js" type="text/javascript"></script>\n                <script src="/js/scripts.js" type="text/javascript"></script>\n                <script src="/js/modal.js" type="text/javascript"></script>\n                <script src="/js/sticky-tables.js" type="text/javascript"></script>\n                <script type="text/javascript" src="/js/slides/ekko-lightbox.min.js"></script>\n                <script src="/js/t4-nav-controller.js" type="text/javascript"></script>\n              \n<script type="text/javascript">\n    $(document).ready(function($) {\n        // delegate calls to data-toggle="lightbox"\n        $(document).delegate(\'*[data-toggle="lightbox"]:not([data-gallery="navigateTo"])\', \'click\', function(event) {\n            event.preventDefault();\n            $(this).ekkoLightbox();\n        });\n        $(\'a[data-click="false"]\').click(function(e) {\n            e.preventDefault();\n            var target = $(this).data(\'target\');\n            $(target).click();\n        });\n    });\n</script>          \n             \n\n\n        \n        <!--<ul></ul> -->\n    <script type="text/javascript" src="/resources/track_downloads.js"></script>\n</body>\n</html>\n '  # noqa: E501
            result = ACH().all()
            self.assertIsInstance(result, pd.DataFrame)


class Test_Check_Services(TestCase):
    def test_get_data(self) -> None:
        url_base = "https://www.federalreserve.gov/paymentsystems/check_"
        url = f"{url_base}postalmosprocannual.htm"
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\xef\xbb\xbf<!doctype html>\n<html lang="en" class="no-js">\n    <head>\n        <meta charset="utf-8">\n        <meta http-equiv="X-UA-Compatible" content="IE=edge"/>\n        <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0 maximum-scale=1.6, user-scalable=1"/>\n        <meta name="keywords" content="Board of Governors of the Federal Reserve System, Federal Reserve Board of Governors, Federal Reserve Board, Federal Reserve" />\n        <meta name="description" content="The Federal Reserve Board of Governors in Washington DC." />\n        <meta property="og:site_name" content="Board of Governors of the Federal Reserve System"/>\n        <meta property="og:type" content="article" /> \n        <meta property="og:image"  content="" /> \n        <meta name="twitter:card" content="summary" />\n        <meta name="twitter:image" content="" />\n        \n\n\n \n        <title>Federal Reserve Board - Postal Money Orders Processed by the Federal Reserve--Annual Data</title>\n        \n    <link href="/css/bootstrap.css" rel="stylesheet" type="text/css">\n    <link href="/css/bluesteel-theme.css" rel="stylesheet" type="text/css">\n    <script src="/js/modernizr-latest.js" type="text/javascript"></script>\n    <script src="/js/grunticon.loader.js"></script>\n    <script>\n        grunticon(["/css/icons.data.svg.css", "/css/icons.data.png.css", "/css/icons.fallback.css"]);\n    </script>\n    <noscript>\n        <link href="/css/icons.fallback.css" rel="stylesheet">\n    </noscript>                \n    <script src="/js/angular.min.js" type="text/javascript"></script>\n    <script src="/js/app.js" type="text/javascript"></script>\n        \n    <!-- For Linear Gradients in IE9 or greater -->\n    <!--[if gte IE 9]>\n        <style type="text/css">\n          .gradient {\n             filter: none;\n          }\n        </style>\n      <![endif]-->\n\n\n \n    <script type="text/javascript" >\n  (function(i,s,o,g,r,a,m){i[\'GoogleAnalyticsObject\']=r;i[r]=i[r]||function(){\n  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),\n  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)\n  })(window,document,\'script\',\'//www.google-analytics.com/analytics.js\',\'ga\');\n  ga(\'create\', \'UA-35121701-1\', \'federalreserve.gov\', {\'cookieExpires\': 0});\n  ga(\'set\', \'anonymizeIp\', true);\n  ga(\'send\', \'pageview\');\n</script></head>\n    <body data-ng-app="pubwebApp">\n        <a href="#content" class="globalskip">Skip to main content</a>\n<nav class="nav-primary-mobile" id="nav-primary-mobile">\n</nav>\n<div class="t1_nav navbar navbar-default navbar-fixed-top" role="navigation">\n    <div class="t1_container clearfix">\n        <a id="logo" class="btn btn-default icon-FRB_logo-bw visible-xs-table-cell t1__btnlogo" href="/default.htm" role="button"><span class="sr-only">Back to Home</span>\n        </a>\n        <a href="/default.htm" id="banner" class="visible-xs-table-cell t1__banner" title="Link to Home Page" role="button">Board of Governors of the Federal Reserve System</a>\n        <ul class="nav navbar-nav visible-md visible-lg t1_toplinks">\n            <li class="navbar-text navbar-text__social">\n                <span class="navbar-link" tabindex="0">Stay Connected</span>\n                <ul class="nav navbar-nav t1_social">\n                    \n\n\n    \n                <li class="social__item"><a data-icon="facebook" href="https://www.facebook.com/federalreserve" class="noIcon"><div class="icon-facebook-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="twitter" href="https://twitter.com/federalreserve" class="noIcon"><div class="icon-twitter-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="youtube" href="https://www.youtube.com/federalreserve" class="noIcon"><div class="icon-youtube-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="flickr" href="https://www.flickr.com/photos/federalreserve/" class="noIcon"><div class="icon-flickr-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="linkedin" href="https://www.linkedin.com/company/federal-reserve-board" class="noIcon"><div class="icon-linkedin-color icon icon__sm"></div><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="rss" href="/feeds/feeds.htm" class="noIcon"><div class="icon-rss-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to RSS</span></a></li>\n            \n                <li class="social__item"><a data-icon="email" href="/subscribe.htm" class="noIcon"><div class="icon-email-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to Email</span></a></li>\n                    \n    \n \n \n                </ul>\n            </li>\n           \r\n                                <li class="navbar-text"><a class="navbar-link" href="/recentpostings.htm">Recent Postings</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/newsevents/calendar.htm">Calendar</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/publications.htm">Publications</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/sitemap.htm">Site Map</a></li>\r\n                \r\n                                <li class="navbar-text"><a class="navbar-link" href="/azindex.htm">A-Z index</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/careers.htm">Careers</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/faqs.htm">FAQs</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/videos.htm">Videos</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/aboutthefed/contact-us-topics.htm">Contact</a></li>\r\n                \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \n        </ul>\n        <form class="nav navbar-form hidden-xs nav__search" role="search" action="//www.fedsearch.org/board_public/search" method="get">\n            <div class="form-group input-group input-group-sm">\n                <label for="t1search" class=\'sr-only\'>Search</label>\n                <input id="t1search" class="nav__input form-control" name="text" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                    <button type="submit" class="nav__button btn btn-default" id="headerTopLinksSearchFormSubmit" name="Search">\n                        <span class="sr-only">Submit Search Button</span>\n                        <span class="icon-magnifying-glass icon icon__xs icon--right"></span>\n                    </button>\n                </span>\n            </div>\n            <div class=\'nav__advanced\'><a class="noIcon" href="//www.fedsearch.org/board_public">Advanced</a></div>\n        </form>        \n        <div class="btn-group visible-xs-table-cell visible-sm-table-cell t1__dropdown">\n            <span class="icon icon__md icon--right dropdown-toggle icon-options-nav" data-toggle="dropdown">\n                <span class="sr-only">Toggle Dropdown Menu</span>\n            </span>\n        </div>\n    </div>\n</div>\n<header class="jumbotron hidden-xs">\n    <a class="jumbotron__link" href="/default.htm" title="Link to Home Page">\n        <div class="container-fluid">\n            <h1 class="jumbotron__heading">Board of Governors of the Federal Reserve System</h1>\n            <p class="jumbotron__content"><em>The Federal Reserve, the central bank of the United States, provides the nation with a safe, flexible, and stable monetary and financial system.</em></p>\n        </div>\n    </a>\n</header>\n<div class="t2__offcanvas visible-xs-block">\n    <div class="navbar-header">\n        <button type="button" class="offcanvas__toggle icon-offcanvas-nav" data-toggle="offcanvas" data-target="#offcanvasT2Menu" data-canvas="body">\n            <span class="sr-only">Main Menu Toggle Button</span>\n        </button>\n        <span class="navbar-brand offcanvas__title">Sections</span>\n        <button type="button" class="offcanvas__search icon-search-nav" data-toggle="collapse" data-target="#navbar-collapse">\n            <span class="sr-only">Search Toggle Button</span>\n        </button>\n    </div>\n    <div class="navbar-collapse collapse mobile-form" id="navbar-collapse">\n        <form method="get" class="navbar-form" role="search" action="//www.fedsearch.org/board_public/search">\n            <div class="form-group input-group mobileSearch">\n                <label class="sr-only" for="t2search">Search</label>\n                <input id="t2search" name="text" class="form-control" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                <span class="sr-only">Search Submit Button</span>\n                <button type="submit" class="btn btn-default">Submit</button>\n                </span>\n            </div>\n        </form>\n    </div>\n</div>\n<nav class="nav-primary navbar hidden-xs" id="nav-primary" role="navigation">\n    <ul class="nav navbar-nav" role="menubar">\n        \n \n<li role="menuitem" class="dropdown nav-about  dropdown--3Col">\n    <a href="/aboutthefed.htm" class="sr-only-focusable" aria-haspopup="true" id="aboutMenu">About<br />the Fed</a>\n    <ul aria-labelledby="aboutMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-4 list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/structure-federal-reserve-system.htm">Structure of the Federal Reserve System</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/pf.htm">Purposes &amp; Functions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/bios/board/default.htm">Board Members</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/advisorydefault.htm">Advisory Councils</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/federal-reserve-system.htm">Federal Reserve Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fract.htm">Federal Reserve Act</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/currency.htm">Currency</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardmeetings/meetingdates.htm">Board Meetings</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardvotes.htm">Board Votes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/diversityinclusion.htm">Diversity & Inclusion</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/careers.htm">Careers</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/procurement/about.htm">Do Business with the Board</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/k8.htm">Holidays Observed - K.8</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/contact-us-topics.htm">Contact </a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/foia/about_foia.htm">Requesting Information (FOIA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/faqs.htm">FAQs</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/educational-tools/default.htm">Economic Education</a>\n</li>\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fed-financial-statements.htm">Fed Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/innovation.htm">Innovation</a>\n</li>\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-news  dropdown--1Col">\n    <a href="/newsevents.htm" class="sr-only-focusable" aria-haspopup="true" id="newsMenu">News<br />&amp; Events</a>\n    <ul aria-labelledby="newsMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        \n<li>\n    <a class="sr-only-focusable" href="/newsevents/pressreleases.htm">Press Releases</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/speeches.htm">Speeches</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/testimony.htm">Testimony</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/calendar.htm">Calendar</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/videos.htm">Videos\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/photogallery.htm">Photo Gallery\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-monetary  dropdown--2Col">\n    <a href="/monetarypolicy.htm" class="sr-only-focusable" aria-haspopup="true" id="monetaryMenu">Monetary<br />Policy</a>\n    <ul aria-labelledby="monetaryMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Federal Open Market Committee</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc.htm">About the FOMC</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomccalendars.htm">Meeting calendars and information</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_historical.htm">Transcripts and other historical materials</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_projectionsfaqs.htm">FAQs</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Monetary Policy Principles and Practice</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/monetary-policy-principles-and-practice.htm">Notes</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Policy Implementation</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policy-normalization.htm">Policy Normalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policytools.htm">Policy Tools</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/mpr_default.htm">Monetary Policy Report</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/beige-book-default.htm">Beige Book</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/quarterly-balance-sheet-developments-report.htm">Quarterly Report on Federal Reserve Balance Sheet Developments</a>\n</li>\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Review of Monetary Policy Strategy, Tools, and Communications</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/review-of-monetary-policy-strategy-tools-and-communications.htm">Overview</a>\n</li>\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-supervision  dropdown--fw">\n    <a href="/supervisionreg.htm"  class="sr-only-focusable" id="supervisionMenu" aria-haspopup="true">Supervision<br />&amp; Regulation</a>\n    <ul aria-labelledby="supervisionMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Institution Supervision</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/community-and-regional-financial-institutions.htm">Community & Regional Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/large-financial-institutions.htm">Large Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/foreign-banking-organizations.htm">Foreign Banking Organizations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/financial-market-utility-supervision.htm">Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/consumer-compliance.htm">Consumer Compliance</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supervision-and-regulation-report.htm">Federal Reserve Supervision and Regulation Report</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reporting Forms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/default.aspx">All Reporting Forms</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reporting-forms-whats-new.htm">Recent Updates</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/review.aspx">Information collections under review</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?xTdTa+FaSlI=">Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?+PhkJwiBjbA=">Applications/structure change</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?1/RIXO/UduI=">Federal Financial Institutions Examination Council (FFIEC)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?L877VwEj9Ss=">Municipal &amp; Government Securities</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Supervision &amp; Regulation Letters</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/srletters/srletters.htm">By Year</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/topics/topics.htm">By Topic</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Banking Applications &amp; Legal Developments</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/application-process.htm">Application Process</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/board-and-reserve-bank-actions.htm">Board & Reserve Bank Action</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/legal-developments.htm">Enforcement Actions & Legal Developments</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Regulatory Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">Regulations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supmanual.htm">Manuals</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/basel/basel-default.htm">Basel Regulatory Framework</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/volcker-rule.htm">Volcker Rule</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/training-and-assistance-landing.htm">Education, Training, and Assistance</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Banking &amp; Data Structure</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/insider.aspx">Beneficial Ownership reports</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/">U.S. Offices of Foreign Entities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/fhc.htm">Financial Holding Companies</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/isb-default.htm">Interstate Branching</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/suds.htm">Securities Underwriting & Dealing Subsidiaries</a>\n</li>\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-payment  dropdown--fw">\n    <a href="/paymentsystems.htm"  class="sr-only-focusable" id="paymentMenu" aria-haspopup="true">Payment<br />Systems</a>\n    <ul aria-labelledby="paymentMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations &amp; Statutes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regcc-about.htm">Regulation CC (Availability of Funds and Collection of Checks)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regii-about.htm">Regulation II (Debit Card Interchange Fees and Routing)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/reghh-about.htm">Regulation HH (Financial Market Utilities)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/other-regulations.htm">Other Regulations and Statutes</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Payment Policies</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pfs_about.htm">Federal Reserve\'s Key Policies for the Provision of Financial Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/oo_about.htm">Overnight Overdrafts</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_about.htm">Payment System Risk</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/telecomm.htm">Sponsorship for Priority Telecommunication Services</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reserve Bank Payment Services &amp; Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/check_about.htm">Check Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_data.htm">Daylight Overdrafts and Fees</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fednow_about.htm">FedNow\xe2\x84\xa0 Service</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedfunds_about.htm">Fedwire Funds Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedsecs_about.htm">Fedwire Securities Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fisagy_about.htm">Fiscal Agency Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/natl_about.htm">National Settlement Service</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Financial Market Utilities &amp; Infrastructures</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/over_about.htm">Supervision & Oversight of Financial Market Infrastructures</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/designated_fmu_about.htm">Designated Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/int_standards.htm">International Standards for Financial Market Infrastructures</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Research, Committees, and Forums</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/payres_about.htm">Payment Research</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fr-payments-study.htm">Federal Reserve Payments Study (FRPS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pspa_committee.htm">Payments System Policy Advisory Committee</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/forums.htm">Forums</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-econ dropdown--right dropdown--2Col">\n    <a href="/econres.htm" class="sr-only-focusable" aria-haspopup="true" id="econMenu">Economic<br />Research</a>\n    <ul aria-labelledby="econMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Working Papers and Notes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/feds/index.htm">Finance and Economics Discussion Series (FEDS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/notes/feds-notes/default.htm">FEDS Notes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/ifdp/index.htm">International Finance Discussion Papers (IFDP)</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/econres/theeconomists.htm">Meet the Economists</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Models and Tools</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/us-models-about.htm">FRB/US Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/edo-models-about.htm">Estimated Dynamic Optimization (EDO) Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n\n\n\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-data dropdown--right dropdown--fw">\n    <a href="/data.htm"  class="sr-only-focusable" id="dataMenu" aria-haspopup="true">Data</a>\n    <ul aria-labelledby="dataMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/datadownload"><span class="icon icon-ddp icon__sm"></span> Data Download Program</a>\n</li>\n\n\n<li class="nav__header">\n    <p>\n        <strong>Bank Assets and Liabilities</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h3/current/default.htm">Aggregate Reserves of Depository Institutions and the Monetary Base - H.3</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h8/current/default.htm">Assets and Liabilities of Commercial Banks in the U.S. - H.8</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/assetliab/current.htm">Assets and Liabilities of U.S. Branches and Agencies of Foreign Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/chargeoff/">Charge-Off and Delinquency Rates on Loans and Leases at Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sfos/sfos.htm">Senior Financial Officer Survey</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sloos.htm">Senior Loan Officer Opinion Survey on Bank Lending Practices</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Bank Structure Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for the U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Business Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/cp/">Commercial Paper</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g20/current/g20.htm">Finance Companies - G.20</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/govsecure/current.htm">New Security Issues, State and Local Governments</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/corpsecure/current.htm">New Security Issues, U.S. Corporations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Dealer Financing Terms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/scoos.htm">Senior Credit Officer Opinion Survey on Dealer Financing Terms</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Exchange Rates and International Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h10/current/">Foreign Exchange Rates - H.10/G.5</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/intlsumm/current.htm">International Summary Statistics</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/secholdtrans/current.htm">Securities Holdings and Transactions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/statbanksus/current.htm">Statistics Reported by Banks and Other Financial Firms in the United States</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Financial Accounts</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/z1/default.htm">Financial Accounts of the United States - Z.1</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Household Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g19/current/default.htm">Consumer Credit - G.19</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/housedebt/default.htm">Household Debt Service and Financial Obligations Ratios</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mortoutstand/current.htm">Mortgage Debt Outstanding</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Industrial Activity</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g17/current/default.htm">Industrial Production and Capacity Utilization - G.17</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Interest Rates</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h15/">Selected Interest Rates - H.15</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Micro Data Reference Manual (MDRM)</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mdrm.htm">Micro and Macro Data Collections</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Money Stock and Reserve Balances</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h41/">Factors Affecting Reserve Balances - H.4.1</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h6/current/default.htm">Money Stock Measures - H.6</a>\n</li>\n\n\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Other</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/yield-curve-models.htm">Yield Curve Models and Data</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-consumer dropdown--right dropdown--4Col">\n    <a href="/consumerscommunities.htm" class="sr-only-focusable" aria-haspopup="true" id="consumerMenu">Consumers<br />&amp; Communities</a>\n    <ul aria-labelledby="consumerMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cra_about.htm">Community Reinvestment Act (CRA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">All Regulations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Supervision&nbsp;&amp; Enforcement</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/caletters/caletters.htm">CA Letters</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/enforcementactions/default.aspx">Enforcement Actions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/independent-foreclosure-review.htm">Independent Foreclosure Review</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Community Development</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/neighborhood-revitalization.htm">Housing and Neighborhood Revitalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/small-business-and-entrepreneurship.htm">Small Business and Entrepreneurship</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/workforce.htm">Employment and Workforce Development</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cdf.htm">Community Development Finance</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/rural-community-economic-development.htm">Rural Community and Economic Development</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Research&nbsp;&amp; Analysis</strong>\n    </p>\n</li>\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/community-development-publications.htm">Research Publications & Data Analysis</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/cac.htm">Community Advisory Council</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Consumer Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/foreclosure.htm">Mortgage and Foreclosure Resources</a>\n</li>\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/comm-dev-system-map.htm">Federal Reserve Community Development Resources</a>\n</li>\n\n\n\n                </ul>\n            </div>\n        </li>             \n    </ul>\n</li>\n \n \n \n \n \n \n \n \n \n \n \n \n    </ul>\n</nav>\n        <!-- BODY CONTENT -->\n        <div id="content" class="container container__main" role="main">\n            <div class="row">\n                <div class="page-header">\n                    <ol class="breadcrumb">\n                            <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/default.htm">Home</a></li>\n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems.htm">Payment Systems</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/check_about.htm">Check Services</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/check_data.htm">Data</a></li>\n                                          \n \n                                          \n \n                                          \n \n                                          \n \n \n                    </ol>\n                    <div class="header-group">\n                        <div id="page-title" class="page-title"> \n                        \n                                \t\t<h2>\n                                \t\t    Check Services\n                                \t\t</h2>\n                            \t\t\n                        </div> \n                        \n                  </div>\n                </div><!-- / .page-header -->\n            \n            </div><!-- / .row -->\n  \n\n\n        <div class=\'row\'>\n        \n                                <div class="col-xs-12 col-sm-4 col-md-3">\n                                  <div id="t4_nav" class="t4_nav list-group sticky">\n                                    \n                                    \r\n\r\n\r\n\r\n<!-- <h3 class="text-uppercase">Chapters</h3> -->\r\n\r\n\r\n\r\n\r\n<a href="/paymentsystems/check_data.htm" class="list-group-item" title="">Data</a>\n                                  </div>\n                                </div>\n                        \n                                                    <div id="article" class="col-xs-12 col-sm-8 col-md-9">\n                                                    <h3>Postal Money Orders Processed by the Federal Reserve--Annual Data</h3>\n                                                \n                                <!-- Article Area -->\n                                    \n                                    <div id="floatRightRail" class="col-xs-12 col-sm-12 col-md-5 pull-right">\n                                        \n                                        \n\n<!-- Text Box Begins -->\n\n<div class="panel panel-related">\n    <div class="panel-heading">\n                   \n                <h5 class="panel-title text-capitalize">\n                    Other Formats  \n                </h5>\n                \n    </div>\n    <div class="panel-body">\n        <ul>\n\t<li><a href="/paymentsystems/files/check_postalmosprocannual.txt">ASCII</a></li>\n\t<li><a href="/paymentsystems/files/check_postalmosprocannual.pdf">PDF (96&nbsp;KB)</a></li>\n</ul>\n    </div>\n</div>\n\n\n\n\n            \n                                    </div>     \n                                    \n                                    \n                                    <p>The daily average volume and average daily value of items are based on the number of business days in each year.<a id="ft1" name="ft1"></a><a href="#fn1" title="footnote 1"><sup>1</sup></a></p>\n\n<div class="data-table" data-table-popout="" id="data-table-0">\n<table class="pubtables" data-other-sticky-ids="navbar-mobile,nav-primary" data-sticky-columns="1" data-sticky-rows="1">\n\t<thead>\n\t\t<tr class="header">\n\t\t\t<th align="center" scope="col" valign="bottom">Year</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Volume<br />\n\t\t\t(millions<br />\n\t\t\tof items)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Volume<br />\n\t\t\tpercent<br />\n\t\t\tchange</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Value<br />\n\t\t\t(billions<br />\n\t\t\tof dollars)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Value<br />\n\t\t\tpercent<br />\n\t\t\tchange</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average daily<br />\n\t\t\tvolume<br />\n\t\t\t(thousands<br />\n\t\t\tof items)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average daily<br />\n\t\t\tvalue<br />\n\t\t\t(millions<br />\n\t\t\tof dollars)</th>\n\t\t\t<th align="center" scope="col" valign="bottom">Average value<br />\n\t\t\tper postal<br />\n\t\t\tmoney order<br />\n\t\t\t(dollars)</th>\n\t\t</tr>\n\t</thead>\n\t<tbody>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2020</strong></td>\n\t\t\t<td class="left2">74</td>\n\t\t\t<td class="left2">-8.3</td>\n\t\t\t<td class="left2">21</td>\n\t\t\t<td class="left2">-4.0</td>\n\t\t\t<td class="left2">288</td>\n\t\t\t<td class="left2">80</td>\n\t\t\t<td class="left2">269</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2019</strong></td>\n\t\t\t<td class="left2">80</td>\n\t\t\t<td class="left2">-2.8</td>\n\t\t\t<td class="left2">21</td>\n\t\t\t<td class="left2">1.8</td>\n\t\t\t<td class="left2">320</td>\n\t\t\t<td class="left2">85</td>\n\t\t\t<td class="left2">266</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2018</strong></td>\n\t\t\t<td class="left2">83</td>\n\t\t\t<td class="left2">-2.8</td>\n\t\t\t<td class="left2">21</td>\n\t\t\t<td class="left2">1.7</td>\n\t\t\t<td class="left2">330</td>\n\t\t\t<td class="left2">84</td>\n\t\t\t<td class="left2">254</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2017</strong></td>\n\t\t\t<td class="left2">85</td>\n\t\t\t<td class="left2">-3.3</td>\n\t\t\t<td class="left2">21</td>\n\t\t\t<td class="left2">0.0</td>\n\t\t\t<td class="left2">341</td>\n\t\t\t<td class="left2">83</td>\n\t\t\t<td class="left2">243</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2016</strong></td>\n\t\t\t<td class="left2">88</td>\n\t\t\t<td class="left2">-4.0</td>\n\t\t\t<td class="left2">21</td>\n\t\t\t<td class="left2">-0.4</td>\n\t\t\t<td class="left2">351</td>\n\t\t\t<td class="left2">82</td>\n\t\t\t<td class="left2">235</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2015</strong></td>\n\t\t\t<td class="left2">92</td>\n\t\t\t<td class="left2">-3.3</td>\n\t\t\t<td class="left2">21</td>\n\t\t\t<td class="left2">-0.7</td>\n\t\t\t<td class="left2">365</td>\n\t\t\t<td class="left2">83</td>\n\t\t\t<td class="left2">226</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2014</strong></td>\n\t\t\t<td class="left2">95</td>\n\t\t\t<td class="left2">-6.3</td>\n\t\t\t<td class="left2">21</td>\n\t\t\t<td class="left2">-6.1</td>\n\t\t\t<td class="left2">378</td>\n\t\t\t<td class="left2">83</td>\n\t\t\t<td class="left2">220</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2013</strong></td>\n\t\t\t<td class="left2">101</td>\n\t\t\t<td class="left2">-5.9</td>\n\t\t\t<td class="left2">22</td>\n\t\t\t<td class="left2">1.5</td>\n\t\t\t<td class="left2">403</td>\n\t\t\t<td class="left2">89</td>\n\t\t\t<td class="left2">220</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2012</strong></td>\n\t\t\t<td class="left2">108</td>\n\t\t\t<td class="left2">-4.8</td>\n\t\t\t<td class="left2">22</td>\n\t\t\t<td class="left2">-1.3</td>\n\t\t\t<td class="left2">429</td>\n\t\t\t<td class="left2">87</td>\n\t\t\t<td class="left2">204</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2011</strong></td>\n\t\t\t<td class="left2">113</td>\n\t\t\t<td class="left2">-6.6</td>\n\t\t\t<td class="left2">22</td>\n\t\t\t<td class="left2">-4.3</td>\n\t\t\t<td class="left2">450</td>\n\t\t\t<td class="left2">89</td>\n\t\t\t<td class="left2">197</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2010</strong></td>\n\t\t\t<td class="left2">121</td>\n\t\t\t<td class="left2">-7.8</td>\n\t\t\t<td class="left2">23</td>\n\t\t\t<td class="left2">-2.0</td>\n\t\t\t<td class="left2">480</td>\n\t\t\t<td class="left2">92</td>\n\t\t\t<td class="left2">192</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2009</strong></td>\n\t\t\t<td class="left2">131</td>\n\t\t\t<td class="left2">-10.1</td>\n\t\t\t<td class="left2">24</td>\n\t\t\t<td class="left2">-7.3</td>\n\t\t\t<td class="left2">521</td>\n\t\t\t<td class="left2">94</td>\n\t\t\t<td class="left2">180</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2008</strong></td>\n\t\t\t<td class="left2">146</td>\n\t\t\t<td class="left2">-10.9</td>\n\t\t\t<td class="left2">26</td>\n\t\t\t<td class="left2">-19.2</td>\n\t\t\t<td class="left2">579</td>\n\t\t\t<td class="left2">101</td>\n\t\t\t<td class="left2">175</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2007</strong></td>\n\t\t\t<td class="left2">164</td>\n\t\t\t<td class="left2">-4.2</td>\n\t\t\t<td class="left2">32</td>\n\t\t\t<td class="left2">12.7</td>\n\t\t\t<td class="left2">653</td>\n\t\t\t<td class="left2">126</td>\n\t\t\t<td class="left2">193</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2006</strong></td>\n\t\t\t<td class="left2">171</td>\n\t\t\t<td class="left2">-3.1</td>\n\t\t\t<td class="left2">28</td>\n\t\t\t<td class="left2">-1.2</td>\n\t\t\t<td class="left2">681</td>\n\t\t\t<td class="left2">112</td>\n\t\t\t<td class="left2">164</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2005</strong></td>\n\t\t\t<td class="left2">176</td>\n\t\t\t<td class="left2">-5.6</td>\n\t\t\t<td class="left2">28</td>\n\t\t\t<td class="left2">-2.2</td>\n\t\t\t<td class="left2">703</td>\n\t\t\t<td class="left2">113</td>\n\t\t\t<td class="left2">161</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2004</strong></td>\n\t\t\t<td class="left2">187</td>\n\t\t\t<td class="left2">-5.7</td>\n\t\t\t<td class="left2">29</td>\n\t\t\t<td class="left2">-0.5</td>\n\t\t\t<td class="left2">739</td>\n\t\t\t<td class="left2">115</td>\n\t\t\t<td class="left2">155</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2003</strong></td>\n\t\t\t<td class="left2">198</td>\n\t\t\t<td class="left2">-8.4</td>\n\t\t\t<td class="left2">29</td>\n\t\t\t<td class="left2">-3.2</td>\n\t\t\t<td class="left2">790</td>\n\t\t\t<td class="left2">116</td>\n\t\t\t<td class="left2">147</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2002</strong></td>\n\t\t\t<td class="left2">216</td>\n\t\t\t<td class="left2">-5.6</td>\n\t\t\t<td class="left2">30</td>\n\t\t\t<td class="left2">-1.0</td>\n\t\t\t<td class="left2">862</td>\n\t\t\t<td class="left2">120</td>\n\t\t\t<td class="left2">139</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2001</strong></td>\n\t\t\t<td class="left2">229</td>\n\t\t\t<td class="left2">-0.3</td>\n\t\t\t<td class="left2">30</td>\n\t\t\t<td class="left2">1.4</td>\n\t\t\t<td class="left2">914</td>\n\t\t\t<td class="left2">121</td>\n\t\t\t<td class="left2">133</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2000</strong></td>\n\t\t\t<td class="left2">230</td>\n\t\t\t<td class="left2">1.9</td>\n\t\t\t<td class="left2">30</td>\n\t\t\t<td class="left2">3.2</td>\n\t\t\t<td class="left2">913</td>\n\t\t\t<td class="left2">119</td>\n\t\t\t<td class="left2">131</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1999</strong></td>\n\t\t\t<td class="left2">226</td>\n\t\t\t<td class="left2">6.2</td>\n\t\t\t<td class="left2">29</td>\n\t\t\t<td class="left2">2.3</td>\n\t\t\t<td class="left2">896</td>\n\t\t\t<td class="left2">116</td>\n\t\t\t<td class="left2">129</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1998</strong></td>\n\t\t\t<td class="left2">213</td>\n\t\t\t<td class="left2">4.4</td>\n\t\t\t<td class="left2">28</td>\n\t\t\t<td class="left2">7.6</td>\n\t\t\t<td class="left2">844</td>\n\t\t\t<td class="left2">113</td>\n\t\t\t<td class="left2">134</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1997</strong></td>\n\t\t\t<td class="left2">204</td>\n\t\t\t<td class="left2">-1.0</td>\n\t\t\t<td class="left2">26</td>\n\t\t\t<td class="left2">2.5</td>\n\t\t\t<td class="left2">812</td>\n\t\t\t<td class="left2">105</td>\n\t\t\t<td class="left2">130</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1996</strong></td>\n\t\t\t<td class="left2">206</td>\n\t\t\t<td class="left2">1.3</td>\n\t\t\t<td class="left2">26</td>\n\t\t\t<td class="left2">4.0</td>\n\t\t\t<td class="left2">816</td>\n\t\t\t<td class="left2">103</td>\n\t\t\t<td class="left2">126</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1995</strong></td>\n\t\t\t<td class="left2">203</td>\n\t\t\t<td class="left2">1.5</td>\n\t\t\t<td class="left2">25</td>\n\t\t\t<td class="left2">4.5</td>\n\t\t\t<td class="left2">809</td>\n\t\t\t<td class="left2">99</td>\n\t\t\t<td class="left2">122</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1994</strong></td>\n\t\t\t<td class="left2">200</td>\n\t\t\t<td class="left2">4.2</td>\n\t\t\t<td class="left2">24</td>\n\t\t\t<td class="left2">7.0</td>\n\t\t\t<td class="left2">797</td>\n\t\t\t<td class="left2">95</td>\n\t\t\t<td class="left2">119</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1993</strong></td>\n\t\t\t<td class="left2">192</td>\n\t\t\t<td class="left2">6.1</td>\n\t\t\t<td class="left2">22</td>\n\t\t\t<td class="left2">10.0</td>\n\t\t\t<td class="left2">762</td>\n\t\t\t<td class="left2">88</td>\n\t\t\t<td class="left2">116</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1992</strong></td>\n\t\t\t<td class="left2">181</td>\n\t\t\t<td class="left2">8.9</td>\n\t\t\t<td class="left2">20</td>\n\t\t\t<td class="left2">14.0</td>\n\t\t\t<td class="left2">715</td>\n\t\t\t<td class="left2">80</td>\n\t\t\t<td class="left2">112</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1991</strong></td>\n\t\t\t<td class="left2">166</td>\n\t\t\t<td class="left2">2.6</td>\n\t\t\t<td class="left2">18</td>\n\t\t\t<td class="left2">7.5</td>\n\t\t\t<td class="left2">662</td>\n\t\t\t<td class="left2">71</td>\n\t\t\t<td class="left2">107</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>1990</strong></td>\n\t\t\t<td class="left2">162</td>\n\t\t\t<td class="left2">9.9</td>\n\t\t\t<td class="left2">16</td>\n\t\t\t<td class="left2">15.4</td>\n\t\t\t<td class="left2">645</td>\n\t\t\t<td class="left2">66</td>\n\t\t\t<td class="left2">102</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>1989</strong></td>\n\t\t\t<td class="left2">147</td>\n\t\t\t<td class="left2">2.1</td>\n\t\t\t<td class="left2">14</td>\n\t\t\t<td class="left2">7.7</td>\n\t\t\t<td class="left2">587</td>\n\t\t\t<td class="left2">57</td>\n\t\t\t<td class="left2">97</td>\n\t\t</tr>\n\t</tbody>\n</table>\n</div>\n\n<h5>Footnotes</h5>\n\n<p><a id="fn1" name="fn1"></a>1. Values are subject to minor changes based on revisions to underlying data. <a href="#ft1">Return to text</a></p>\n\n                                    \n                                    \n                                </div>\n                                \n        </div>            \n        <a id="back-top" class="icon__backTop icon-backtop" href="#" title="Scroll To Top"><span class="sr-only">Back to Top</span></a>\n\n        \n        <div class="row">\n            <div class=\'lastUpdate\' id="lastUpdate">Last Update:\n              \n                            March 09, 2021\n                          \n            </div>\n        </div>\n        </div><!-- / #content .container -->\n        <footer class="container footer">\n    <div class="row footer__content">\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="footer__heading"><span class="text-uppercase">Board of Governors</span> <em>of the</em> <span class="text-uppercase">Federal Reserve System</span></h6>\n            <ul class="list-unstyled">\n                <li class=\'footer__listItem\'><a class="footer__link" href="/aboutthefed.htm">About the Fed</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/newsevents.htm">News & Events</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/monetarypolicy.htm">Monetary Policy</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/supervisionreg.htm">Supervision & Regulation</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/paymentsystems.htm">Payment Systems</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/econres.htm">Economic Research</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/data.htm">Data</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/consumerscommunities.htm">Consumers & Communities</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/financial-stability.htm">Financial Stability</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/economic-disparities-work.htm">Federal Reserve\'s Work Related to Economic Disparities</a></li> \n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="text-uppercase footer__heading">Tools and Information</h6>\n            <ul class="list-unstyled">\n                 <li class=\'footer__listItem\'><a href="/aboutthefed/contact-us-topics.htm" class="footer__link">Contact</a></li>\n                <li class=\'footer__listItem\'><a href="/publications.htm" class="footer__link">Publications</a></li>\n                <li class=\'footer__listItem\'><a href="/foia/about_foia.htm" class="footer__link">Freedom of Information (FOIA)</a></li>\n                <li class=\'footer__listItem\'><a href="https://oig.federalreserve.gov/" class="footer__link">Office of Inspector General</a></li>\n                <li class=\'footer__listItem\'><a href="/publications/annual-report.htm" class="footer__link">Budget &amp; Performance</a> | <a href="/regreform/audit.htm" class="footer__link">Audit</a></li>\n                <li class=\'footer__listItem\'><a href="/eeo.htm" class="footer__link">No FEAR Act</a></li>\n                <li class=\'footer__listItem\'><a href="/espanol.htm" class="footer__link">Espa\xc3\xb1ol</a></li>\n                <li class=\'footer__listItem\'><a href="/website-linking-policies.htm" class="footer__link">Website Policies</a>  | <a href="/privacy.htm" class="footer__link">Privacy Program</a></li>\n                <li class=\'footer__listItem\'><a href="/accessibility.htm" class="footer__link">Accessibility</a></li>\n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <div class="footer__social">\n                <h6 class="text-uppercase footer__heading footer__heading--social">Stay Connected</h6>\n                \n\n\n    \n                <a class="icon__md icon footer__btn icon-facebook-color" href="https://www.facebook.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-twitter-color" href="https://twitter.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-youtube-color" href="https://www.youtube.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-flickr-color" href="https://www.flickr.com/photos/federalreserve/"><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-linkedin-color" href="https://www.linkedin.com/company/federal-reserve-board"><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-rss-color" href="/feeds/feeds.htm"><span class=\'sr-only\'>Subscribe to RSS</span></a>\n            \n                <a class="icon__md icon footer__btn icon-email-color" href="/subscribe.htm"><span class=\'sr-only\'>Subscribe to Email</span></a>\n                    \n    \n \n \n            </div>\n            <a href="https://www.usa.gov/" target="_blank" title="Link to USA.gov">\n                <img class=\'footer__image\' src="/images/USAGov%402x.png" alt="Link to USA.gov" />\n            </a>\n            <a href="/open/open.htm" target="_blank" title="Link to Open.gov">\n                <img class=\'footer__image\' src="/images/OpenGov%402x.png" alt="Link to Open.gov" />\n            </a>\n        </div>\n    </div>\n    <div class="row">\n        <div class="col-xs-12 footer__footer">\n            <p class="text-uppercase footer__text footer__text--left">Board of Governors <em class="text-lowercase">of the</em> Federal Reserve System</p>\n            <p class="footer__text footer__text--right">20th Street and Constitution Avenue N.W., Washington, DC 20551</p>\n        </div>\n    </div>\n</footer>\n\n\n                <script src="/js/jquery1.11.1.min.js" type="text/javascript"></script>\n                <script src="/js/bootstrap.min.js" type="text/javascript"></script>\n                <script src="/js/scripts.js" type="text/javascript"></script>\n                <script src="/js/modal.js" type="text/javascript"></script>\n                <script src="/js/sticky-tables.js" type="text/javascript"></script>\n                <script type="text/javascript" src="/js/slides/ekko-lightbox.min.js"></script>\n                <script src="/js/t4-nav-controller.js" type="text/javascript"></script>\n              \n<script type="text/javascript">\n    $(document).ready(function($) {\n        // delegate calls to data-toggle="lightbox"\n        $(document).delegate(\'*[data-toggle="lightbox"]:not([data-gallery="navigateTo"])\', \'click\', function(event) {\n            event.preventDefault();\n            $(this).ekkoLightbox();\n        });\n        $(\'a[data-click="false"]\').click(function(e) {\n            e.preventDefault();\n            var target = $(this).data(\'target\');\n            $(target).click();\n        });\n    });\n</script>          \n             \n\n\n        \n        <!--<ul></ul> -->\n    <script type="text/javascript" src="/resources/track_downloads.js"></script>\n</body>\n</html>\n '  # noqa: E501
            result = Check_Services().get_data(url)
            self.assertIsInstance(result, list)
            self.assertEqual(len(result), 3)

    def test_all(self) -> None:
        with mock.patch('FedPy.Payment_Systems.Check_Services.Check_Services.pmo') as pmo_patch:
            pmo_patch.return_value = util.to_DataFrame([
                ['Year', 'Volume (millions of items)', 'Volume percent change', 'Value (billions of dollars)',
                 'Value percent change', 'Average daily volume (thousands of items)', 'Average daily value (millions of dollars)',
                 'Average value per postal money order (dollars)'],
                ['2020', '2019', '2018'],
                [[74.0, -8.3, 21.0, -4.0, 288.0, 80.0, 269.0],
                 [80.0, -2.8, 21.0, 1.8, 320.0, 85.0, 266.0],
                 [83.0, -2.8, 21.0, 1.7, 330.0, 84.0, 254.0]],
            ])
            with mock.patch('FedPy.Payment_Systems.Check_Services.Check_Services.cc') as cc_patch:
                cc_patch.return_value = util.to_DataFrame([
                    ['Year', 'Volume (millions of items)', 'Volume percent change', 'Value (billions of dollars)',
                     'Value percent change', 'Average daily volume (millions of items)',
                     'Average daily value (billions of dollars)', 'Average value per check (dollars)'],
                    ['2020', '2019', '2018'],
                    [[3767.0, -14.2, 7875.0, -5.3, 14.7, 30.8, 2091.0],
                     [4389.0, -7.4, 8318.0, -2.0, 17.5, 33.1, 1895.0],
                     [4740.0, -8.0, 8485.0, 0.6, 18.9, 33.8, 1790.0]],
                ])
                with mock.patch('FedPy.Payment_Systems.Check_Services.Check_Services.cr') as cr_patch:
                    cr_patch.return_value = util.to_DataFrame([
                        ['Year', 'Volume (millions of items)', 'Volume percent change', 'Value (billions of dollars)',
                         'Value percent change', 'Average daily volume (thousands of items)',
                         'Average daily value (millions of dollars)', 'Average value per check (dollars)'],
                        ['2020', '2019', '2018'],
                        [[20.0, -26.5, 56.0, -3.4, 78.0, 218.0, 2794.0],
                         [27.0, -6.3, 58.0, 2.2, 108.0, 230.0, 2125.0],
                         [29.0, -7.3, 57.0, 6.0, 116.0, 225.0, 1949.0]],
                    ])
                    with mock.patch('FedPy.Payment_Systems.Check_Services.Check_Services.gc') as gc_patch:
                        gc_patch.return_value = util.to_DataFrame([
                             ['Year', 'Volume (millions of items)', 'Volume percent change', 'Value (billions of dollars)',
                              'Value percent change', 'Average daily volume (thousands of items)',
                              'Average daily value (millions of dollars)', 'Average value per postal money order (dollars)'],
                             ['2020', '2019', '2018'],
                             [[74.0, -8.3, 21.0, -4.0, 288.0, 80.0, 269.0],
                              [80.0, -2.8, 21.0, 1.8, 320.0, 85.0, 266.0],
                              [83.0, -2.8, 21.0, 1.7, 330.0, 84.0, 254.0]],
                        ])
                        result = Check_Services().all()
                        self.assertIsInstance(result, pd.DataFrame)

    def test_pmo(self) -> None:
        with mock.patch('FedPy.Payment_Systems.Check_Services.Check_Services.get_data') as patched:
            patched.return_value = [
                ['Year', 'Volume (millions of items)', 'Volume percent change', 'Value (billions of dollars)',
                 'Value percent change', 'Average daily volume (thousands of items)', 'Average daily value (millions of dollars)',
                 'Average value per postal money order (dollars)'],
                ['2020', '2019', '2018'],
                [[74.0, -8.3, 21.0, -4.0, 288.0, 80.0, 269.0],
                 [80.0, -2.8, 21.0, 1.8, 320.0, 85.0, 266.0],
                 [83.0, -2.8, 21.0, 1.7, 330.0, 84.0, 254.0]],
            ]
            result = Check_Services().pmo()
            self.assertIsInstance(result, pd.DataFrame)

    def test_cc(self) -> None:
        with mock.patch('FedPy.Payment_Systems.Check_Services.Check_Services.get_data') as patched:
            patched.return_value = [
                ['Year', 'Volume (millions of items)', 'Volume percent change', 'Value (billions of dollars)',
                 'Value percent change', 'Average daily volume (millions of items)',
                 'Average daily value (billions of dollars)', 'Average value per check (dollars)'],
                ['2020', '2019', '2018'],
                [[3767.0, -14.2, 7875.0, -5.3, 14.7, 30.8, 2091.0],
                 [4389.0, -7.4, 8318.0, -2.0, 17.5, 33.1, 1895.0],
                 [4740.0, -8.0, 8485.0, 0.6, 18.9, 33.8, 1790.0]],
            ]
            result = Check_Services().cc()
            self.assertIsInstance(result, pd.DataFrame)

    def test_cr(self) -> None:
        with mock.patch('FedPy.Payment_Systems.Check_Services.Check_Services.get_data') as patched:
            patched.return_value = [
                ['Year', 'Volume (millions of items)', 'Volume percent change', 'Value (billions of dollars)',
                 'Value percent change', 'Average daily volume (thousands of items)',
                 'Average daily value (millions of dollars)', 'Average value per check (dollars)'],
                ['2020', '2019', '2018'],
                [[20.0, -26.5, 56.0, -3.4, 78.0, 218.0, 2794.0],
                 [27.0, -6.3, 58.0, 2.2, 108.0, 230.0, 2125.0],
                 [29.0, -7.3, 57.0, 6.0, 116.0, 225.0, 1949.0]],
            ]
            result = Check_Services().cr()
            self.assertIsInstance(result, pd.DataFrame)

    def test_gc(self) -> None:
        with mock.patch('FedPy.Payment_Systems.Check_Services.Check_Services.get_data') as patched:
            patched.return_value = [
                 ['Year', 'Volume (millions of items)', 'Volume percent change', 'Value (billions of dollars)',
                  'Value percent change', 'Average daily volume (thousands of items)',
                  'Average daily value (millions of dollars)', 'Average value per postal money order (dollars)'],
                 ['2020', '2019', '2018'],
                 [[74.0, -8.3, 21.0, -4.0, 288.0, 80.0, 269.0],
                  [80.0, -2.8, 21.0, 1.8, 320.0, 85.0, 266.0],
                  [83.0, -2.8, 21.0, 1.7, 330.0, 84.0, 254.0]],
            ]
            result = Check_Services().gc()
            self.assertIsInstance(result, pd.DataFrame)


class Test_NSS(TestCase):
    def test_get_data(self) -> None:
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\n\n<!doctype html>\n<html lang="en">\n<head>\n<meta charset="UTF-8">\n<meta name="viewport" content="width=device-width,initial-scale=1">\n<meta http-equiv="X-UA-Compatible" content="IE=edge">\n<link rel="icon" href="/webfiles/1626607380014/images/global/favicon.ico" type="image/x-icon">\n<link rel="shortcut icon" href="/webfiles/1626607380014/images/global/favicon.ico" type="image/x-icon">\n<!--[if lt IE 9]>\n    <script src="/js/html5shiv.js"></script>\n<![endif]-->\n<!-- Google Tag Manager -->\n<script>(function(w,d,s,l,i){w[l]=w[l]||[];w[l].push({\'gtm.start\': new Date().getTime(),event:\'gtm.js\'});var f=d.getElementsByTagName(s)[0], j=d.createElement(s),dl=l!=\'dataLayer\'?\'&l=\'+l:\'\';j.async=true;j.src=\'https://www.googletagmanager.com/gtm.js?id=\'+i+dl;f.parentNode.insertBefore(j,f);})(window,document,\'script\',\'dataLayer\',\'GTM-KXGSJGD\');</script>\n<!-- End Google Tag Manager -->\n<link href="https://fonts.googleapis.com/css?family=Open+Sans:300,400,700" rel="stylesheet">\n<link rel="stylesheet" type="text/css" href="/webfiles/1626607380014/css/bootstrap.css">\n<link rel="stylesheet" type="text/css" href="/webfiles/1626607380014/css/default.css">\n<style>\n        .pagesettings {\n            width: 100%;\n            position: relative;\n            background-color: silver;\n            color: white;\n            font-size: 18px;\n            font-weight: bold;\n            font-style: italic;\n            text-align: center;\n            padding: 1em 0;\n        }\n    </style>\n<title>National Settlement Service Service - Annual Statistics</title>\n<meta name="description" content="Annual National Settlement Service statistics"/>\n<meta property="og:description" content="Annual National Settlement Service statistics"/>\n<link rel="canonical" href="https://www.frbservices.org/resources/financial-services/national-settlement-service/volume-value-stats/annual-stats.html"/>\n</head>\n<body>\n<!-- Google Tag Manager (noscript) -->\n<noscript><iframe src="https://www.googletagmanager.com/ns.html?id=GTM-KXGSJGD" height="0" width="0" style="display:none;visibility:hidden"></iframe></noscript>\n<!-- End Google Tag Manager (noscript) -->\n<a href="#content" class="sr-only sr-only-focusable">Skip to main content</a>\n<div class="container">\n    <div class="component-container">\n\n\n\n\n\n\n\n\n\n\n    </div>\n\n\n\n\n\n<nav class="navbar-static-top navbar" id="navbar-frbs">\n    <div class="navbar-header"> <a href="https://www.frbservices.org/"><img src="https://www.frbservices.org/webfiles/1626607380014/images/federal-reserve-logo-frbservices.svg" alt="The Federal Reserve" class="logo"></a>\n        <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar" aria-expanded="false" aria-controls="navbar"> <span class="sr-only">Toggle navigation</span> <span class="icon-bar"></span> <span class="icon-bar"></span> <span class="icon-bar"></span></button>\n    </div>\n    <div class="navbar__utility hidden-xs" style="z-index: 2000;">\n        <form role="search" class="search" id="search-level-one" action="/search/">\n            <label for="search-query" class="item-invisible">Search</label>\n            <input id="search-query" name="q" type="text" class="form-control" autocomplete="off" placeholder="Begin your search...">\n            <button class="btn btn-default btn-search" type="submit" aria-label="Search"><span class="glyphicon glyphicon-search" aria-hidden="true"></span></button>\n        </form>\n\n\n\n        <div class="btn-group-sm btn-group--app">\n                    <a class="btn btn-default btn-sm dropdown-toggle" data-toggle="dropdown" href="">\n                        Application Sign In\n                        <span class="caret"></span>\n                    </a>\n                        <ul class="dropdown-menu">\n                                <li><a href="/Electronic-Access/AccessFedLine.html">FedLine Web</a></li>\n                                <li><a href="/sign-in/reporting-central.html">Reporting Central</a></li>\n                                <li><a href="/sign-in/reserves-central.html">Reserves Central</a></li>\n                                <li><a href="/sign-in/survey-central.html">Survey Central</a></li>\n                                <li><a href="/sign-in/term-deposit-facility.html">Term Deposit Facility</a></li>\n                        </ul>\n                    <a class="btn btn-default btn-service-status" href="/app/status/serviceStatus.do" style="background-image: url(/app/status/serviceStatusLink)">\n                        <span class="glyphicon glyphicon-check" aria-hidden="true"></span> Service Status\n                    </a>\n                    <a class="btn btn-default "  href="/contacts/index.jsp">\n                        Contact\n                        \n                    </a>\n        </div>\n    </div>\n</nav>\n\n\n\n<div id="navbar" class="navbar-collapse collapse collapse-blue">\n    <ul class="nav navbar-nav" data-smartmenus-id="159500933331223">\n        <li class="visible-xs-block">\n            <form role="search" class="form-search" id="search-level-two" action="/search/">\n                <label for="search-query-two" class="item-invisible">Search</label>\n                <input id="search-query-two" name="q" type="text" class="form-control form-search__input" autocomplete="off" placeholder="Begin your search...">\n                <button class="btn btn-default btn-search" type="submit" aria-label="Search"><span class="glyphicon glyphicon-search" aria-hidden="true"></span></button>\n            </form>\n        </li>\n\n\n\n                <li class="visible-xs-block">\n                    <a class="btn-sm dropdown-toggle" href="">\n                        Application Sign In\n                    </a>\n                        <ul class="dropdown-menu">\n                                <li><a href="/Electronic-Access/AccessFedLine.html">FedLine Web</a></li>\n                                <li><a href="/sign-in/reporting-central.html">Reporting Central</a></li>\n                                <li><a href="/sign-in/reserves-central.html">Reserves Central</a></li>\n                                <li><a href="/sign-in/survey-central.html">Survey Central</a></li>\n                                <li><a href="/sign-in/term-deposit-facility.html">Term Deposit Facility</a></li>\n                        </ul>\n                </li>\n                <li class="visible-xs-block">\n                    <a class="btn-service-status" href="/app/status/serviceStatus.do" style="background-image: url(/app/status/serviceStatusLink)">\n                        Service Status\n                    </a>\n                </li>\n                <li class="visible-xs-block">\n                    <a class="" href="/contacts/index.jsp">\n                        Contact\n                    </a>\n                </li>\n                     \n                        <li class="dropdown" id="Financial Services"><a href="https://www.frbservices.org/financial-services" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Financial Services<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Financial Services Home</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/accounting" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Accounting<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting">Accounting Services Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/account-management-info.html">Account Management Information</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/daily-statement.html">Daily Statement of Account</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/daylight-overdraft-report.html">Daylight Overdraft Reports</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/ami-premium.html">Premium Accounting Information Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/statement-service-charges.html">Statement of Service Charges</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/accounting-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/ami-business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/accounting">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/accounting">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/accounting/service-setup">Service Setup</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/ach" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">ACH<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach">FedACH Products & Services Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/exception-resolution.html">FedACH Exception Resolution Service</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/file-services.html">FedACH Information File Service</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/origination-receipt.html">FedACH Origination & Receipt</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/risk">FedACH Risk Management Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/same-day-service.html">FedACH SameDay Service</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/fedglobal">FedGlobal ACH Payments</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/fedpayments-reporter">FedPayments Reporter Service for FedACH Services</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/ach-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/ach">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/ach">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/service-setup">Service Setup</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/ach/testing-opportunities.html">Testing Opportunities</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/bundled-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Bundled Solutions<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/bundled-solutions">Bundled Solutions Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/bundled-solutions/fedcomplete">FedComplete</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/bundled-solutions/fedtransaction-analyzer.html">FedTransaction Analyzer</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/fedcomplete-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/bundled-solutions">Resources</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/cash" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Cash<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash">FedCash Services Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/coin.html">Coin</a></li>\n                                                            <li><a href="/financial-services/cash/coin-allocation">Coin Allocation</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/coin-terminal.html">Coin Terminal Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/currency.html">Currency</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/fedline-web">FedCash Services via the FedLine Web Solution</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/cash-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/business-continuity">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/cash">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/cash">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/cash/service-setup">Service Setup</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/check" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Check<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/check">Check Products & Services Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/check21.html">Check 21-Enabled Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/check-adjustments.html">Check Adjustments Services</a></li>\n                                                            <li><a href="/financial-services/check/feddetect">FedDetect Duplicate Treasury Check Notifier Service</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/fedforward-image-deposit.html">FedForward Image Deposit Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/fedimage-electronic-check">FedImage and Electronic Check Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/fedpayments-reporter">FedPayments Reporter Service for Check Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/fedreceipt">FedReceipt Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/fedreturn.html">FedReturn Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/foreign-canadian.html">Foreign and Canadian Check Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/image-enabled-savings-bond.html">Image-Enabled Savings Bond Processing</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/paper-clearing">Paper Check Clearing Services</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/check-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/check">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/check">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/service-setup">Service Setup</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/check/testing-opportunities.html">Testing Opportunities</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/fednow" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedNow Service<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow">FedNow Service Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/about.html">About FedNow</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/blog">Blog Articles</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/community">FedNow Community</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/in-the-news.html">In the News</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/instant-payments-education">Instant Payments Education</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/fednow/prepare-for-fednow.html">Prepare for FedNow</a></li>\n                                                            <li><a href="/financial-services/fednow/research">Research</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/news/email-notifications.html">Sign Up for FedNow Emails</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/national-settlement-service" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">National Settlement Service<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/national-settlement-service">National Settlement Service Home</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/national-settlement-service-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/national-settlement-service/business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/national-settlement-service">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/national-settlement-service">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/national-settlement-service/service-setup">Service Setup</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/securities" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Securities<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/securities">Fedwire Securities Service Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/securities/joint-custody.html">Joint Custody Service</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/securities-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/securities/business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/securities">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/securities">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/securities/service-setup.html">Service Setup</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/wires/testing">Testing Opportunities</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/financial-services/wires" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Wires<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/financial-services/wires">Fedwire Funds Service Home</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fees/wires-2021.html">2021 Fees</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/wires/business-continuity">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/wires">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/wires">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/wires/service-setup">Service Setup</a></li>\n                                                            <li><a href="https://www.frbservices.org/financial-services/wires/testing">Testing Opportunities</a></li>\n                                                </ul>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="Central Bank"><a href="https://www.frbservices.org/central-bank" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Central Bank<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/central-bank" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Central Bank Home</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbdiscountwindow.org/" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Lending Central (Off-site)</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/central-bank/reporting-central" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Reporting Central<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/central-bank/reporting-central">Reporting Central Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reporting-central/user-guides.html">Reporting Central User Guides</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reporting-central/stats-reporting-disruption.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/central-bank">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/central-bank">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reporting-central/service-setup">Service Setup</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/central-bank/reserves-central" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Reserves Central<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/central-bank/reserves-central">Reserves Central Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reserves-central/excess-balance-account.html">Excess Balance Account</a></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reserves-central/term-deposit-facility">Term Deposit Facility</a></li>\n                                                            <li role="separator" class="divider"></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reserves-central/reserve-admin-business-continuity.html">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/forms/central-bank">Forms</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/central-bank">Resources</a></li>\n                                                            <li><a href="https://www.frbservices.org/central-bank/reserves-central/service-setup">Service Setup</a></li>\n                                                </ul>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="Treasury Services"><a href="https://www.frbservices.org/treasury-services" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Treasury Services<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury Services Home</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/collateral-services.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Collateral Services</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/savings-bonds-financial-institutions.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Savings Bonds For Financial Institutions</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/savings-bond-news.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Savings Bonds News You Can Use</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/ach-reclamation.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury ACH Reclamation</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/auctions.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury Auctions</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/check-reclamation.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury Check Reclamation</a>\n                                        </li>\n                                        <li role="separator" class="divider"></li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/business-continuity.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Business Continuity</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/treasury" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Forms</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/treasury" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Resources</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/treasury-services/service-setup" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Service Setup</a>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="Fedline Solutions"><a href="https://www.frbservices.org/fedline-solutions" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Fedline Solutions<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Solutions Home</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/fedline-advantage.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Advantage</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/fedline-command.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Command</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/fedline-direct.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Direct</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/fedline-web.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Web</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/fedmail.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedMail</a>\n                                        </li>\n                                        <li role="separator" class="divider"></li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/fees/fedline-solutions-2021.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">2021 Fees</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/fedline-solutions/application-connectivity-testing.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Application and Connectivity Testing</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/business-continuity" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Business Continuity</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/fedline-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Forms</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/fedline-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Resources</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/fedline-solutions/service-setup" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Service Setup</a>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="Forms"><a href="https://www.frbservices.org/forms" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Forms<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Forms Home</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/accounting" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Accounting</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/ach" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">ACH</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/cash" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Cash</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/central-bank" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Central Bank</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/check" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Check</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/district-information" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">District Information</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/fedline-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Solutions</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/national-settlement-service" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">National Settlement Service</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/securities" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Securities</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/treasury" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury Services</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/forms/wires" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Wires</a>\n                                        </li>\n                            </ul>\n                        </li>\n                     \n                        <li class="dropdown" id="Resources"><a href="https://www.frbservices.org/resources" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Resources<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/central-bank" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Central Bank<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/central-bank">Central Bank Resources Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/central-bank/faq">Frequently Asked Questions</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/central-bank/reporting-central.html">Reporting Central Resources</a></li>\n                                                            <li><a href="/resources/central-bank/survey-central.html">Survey Central Resources</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="/resources/covid-19-resources/" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">COVID-19</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/routing-number-directory" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">E-Payments Routing Directory</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/fedline-solutions" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">FedLine Solutions<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions">FedLine Solutions Resources Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions/euac.html">End User Authorization Contact (EUAC) Support</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions/command-matrix.html">FedLine Command Environment and Configuration Change Matrix</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions/direct-file-matrix.html">FedLine Direct File Environment and Configuration Change Matrix</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions/direct-message-matrix.html">FedLine Direct Message Environment and Configuration Change Matrix</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/fedline-solutions/faq">Frequently Asked Questions</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/fees" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Fees</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/financial-services" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Financial Services<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/accounting">Accounting</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/ach">ACH</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/bundled-solutions">Bundled Solutions</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/cash">Cash</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/check">Check</a></li>\n                                                            <li><a href="/app/fedachfx/home.html">Foreign Exchange Rates</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/national-settlement-service">National Settlement Service</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/securities">Securities</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/financial-services/wires">Wires</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/industry-links" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Industry Links</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/resource-centers" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Resource Centers<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers">Resource Centers Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers/business-banking-toolbox">Business Banking Toolbox</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers/business-continuity">Business Continuity</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers/international-payments">International Payments</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers/risk-mgmt-toolbox">Risk Management Toolbox</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/resource-centers/same-day-ach">Same Day ACH</a></li>\n                                                            <li><a href="/resources/resource-centers/security-resiliency-assurance-program">Security and Resiliency Assurance Program</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/rules-regulations" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Rules & Regulations<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/rules-regulations">Rules and Regulations Resources Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/rules-regulations/operating-circulars.html">Operating Circulars</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/rules-regulations/regulations.html">Regulations</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/service-setup" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Service & Access Setup<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/service-setup">Service and Access Setup Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/service-setup/current-customer.html">Current Financial Services Customer</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/service-setup/merger.html">Financial Institution Merger</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/service-setup/new-customer.html">New Financial Services Customer</a></li>\n                                                </ul>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/resources/treasury" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Treasury Services<span class="caret"></span></a>\n                                                <ul class="dropdown-menu" id="sm-159500933331223-4" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-3" aria-expanded="false">\n                                                            <li><a href="https://www.frbservices.org/resources/treasury">Treasury Services Resources Home</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/treasury/collateral-services.html">Collateral Services</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/treasury/faq">Frequently Asked Questions</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/treasury/savings-bonds-financial-institutions.html">Savings Bonds for Financial Institutions</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/treasury/treasury-auctions.html">Treasury Auctions</a></li>\n                                                            <li><a href="https://www.frbservices.org/resources/treasury/where-to-send-security-deposits.html">Where to Send Security Deposits</a></li>\n                                                </ul>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="Education"><a href="https://www.frbservices.org/education" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">Education<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/education/webinars" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Federal Reserve Bank Webinars</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/education/industry-events.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Industry Events</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/education/products-services-education" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Products & Services Education</a>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="News"><a href="https://www.frbservices.org/news" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">News<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/news/communications" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Communications</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/news/email-notifications.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Email Notifications</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/news/fed360" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Fed360</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/news/press-releases" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Press Releases</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/news/research.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Research Studies</a>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n                        <li class="dropdown" id="About"><a href="https://www.frbservices.org/about" class="dropdown-toggle has-submenu" aria-haspopup="true" aria-expanded="false" id="sm-159500933331223-1" aria-controls="sm-159500933331223-2">About<span class="caret"></span></a>\n                            <ul class="dropdown-menu sm-nowrap" id="sm-159500933331223-2" role="group" aria-hidden="true" aria-labelledby="sm-159500933331223-1" aria-expanded="false">\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/about/frb-bank-offices.html" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Bank Offices</a>\n                                        </li>\n                                        <li class="dropdown dropdown-submenu"><a href="https://www.frbservices.org/about/holiday-schedules" class="dropdown-toggle has-submenu" id="sm-159500933331223-3" aria-haspopup="true" aria-controls="sm-159500933331223-4" aria-expanded="false">Holiday Schedules</a>\n                                        </li>\n                            </ul>\n                        </li>\n                    \n    </ul>\n</div>\n<!--HstSiteMapItemService{canonicalPath=\'/hst:crsocms/hst:configurations/crsocms/hst:workspace/hst:sitemap/resources/financial-services/national-settlement-service/volume-value-stats/annual-stats.html\', site=HstSiteService{name=\'crsocms\',configurationPath=\'/hst:crsocms/hst:configurations/crsocms\'}}-->\n<!---->\n    <nav class="navbar navbar-static-top navbar__margin-bottom" id="subnav">\n        <div class="navbar-header navbar-header-inner">\n                <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar-inner" aria-expanded="false" aria-controls="navbar">Resources<span class="caret"></span> </button>\n        </div>\n        <div id="navbar-inner" class="navbar-collapse collapse">\n            <ul class="nav navbar-nav" data-smartmenus-id="16098773335059512">\n                            <li><a href="https://www.frbservices.org/resources/central-bank" aria-haspopup="true" aria-expanded="false" class="has-submenu">Central Bank<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/central-bank">Central Bank Resources Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/central-bank/faq">Frequently Asked Questions</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/central-bank/reporting-central.html">Reporting Central Resources</a></li>\n                                                <li><a href="/resources/central-bank/survey-central.html">Survey Central Resources</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="/resources/covid-19-resources/" aria-haspopup="true" aria-expanded="false" >COVID-19</a>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/routing-number-directory" aria-haspopup="true" aria-expanded="false" >E-Payments Routing Directory</a>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/fedline-solutions" aria-haspopup="true" aria-expanded="false" class="has-submenu">FedLine Solutions<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions">FedLine Solutions Resources Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions/euac.html">End User Authorization Contact (EUAC) Support</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions/command-matrix.html">FedLine Command Environment and Configuration Change Matrix</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions/direct-file-matrix.html">FedLine Direct File Environment and Configuration Change Matrix</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions/direct-message-matrix.html">FedLine Direct Message Environment and Configuration Change Matrix</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/fedline-solutions/faq">Frequently Asked Questions</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/fees" aria-haspopup="true" aria-expanded="false" >Fees</a>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/financial-services" aria-haspopup="true" aria-expanded="false" class="has-submenu">Financial Services<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/accounting">Accounting</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/ach">ACH</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/bundled-solutions">Bundled Solutions</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/cash">Cash</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/check">Check</a></li>\n                                                <li><a href="/app/fedachfx/home.html">Foreign Exchange Rates</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/national-settlement-service">National Settlement Service</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/securities">Securities</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/financial-services/wires">Wires</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/industry-links" aria-haspopup="true" aria-expanded="false" >Industry Links</a>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/resource-centers" aria-haspopup="true" aria-expanded="false" class="has-submenu">Resource Centers<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers">Resource Centers Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers/business-banking-toolbox">Business Banking Toolbox</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers/business-continuity">Business Continuity</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers/international-payments">International Payments</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers/risk-mgmt-toolbox">Risk Management Toolbox</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/resource-centers/same-day-ach">Same Day ACH</a></li>\n                                                <li><a href="/resources/resource-centers/security-resiliency-assurance-program">Security and Resiliency Assurance Program</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/rules-regulations" aria-haspopup="true" aria-expanded="false" class="has-submenu">Rules & Regulations<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/rules-regulations">Rules and Regulations Resources Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/rules-regulations/operating-circulars.html">Operating Circulars</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/rules-regulations/regulations.html">Regulations</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/service-setup" aria-haspopup="true" aria-expanded="false" class="has-submenu">Service & Access Setup<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/service-setup">Service and Access Setup Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/service-setup/current-customer.html">Current Financial Services Customer</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/service-setup/merger.html">Financial Institution Merger</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/service-setup/new-customer.html">New Financial Services Customer</a></li>\n                                    </ul>\n                            </li>\n                            <li><a href="https://www.frbservices.org/resources/treasury" aria-haspopup="true" aria-expanded="false" class="has-submenu">Treasury Services<span class="caret"></span></a>\n                                    <ul class="dropdown-menu" role="group" aria-hidden="true" aria-expanded="false">\n                                                <li><a href="https://www.frbservices.org/resources/treasury">Treasury Services Resources Home</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/treasury/collateral-services.html">Collateral Services</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/treasury/faq">Frequently Asked Questions</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/treasury/savings-bonds-financial-institutions.html">Savings Bonds for Financial Institutions</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/treasury/treasury-auctions.html">Treasury Auctions</a></li>\n                                                <li><a href="https://www.frbservices.org/resources/treasury/where-to-send-security-deposits.html">Where to Send Security Deposits</a></li>\n                                    </ul>\n                            </li>\n            </ul>\n        </div>\n    </nav>\n    <div id="content" role="main">\n\n\n\n\n\n\n<div class="row hidden-print" id="content">\n\t<div class="col-sm-12">\n\t\t<ol class="breadcrumb">\n\t\t\t<li><a href="/">Home</a></li>\n\t\t\t\t\t<li><a href="/resources">Resources</a></li>\n\t\t\t\t\t<li><a href="/resources/financial-services">Financial Services</a></li>\n\t\t\t\t\t<li><a href="/resources/financial-services/national-settlement-service">National Settlement Service</a></li>\n\t\t\t\t\t<li><a href="/resources/financial-services/national-settlement-service/volume-value-stats">National Settlement Service Volume and Value Statistics</a></li>\n  \t\t\t\t\t<li>National Settlement Service Service - Annual Statistics</li>\n\t\t</ol>\n\t</div>\n</div>    <div class="row">\n        <div class="col-sm-8 col-md-9 col-sm-push-4 col-md-push-3">\n\n    <div>\n                <div>\n                    \n\n\n    <h1>National Settlement Service Service - Annual Statistics</h1>\r\n      <table class="table table-striped table-responsive table-hover">\r\n        <caption>\r\n        Annual National Settlement Service Statistics\r\n        </caption>\r\n        <thead>\r\n          <tr>\r\n            <th scope="col">Year</th>\r\n            <th scope="col">Volume of NSS entries processed<br/>\r\n              (number)</th>\r\n            <th scope="col">Annual volume growth of NSS entries<br/>\r\n              (percent)</th>\r\n            <th scope="col">Volume of NSS files processed<br/>\r\n              (number)</th>\r\n            <th scope="col">Value of total settlement<a href="#1ftn"><sup>1</sup></a><br/>\r\n              ($millions)</th>\r\n            <th scope="col">Annual value growth<br/>\r\n              (percent)</th>\r\n            <th scope="col">Average daily number of entries<a href="#2ftn"><sup>2</sup></a><br/>\r\n              (number)</th>\r\n            <th scope="col">Average daily Number of files<a href="#2ftn"><sup>2</sup></a><br/>\r\n              (number)</th>\r\n\t\t\t<th scope="col">Average daily settlement value<a href="#2ftn"><sup>2</sup></a><br/>\r\n\t\t\t  ($millions)</th>\r\n          </tr>\r\n        </thead>\r\n        <tbody>\r\n\t\t  <tr>\r\n\t\t    <th scope="row">2020</th>\r\n\t\t    <td>551,371</td>\r\n\t\t\t<td>(1.2)</td>\r\n\t\t\t<td>9,468</td>\r\n\t\t\t<td>23,465,177</td>\r\n\t\t\t<td>7.9</td>\r\n\t\t\t<td>2,179</td>\r\n\t\t\t<td>37</td>\r\n\t\t\t<td>92,748</td>\r\n\t\t</tr>\r\n <tr>\r\n\t\t    <th scope="row">2019</th>\r\n\t\t    <td>558,240</td>\r\n\t\t\t<td>(3.7)</td>\r\n\t\t\t<td>9,675</td>\r\n\t\t\t<td>21,744,920</td>\r\n\t\t\t<td>4.1</td>\r\n\t\t\t<td>2,224</td>\r\n\t\t\t<td>39</td>\r\n\t\t\t<td>86,633</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2018</th>\r\n\t\t\t<td>579,394</td>\r\n\t\t\t<td>0.3</td>\r\n\t\t\t<td>9,674</td>\r\n\t\t\t<td>20,886,077</td>\r\n\t\t\t<td>7.8</td>\r\n\t\t\t<td>2,308</td>\r\n\t\t\t<td>39</td>\r\n\t\t\t<td>83,211</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2017</th>\r\n\t\t\t<td>577,699</td>\r\n\t\t\t<td>3.0</td>\r\n\t\t\t<td>9,238</td>\r\n\t\t\t<td>19,366,604</td>\r\n\t\t\t<td>7.7</td>\r\n\t\t\t<td>2,302</td>\r\n\t\t\t<td>37</td>\r\n\t\t\t<td>77,158</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2016</th>\r\n\t\t\t<td>560,735</td>\r\n\t\t\t<td>(1.5)</td>\r\n\t\t\t<td>9,333</td>\r\n\t\t\t<td>17,983,149</td>\r\n\t\t\t<td>(0.6)</td>\r\n\t\t\t<td>2,234</td>\r\n\t\t\t<td>37</td>\r\n\t\t\t<td>71,646</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2015</th>\r\n\t\t\t<td>569,250</td>\r\n\t\t\t<td>(13.8)</td>\r\n\t\t\t<td>9,372</td>\r\n\t\t\t<td>18,085,534</td>\r\n\t\t\t<td>3.0</td>\r\n\t\t\t<td>2,259</td>\r\n\t\t\t<td>37</td>\r\n\t\t\t<td>71,768</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2014</th>\r\n\t\t\t<td>660,062</td>\r\n\t\t\t<td>(9.0)</td>\r\n\t\t\t<td>9,896</td>\r\n\t\t\t<td>17,551,626</td>\r\n\t\t\t<td>1.6</td>\r\n\t\t\t<td>2,630</td>\r\n\t\t\t<td>39</td>\r\n\t\t\t<td>69,927</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2013</th>\r\n\t\t\t<td>725,082</td>\r\n\t\t\t<td>9.4</td>\r\n\t\t\t<td>10,152</td>\r\n\t\t\t<td>17,281,065</td>\r\n\t\t\t<td>7.4</td>\r\n\t\t\t<td>2,889</td>\r\n\t\t\t<td>40</td>\r\n\t\t\t<td>68,849</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2012</th>\r\n\t\t\t<td>662,912</td>\r\n\t\t\t<td>16.0</td>\r\n\t\t\t<td>8,503</td>\r\n\t\t\t<td>16,084,794</td>\r\n\t\t\t<td>2.2</td>\r\n\t\t\t<td>2,641</td>\r\n\t\t\t<td>34</td>\r\n\t\t\t<td>64,083</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2011</th>\r\n\t\t\t<td>571,279</td>\r\n\t\t\t<td>10.0</td>\r\n\t\t\t<td>6,904</td>\r\n\t\t\t<td>15,738,400</td>\r\n\t\t\t<td>8.3</td>\r\n\t\t\t<td>2,276</td>\r\n\t\t\t<td>28</td>\r\n\t\t\t<td>62,703</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2010</th>\r\n\t\t\t<td>519,444</td>\r\n\t\t\t<td>12.0</td>\r\n\t\t\t<td>6,944</td>\r\n\t\t\t<td>14,526,482</td>\r\n\t\t\t<td>(12.0)</td>\r\n\t\t\t<td>2,061</td>\r\n\t\t\t<td>28</td>\r\n\t\t\t<td>57,645</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2009</th>\r\n\t\t\t<td>463,730</td>\r\n\t\t\t<td>(1.0)</td>\r\n\t\t\t<td>10,521</td>\r\n\t\t\t<td>16,502,771</td>\r\n\t\t\t<td>(21.3)</td>\r\n\t\t\t<td>1,840</td>\r\n\t\t\t<td>42</td>\r\n\t\t\t<td>65,487</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2008</th>\r\n\t\t\t<td>468,534</td>\r\n\t\t\t<td>(7.2)</td>\r\n\t\t\t<td>15,194</td>\r\n\t\t\t<td>20,960,779</td>\r\n\t\t\t<td>37.2</td>\r\n\t\t\t<td>1,859</td>\r\n\t\t\t<td>60</td>\r\n\t\t\t<td>83,178</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2007</th>\r\n\t\t\t<td>504,967</td>\r\n\t\t\t<td>7.4</td>\r\n\t\t\t<td>17,181</td>\r\n\t\t\t<td>15,278,205</td>\r\n\t\t\t<td>161.5</td>\r\n\t\t\t<td>2,012</td>\r\n\t\t\t<td>68</td>\r\n\t\t\t<td>60,869</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2006</th>\r\n\t\t\t<td>470,258</td>\r\n\t\t\t<td>6.8</td>\r\n\t\t\t<td>17,366</td>\r\n\t\t\t<td>5,843,314</td>\r\n\t\t\t<td>3.4</td>\r\n\t\t\t<td>1,874</td>\r\n\t\t\t<td>69</td>\r\n\t\t\t<td>23,280</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2005</th>\r\n\t\t\t<td>440,304</td>\r\n\t\t\t<td>1.3</td>\r\n\t\t\t<td>17,490</td>\r\n\t\t\t<td>5,649,184</td>\r\n\t\t\t<td>7.3</td>\r\n\t\t\t<td>1,754</td>\r\n\t\t\t<td>70</td>\r\n\t\t\t<td>22,507</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2004</th>\r\n\t\t\t<td>434,552</td>\r\n\t\t\t<td>2.8</td>\r\n\t\t\t<td>20,343</td>\r\n\t\t\t<td>5,262,592</td>\r\n\t\t\t<td>17.0</td>\r\n\t\t\t<td>1,718</td>\r\n\t\t\t<td>80</td>\r\n\t\t\t<td>20,801</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2003</th>\r\n\t\t\t<td>422,751</td>\r\n\t\t\t<td>1.8</td>\r\n\t\t\t<td>21,478</td>\r\n\t\t\t<td>4,497,362</td>\r\n\t\t\t<td>16.6</td>\r\n\t\t\t<td>1,684</td>\r\n\t\t\t<td>86</td>\r\n\t\t\t<td>17,918</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2002</th>\r\n\t\t\t<td>415,343</td>\r\n\t\t\t<td>34.3</td>\r\n\t\t\t<td>22,913</td>\r\n\t\t\t<td>3,856,291</td>\r\n\t\t\t<td>24.9</td>\r\n\t\t\t<td>1,655</td>\r\n\t\t\t<td>91</td>\r\n\t\t\t<td>15,364</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2001</th>\r\n\t\t\t<td>309,287</td>\r\n\t\t\t<td>65.8</td>\r\n\t\t\t<td>15,755</td>\r\n\t\t\t<td>3,087,277</td>\r\n\t\t\t<td>95.6</td>\r\n\t\t\t<td>1,232</td>\r\n\t\t\t<td>63</td>\r\n\t\t\t<td>12,300</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">2000</th>\r\n\t\t\t<td>186,530</td>\r\n\t\t\t<td>342.4</td>\r\n\t\t\t<td>6,817</td>\r\n\t\t\t<td>1,578,081</td>\r\n\t\t\t<td>639.1</td>\r\n\t\t\t<td>740</td>\r\n\t\t\t<td>27</td>\r\n\t\t\t<td>6,262</td>\r\n\t\t</tr>\r\n\t\t<tr>\r\n\t\t\t<th scope="row">1999</th>\r\n\t\t\t<td>42,164</td>\r\n\t\t\t<td>---</td>\r\n\t\t\t<td>1,071</td>\r\n\t\t\t<td>213,516</td>\r\n\t\t\t<td>---</td>\r\n\t\t\t<td>167</td>\r\n\t\t\t<td>4</td>\r\n\t\t\t<td>847</td>\r\n\t\t</tr>\r\n        </tbody>\r\n      </table>\r\n      <h2>Notes</h2>\r\n      <p><sup id="1ftn">1</sup> Each file submitted to NSS contains a number of debit and credit entries which net to zero.  The "settlement value" represented in these statistics includes the total dollar value (debits only) of settlement files processed through NSS.</p>\r\n\t  <p><sup id="2ftn">2</sup> Based on the number of business days per period.</p>\r\n      <p> <a href="#content">Top of Page</a> </p>\n\n                </div>\n    </div>\n        </div>\n        <div class="col-sm-4 col-md-3 col-sm-pull-8 col-md-pull-9 hidden-print">\n\n    <div>\n                <div>\n                    \n\n\n\t<div class="spotlight spotlight--nss">\n\t<h2>National Settlement Service Resources</h2>\n\t\n\n\t<ul class="list--hide">\n\t\t\t\t\t\t<li><a href="/resources/financial-services/national-settlement-service" target="">National Settlement Service Resources Home</a></li>\n\t\t\t\t\t\t<li><a href="/financial-services/national-settlement-service/service-setup" target="">High-Level Onboarding Process for National Settlement Service</a></li>\n\t\t\t\t\t\t<li><a href="/resources/financial-services/wires/extension-guidelines.html" target="">National Settlement Services Extension Guidelines</a></li>\n\t\t\t\t\t<li><a data-toggle="collapse" data-parent="#accordion" aria-expanded="false" href="#spotlight1" target="">National Settlement Services Volume and Value Statistics <span class="caret"></span></a></li>\n\t\t\t\t\n\t\t\t\t<div id="spotlight1" class="spotlight collapse">\n\t\t\t\t\t\n\t\t\t\t\t<ul class="hide-list-type">\n\t\t\t\t\t\t\t<li><a href="/resources/financial-services/national-settlement-service/volume-value-stats" target="">National Settlement Service Volume and Value Statistics Home</a></li>\n\t\t\t\t\t\t\t<li><a href="/resources/financial-services/national-settlement-service/volume-value-stats/monthly-stats.html" target="">National Settlement Service \xe2\x80\x93 Monthly Statistics</a></li>\n\t\t\t\t\t\t\t<li><a href="/resources/financial-services/national-settlement-service/volume-value-stats/quarterly-stats.html" target="">National Settlement Service \xe2\x80\x93 Quarterly Statistics</a></li>\n\t\t\t\t\t\t\t<li><a href="/resources/financial-services/national-settlement-service/volume-value-stats/annual-stats.html" target="">National Settlement Service \xe2\x80\x93 Annual Statistics</a></li>\n\t\t\t\t\t</ul>\n\t\t\t\t</div>\n\t</ul>\n</div>\n\n                </div>\n    </div>\n        </div>\n    </div>\n    </div>\n\n\n\n\n<footer class="hidden-print">\n    <div class="row row--no-margin">\n        <div class="twitter col-sm-12 text-right">\n            FOLLOW US\n            <a class="twitter" href="https://twitter.com/frbservices">\n                <img src="https://www.frbservices.org/webfiles/1626607380014/images/twitter-logo-blue.svg" alt="Follow us on Twitter" height="42" width="36">\n            </a>\n            <a class="twitter" href="https://www.linkedin.com/company/federal-reserve-financial-services/ ">\n                <img src="https://www.frbservices.org/webfiles/1626607380014/images/lI-In-bug.png" alt="Go to Federal Reserve Financial Services on LinkedIn" height="30" width="38">\n            </a>\n        </div>\n    </div>\n    <div class="row row--no-margin">\n        <div class="col-xs-12">\n            <hr>\n        </div>\n    </div>\n    <div class="row row--no-margin" style="position: relative">\n\n\n\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/financial-services" class="footer__list-header">Financial Services</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/accounting">Accounting</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/ach">ACH</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/bundled-solutions">Bundled Solutions</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/cash">Cash</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/check">Check</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/fednow">FedNow Service</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/national-settlement-service">National Settlement Service</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/securities">Securities</a></li>\n                            <li><a href="https://www.frbservices.org/financial-services/wires">Wires</a></li>\n                        <li><a href="https://www.frbservices.org/central-bank" class="footer__list-header">Central Bank</a></li>\n                            <li><a href="https://www.frbdiscountwindow.org/">Lending Central (Off-Site)</a></li>\n                            <li><a href="https://www.frbservices.org/central-bank/reporting-central">Reporting Central</a></li>\n                            <li><a href="https://www.frbservices.org/central-bank/reserves-central">Reserves Central</a></li>\n                </ul>\n            </div>\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/treasury-services" class="footer__list-header">Treasury Services</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/ach-reclamation.html">ACH Reclamation</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/auctions.html">Auctions</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/check-reclamation.html">Check Reclamation</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/collateral-services.html">Collateral Services</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/savings-bonds-financial-institutions.html">Savings Bonds For Financial Institutions</a></li>\n                            <li><a href="https://www.frbservices.org/treasury-services/savings-bond-news.html">Savings Bonds News You Can Use</a></li>\n                        <li><a href="https://www.frbservices.org/fedline-solutions" class="footer__list-header">Fedline<sup>\xc2\xae</sup> Solutions</a></li>\n                            <li><a href="https://www.frbservices.org/fedline-solutions/fedline-advantage.html">FedLine Advantage<sup>\xc2\xae</sup></a></li>\n                            <li><a href="https://www.frbservices.org/fedline-solutions/fedline-command.html">FedLine Command<sup>\xc2\xae</sup></a></li>\n                            <li><a href="https://www.frbservices.org/fedline-solutions/fedline-direct.html">FedLine Direct<sup>\xc2\xae</sup></a></li>\n                            <li><a href="https://www.frbservices.org/fedline-solutions/fedline-web.html">FedLine Web<sup>\xc2\xae</sup></a></li>\n                            <li><a href="https://www.frbservices.org/fedline-solutions/fedmail.html">FedMail<sup>\xc2\xae</sup></a></li>\n                </ul>\n            </div>\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/forms" class="footer__list-header">Forms</a></li>\n                            <li><a href="https://www.frbservices.org/forms/accounting">Accounting</a></li>\n                            <li><a href="https://www.frbservices.org/forms/ach">ACH</a></li>\n                            <li><a href="https://www.frbservices.org/forms/cash">Cash</a></li>\n                            <li><a href="https://www.frbservices.org/forms/central-bank">Central Bank</a></li>\n                            <li><a href="https://www.frbservices.org/forms/check">Check</a></li>\n                            <li><a href="https://www.frbservices.org/forms/district-information">District Information</a></li>\n                            <li><a href="https://www.frbservices.org/forms/fedline-solutions">FedLine<sup>\xc2\xae</sup> Solutions</a></li>\n                            <li><a href="https://www.frbservices.org/forms/national-settlement-service">National Settlement Service</a></li>\n                            <li><a href="https://www.frbservices.org/forms/securities">Securities</a></li>\n                            <li><a href="https://www.frbservices.org/forms/treasury">Treasury Services</a></li>\n                            <li><a href="https://www.frbservices.org/forms/wires">Wires</a></li>\n                </ul>\n            </div>\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/resources" class="footer__list-header">Resources</a></li>\n                            <li><a href="https://www.frbservices.org/resources/central-bank">Central Bank</a></li>\n                            <li><a href="https://www.frbservices.org/resources/fedline-solutions">FedLine<sup>\xc2\xae</sup> Solutions</a></li>\n                            <li><a href="https://www.frbservices.org/resources/fees">Fees</a></li>\n                            <li><a href="https://www.frbservices.org/resources/financial-services">Financial Services</a></li>\n                            <li><a href="https://www.frbservices.org/resources/industry-links">Industry Links</a></li>\n                            <li><a href="https://www.frbservices.org/resources/resource-centers">Resource Centers</a></li>\n                            <li><a href="https://www.frbservices.org/resources/routing-number-directory">E-Payments Routing Directory</a></li>\n                            <li><a href="https://www.frbservices.org/resources/rules-regulations">Rules & Regulations</a></li>\n                            <li><a href="https://www.frbservices.org/resources/service-setup">Service & Access Setup</a></li>\n                            <li><a href="https://www.frbservices.org/resources/treasury">Treasury Services</a></li>\n                </ul>\n            </div>\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/education" class="footer__list-header">Education</a></li>\n                            <li><a href="https://www.frbservices.org/education/industry-events.html">Events</a></li>\n                            <li><a href="https://www.frbservices.org/education/products-services-education">Products & Services Education</a></li>\n                            <li><a href="https://www.frbservices.org/education/webinars">Webinars</a></li>\n                </ul>\n            </div>\n            <div class="col-md-2 col-sm-4 col-xs-6">\n                <ul class="list-unstyled">\n                        <li><a href="https://www.frbservices.org/news" class="footer__list-header">News</a></li>\n                            <li><a href="https://www.frbservices.org/news/communications">Communications</a></li>\n                            <li><a href="https://www.frbservices.org/news/email-notifications.html">Email Notifications</a></li>\n                            <li><a href="https://www.frbservices.org/news/fed360">Fed360<sup>\xc2\xae</sup></a></li>\n                            <li><a href="https://www.frbservices.org/news/press-releases">Press Releases</a></li>\n                            <li><a href="https://www.frbservices.org/news/research.html">Research Studies</a></li>\n                        <li><a href="https://www.frbservices.org/about" class="footer__list-header">About Us</a></li>\n                            <li><a href="https://www.frbservices.org/about/frb-bank-offices.html">Bank Offices</a></li>\n                            <li><a href="https://www.frbservices.org/about/holiday-schedules">Holiday Schedules</a></li>\n                        <li><a href="/contacts/index.jsp" class="footer__list-header">Contact</a></li>\n                        <li><a href="https://www.frbservices.org/sitemap" class="footer__list-header">Sitemap</a></li>\n                </ul>\n            </div>\n    </div>\n    <div class="row row--no-margin">\n        <div class="col-xs-12">\n            <hr>\n        </div>\n    </div>\n    <div class="row row--no-margin">\n        <div class="footnote col-sm-6 col-xs-12">\n            <p>\n                This site is a product of the Federal Reserve Banks.<br>\n\n\n\n                <a href="https://www.frbservices.org/terms">Terms of Use</a> | \n                <a href="https://www.frbservices.org/terms/supported-browsers.html">Supported Browsers</a> | \n                <a href="https://www.frbservices.org/terms/privacy.html">Privacy Policy</a>\n            <br>\n                <a href="https://www.frbservices.org/terms/data-privacy.html">General Reserve Bank Data Privacy Notice</a> | \n                <a href="https://www.frbservices.org/terms/reprinting.html">Reprinting Articles</a>\n            <br>\n            </p>\n        </div>\n        <div class="footnote text-right col-sm-6 col-xs-12">\n            <p>Address comments and questions to the<br>\n                <a href="mailto:siteinfo@frbservices.org">FRBservices.org Webmaster</a><br>\n                \xc2\xa92021 Federal Reserve Banks</p>\n        </div>\n    </div>\n</footer>    <script src="/webfiles/1626607380014/js/jquery.min.js"></script>\n    <script src="/webfiles/1626607380014/js/jquery.frbservices.min.js"></script>\n    <script src="/webfiles/1626607380014/js/bootstrap.min.js"></script>\n    <script src="/webfiles/1626607380014/js/html5shiv.js"></script>\n    <script src="/webfiles/1626607380014/js/jquery.smartmenus.min.js"></script>\n    <script src="/webfiles/1626607380014/js/jquery.smartmenus.bootstrap.min.js"></script>\n    <script src="/webfiles/1626607380014/js/abasearch.js"></script>\n</div>\n</body>\n</html>'  # noqa: E501
            result = NSS().get_data()
            self.assertIsInstance(result, list)
            self.assertEqual(len(result), 3)


class Test_Currency(TestCase):
    def test_ic_get_data(self) -> None:
        url = "https://www.federalreserve.gov/paymentsystems/coin_currcircvalue.htm"
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\xef\xbb\xbf<!doctype html>\n<html lang="en" class="no-js">\n    <head>\n        <meta charset="utf-8">\n        <meta http-equiv="X-UA-Compatible" content="IE=edge"/>\n        <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0 maximum-scale=1.6, user-scalable=1"/>\n        <meta name="keywords" content="Board of Governors of the Federal Reserve System, Federal Reserve Board of Governors, Federal Reserve Board, Federal Reserve" />\n        <meta name="description" content="The Federal Reserve Board of Governors in Washington DC." />\n        <meta property="og:site_name" content="Board of Governors of the Federal Reserve System"/>\n        <meta property="og:type" content="article" /> \n        <meta property="og:image"  content="" /> \n        <meta name="twitter:card" content="summary" />\n        <meta name="twitter:image" content="" />\n        \n\n\n \n        <title>Federal Reserve Board - Currency in Circulation: Value</title>\n        \n    <link href="/css/bootstrap.css" rel="stylesheet" type="text/css">\n    <link href="/css/bluesteel-theme.css" rel="stylesheet" type="text/css">\n    <script src="/js/modernizr-latest.js" type="text/javascript"></script>\n    <script src="/js/grunticon.loader.js"></script>\n    <script>\n        grunticon(["/css/icons.data.svg.css", "/css/icons.data.png.css", "/css/icons.fallback.css"]);\n    </script>\n    <noscript>\n        <link href="/css/icons.fallback.css" rel="stylesheet">\n    </noscript>                \n    <script src="/js/angular.min.js" type="text/javascript"></script>\n    <script src="/js/app.js" type="text/javascript"></script>\n        \n    <!-- For Linear Gradients in IE9 or greater -->\n    <!--[if gte IE 9]>\n        <style type="text/css">\n          .gradient {\n             filter: none;\n          }\n        </style>\n      <![endif]-->\n\n\n \n    <script type="text/javascript" >\n  (function(i,s,o,g,r,a,m){i[\'GoogleAnalyticsObject\']=r;i[r]=i[r]||function(){\n  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),\n  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)\n  })(window,document,\'script\',\'//www.google-analytics.com/analytics.js\',\'ga\');\n  ga(\'create\', \'UA-35121701-1\', \'federalreserve.gov\', {\'cookieExpires\': 0});\n  ga(\'set\', \'anonymizeIp\', true);\n  ga(\'send\', \'pageview\');\n</script></head>\n    <body data-ng-app="pubwebApp">\n        <a href="#content" class="globalskip">Skip to main content</a>\n<nav class="nav-primary-mobile" id="nav-primary-mobile">\n</nav>\n<div class="t1_nav navbar navbar-default navbar-fixed-top" role="navigation">\n    <div class="t1_container clearfix">\n        <a id="logo" class="btn btn-default icon-FRB_logo-bw visible-xs-table-cell t1__btnlogo" href="/default.htm" role="button"><span class="sr-only">Back to Home</span>\n        </a>\n        <a href="/default.htm" id="banner" class="visible-xs-table-cell t1__banner" title="Link to Home Page" role="button">Board of Governors of the Federal Reserve System</a>\n        <ul class="nav navbar-nav visible-md visible-lg t1_toplinks">\n            <li class="navbar-text navbar-text__social">\n                <span class="navbar-link" tabindex="0">Stay Connected</span>\n                <ul class="nav navbar-nav t1_social">\n                    \n\n\n    \n                <li class="social__item"><a data-icon="facebook" href="https://www.facebook.com/federalreserve" class="noIcon"><div class="icon-facebook-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="twitter" href="https://twitter.com/federalreserve" class="noIcon"><div class="icon-twitter-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="youtube" href="https://www.youtube.com/federalreserve" class="noIcon"><div class="icon-youtube-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="flickr" href="https://www.flickr.com/photos/federalreserve/" class="noIcon"><div class="icon-flickr-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="linkedin" href="https://www.linkedin.com/company/federal-reserve-board" class="noIcon"><div class="icon-linkedin-color icon icon__sm"></div><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="rss" href="/feeds/feeds.htm" class="noIcon"><div class="icon-rss-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to RSS</span></a></li>\n            \n                <li class="social__item"><a data-icon="email" href="/subscribe.htm" class="noIcon"><div class="icon-email-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to Email</span></a></li>\n                    \n    \n \n \n                </ul>\n            </li>\n           \r\n                                <li class="navbar-text"><a class="navbar-link" href="/recentpostings.htm">Recent Postings</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/newsevents/calendar.htm">Calendar</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/publications.htm">Publications</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/sitemap.htm">Site Map</a></li>\r\n                \r\n                                <li class="navbar-text"><a class="navbar-link" href="/azindex.htm">A-Z index</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/careers.htm">Careers</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/faqs.htm">FAQs</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/videos.htm">Videos</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/aboutthefed/contact-us-topics.htm">Contact</a></li>\r\n                \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \n        </ul>\n        <form class="nav navbar-form hidden-xs nav__search" role="search" action="//www.fedsearch.org/board_public/search" method="get">\n            <div class="form-group input-group input-group-sm">\n                <label for="t1search" class=\'sr-only\'>Search</label>\n                <input id="t1search" class="nav__input form-control" name="text" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                    <button type="submit" class="nav__button btn btn-default" id="headerTopLinksSearchFormSubmit" name="Search">\n                        <span class="sr-only">Submit Search Button</span>\n                        <span class="icon-magnifying-glass icon icon__xs icon--right"></span>\n                    </button>\n                </span>\n            </div>\n            <div class=\'nav__advanced\'><a class="noIcon" href="//www.fedsearch.org/board_public">Advanced</a></div>\n        </form>        \n        <div class="btn-group visible-xs-table-cell visible-sm-table-cell t1__dropdown">\n            <span class="icon icon__md icon--right dropdown-toggle icon-options-nav" data-toggle="dropdown">\n                <span class="sr-only">Toggle Dropdown Menu</span>\n            </span>\n        </div>\n    </div>\n</div>\n<header class="jumbotron hidden-xs">\n    <a class="jumbotron__link" href="/default.htm" title="Link to Home Page">\n        <div class="container-fluid">\n            <h1 class="jumbotron__heading">Board of Governors of the Federal Reserve System</h1>\n            <p class="jumbotron__content"><em>The Federal Reserve, the central bank of the United States, provides the nation with a safe, flexible, and stable monetary and financial system.</em></p>\n        </div>\n    </a>\n</header>\n<div class="t2__offcanvas visible-xs-block">\n    <div class="navbar-header">\n        <button type="button" class="offcanvas__toggle icon-offcanvas-nav" data-toggle="offcanvas" data-target="#offcanvasT2Menu" data-canvas="body">\n            <span class="sr-only">Main Menu Toggle Button</span>\n        </button>\n        <span class="navbar-brand offcanvas__title">Sections</span>\n        <button type="button" class="offcanvas__search icon-search-nav" data-toggle="collapse" data-target="#navbar-collapse">\n            <span class="sr-only">Search Toggle Button</span>\n        </button>\n    </div>\n    <div class="navbar-collapse collapse mobile-form" id="navbar-collapse">\n        <form method="get" class="navbar-form" role="search" action="//www.fedsearch.org/board_public/search">\n            <div class="form-group input-group mobileSearch">\n                <label class="sr-only" for="t2search">Search</label>\n                <input id="t2search" name="text" class="form-control" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                <span class="sr-only">Search Submit Button</span>\n                <button type="submit" class="btn btn-default">Submit</button>\n                </span>\n            </div>\n        </form>\n    </div>\n</div>\n<nav class="nav-primary navbar hidden-xs" id="nav-primary" role="navigation">\n    <ul class="nav navbar-nav" role="menubar">\n        \n \n<li role="menuitem" class="dropdown nav-about  dropdown--3Col">\n    <a href="/aboutthefed.htm" class="sr-only-focusable" aria-haspopup="true" id="aboutMenu">About<br />the Fed</a>\n    <ul aria-labelledby="aboutMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-4 list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/structure-federal-reserve-system.htm">Structure of the Federal Reserve System</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/pf.htm">Purposes &amp; Functions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/bios/board/default.htm">Board Members</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/advisorydefault.htm">Advisory Councils</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/federal-reserve-system.htm">Federal Reserve Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fract.htm">Federal Reserve Act</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/currency.htm">Currency</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardmeetings/meetingdates.htm">Board Meetings</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardvotes.htm">Board Votes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/diversityinclusion.htm">Diversity & Inclusion</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/careers.htm">Careers</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/procurement/about.htm">Do Business with the Board</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/k8.htm">Holidays Observed - K.8</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/contact-us-topics.htm">Contact </a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/foia/about_foia.htm">Requesting Information (FOIA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/faqs.htm">FAQs</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/educational-tools/default.htm">Economic Education</a>\n</li>\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fed-financial-statements.htm">Fed Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/innovation.htm">Innovation</a>\n</li>\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-news  dropdown--1Col">\n    <a href="/newsevents.htm" class="sr-only-focusable" aria-haspopup="true" id="newsMenu">News<br />&amp; Events</a>\n    <ul aria-labelledby="newsMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        \n<li>\n    <a class="sr-only-focusable" href="/newsevents/pressreleases.htm">Press Releases</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/speeches.htm">Speeches</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/testimony.htm">Testimony</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/calendar.htm">Calendar</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/videos.htm">Videos\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/photogallery.htm">Photo Gallery\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n\n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-monetary  dropdown--2Col">\n    <a href="/monetarypolicy.htm" class="sr-only-focusable" aria-haspopup="true" id="monetaryMenu">Monetary<br />Policy</a>\n    <ul aria-labelledby="monetaryMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Federal Open Market Committee</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc.htm">About the FOMC</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomccalendars.htm">Meeting calendars and information</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_historical.htm">Transcripts and other historical materials</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_projectionsfaqs.htm">FAQs</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Monetary Policy Principles and Practice</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/monetary-policy-principles-and-practice.htm">Notes</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Policy Implementation</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policy-normalization.htm">Policy Normalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policytools.htm">Policy Tools</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/mpr_default.htm">Monetary Policy Report</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/beige-book-default.htm">Beige Book</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/quarterly-balance-sheet-developments-report.htm">Quarterly Report on Federal Reserve Balance Sheet Developments</a>\n</li>\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Review of Monetary Policy Strategy, Tools, and Communications</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/review-of-monetary-policy-strategy-tools-and-communications.htm">Overview</a>\n</li>\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-supervision  dropdown--fw">\n    <a href="/supervisionreg.htm"  class="sr-only-focusable" id="supervisionMenu" aria-haspopup="true">Supervision<br />&amp; Regulation</a>\n    <ul aria-labelledby="supervisionMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Institution Supervision</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/community-and-regional-financial-institutions.htm">Community & Regional Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/large-financial-institutions.htm">Large Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/foreign-banking-organizations.htm">Foreign Banking Organizations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/financial-market-utility-supervision.htm">Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/consumer-compliance.htm">Consumer Compliance</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supervision-and-regulation-report.htm">Federal Reserve Supervision and Regulation Report</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reporting Forms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/default.aspx">All Reporting Forms</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reporting-forms-whats-new.htm">Recent Updates</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/review.aspx">Information collections under review</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?xTdTa+FaSlI=">Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?+PhkJwiBjbA=">Applications/structure change</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?1/RIXO/UduI=">Federal Financial Institutions Examination Council (FFIEC)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?L877VwEj9Ss=">Municipal &amp; Government Securities</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Supervision &amp; Regulation Letters</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/srletters/srletters.htm">By Year</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/topics/topics.htm">By Topic</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Banking Applications &amp; Legal Developments</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/application-process.htm">Application Process</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/board-and-reserve-bank-actions.htm">Board & Reserve Bank Action</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/legal-developments.htm">Enforcement Actions & Legal Developments</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Regulatory Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">Regulations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supmanual.htm">Manuals</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/basel/basel-default.htm">Basel Regulatory Framework</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/volcker-rule.htm">Volcker Rule</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/training-and-assistance-landing.htm">Education, Training, and Assistance</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Banking &amp; Data Structure</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/insider.aspx">Beneficial Ownership reports</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/">U.S. Offices of Foreign Entities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/fhc.htm">Financial Holding Companies</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/isb-default.htm">Interstate Branching</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/suds.htm">Securities Underwriting & Dealing Subsidiaries</a>\n</li>\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-payment  dropdown--fw">\n    <a href="/paymentsystems.htm"  class="sr-only-focusable" id="paymentMenu" aria-haspopup="true">Payment<br />Systems</a>\n    <ul aria-labelledby="paymentMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations &amp; Statutes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regcc-about.htm">Regulation CC (Availability of Funds and Collection of Checks)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regii-about.htm">Regulation II (Debit Card Interchange Fees and Routing)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/reghh-about.htm">Regulation HH (Financial Market Utilities)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/other-regulations.htm">Other Regulations and Statutes</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Payment Policies</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pfs_about.htm">Federal Reserve\'s Key Policies for the Provision of Financial Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/oo_about.htm">Overnight Overdrafts</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_about.htm">Payment System Risk</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/telecomm.htm">Sponsorship for Priority Telecommunication Services</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reserve Bank Payment Services &amp; Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/check_about.htm">Check Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_data.htm">Daylight Overdrafts and Fees</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fednow_about.htm">FedNow\xe2\x84\xa0 Service</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedfunds_about.htm">Fedwire Funds Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedsecs_about.htm">Fedwire Securities Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fisagy_about.htm">Fiscal Agency Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/natl_about.htm">National Settlement Service</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Financial Market Utilities &amp; Infrastructures</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/over_about.htm">Supervision & Oversight of Financial Market Infrastructures</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/designated_fmu_about.htm">Designated Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/int_standards.htm">International Standards for Financial Market Infrastructures</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Research, Committees, and Forums</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/payres_about.htm">Payment Research</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fr-payments-study.htm">Federal Reserve Payments Study (FRPS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pspa_committee.htm">Payments System Policy Advisory Committee</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/forums.htm">Forums</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-econ dropdown--right dropdown--2Col">\n    <a href="/econres.htm" class="sr-only-focusable" aria-haspopup="true" id="econMenu">Economic<br />Research</a>\n    <ul aria-labelledby="econMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Working Papers and Notes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/feds/index.htm">Finance and Economics Discussion Series (FEDS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/notes/feds-notes/default.htm">FEDS Notes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/ifdp/index.htm">International Finance Discussion Papers (IFDP)</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/econres/theeconomists.htm">Meet the Economists</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Models and Tools</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/us-models-about.htm">FRB/US Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/edo-models-about.htm">Estimated Dynamic Optimization (EDO) Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n\n\n\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-data dropdown--right dropdown--fw">\n    <a href="/data.htm"  class="sr-only-focusable" id="dataMenu" aria-haspopup="true">Data</a>\n    <ul aria-labelledby="dataMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/datadownload"><span class="icon icon-ddp icon__sm"></span> Data Download Program</a>\n</li>\n\n\n<li class="nav__header">\n    <p>\n        <strong>Bank Assets and Liabilities</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h3/current/default.htm">Aggregate Reserves of Depository Institutions and the Monetary Base - H.3</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h8/current/default.htm">Assets and Liabilities of Commercial Banks in the U.S. - H.8</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/assetliab/current.htm">Assets and Liabilities of U.S. Branches and Agencies of Foreign Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/chargeoff/">Charge-Off and Delinquency Rates on Loans and Leases at Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sfos/sfos.htm">Senior Financial Officer Survey</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sloos.htm">Senior Loan Officer Opinion Survey on Bank Lending Practices</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Bank Structure Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for the U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Business Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/cp/">Commercial Paper</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g20/current/g20.htm">Finance Companies - G.20</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/govsecure/current.htm">New Security Issues, State and Local Governments</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/corpsecure/current.htm">New Security Issues, U.S. Corporations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Dealer Financing Terms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/scoos.htm">Senior Credit Officer Opinion Survey on Dealer Financing Terms</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Exchange Rates and International Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h10/current/">Foreign Exchange Rates - H.10/G.5</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/intlsumm/current.htm">International Summary Statistics</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/secholdtrans/current.htm">Securities Holdings and Transactions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/statbanksus/current.htm">Statistics Reported by Banks and Other Financial Firms in the United States</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Financial Accounts</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/z1/default.htm">Financial Accounts of the United States - Z.1</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Household Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g19/current/default.htm">Consumer Credit - G.19</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/housedebt/default.htm">Household Debt Service and Financial Obligations Ratios</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mortoutstand/current.htm">Mortgage Debt Outstanding</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Industrial Activity</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g17/current/default.htm">Industrial Production and Capacity Utilization - G.17</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Interest Rates</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h15/">Selected Interest Rates - H.15</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Micro Data Reference Manual (MDRM)</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mdrm.htm">Micro and Macro Data Collections</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Money Stock and Reserve Balances</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h41/">Factors Affecting Reserve Balances - H.4.1</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h6/current/default.htm">Money Stock Measures - H.6</a>\n</li>\n\n\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Other</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/yield-curve-models.htm">Yield Curve Models and Data</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-consumer dropdown--right dropdown--4Col">\n    <a href="/consumerscommunities.htm" class="sr-only-focusable" aria-haspopup="true" id="consumerMenu">Consumers<br />&amp; Communities</a>\n    <ul aria-labelledby="consumerMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cra_about.htm">Community Reinvestment Act (CRA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">All Regulations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Supervision&nbsp;&amp; Enforcement</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/caletters/caletters.htm">CA Letters</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/enforcementactions/default.aspx">Enforcement Actions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/independent-foreclosure-review.htm">Independent Foreclosure Review</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Community Development</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/neighborhood-revitalization.htm">Housing and Neighborhood Revitalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/small-business-and-entrepreneurship.htm">Small Business and Entrepreneurship</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/workforce.htm">Employment and Workforce Development</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cdf.htm">Community Development Finance</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/rural-community-economic-development.htm">Rural Community and Economic Development</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Research&nbsp;&amp; Analysis</strong>\n    </p>\n</li>\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/community-development-publications.htm">Research Publications & Data Analysis</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/cac.htm">Community Advisory Council</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Consumer Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/foreclosure.htm">Mortgage and Foreclosure Resources</a>\n</li>\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/comm-dev-system-map.htm">Federal Reserve Community Development Resources</a>\n</li>\n\n\n\n                </ul>\n            </div>\n        </li>             \n    </ul>\n</li>\n \n \n \n \n \n \n \n \n \n \n \n \n    </ul>\n</nav>\n        <!-- BODY CONTENT -->\n        <div id="content" class="container container__main" role="main">\n            <div class="row">\n                <div class="page-header">\n                    <ol class="breadcrumb">\n                            <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/default.htm">Home</a></li>\n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems.htm">Payment Systems</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/coin_data.htm">Data</a></li>\n                                          \n \n                                          \n \n                                          \n \n                                          \n \n \n                    </ol>\n                    <div class="header-group">\n                        <div id="page-title" class="page-title"> \n                        \n                                \t\t<h2>\n                                \t\t    Currency and Coin Services\n                                \t\t</h2>\n                            \t\t\n                        </div> \n                        \n                  </div>\n                </div><!-- / .page-header -->\n            \n            </div><!-- / .row -->\n  \n\n\n        <div class=\'row\'>\n        \n                                <div class="col-xs-12 col-sm-4 col-md-3">\n                                  <div id="t4_nav" class="t4_nav list-group sticky">\n                                    \n                                    \r\n\r\n\r\n\r\n<!-- <h3 class="text-uppercase">Chapters</h3> -->\r\n\r\n\r\n\r\n\r\n<a href="/paymentsystems/coin_data.htm" class="list-group-item" title="">Data</a><a href="/paymentsystems/coin_currency_orders.htm" class="list-group-item" title="">Currency Print Orders</a><a href="/paymentsystems/coin_addinfo.htm" class="list-group-item" title="">Additional Information</a>\n                                  </div>\n                                </div>\n                        \n                                                    <div id="article" class="col-xs-12 col-sm-8 col-md-9">\n                                                    <h3>Currency in Circulation: Value</h3>\n                                                \n                                <!-- Article Area -->\n                                    \n                                    <div id="floatRightRail" class="col-xs-12 col-sm-12 col-md-5 pull-right">\n                                        \n                                        \n\n<!-- Text Box Begins -->\n\n<div class="panel panel-related">\n    <div class="panel-heading">\n                   \n                <h5 class="panel-title text-capitalize">\n                    Other Formats  \n                </h5>\n                \n    </div>\n    <div class="panel-body">\n        <ul>\n\t<li><a href="/paymentsystems/files/coin_currcircvalue.txt">ASCII</a></li>\n\t<li><a href="/paymentsystems/files/coin_currcircvalue.pdf">PDF (69&nbsp;KB)</a></li>\n</ul>\n    </div>\n</div>\n\n\n\n\n            \n                                    </div>     \n                                    \n                                    \n                                    <p>(in billions of dollars, as of December 31 of each year)</p>\n\n<p>&nbsp;</p>\n\n<div class="data-table" data-table-popout="" id="data-table-0">\n<table class="pubtables" data-other-sticky-ids="navbar-mobile,nav-primary" data-sticky-columns="1" data-sticky-rows="1">\n\t<thead>\n\t\t<tr class="header" valign="bottom">\n\t\t\t<th align="left" scope="col">Year</th>\n\t\t\t<th scope="col">$1</th>\n\t\t\t<th scope="col">$2</th>\n\t\t\t<th scope="col">$5</th>\n\t\t\t<th scope="col">$10</th>\n\t\t\t<th scope="col">$20</th>\n\t\t\t<th scope="col">$50</th>\n\t\t\t<th scope="col">$100</th>\n\t\t\t<th scope="col">$500 to<br />\n\t\t\t$10,000</th>\n\t\t\t<th scope="col">Total</th>\n\t\t</tr>\n\t</thead>\n\t<tbody>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2020</strong></td>\n\t\t\t<td class="left2">$13.1</td>\n\t\t\t<td class="left2">$2.7</td>\n\t\t\t<td class="left2">$16.1</td>\n\t\t\t<td class="left2">$22.5</td>\n\t\t\t<td class="left2">$234.9</td>\n\t\t\t<td class="left2">$114.2</td>\n\t\t\t<td class="left2">$1,636.8</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$2,040.7</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2019</strong></td>\n\t\t\t<td class="left2">$12.7</td>\n\t\t\t<td class="left2">$2.7</td>\n\t\t\t<td class="left2">$15.8</td>\n\t\t\t<td class="left2">$20.9</td>\n\t\t\t<td class="left2">$190.5</td>\n\t\t\t<td class="left2">$92.3</td>\n\t\t\t<td class="left2">$1,424.7</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$1,759.8</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2018</strong></td>\n\t\t\t<td class="left2">$12.4</td>\n\t\t\t<td class="left2">$2.5</td>\n\t\t\t<td class="left2">$15.3</td>\n\t\t\t<td class="left2">$20.1</td>\n\t\t\t<td class="left2">$188.5</td>\n\t\t\t<td class="left2">$89.2</td>\n\t\t\t<td class="left2">$1,343.5</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$1,671.9</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2017</strong></td>\n\t\t\t<td class="left2">$12.1</td>\n\t\t\t<td class="left2">$2.4</td>\n\t\t\t<td class="left2">$14.8</td>\n\t\t\t<td class="left2">$19.6</td>\n\t\t\t<td class="left2">$183.8</td>\n\t\t\t<td class="left2">$86.4</td>\n\t\t\t<td class="left2">$1,251.7</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$1,571.1</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2016</strong></td>\n\t\t\t<td class="left2">$11.7</td>\n\t\t\t<td class="left2">$2.3</td>\n\t\t\t<td class="left2">$14.2</td>\n\t\t\t<td class="left2">$19.2</td>\n\t\t\t<td class="left2">$177.2</td>\n\t\t\t<td class="left2">$83.5</td>\n\t\t\t<td class="left2">$1,154.8</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$1,463.4</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2015</strong></td>\n\t\t\t<td class="left2">$11.4</td>\n\t\t\t<td class="left2">$2.3</td>\n\t\t\t<td class="left2">$13.7</td>\n\t\t\t<td class="left2">$19.0</td>\n\t\t\t<td class="left2">$171.3</td>\n\t\t\t<td class="left2">$79.8</td>\n\t\t\t<td class="left2">$1,082.2</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$1,380.0</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2014</strong></td>\n\t\t\t<td class="left2">$11.0</td>\n\t\t\t<td class="left2">$2.2</td>\n\t\t\t<td class="left2">$13.1</td>\n\t\t\t<td class="left2">$18.9</td>\n\t\t\t<td class="left2">$162.2</td>\n\t\t\t<td class="left2">$76.9</td>\n\t\t\t<td class="left2">$1,014.5</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$1,299.1</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2013</strong></td>\n\t\t\t<td class="left2">$10.6</td>\n\t\t\t<td class="left2">$2.1</td>\n\t\t\t<td class="left2">$12.7</td>\n\t\t\t<td class="left2">$18.5</td>\n\t\t\t<td class="left2">$155.0</td>\n\t\t\t<td class="left2">$74.5</td>\n\t\t\t<td class="left2">$924.7</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$1,198.3</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2012</strong></td>\n\t\t\t<td class="left2">$10.3</td>\n\t\t\t<td class="left2">$2.0</td>\n\t\t\t<td class="left2">$12.2</td>\n\t\t\t<td class="left2">$17.7</td>\n\t\t\t<td class="left2">$148.9</td>\n\t\t\t<td class="left2">$72.5</td>\n\t\t\t<td class="left2">$863.1</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$1,127.1</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2011</strong></td>\n\t\t\t<td class="left2">$10.0</td>\n\t\t\t<td class="left2">$1.9</td>\n\t\t\t<td class="left2">$11.8</td>\n\t\t\t<td class="left2">$17.2</td>\n\t\t\t<td class="left2">$141.1</td>\n\t\t\t<td class="left2">$69.6</td>\n\t\t\t<td class="left2">$782.6</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$1,034.5</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2010</strong></td>\n\t\t\t<td class="left2">$9.7</td>\n\t\t\t<td class="left2">$1.8</td>\n\t\t\t<td class="left2">$11.5</td>\n\t\t\t<td class="left2">$16.6</td>\n\t\t\t<td class="left2">$130.6</td>\n\t\t\t<td class="left2">$66.9</td>\n\t\t\t<td class="left2">$704.6</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$942.0</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2009</strong></td>\n\t\t\t<td class="left2">$9.6</td>\n\t\t\t<td class="left2">$1.7</td>\n\t\t\t<td class="left2">$11.2</td>\n\t\t\t<td class="left2">$16.2</td>\n\t\t\t<td class="left2">$127.5</td>\n\t\t\t<td class="left2">$65.3</td>\n\t\t\t<td class="left2">$656.4</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$888.3</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2008</strong></td>\n\t\t\t<td class="left2">$9.5</td>\n\t\t\t<td class="left2">$1.7</td>\n\t\t\t<td class="left2">$11.0</td>\n\t\t\t<td class="left2">$16.3</td>\n\t\t\t<td class="left2">$125.1</td>\n\t\t\t<td class="left2">$64.7</td>\n\t\t\t<td class="left2">$625.0</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$853.2</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2007</strong></td>\n\t\t\t<td class="left2">$9.3</td>\n\t\t\t<td class="left2">$1.6</td>\n\t\t\t<td class="left2">$10.8</td>\n\t\t\t<td class="left2">$16.2</td>\n\t\t\t<td class="left2">$121.8</td>\n\t\t\t<td class="left2">$63.0</td>\n\t\t\t<td class="left2">$569.3</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$792.2</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2006</strong></td>\n\t\t\t<td class="left2">$9.0</td>\n\t\t\t<td class="left2">$1.5</td>\n\t\t\t<td class="left2">$10.5</td>\n\t\t\t<td class="left2">$16.0</td>\n\t\t\t<td class="left2">$119.2</td>\n\t\t\t<td class="left2">$62.8</td>\n\t\t\t<td class="left2">$564.1</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$783.5</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2005</strong></td>\n\t\t\t<td class="left2">$8.8</td>\n\t\t\t<td class="left2">$1.5</td>\n\t\t\t<td class="left2">$10.3</td>\n\t\t\t<td class="left2">$15.5</td>\n\t\t\t<td class="left2">$115.4</td>\n\t\t\t<td class="left2">$62.1</td>\n\t\t\t<td class="left2">$545.0</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$758.8</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2004</strong></td>\n\t\t\t<td class="left2">$8.3</td>\n\t\t\t<td class="left2">$1.4</td>\n\t\t\t<td class="left2">$9.8</td>\n\t\t\t<td class="left2">$15.1</td>\n\t\t\t<td class="left2">$107.6</td>\n\t\t\t<td class="left2">$60.6</td>\n\t\t\t<td class="left2">$516.7</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$719.9</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2003</strong></td>\n\t\t\t<td class="left2">$8.2</td>\n\t\t\t<td class="left2">$1.3</td>\n\t\t\t<td class="left2">$9.7</td>\n\t\t\t<td class="left2">$15.1</td>\n\t\t\t<td class="left2">$107.8</td>\n\t\t\t<td class="left2">$59.9</td>\n\t\t\t<td class="left2">$487.8</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$690.2</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2002</strong></td>\n\t\t\t<td class="left2">$8.0</td>\n\t\t\t<td class="left2">$1.3</td>\n\t\t\t<td class="left2">$9.4</td>\n\t\t\t<td class="left2">$14.9</td>\n\t\t\t<td class="left2">$103.7</td>\n\t\t\t<td class="left2">$58.5</td>\n\t\t\t<td class="left2">$458.7</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$654.8</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2001</strong></td>\n\t\t\t<td class="left2">$7.8</td>\n\t\t\t<td class="left2">$1.3</td>\n\t\t\t<td class="left2">$9.2</td>\n\t\t\t<td class="left2">$14.7</td>\n\t\t\t<td class="left2">$100.9</td>\n\t\t\t<td class="left2">$57.0</td>\n\t\t\t<td class="left2">$421.1</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$612.3</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2000</strong></td>\n\t\t\t<td class="left2">$7.7</td>\n\t\t\t<td class="left2">$1.2</td>\n\t\t\t<td class="left2">$8.9</td>\n\t\t\t<td class="left2">$14.5</td>\n\t\t\t<td class="left2">$98.6</td>\n\t\t\t<td class="left2">$55.0</td>\n\t\t\t<td class="left2">$377.7</td>\n\t\t\t<td class="left2">$0.3</td>\n\t\t\t<td class="left2">$563.9</td>\n\t\t</tr>\n\t</tbody>\n</table>\n</div>\n\n<p>Includes Federal Reserve notes, U.S. notes, and currency no longer issued.</p>\n\n                                    \n                                    \n                                </div>\n                                \n        </div>            \n        <a id="back-top" class="icon__backTop icon-backtop" href="#" title="Scroll To Top"><span class="sr-only">Back to Top</span></a>\n\n        \n        <div class="row">\n            <div class=\'lastUpdate\' id="lastUpdate">Last Update:\n              \n                            April 21, 2021\n                          \n            </div>\n        </div>\n        </div><!-- / #content .container -->\n        <footer class="container footer">\n    <div class="row footer__content">\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="footer__heading"><span class="text-uppercase">Board of Governors</span> <em>of the</em> <span class="text-uppercase">Federal Reserve System</span></h6>\n            <ul class="list-unstyled">\n                <li class=\'footer__listItem\'><a class="footer__link" href="/aboutthefed.htm">About the Fed</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/newsevents.htm">News & Events</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/monetarypolicy.htm">Monetary Policy</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/supervisionreg.htm">Supervision & Regulation</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/paymentsystems.htm">Payment Systems</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/econres.htm">Economic Research</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/data.htm">Data</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/consumerscommunities.htm">Consumers & Communities</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/financial-stability.htm">Financial Stability</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/economic-disparities-work.htm">Federal Reserve\'s Work Related to Economic Disparities</a></li> \n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="text-uppercase footer__heading">Tools and Information</h6>\n            <ul class="list-unstyled">\n                 <li class=\'footer__listItem\'><a href="/aboutthefed/contact-us-topics.htm" class="footer__link">Contact</a></li>\n                <li class=\'footer__listItem\'><a href="/publications.htm" class="footer__link">Publications</a></li>\n                <li class=\'footer__listItem\'><a href="/foia/about_foia.htm" class="footer__link">Freedom of Information (FOIA)</a></li>\n                <li class=\'footer__listItem\'><a href="https://oig.federalreserve.gov/" class="footer__link">Office of Inspector General</a></li>\n                <li class=\'footer__listItem\'><a href="/publications/annual-report.htm" class="footer__link">Budget &amp; Performance</a> | <a href="/regreform/audit.htm" class="footer__link">Audit</a></li>\n                <li class=\'footer__listItem\'><a href="/eeo.htm" class="footer__link">No FEAR Act</a></li>\n                <li class=\'footer__listItem\'><a href="/espanol.htm" class="footer__link">Espa\xc3\xb1ol</a></li>\n                <li class=\'footer__listItem\'><a href="/website-linking-policies.htm" class="footer__link">Website Policies</a>  | <a href="/privacy.htm" class="footer__link">Privacy Program</a></li>\n                <li class=\'footer__listItem\'><a href="/accessibility.htm" class="footer__link">Accessibility</a></li>\n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <div class="footer__social">\n                <h6 class="text-uppercase footer__heading footer__heading--social">Stay Connected</h6>\n                \n\n\n    \n                <a class="icon__md icon footer__btn icon-facebook-color" href="https://www.facebook.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-twitter-color" href="https://twitter.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-youtube-color" href="https://www.youtube.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-flickr-color" href="https://www.flickr.com/photos/federalreserve/"><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-linkedin-color" href="https://www.linkedin.com/company/federal-reserve-board"><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-rss-color" href="/feeds/feeds.htm"><span class=\'sr-only\'>Subscribe to RSS</span></a>\n            \n                <a class="icon__md icon footer__btn icon-email-color" href="/subscribe.htm"><span class=\'sr-only\'>Subscribe to Email</span></a>\n                    \n    \n \n \n            </div>\n            <a href="https://www.usa.gov/" target="_blank" title="Link to USA.gov">\n                <img class=\'footer__image\' src="/images/USAGov%402x.png" alt="Link to USA.gov" />\n            </a>\n            <a href="/open/open.htm" target="_blank" title="Link to Open.gov">\n                <img class=\'footer__image\' src="/images/OpenGov%402x.png" alt="Link to Open.gov" />\n            </a>\n        </div>\n    </div>\n    <div class="row">\n        <div class="col-xs-12 footer__footer">\n            <p class="text-uppercase footer__text footer__text--left">Board of Governors <em class="text-lowercase">of the</em> Federal Reserve System</p>\n            <p class="footer__text footer__text--right">20th Street and Constitution Avenue N.W., Washington, DC 20551</p>\n        </div>\n    </div>\n</footer>\n\n\n                <script src="/js/jquery1.11.1.min.js" type="text/javascript"></script>\n                <script src="/js/bootstrap.min.js" type="text/javascript"></script>\n                <script src="/js/scripts.js" type="text/javascript"></script>\n                <script src="/js/modal.js" type="text/javascript"></script>\n                <script src="/js/sticky-tables.js" type="text/javascript"></script>\n                <script type="text/javascript" src="/js/slides/ekko-lightbox.min.js"></script>\n                <script src="/js/t4-nav-controller.js" type="text/javascript"></script>\n              \n<script type="text/javascript">\n    $(document).ready(function($) {\n        // delegate calls to data-toggle="lightbox"\n        $(document).delegate(\'*[data-toggle="lightbox"]:not([data-gallery="navigateTo"])\', \'click\', function(event) {\n            event.preventDefault();\n            $(this).ekkoLightbox();\n        });\n        $(\'a[data-click="false"]\').click(function(e) {\n            e.preventDefault();\n            var target = $(this).data(\'target\');\n            $(target).click();\n        });\n    });\n</script>          \n             \n\n\n        \n        <!--<ul></ul> -->\n    <script type="text/javascript" src="/resources/track_downloads.js"></script>\n</body>\n</html>\n '  # noqa: E501
            result = Currency().ic_get_data(url)
            self.assertIsInstance(result, list)
            self.assertEqual(len(result), 3)

    def test_in_circulation(self) -> None:
        with mock.patch('FedPy.Payment_Systems.Currency.Currency.ic_get_data') as patched:
            patched.return_value = [
                ['Year', '$1', '$2', '$5', '$10', '$20', '$50', '$100', '$500 to $10,000', 'Total'],
                ['2020', '2019', '2018'],
                [['13.1', '1.4', '3.2', '2.3', '11.7', '2.3', '16.4', '0.0004', '50.3'],
                 ['12.7', '1.3', '3.2', '2.1', '9.5', '1.8', '14.2', '0.0004', '44.9'],
                 ['12.4', '1.3', '3.1', '2.0', '9.4', '1.8', '13.4', '0.0004', '43.4']],
            ]
            result = Currency().in_circulation("vol")
            self.assertIsInstance(result, pd.DataFrame)
            patched.assert_called_with("https://www.federalreserve.gov/paymentsystems/coin_currcircvolume.htm")

            patched.return_value = [
                ['Year', '$1', '$2', '$5', '$10', '$20', '$50', '$100', '$500 to $10,000', 'Total'],
                ['2020', '2019', '2018'],
                [['$131000000000', '$27000000000', '$161000000000', '$225000000000', '$2349000000000', '$1142000000000',
                  '$16368000000000', '$03000000000', '$20407000000000'],
                 ['$127000000000', '$27000000000', '$158000000000', '$209000000000', '$1905000000000', '$923000000000',
                  '$14247000000000', '$03000000000', '$17598000000000'],
                 ['$124000000000', '$25000000000', '$153000000000', '$201000000000', '$1885000000000', '$892000000000',
                  '$13435000000000', '$03000000000', '$16719000000000']],
            ]
            result = Currency().in_circulation("val")
            self.assertIsInstance(result, pd.DataFrame)
            patched.assert_called_with("https://www.federalreserve.gov/paymentsystems/coin_currcircvalue.htm")

    def test_pc_get_data(self) -> None:
        url = "https://www.federalreserve.gov/paymentsystems/coin_costnewcurr.htm"
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\xef\xbb\xbf<!doctype html>\n<html lang="en" class="no-js">\n    <head>\n        <meta charset="utf-8">\n        <meta http-equiv="X-UA-Compatible" content="IE=edge"/>\n        <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0 maximum-scale=1.6, user-scalable=1"/>\n        <meta name="keywords" content="Board of Governors of the Federal Reserve System, Federal Reserve Board of Governors, Federal Reserve Board, Federal Reserve" />\n        <meta name="description" content="The Federal Reserve Board of Governors in Washington DC." />\n        <meta property="og:site_name" content="Board of Governors of the Federal Reserve System"/>\n        <meta property="og:type" content="article" /> \n        <meta property="og:image"  content="" /> \n        <meta name="twitter:card" content="summary" />\n        <meta name="twitter:image" content="" />\n        \n\n\n \n        <title>Federal Reserve Board - Cost of New Currency</title>\n        \n    <link href="/css/bootstrap.css" rel="stylesheet" type="text/css">\n    <link href="/css/bluesteel-theme.css" rel="stylesheet" type="text/css">\n    <script src="/js/modernizr-latest.js" type="text/javascript"></script>\n    <script src="/js/grunticon.loader.js"></script>\n    <script>\n        grunticon(["/css/icons.data.svg.css", "/css/icons.data.png.css", "/css/icons.fallback.css"]);\n    </script>\n    <noscript>\n        <link href="/css/icons.fallback.css" rel="stylesheet">\n    </noscript>                \n    <script src="/js/angular.min.js" type="text/javascript"></script>\n    <script src="/js/app.js" type="text/javascript"></script>\n        \n    <!-- For Linear Gradients in IE9 or greater -->\n    <!--[if gte IE 9]>\n        <style type="text/css">\n          .gradient {\n             filter: none;\n          }\n        </style>\n      <![endif]-->\n\n\n \n    <script type="text/javascript" >\n  (function(i,s,o,g,r,a,m){i[\'GoogleAnalyticsObject\']=r;i[r]=i[r]||function(){\n  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),\n  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)\n  })(window,document,\'script\',\'//www.google-analytics.com/analytics.js\',\'ga\');\n  ga(\'create\', \'UA-35121701-1\', \'federalreserve.gov\', {\'cookieExpires\': 0});\n  ga(\'set\', \'anonymizeIp\', true);\n  ga(\'send\', \'pageview\');\n</script></head>\n    <body data-ng-app="pubwebApp">\n        <a href="#content" class="globalskip">Skip to main content</a>\n<nav class="nav-primary-mobile" id="nav-primary-mobile">\n</nav>\n<div class="t1_nav navbar navbar-default navbar-fixed-top" role="navigation">\n    <div class="t1_container clearfix">\n        <a id="logo" class="btn btn-default icon-FRB_logo-bw visible-xs-table-cell t1__btnlogo" href="/default.htm" role="button"><span class="sr-only">Back to Home</span>\n        </a>\n        <a href="/default.htm" id="banner" class="visible-xs-table-cell t1__banner" title="Link to Home Page" role="button">Board of Governors of the Federal Reserve System</a>\n        <ul class="nav navbar-nav visible-md visible-lg t1_toplinks">\n            <li class="navbar-text navbar-text__social">\n                <span class="navbar-link" tabindex="0">Stay Connected</span>\n                <ul class="nav navbar-nav t1_social">\n                    \n\n\n    \n                <li class="social__item"><a data-icon="facebook" href="https://www.facebook.com/federalreserve" class="noIcon"><div class="icon-facebook-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="twitter" href="https://twitter.com/federalreserve" class="noIcon"><div class="icon-twitter-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="youtube" href="https://www.youtube.com/federalreserve" class="noIcon"><div class="icon-youtube-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="flickr" href="https://www.flickr.com/photos/federalreserve/" class="noIcon"><div class="icon-flickr-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="linkedin" href="https://www.linkedin.com/company/federal-reserve-board" class="noIcon"><div class="icon-linkedin-color icon icon__sm"></div><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="rss" href="/feeds/feeds.htm" class="noIcon"><div class="icon-rss-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to RSS</span></a></li>\n            \n                <li class="social__item"><a data-icon="email" href="/subscribe.htm" class="noIcon"><div class="icon-email-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to Email</span></a></li>\n                    \n    \n \n \n                </ul>\n            </li>\n           \r\n                                <li class="navbar-text"><a class="navbar-link" href="/recentpostings.htm">Recent Postings</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/newsevents/calendar.htm">Calendar</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/publications.htm">Publications</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/sitemap.htm">Site Map</a></li>\r\n                \r\n                                <li class="navbar-text"><a class="navbar-link" href="/azindex.htm">A-Z index</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/careers.htm">Careers</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/faqs.htm">FAQs</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/videos.htm">Videos</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/aboutthefed/contact-us-topics.htm">Contact</a></li>\r\n                \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \n        </ul>\n        <form class="nav navbar-form hidden-xs nav__search" role="search" action="//www.fedsearch.org/board_public/search" method="get">\n            <div class="form-group input-group input-group-sm">\n                <label for="t1search" class=\'sr-only\'>Search</label>\n                <input id="t1search" class="nav__input form-control" name="text" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                    <button type="submit" class="nav__button btn btn-default" id="headerTopLinksSearchFormSubmit" name="Search">\n                        <span class="sr-only">Submit Search Button</span>\n                        <span class="icon-magnifying-glass icon icon__xs icon--right"></span>\n                    </button>\n                </span>\n            </div>\n            <div class=\'nav__advanced\'><a class="noIcon" href="//www.fedsearch.org/board_public">Advanced</a></div>\n        </form>        \n        <div class="btn-group visible-xs-table-cell visible-sm-table-cell t1__dropdown">\n            <span class="icon icon__md icon--right dropdown-toggle icon-options-nav" data-toggle="dropdown">\n                <span class="sr-only">Toggle Dropdown Menu</span>\n            </span>\n        </div>\n    </div>\n</div>\n<header class="jumbotron hidden-xs">\n    <a class="jumbotron__link" href="/default.htm" title="Link to Home Page">\n        <div class="container-fluid">\n            <h1 class="jumbotron__heading">Board of Governors of the Federal Reserve System</h1>\n            <p class="jumbotron__content"><em>The Federal Reserve, the central bank of the United States, provides the nation with a safe, flexible, and stable monetary and financial system.</em></p>\n        </div>\n    </a>\n</header>\n<div class="t2__offcanvas visible-xs-block">\n    <div class="navbar-header">\n        <button type="button" class="offcanvas__toggle icon-offcanvas-nav" data-toggle="offcanvas" data-target="#offcanvasT2Menu" data-canvas="body">\n            <span class="sr-only">Main Menu Toggle Button</span>\n        </button>\n        <span class="navbar-brand offcanvas__title">Sections</span>\n        <button type="button" class="offcanvas__search icon-search-nav" data-toggle="collapse" data-target="#navbar-collapse">\n            <span class="sr-only">Search Toggle Button</span>\n        </button>\n    </div>\n    <div class="navbar-collapse collapse mobile-form" id="navbar-collapse">\n        <form method="get" class="navbar-form" role="search" action="//www.fedsearch.org/board_public/search">\n            <div class="form-group input-group mobileSearch">\n                <label class="sr-only" for="t2search">Search</label>\n                <input id="t2search" name="text" class="form-control" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                <span class="sr-only">Search Submit Button</span>\n                <button type="submit" class="btn btn-default">Submit</button>\n                </span>\n            </div>\n        </form>\n    </div>\n</div>\n<nav class="nav-primary navbar hidden-xs" id="nav-primary" role="navigation">\n    <ul class="nav navbar-nav" role="menubar">\n        \n \n<li role="menuitem" class="dropdown nav-about  dropdown--3Col">\n    <a href="/aboutthefed.htm" class="sr-only-focusable" aria-haspopup="true" id="aboutMenu">About<br />the Fed</a>\n    <ul aria-labelledby="aboutMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-4 list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/structure-federal-reserve-system.htm">Structure of the Federal Reserve System</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/pf.htm">Purposes &amp; Functions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/bios/board/default.htm">Board Members</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/advisorydefault.htm">Advisory Councils</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/federal-reserve-system.htm">Federal Reserve Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fract.htm">Federal Reserve Act</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/currency.htm">Currency</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardmeetings/meetingdates.htm">Board Meetings</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardvotes.htm">Board Votes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/diversityinclusion.htm">Diversity & Inclusion</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/careers.htm">Careers</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/procurement/about.htm">Do Business with the Board</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/k8.htm">Holidays Observed - K.8</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/contact-us-topics.htm">Contact </a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/foia/about_foia.htm">Requesting Information (FOIA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/faqs.htm">FAQs</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/educational-tools/default.htm">Economic Education</a>\n</li>\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fed-financial-statements.htm">Fed Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/innovation.htm">Innovation</a>\n</li>\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-news  dropdown--1Col">\n    <a href="/newsevents.htm" class="sr-only-focusable" aria-haspopup="true" id="newsMenu">News<br />&amp; Events</a>\n    <ul aria-labelledby="newsMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        \n<li>\n    <a class="sr-only-focusable" href="/newsevents/pressreleases.htm">Press Releases</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/speeches.htm">Speeches</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/testimony.htm">Testimony</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/calendar.htm">Calendar</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/videos.htm">Videos\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/photogallery.htm">Photo Gallery\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n\n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-monetary  dropdown--2Col">\n    <a href="/monetarypolicy.htm" class="sr-only-focusable" aria-haspopup="true" id="monetaryMenu">Monetary<br />Policy</a>\n    <ul aria-labelledby="monetaryMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Federal Open Market Committee</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc.htm">About the FOMC</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomccalendars.htm">Meeting calendars and information</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_historical.htm">Transcripts and other historical materials</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_projectionsfaqs.htm">FAQs</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Monetary Policy Principles and Practice</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/monetary-policy-principles-and-practice.htm">Notes</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Policy Implementation</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policy-normalization.htm">Policy Normalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policytools.htm">Policy Tools</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/mpr_default.htm">Monetary Policy Report</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/beige-book-default.htm">Beige Book</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/quarterly-balance-sheet-developments-report.htm">Quarterly Report on Federal Reserve Balance Sheet Developments</a>\n</li>\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Review of Monetary Policy Strategy, Tools, and Communications</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/review-of-monetary-policy-strategy-tools-and-communications.htm">Overview</a>\n</li>\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-supervision  dropdown--fw">\n    <a href="/supervisionreg.htm"  class="sr-only-focusable" id="supervisionMenu" aria-haspopup="true">Supervision<br />&amp; Regulation</a>\n    <ul aria-labelledby="supervisionMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Institution Supervision</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/community-and-regional-financial-institutions.htm">Community & Regional Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/large-financial-institutions.htm">Large Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/foreign-banking-organizations.htm">Foreign Banking Organizations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/financial-market-utility-supervision.htm">Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/consumer-compliance.htm">Consumer Compliance</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supervision-and-regulation-report.htm">Federal Reserve Supervision and Regulation Report</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reporting Forms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/default.aspx">All Reporting Forms</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reporting-forms-whats-new.htm">Recent Updates</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/review.aspx">Information collections under review</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?xTdTa+FaSlI=">Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?+PhkJwiBjbA=">Applications/structure change</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?1/RIXO/UduI=">Federal Financial Institutions Examination Council (FFIEC)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?L877VwEj9Ss=">Municipal &amp; Government Securities</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Supervision &amp; Regulation Letters</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/srletters/srletters.htm">By Year</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/topics/topics.htm">By Topic</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Banking Applications &amp; Legal Developments</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/application-process.htm">Application Process</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/board-and-reserve-bank-actions.htm">Board & Reserve Bank Action</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/legal-developments.htm">Enforcement Actions & Legal Developments</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Regulatory Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">Regulations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supmanual.htm">Manuals</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/basel/basel-default.htm">Basel Regulatory Framework</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/volcker-rule.htm">Volcker Rule</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/training-and-assistance-landing.htm">Education, Training, and Assistance</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Banking &amp; Data Structure</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/insider.aspx">Beneficial Ownership reports</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/">U.S. Offices of Foreign Entities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/fhc.htm">Financial Holding Companies</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/isb-default.htm">Interstate Branching</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/suds.htm">Securities Underwriting & Dealing Subsidiaries</a>\n</li>\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-payment  dropdown--fw">\n    <a href="/paymentsystems.htm"  class="sr-only-focusable" id="paymentMenu" aria-haspopup="true">Payment<br />Systems</a>\n    <ul aria-labelledby="paymentMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations &amp; Statutes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regcc-about.htm">Regulation CC (Availability of Funds and Collection of Checks)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regii-about.htm">Regulation II (Debit Card Interchange Fees and Routing)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/reghh-about.htm">Regulation HH (Financial Market Utilities)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/other-regulations.htm">Other Regulations and Statutes</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Payment Policies</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pfs_about.htm">Federal Reserve\'s Key Policies for the Provision of Financial Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/oo_about.htm">Overnight Overdrafts</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_about.htm">Payment System Risk</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/telecomm.htm">Sponsorship for Priority Telecommunication Services</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reserve Bank Payment Services &amp; Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/check_about.htm">Check Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_data.htm">Daylight Overdrafts and Fees</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fednow_about.htm">FedNow\xe2\x84\xa0 Service</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedfunds_about.htm">Fedwire Funds Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedsecs_about.htm">Fedwire Securities Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fisagy_about.htm">Fiscal Agency Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/natl_about.htm">National Settlement Service</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Financial Market Utilities &amp; Infrastructures</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/over_about.htm">Supervision & Oversight of Financial Market Infrastructures</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/designated_fmu_about.htm">Designated Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/int_standards.htm">International Standards for Financial Market Infrastructures</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Research, Committees, and Forums</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/payres_about.htm">Payment Research</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fr-payments-study.htm">Federal Reserve Payments Study (FRPS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pspa_committee.htm">Payments System Policy Advisory Committee</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/forums.htm">Forums</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-econ dropdown--right dropdown--2Col">\n    <a href="/econres.htm" class="sr-only-focusable" aria-haspopup="true" id="econMenu">Economic<br />Research</a>\n    <ul aria-labelledby="econMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Working Papers and Notes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/feds/index.htm">Finance and Economics Discussion Series (FEDS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/notes/feds-notes/default.htm">FEDS Notes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/ifdp/index.htm">International Finance Discussion Papers (IFDP)</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/econres/theeconomists.htm">Meet the Economists</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Models and Tools</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/us-models-about.htm">FRB/US Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/edo-models-about.htm">Estimated Dynamic Optimization (EDO) Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n\n\n\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-data dropdown--right dropdown--fw">\n    <a href="/data.htm"  class="sr-only-focusable" id="dataMenu" aria-haspopup="true">Data</a>\n    <ul aria-labelledby="dataMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/datadownload"><span class="icon icon-ddp icon__sm"></span> Data Download Program</a>\n</li>\n\n\n<li class="nav__header">\n    <p>\n        <strong>Bank Assets and Liabilities</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h3/current/default.htm">Aggregate Reserves of Depository Institutions and the Monetary Base - H.3</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h8/current/default.htm">Assets and Liabilities of Commercial Banks in the U.S. - H.8</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/assetliab/current.htm">Assets and Liabilities of U.S. Branches and Agencies of Foreign Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/chargeoff/">Charge-Off and Delinquency Rates on Loans and Leases at Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sfos/sfos.htm">Senior Financial Officer Survey</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sloos.htm">Senior Loan Officer Opinion Survey on Bank Lending Practices</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Bank Structure Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for the U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Business Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/cp/">Commercial Paper</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g20/current/g20.htm">Finance Companies - G.20</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/govsecure/current.htm">New Security Issues, State and Local Governments</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/corpsecure/current.htm">New Security Issues, U.S. Corporations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Dealer Financing Terms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/scoos.htm">Senior Credit Officer Opinion Survey on Dealer Financing Terms</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Exchange Rates and International Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h10/current/">Foreign Exchange Rates - H.10/G.5</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/intlsumm/current.htm">International Summary Statistics</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/secholdtrans/current.htm">Securities Holdings and Transactions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/statbanksus/current.htm">Statistics Reported by Banks and Other Financial Firms in the United States</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Financial Accounts</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/z1/default.htm">Financial Accounts of the United States - Z.1</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Household Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g19/current/default.htm">Consumer Credit - G.19</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/housedebt/default.htm">Household Debt Service and Financial Obligations Ratios</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mortoutstand/current.htm">Mortgage Debt Outstanding</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Industrial Activity</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g17/current/default.htm">Industrial Production and Capacity Utilization - G.17</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Interest Rates</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h15/">Selected Interest Rates - H.15</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Micro Data Reference Manual (MDRM)</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mdrm.htm">Micro and Macro Data Collections</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Money Stock and Reserve Balances</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h41/">Factors Affecting Reserve Balances - H.4.1</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h6/current/default.htm">Money Stock Measures - H.6</a>\n</li>\n\n\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Other</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/yield-curve-models.htm">Yield Curve Models and Data</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-consumer dropdown--right dropdown--4Col">\n    <a href="/consumerscommunities.htm" class="sr-only-focusable" aria-haspopup="true" id="consumerMenu">Consumers<br />&amp; Communities</a>\n    <ul aria-labelledby="consumerMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cra_about.htm">Community Reinvestment Act (CRA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">All Regulations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Supervision&nbsp;&amp; Enforcement</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/caletters/caletters.htm">CA Letters</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/enforcementactions/default.aspx">Enforcement Actions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/independent-foreclosure-review.htm">Independent Foreclosure Review</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Community Development</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/neighborhood-revitalization.htm">Housing and Neighborhood Revitalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/small-business-and-entrepreneurship.htm">Small Business and Entrepreneurship</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/workforce.htm">Employment and Workforce Development</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cdf.htm">Community Development Finance</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/rural-community-economic-development.htm">Rural Community and Economic Development</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Research&nbsp;&amp; Analysis</strong>\n    </p>\n</li>\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/community-development-publications.htm">Research Publications & Data Analysis</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/cac.htm">Community Advisory Council</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Consumer Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/foreclosure.htm">Mortgage and Foreclosure Resources</a>\n</li>\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/comm-dev-system-map.htm">Federal Reserve Community Development Resources</a>\n</li>\n\n\n\n                </ul>\n            </div>\n        </li>             \n    </ul>\n</li>\n \n \n \n \n \n \n \n \n \n \n \n \n    </ul>\n</nav>\n        <!-- BODY CONTENT -->\n        <div id="content" class="container container__main" role="main">\n            <div class="row">\n                <div class="page-header">\n                    <ol class="breadcrumb">\n                            <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/default.htm">Home</a></li>\n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems.htm">Payment Systems</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/coin_data.htm">Data</a></li>\n                                          \n \n                                          \n \n                                          \n \n                                          \n \n \n                    </ol>\n                    <div class="header-group">\n                        <div id="page-title" class="page-title"> \n                        \n                                \t\t<h2>\n                                \t\t    Currency and Coin Services\n                                \t\t</h2>\n                            \t\t\n                        </div> \n                        \n                  </div>\n                </div><!-- / .page-header -->\n            \n            </div><!-- / .row -->\n  \n\n\n        <div class=\'row\'>\n        \n                                <div class="col-xs-12 col-sm-4 col-md-3">\n                                  <div id="t4_nav" class="t4_nav list-group sticky">\n                                    \n                                    \r\n\r\n\r\n\r\n<!-- <h3 class="text-uppercase">Chapters</h3> -->\r\n\r\n\r\n\r\n\r\n<a href="/paymentsystems/coin_data.htm" class="list-group-item" title="">Data</a><a href="/paymentsystems/coin_currency_orders.htm" class="list-group-item" title="">Currency Print Orders</a><a href="/paymentsystems/coin_addinfo.htm" class="list-group-item" title="">Additional Information</a>\n                                  </div>\n                                </div>\n                        \n                                                    <div id="article" class="col-xs-12 col-sm-8 col-md-9">\n                                                    <h3>Cost of New Currency</h3>\n                                                \n                                <!-- Article Area -->\n                                    \n                                    <div id="floatRightRail" class="col-xs-12 col-sm-12 col-md-5 pull-right">\n                                        \n                                        \n\n<!-- Text Box Begins -->\n\n<div class="panel panel-related">\n    <div class="panel-heading">\n                   \n                <h5 class="panel-title text-capitalize">\n                    Other Formats  \n                </h5>\n                \n    </div>\n    <div class="panel-body">\n        <ul>\n\t<li><a href="/paymentsystems/files/coin_costnewcurr.txt">ASCII</a></li>\n\t<li><a href="/paymentsystems/files/coin_costnewcurr.pdf">PDF (99&nbsp;KB)</a></li>\n</ul>\n    </div>\n</div>\n\n\n\n\n            \n                                    </div>     \n                                    \n                                    \n                                    <p>Variable and fixed costs of printing new currency&nbsp;(in millions of dollars)</p>\n\n<div class="data-table" data-table-popout="" id="data-table-0">\n<table class="pubtables" data-other-sticky-ids="navbar-mobile,nav-primary" data-sticky-columns="1" data-sticky-rows="1">\n\t<thead>\n\t\t<tr class="header">\n\t\t\t<th align="left" scope="col">Year</th>\n\t\t\t<th scope="col">Amount</th>\n\t\t</tr>\n\t</thead>\n\t<tbody>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2020</strong></td>\n\t\t\t<td class="left2">$778</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2019</strong></td>\n\t\t\t<td class="left2">$641</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2018</strong></td>\n\t\t\t<td class="left2">$800</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2017</strong></td>\n\t\t\t<td class="left2">$674</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2016</strong></td>\n\t\t\t<td class="left2">$660</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2015</strong></td>\n\t\t\t<td class="left2">$689</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2014</strong></td>\n\t\t\t<td class="left2">$707</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2013</strong></td>\n\t\t\t<td class="left2">$717</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2012</strong></td>\n\t\t\t<td class="left2">$721</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2011</strong></td>\n\t\t\t<td class="left2">$650</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2010</strong></td>\n\t\t\t<td class="left2">$598</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2009</strong></td>\n\t\t\t<td class="left2">$503</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2008</strong></td>\n\t\t\t<td class="left2">$500</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2007</strong></td>\n\t\t\t<td class="left2">$576</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2006</strong></td>\n\t\t\t<td class="left2">$489</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2005</strong></td>\n\t\t\t<td class="left2">$497</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2004</strong></td>\n\t\t\t<td class="left2">$514</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2003</strong></td>\n\t\t\t<td class="left2">$514</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2002</strong></td>\n\t\t\t<td class="left2">$430</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2001</strong></td>\n\t\t\t<td class="left2">$344</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2000</strong></td>\n\t\t\t<td class="left2">$456</td>\n\t\t</tr>\n\t</tbody>\n</table>\n</div>\n\n                                    \n                                    \n                                </div>\n                                \n        </div>            \n        <a id="back-top" class="icon__backTop icon-backtop" href="#" title="Scroll To Top"><span class="sr-only">Back to Top</span></a>\n\n        \n        <div class="row">\n            <div class=\'lastUpdate\' id="lastUpdate">Last Update:\n              \n                            April 21, 2021\n                          \n            </div>\n        </div>\n        </div><!-- / #content .container -->\n        <footer class="container footer">\n    <div class="row footer__content">\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="footer__heading"><span class="text-uppercase">Board of Governors</span> <em>of the</em> <span class="text-uppercase">Federal Reserve System</span></h6>\n            <ul class="list-unstyled">\n                <li class=\'footer__listItem\'><a class="footer__link" href="/aboutthefed.htm">About the Fed</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/newsevents.htm">News & Events</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/monetarypolicy.htm">Monetary Policy</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/supervisionreg.htm">Supervision & Regulation</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/paymentsystems.htm">Payment Systems</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/econres.htm">Economic Research</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/data.htm">Data</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/consumerscommunities.htm">Consumers & Communities</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/financial-stability.htm">Financial Stability</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/economic-disparities-work.htm">Federal Reserve\'s Work Related to Economic Disparities</a></li> \n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="text-uppercase footer__heading">Tools and Information</h6>\n            <ul class="list-unstyled">\n                 <li class=\'footer__listItem\'><a href="/aboutthefed/contact-us-topics.htm" class="footer__link">Contact</a></li>\n                <li class=\'footer__listItem\'><a href="/publications.htm" class="footer__link">Publications</a></li>\n                <li class=\'footer__listItem\'><a href="/foia/about_foia.htm" class="footer__link">Freedom of Information (FOIA)</a></li>\n                <li class=\'footer__listItem\'><a href="https://oig.federalreserve.gov/" class="footer__link">Office of Inspector General</a></li>\n                <li class=\'footer__listItem\'><a href="/publications/annual-report.htm" class="footer__link">Budget &amp; Performance</a> | <a href="/regreform/audit.htm" class="footer__link">Audit</a></li>\n                <li class=\'footer__listItem\'><a href="/eeo.htm" class="footer__link">No FEAR Act</a></li>\n                <li class=\'footer__listItem\'><a href="/espanol.htm" class="footer__link">Espa\xc3\xb1ol</a></li>\n                <li class=\'footer__listItem\'><a href="/website-linking-policies.htm" class="footer__link">Website Policies</a>  | <a href="/privacy.htm" class="footer__link">Privacy Program</a></li>\n                <li class=\'footer__listItem\'><a href="/accessibility.htm" class="footer__link">Accessibility</a></li>\n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <div class="footer__social">\n                <h6 class="text-uppercase footer__heading footer__heading--social">Stay Connected</h6>\n                \n\n\n    \n                <a class="icon__md icon footer__btn icon-facebook-color" href="https://www.facebook.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-twitter-color" href="https://twitter.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-youtube-color" href="https://www.youtube.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-flickr-color" href="https://www.flickr.com/photos/federalreserve/"><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-linkedin-color" href="https://www.linkedin.com/company/federal-reserve-board"><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-rss-color" href="/feeds/feeds.htm"><span class=\'sr-only\'>Subscribe to RSS</span></a>\n            \n                <a class="icon__md icon footer__btn icon-email-color" href="/subscribe.htm"><span class=\'sr-only\'>Subscribe to Email</span></a>\n                    \n    \n \n \n            </div>\n            <a href="https://www.usa.gov/" target="_blank" title="Link to USA.gov">\n                <img class=\'footer__image\' src="/images/USAGov%402x.png" alt="Link to USA.gov" />\n            </a>\n            <a href="/open/open.htm" target="_blank" title="Link to Open.gov">\n                <img class=\'footer__image\' src="/images/OpenGov%402x.png" alt="Link to Open.gov" />\n            </a>\n        </div>\n    </div>\n    <div class="row">\n        <div class="col-xs-12 footer__footer">\n            <p class="text-uppercase footer__text footer__text--left">Board of Governors <em class="text-lowercase">of the</em> Federal Reserve System</p>\n            <p class="footer__text footer__text--right">20th Street and Constitution Avenue N.W., Washington, DC 20551</p>\n        </div>\n    </div>\n</footer>\n\n\n                <script src="/js/jquery1.11.1.min.js" type="text/javascript"></script>\n                <script src="/js/bootstrap.min.js" type="text/javascript"></script>\n                <script src="/js/scripts.js" type="text/javascript"></script>\n                <script src="/js/modal.js" type="text/javascript"></script>\n                <script src="/js/sticky-tables.js" type="text/javascript"></script>\n                <script type="text/javascript" src="/js/slides/ekko-lightbox.min.js"></script>\n                <script src="/js/t4-nav-controller.js" type="text/javascript"></script>\n              \n<script type="text/javascript">\n    $(document).ready(function($) {\n        // delegate calls to data-toggle="lightbox"\n        $(document).delegate(\'*[data-toggle="lightbox"]:not([data-gallery="navigateTo"])\', \'click\', function(event) {\n            event.preventDefault();\n            $(this).ekkoLightbox();\n        });\n        $(\'a[data-click="false"]\').click(function(e) {\n            e.preventDefault();\n            var target = $(this).data(\'target\');\n            $(target).click();\n        });\n    });\n</script>          \n             \n\n\n        \n        <!--<ul></ul> -->\n    <script type="text/javascript" src="/resources/track_downloads.js"></script>\n</body>\n</html>\n '  # noqa: E501
            result = Currency().pc_get_data(url)
            self.assertIsInstance(result, list)
            self.assertEqual(len(result), 3)

    def test_printing_cost(self) -> None:
        with mock.patch('FedPy.Payment_Systems.Currency.Currency.pc_get_data') as patched:
            patched.return_value = [
                ['Year', 'Amount'],
                ['2020', '2019', '2018'],
                [[778000000], [641000000], [800000000]],
            ]
            result = Currency().printing_cost()
            patched.assert_called_with("https://www.federalreserve.gov/paymentsystems/coin_costnewcurr.htm")
            self.assertIsInstance(result, pd.DataFrame)

    def test_operation_expenses(self) -> None:
        with mock.patch('FedPy.Payment_Systems.Currency.Currency.pc_get_data') as patched:
            patched.return_value = [
                ['Year', 'Amount'],
                ['2020', '2019', '2018'],
                [[778000000], [641000000], [800000000]],
            ]
            result = Currency().operation_expenses()
            patched.assert_called_with('https://www.federalreserve.gov/paymentsystems/coin_expcashops.htm')
            self.assertIsInstance(result, pd.DataFrame)

    def test_payments_get_data(self) -> None:
        url = "https://www.federalreserve.gov/paymentsystems/coin_paycurrcircvalue.htm"
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\xef\xbb\xbf<!doctype html>\n<html lang="en" class="no-js">\n    <head>\n        <meta charset="utf-8">\n        <meta http-equiv="X-UA-Compatible" content="IE=edge"/>\n        <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0 maximum-scale=1.6, user-scalable=1"/>\n        <meta name="keywords" content="Board of Governors of the Federal Reserve System, Federal Reserve Board of Governors, Federal Reserve Board, Federal Reserve" />\n        <meta name="description" content="The Federal Reserve Board of Governors in Washington DC." />\n        <meta property="og:site_name" content="Board of Governors of the Federal Reserve System"/>\n        <meta property="og:type" content="article" /> \n        <meta property="og:image"  content="" /> \n        <meta name="twitter:card" content="summary" />\n        <meta name="twitter:image" content="" />\n        \n\n\n \n        <title>Federal Reserve Board - Payments of Currency to Circulation: Value</title>\n        \n    <link href="/css/bootstrap.css" rel="stylesheet" type="text/css">\n    <link href="/css/bluesteel-theme.css" rel="stylesheet" type="text/css">\n    <script src="/js/modernizr-latest.js" type="text/javascript"></script>\n    <script src="/js/grunticon.loader.js"></script>\n    <script>\n        grunticon(["/css/icons.data.svg.css", "/css/icons.data.png.css", "/css/icons.fallback.css"]);\n    </script>\n    <noscript>\n        <link href="/css/icons.fallback.css" rel="stylesheet">\n    </noscript>                \n    <script src="/js/angular.min.js" type="text/javascript"></script>\n    <script src="/js/app.js" type="text/javascript"></script>\n        \n    <!-- For Linear Gradients in IE9 or greater -->\n    <!--[if gte IE 9]>\n        <style type="text/css">\n          .gradient {\n             filter: none;\n          }\n        </style>\n      <![endif]-->\n\n\n \n    <script type="text/javascript" >\n  (function(i,s,o,g,r,a,m){i[\'GoogleAnalyticsObject\']=r;i[r]=i[r]||function(){\n  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),\n  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)\n  })(window,document,\'script\',\'//www.google-analytics.com/analytics.js\',\'ga\');\n  ga(\'create\', \'UA-35121701-1\', \'federalreserve.gov\', {\'cookieExpires\': 0});\n  ga(\'set\', \'anonymizeIp\', true);\n  ga(\'send\', \'pageview\');\n</script></head>\n    <body data-ng-app="pubwebApp">\n        <a href="#content" class="globalskip">Skip to main content</a>\n<nav class="nav-primary-mobile" id="nav-primary-mobile">\n</nav>\n<div class="t1_nav navbar navbar-default navbar-fixed-top" role="navigation">\n    <div class="t1_container clearfix">\n        <a id="logo" class="btn btn-default icon-FRB_logo-bw visible-xs-table-cell t1__btnlogo" href="/default.htm" role="button"><span class="sr-only">Back to Home</span>\n        </a>\n        <a href="/default.htm" id="banner" class="visible-xs-table-cell t1__banner" title="Link to Home Page" role="button">Board of Governors of the Federal Reserve System</a>\n        <ul class="nav navbar-nav visible-md visible-lg t1_toplinks">\n            <li class="navbar-text navbar-text__social">\n                <span class="navbar-link" tabindex="0">Stay Connected</span>\n                <ul class="nav navbar-nav t1_social">\n                    \n\n\n    \n                <li class="social__item"><a data-icon="facebook" href="https://www.facebook.com/federalreserve" class="noIcon"><div class="icon-facebook-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="twitter" href="https://twitter.com/federalreserve" class="noIcon"><div class="icon-twitter-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="youtube" href="https://www.youtube.com/federalreserve" class="noIcon"><div class="icon-youtube-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="flickr" href="https://www.flickr.com/photos/federalreserve/" class="noIcon"><div class="icon-flickr-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="linkedin" href="https://www.linkedin.com/company/federal-reserve-board" class="noIcon"><div class="icon-linkedin-color icon icon__sm"></div><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="rss" href="/feeds/feeds.htm" class="noIcon"><div class="icon-rss-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to RSS</span></a></li>\n            \n                <li class="social__item"><a data-icon="email" href="/subscribe.htm" class="noIcon"><div class="icon-email-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to Email</span></a></li>\n                    \n    \n \n \n                </ul>\n            </li>\n           \r\n                                <li class="navbar-text"><a class="navbar-link" href="/recentpostings.htm">Recent Postings</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/newsevents/calendar.htm">Calendar</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/publications.htm">Publications</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/sitemap.htm">Site Map</a></li>\r\n                \r\n                                <li class="navbar-text"><a class="navbar-link" href="/azindex.htm">A-Z index</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/careers.htm">Careers</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/faqs.htm">FAQs</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/videos.htm">Videos</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/aboutthefed/contact-us-topics.htm">Contact</a></li>\r\n                \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \n        </ul>\n        <form class="nav navbar-form hidden-xs nav__search" role="search" action="//www.fedsearch.org/board_public/search" method="get">\n            <div class="form-group input-group input-group-sm">\n                <label for="t1search" class=\'sr-only\'>Search</label>\n                <input id="t1search" class="nav__input form-control" name="text" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                    <button type="submit" class="nav__button btn btn-default" id="headerTopLinksSearchFormSubmit" name="Search">\n                        <span class="sr-only">Submit Search Button</span>\n                        <span class="icon-magnifying-glass icon icon__xs icon--right"></span>\n                    </button>\n                </span>\n            </div>\n            <div class=\'nav__advanced\'><a class="noIcon" href="//www.fedsearch.org/board_public">Advanced</a></div>\n        </form>        \n        <div class="btn-group visible-xs-table-cell visible-sm-table-cell t1__dropdown">\n            <span class="icon icon__md icon--right dropdown-toggle icon-options-nav" data-toggle="dropdown">\n                <span class="sr-only">Toggle Dropdown Menu</span>\n            </span>\n        </div>\n    </div>\n</div>\n<header class="jumbotron hidden-xs">\n    <a class="jumbotron__link" href="/default.htm" title="Link to Home Page">\n        <div class="container-fluid">\n            <h1 class="jumbotron__heading">Board of Governors of the Federal Reserve System</h1>\n            <p class="jumbotron__content"><em>The Federal Reserve, the central bank of the United States, provides the nation with a safe, flexible, and stable monetary and financial system.</em></p>\n        </div>\n    </a>\n</header>\n<div class="t2__offcanvas visible-xs-block">\n    <div class="navbar-header">\n        <button type="button" class="offcanvas__toggle icon-offcanvas-nav" data-toggle="offcanvas" data-target="#offcanvasT2Menu" data-canvas="body">\n            <span class="sr-only">Main Menu Toggle Button</span>\n        </button>\n        <span class="navbar-brand offcanvas__title">Sections</span>\n        <button type="button" class="offcanvas__search icon-search-nav" data-toggle="collapse" data-target="#navbar-collapse">\n            <span class="sr-only">Search Toggle Button</span>\n        </button>\n    </div>\n    <div class="navbar-collapse collapse mobile-form" id="navbar-collapse">\n        <form method="get" class="navbar-form" role="search" action="//www.fedsearch.org/board_public/search">\n            <div class="form-group input-group mobileSearch">\n                <label class="sr-only" for="t2search">Search</label>\n                <input id="t2search" name="text" class="form-control" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                <span class="sr-only">Search Submit Button</span>\n                <button type="submit" class="btn btn-default">Submit</button>\n                </span>\n            </div>\n        </form>\n    </div>\n</div>\n<nav class="nav-primary navbar hidden-xs" id="nav-primary" role="navigation">\n    <ul class="nav navbar-nav" role="menubar">\n        \n \n<li role="menuitem" class="dropdown nav-about  dropdown--3Col">\n    <a href="/aboutthefed.htm" class="sr-only-focusable" aria-haspopup="true" id="aboutMenu">About<br />the Fed</a>\n    <ul aria-labelledby="aboutMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-4 list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/structure-federal-reserve-system.htm">Structure of the Federal Reserve System</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/pf.htm">Purposes &amp; Functions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/bios/board/default.htm">Board Members</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/advisorydefault.htm">Advisory Councils</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/federal-reserve-system.htm">Federal Reserve Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fract.htm">Federal Reserve Act</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/currency.htm">Currency</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardmeetings/meetingdates.htm">Board Meetings</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardvotes.htm">Board Votes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/diversityinclusion.htm">Diversity & Inclusion</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/careers.htm">Careers</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/procurement/about.htm">Do Business with the Board</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/k8.htm">Holidays Observed - K.8</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/contact-us-topics.htm">Contact </a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/foia/about_foia.htm">Requesting Information (FOIA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/faqs.htm">FAQs</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/educational-tools/default.htm">Economic Education</a>\n</li>\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fed-financial-statements.htm">Fed Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/innovation.htm">Innovation</a>\n</li>\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-news  dropdown--1Col">\n    <a href="/newsevents.htm" class="sr-only-focusable" aria-haspopup="true" id="newsMenu">News<br />&amp; Events</a>\n    <ul aria-labelledby="newsMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        \n<li>\n    <a class="sr-only-focusable" href="/newsevents/pressreleases.htm">Press Releases</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/speeches.htm">Speeches</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/testimony.htm">Testimony</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/calendar.htm">Calendar</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/videos.htm">Videos\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/photogallery.htm">Photo Gallery\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n\n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-monetary  dropdown--2Col">\n    <a href="/monetarypolicy.htm" class="sr-only-focusable" aria-haspopup="true" id="monetaryMenu">Monetary<br />Policy</a>\n    <ul aria-labelledby="monetaryMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Federal Open Market Committee</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc.htm">About the FOMC</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomccalendars.htm">Meeting calendars and information</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_historical.htm">Transcripts and other historical materials</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_projectionsfaqs.htm">FAQs</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Monetary Policy Principles and Practice</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/monetary-policy-principles-and-practice.htm">Notes</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Policy Implementation</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policy-normalization.htm">Policy Normalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policytools.htm">Policy Tools</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/mpr_default.htm">Monetary Policy Report</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/beige-book-default.htm">Beige Book</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/quarterly-balance-sheet-developments-report.htm">Quarterly Report on Federal Reserve Balance Sheet Developments</a>\n</li>\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Review of Monetary Policy Strategy, Tools, and Communications</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/review-of-monetary-policy-strategy-tools-and-communications.htm">Overview</a>\n</li>\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-supervision  dropdown--fw">\n    <a href="/supervisionreg.htm"  class="sr-only-focusable" id="supervisionMenu" aria-haspopup="true">Supervision<br />&amp; Regulation</a>\n    <ul aria-labelledby="supervisionMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Institution Supervision</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/community-and-regional-financial-institutions.htm">Community & Regional Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/large-financial-institutions.htm">Large Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/foreign-banking-organizations.htm">Foreign Banking Organizations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/financial-market-utility-supervision.htm">Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/consumer-compliance.htm">Consumer Compliance</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supervision-and-regulation-report.htm">Federal Reserve Supervision and Regulation Report</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reporting Forms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/default.aspx">All Reporting Forms</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reporting-forms-whats-new.htm">Recent Updates</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/review.aspx">Information collections under review</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?xTdTa+FaSlI=">Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?+PhkJwiBjbA=">Applications/structure change</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?1/RIXO/UduI=">Federal Financial Institutions Examination Council (FFIEC)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?L877VwEj9Ss=">Municipal &amp; Government Securities</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Supervision &amp; Regulation Letters</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/srletters/srletters.htm">By Year</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/topics/topics.htm">By Topic</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Banking Applications &amp; Legal Developments</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/application-process.htm">Application Process</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/board-and-reserve-bank-actions.htm">Board & Reserve Bank Action</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/legal-developments.htm">Enforcement Actions & Legal Developments</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Regulatory Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">Regulations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supmanual.htm">Manuals</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/basel/basel-default.htm">Basel Regulatory Framework</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/volcker-rule.htm">Volcker Rule</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/training-and-assistance-landing.htm">Education, Training, and Assistance</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Banking &amp; Data Structure</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/insider.aspx">Beneficial Ownership reports</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/">U.S. Offices of Foreign Entities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/fhc.htm">Financial Holding Companies</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/isb-default.htm">Interstate Branching</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/suds.htm">Securities Underwriting & Dealing Subsidiaries</a>\n</li>\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-payment  dropdown--fw">\n    <a href="/paymentsystems.htm"  class="sr-only-focusable" id="paymentMenu" aria-haspopup="true">Payment<br />Systems</a>\n    <ul aria-labelledby="paymentMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations &amp; Statutes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regcc-about.htm">Regulation CC (Availability of Funds and Collection of Checks)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regii-about.htm">Regulation II (Debit Card Interchange Fees and Routing)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/reghh-about.htm">Regulation HH (Financial Market Utilities)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/other-regulations.htm">Other Regulations and Statutes</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Payment Policies</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pfs_about.htm">Federal Reserve\'s Key Policies for the Provision of Financial Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/oo_about.htm">Overnight Overdrafts</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_about.htm">Payment System Risk</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/telecomm.htm">Sponsorship for Priority Telecommunication Services</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reserve Bank Payment Services &amp; Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/check_about.htm">Check Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_data.htm">Daylight Overdrafts and Fees</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fednow_about.htm">FedNow\xe2\x84\xa0 Service</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedfunds_about.htm">Fedwire Funds Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedsecs_about.htm">Fedwire Securities Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fisagy_about.htm">Fiscal Agency Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/natl_about.htm">National Settlement Service</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Financial Market Utilities &amp; Infrastructures</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/over_about.htm">Supervision & Oversight of Financial Market Infrastructures</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/designated_fmu_about.htm">Designated Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/int_standards.htm">International Standards for Financial Market Infrastructures</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Research, Committees, and Forums</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/payres_about.htm">Payment Research</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fr-payments-study.htm">Federal Reserve Payments Study (FRPS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pspa_committee.htm">Payments System Policy Advisory Committee</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/forums.htm">Forums</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-econ dropdown--right dropdown--2Col">\n    <a href="/econres.htm" class="sr-only-focusable" aria-haspopup="true" id="econMenu">Economic<br />Research</a>\n    <ul aria-labelledby="econMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Working Papers and Notes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/feds/index.htm">Finance and Economics Discussion Series (FEDS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/notes/feds-notes/default.htm">FEDS Notes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/ifdp/index.htm">International Finance Discussion Papers (IFDP)</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/econres/theeconomists.htm">Meet the Economists</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Models and Tools</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/us-models-about.htm">FRB/US Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/edo-models-about.htm">Estimated Dynamic Optimization (EDO) Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n\n\n\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-data dropdown--right dropdown--fw">\n    <a href="/data.htm"  class="sr-only-focusable" id="dataMenu" aria-haspopup="true">Data</a>\n    <ul aria-labelledby="dataMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/datadownload"><span class="icon icon-ddp icon__sm"></span> Data Download Program</a>\n</li>\n\n\n<li class="nav__header">\n    <p>\n        <strong>Bank Assets and Liabilities</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h3/current/default.htm">Aggregate Reserves of Depository Institutions and the Monetary Base - H.3</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h8/current/default.htm">Assets and Liabilities of Commercial Banks in the U.S. - H.8</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/assetliab/current.htm">Assets and Liabilities of U.S. Branches and Agencies of Foreign Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/chargeoff/">Charge-Off and Delinquency Rates on Loans and Leases at Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sfos/sfos.htm">Senior Financial Officer Survey</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sloos.htm">Senior Loan Officer Opinion Survey on Bank Lending Practices</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Bank Structure Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for the U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Business Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/cp/">Commercial Paper</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g20/current/g20.htm">Finance Companies - G.20</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/govsecure/current.htm">New Security Issues, State and Local Governments</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/corpsecure/current.htm">New Security Issues, U.S. Corporations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Dealer Financing Terms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/scoos.htm">Senior Credit Officer Opinion Survey on Dealer Financing Terms</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Exchange Rates and International Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h10/current/">Foreign Exchange Rates - H.10/G.5</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/intlsumm/current.htm">International Summary Statistics</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/secholdtrans/current.htm">Securities Holdings and Transactions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/statbanksus/current.htm">Statistics Reported by Banks and Other Financial Firms in the United States</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Financial Accounts</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/z1/default.htm">Financial Accounts of the United States - Z.1</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Household Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g19/current/default.htm">Consumer Credit - G.19</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/housedebt/default.htm">Household Debt Service and Financial Obligations Ratios</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mortoutstand/current.htm">Mortgage Debt Outstanding</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Industrial Activity</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g17/current/default.htm">Industrial Production and Capacity Utilization - G.17</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Interest Rates</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h15/">Selected Interest Rates - H.15</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Micro Data Reference Manual (MDRM)</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mdrm.htm">Micro and Macro Data Collections</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Money Stock and Reserve Balances</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h41/">Factors Affecting Reserve Balances - H.4.1</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h6/current/default.htm">Money Stock Measures - H.6</a>\n</li>\n\n\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Other</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/yield-curve-models.htm">Yield Curve Models and Data</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-consumer dropdown--right dropdown--4Col">\n    <a href="/consumerscommunities.htm" class="sr-only-focusable" aria-haspopup="true" id="consumerMenu">Consumers<br />&amp; Communities</a>\n    <ul aria-labelledby="consumerMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cra_about.htm">Community Reinvestment Act (CRA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">All Regulations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Supervision&nbsp;&amp; Enforcement</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/caletters/caletters.htm">CA Letters</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/enforcementactions/default.aspx">Enforcement Actions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/independent-foreclosure-review.htm">Independent Foreclosure Review</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Community Development</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/neighborhood-revitalization.htm">Housing and Neighborhood Revitalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/small-business-and-entrepreneurship.htm">Small Business and Entrepreneurship</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/workforce.htm">Employment and Workforce Development</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cdf.htm">Community Development Finance</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/rural-community-economic-development.htm">Rural Community and Economic Development</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Research&nbsp;&amp; Analysis</strong>\n    </p>\n</li>\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/community-development-publications.htm">Research Publications & Data Analysis</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/cac.htm">Community Advisory Council</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Consumer Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/foreclosure.htm">Mortgage and Foreclosure Resources</a>\n</li>\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/comm-dev-system-map.htm">Federal Reserve Community Development Resources</a>\n</li>\n\n\n\n                </ul>\n            </div>\n        </li>             \n    </ul>\n</li>\n \n \n \n \n \n \n \n \n \n \n \n \n    </ul>\n</nav>\n        <!-- BODY CONTENT -->\n        <div id="content" class="container container__main" role="main">\n            <div class="row">\n                <div class="page-header">\n                    <ol class="breadcrumb">\n                            <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/default.htm">Home</a></li>\n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems.htm">Payment Systems</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/coin_data.htm">Data</a></li>\n                                          \n \n                                          \n \n                                          \n \n                                          \n \n \n                    </ol>\n                    <div class="header-group">\n                        <div id="page-title" class="page-title"> \n                        \n                                \t\t<h2>\n                                \t\t    Currency and Coin Services\n                                \t\t</h2>\n                            \t\t\n                        </div> \n                        \n                  </div>\n                </div><!-- / .page-header -->\n            \n            </div><!-- / .row -->\n  \n\n\n        <div class=\'row\'>\n        \n                                <div class="col-xs-12 col-sm-4 col-md-3">\n                                  <div id="t4_nav" class="t4_nav list-group sticky">\n                                    \n                                    \r\n\r\n\r\n\r\n<!-- <h3 class="text-uppercase">Chapters</h3> -->\r\n\r\n\r\n\r\n\r\n<a href="/paymentsystems/coin_data.htm" class="list-group-item" title="">Data</a><a href="/paymentsystems/coin_currency_orders.htm" class="list-group-item" title="">Currency Print Orders</a><a href="/paymentsystems/coin_addinfo.htm" class="list-group-item" title="">Additional Information</a>\n                                  </div>\n                                </div>\n                        \n                                                    <div id="article" class="col-xs-12 col-sm-8 col-md-9">\n                                                    <h3>Payments of Currency to Circulation: Value</h3>\n                                                \n                                <!-- Article Area -->\n                                    \n                                    <div id="floatRightRail" class="col-xs-12 col-sm-12 col-md-5 pull-right">\n                                        \n                                        \n\n<!-- Text Box Begins -->\n\n<div class="panel panel-related">\n    <div class="panel-heading">\n                   \n                <h5 class="panel-title text-capitalize">\n                    Other Formats  \n                </h5>\n                \n    </div>\n    <div class="panel-body">\n        <ul>\n\t<li><a href="/paymentsystems/files/coin_paycurrcircvalue.pdf">PDF (79&nbsp;KB)</a></li>\n\t<li><a href="/paymentsystems/files/coin_paycurrcircvalue.txt">ASCII</a></li>\n</ul>\n    </div>\n</div>\n\n\n\n\n            \n                                    </div>     \n                                    \n                                    \n                                    <p>Federal Reserve Payments of Currency to Circulation, in billions of dollars<a id="f1" name="f1"></a><a href="#fn1" title="footnote 1"><sup>1</sup></a></p>\n\n<div class="data-table" data-table-popout="" id="data-table-0">\n<table class="pubtables" data-other-sticky-ids="navbar-mobile,nav-primary" data-sticky-columns="1" data-sticky-rows="1">\n\t<thead>\n\t\t<tr class="header" valign="bottom">\n\t\t\t<th align="left" scope="col">Year</th>\n\t\t\t<th scope="col">$1</th>\n\t\t\t<th scope="col">$2</th>\n\t\t\t<th scope="col">$5</th>\n\t\t\t<th scope="col">$10</th>\n\t\t\t<th scope="col">$20</th>\n\t\t\t<th scope="col">$50</th>\n\t\t\t<th scope="col">$100</th>\n\t\t\t<th scope="col">Total</th>\n\t\t</tr>\n\t</thead>\n\t<tbody>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2020</strong></td>\n\t\t\t<td class="left2">$7.8</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$12.4</td>\n\t\t\t<td class="left2">$15.4</td>\n\t\t\t<td class="left2">$308.3</td>\n\t\t\t<td class="left2">$95.9</td>\n\t\t\t<td class="left2">$432.8</td>\n\t\t\t<td class="left2">$872.8</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2019</strong></td>\n\t\t\t<td class="left2">$10.8</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$16.1</td>\n\t\t\t<td class="left2">$17.1</td>\n\t\t\t<td class="left2">$293.2</td>\n\t\t\t<td class="left2">$78.9</td>\n\t\t\t<td class="left2">$367.7</td>\n\t\t\t<td class="left2">$784.1</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2018</strong></td>\n\t\t\t<td class="left2">$11.3</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$16.6</td>\n\t\t\t<td class="left2">$17.0</td>\n\t\t\t<td class="left2">$302.9</td>\n\t\t\t<td class="left2">$75.4</td>\n\t\t\t<td class="left2">$369.2</td>\n\t\t\t<td class="left2">$792.6</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2017</strong></td>\n\t\t\t<td class="left2">$11.6</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$16.5</td>\n\t\t\t<td class="left2">$17.1</td>\n\t\t\t<td class="left2">$303.5</td>\n\t\t\t<td class="left2">$71.9</td>\n\t\t\t<td class="left2">$368.8</td>\n\t\t\t<td class="left2">$789.7</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2016</strong></td>\n\t\t\t<td class="left2">$11.9</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$16.1</td>\n\t\t\t<td class="left2">$17.2</td>\n\t\t\t<td class="left2">$296.3</td>\n\t\t\t<td class="left2">$64.5</td>\n\t\t\t<td class="left2">$323.8</td>\n\t\t\t<td class="left2">$730.0</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2015</strong></td>\n\t\t\t<td class="left2">$12.2</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$15.8</td>\n\t\t\t<td class="left2">$18.6</td>\n\t\t\t<td class="left2">$299.5</td>\n\t\t\t<td class="left2">$61.0</td>\n\t\t\t<td class="left2">$328.6</td>\n\t\t\t<td class="left2">$736.0</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2014</strong></td>\n\t\t\t<td class="left2">$12.3</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$15.3</td>\n\t\t\t<td class="left2">$19.8</td>\n\t\t\t<td class="left2">$304.5</td>\n\t\t\t<td class="left2">$61.0</td>\n\t\t\t<td class="left2">$363.6</td>\n\t\t\t<td class="left2">$776.8</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2013</strong></td>\n\t\t\t<td class="left2">$12.6</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$15.4</td>\n\t\t\t<td class="left2">$19.7</td>\n\t\t\t<td class="left2">$298.9</td>\n\t\t\t<td class="left2">$60.7</td>\n\t\t\t<td class="left2">$340.8</td>\n\t\t\t<td class="left2">$748.3</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2012</strong></td>\n\t\t\t<td class="left2">$12.7</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$15.2</td>\n\t\t\t<td class="left2">$19.5</td>\n\t\t\t<td class="left2">$299.9</td>\n\t\t\t<td class="left2">$60.7</td>\n\t\t\t<td class="left2">$338.9</td>\n\t\t\t<td class="left2">$747.2</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2011</strong></td>\n\t\t\t<td class="left2">$12.6</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$14.8</td>\n\t\t\t<td class="left2">$19.5</td>\n\t\t\t<td class="left2">$296.1</td>\n\t\t\t<td class="left2">$60.9</td>\n\t\t\t<td class="left2">$330.2</td>\n\t\t\t<td class="left2">$734.3</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2010</strong></td>\n\t\t\t<td class="left2">$12.6</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$14.4</td>\n\t\t\t<td class="left2">$19.4</td>\n\t\t\t<td class="left2">$295.0</td>\n\t\t\t<td class="left2">$59.3</td>\n\t\t\t<td class="left2">$286.9</td>\n\t\t\t<td class="left2">$687.8</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2009</strong></td>\n\t\t\t<td class="left2">$12.6</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$14.0</td>\n\t\t\t<td class="left2">$19.6</td>\n\t\t\t<td class="left2">$290.8</td>\n\t\t\t<td class="left2">$57.4</td>\n\t\t\t<td class="left2">$270.0</td>\n\t\t\t<td class="left2">$664.6</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2008</strong></td>\n\t\t\t<td class="left2">$13.2</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$14.4</td>\n\t\t\t<td class="left2">$21.1</td>\n\t\t\t<td class="left2">$300.0</td>\n\t\t\t<td class="left2">$64.7</td>\n\t\t\t<td class="left2">$318.6</td>\n\t\t\t<td class="left2">$732.1</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2007</strong></td>\n\t\t\t<td class="left2">$13.5</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$14.4</td>\n\t\t\t<td class="left2">$21.8</td>\n\t\t\t<td class="left2">$318.9</td>\n\t\t\t<td class="left2">$64.2</td>\n\t\t\t<td class="left2">$271.3</td>\n\t\t\t<td class="left2">$704.1</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2006</strong></td>\n\t\t\t<td class="left2">$13.5</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$14.3</td>\n\t\t\t<td class="left2">$23.0</td>\n\t\t\t<td class="left2">$328.3</td>\n\t\t\t<td class="left2">$65.7</td>\n\t\t\t<td class="left2">$266.2</td>\n\t\t\t<td class="left2">$711.2</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2005</strong></td>\n\t\t\t<td class="left2">$13.3</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$14.1</td>\n\t\t\t<td class="left2">$22.7</td>\n\t\t\t<td class="left2">$324.1</td>\n\t\t\t<td class="left2">$66.4</td>\n\t\t\t<td class="left2">$257.7</td>\n\t\t\t<td class="left2">$698.4</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2004</strong></td>\n\t\t\t<td class="left2">$13.0</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$13.5</td>\n\t\t\t<td class="left2">$23.0</td>\n\t\t\t<td class="left2">$320.2</td>\n\t\t\t<td class="left2">$66.8</td>\n\t\t\t<td class="left2">$249.6</td>\n\t\t\t<td class="left2">$686.4</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2003</strong></td>\n\t\t\t<td class="left2">$12.7</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$12.8</td>\n\t\t\t<td class="left2">$22.4</td>\n\t\t\t<td class="left2">$314.4</td>\n\t\t\t<td class="left2">$63.1</td>\n\t\t\t<td class="left2">$207.8</td>\n\t\t\t<td class="left2">$633.4</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2002</strong></td>\n\t\t\t<td class="left2">$12.3</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$12.4</td>\n\t\t\t<td class="left2">$22.5</td>\n\t\t\t<td class="left2">$302.6</td>\n\t\t\t<td class="left2">$62.0</td>\n\t\t\t<td class="left2">$204.3</td>\n\t\t\t<td class="left2">$616.1</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2001</strong></td>\n\t\t\t<td class="left2">$11.9</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$12.3</td>\n\t\t\t<td class="left2">$23.1</td>\n\t\t\t<td class="left2">$289.8</td>\n\t\t\t<td class="left2">$61.1</td>\n\t\t\t<td class="left2">$201.4</td>\n\t\t\t<td class="left2">$599.6</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2000</strong></td>\n\t\t\t<td class="left2">$11.5</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$11.8</td>\n\t\t\t<td class="left2">$23.0</td>\n\t\t\t<td class="left2">$264.3</td>\n\t\t\t<td class="left2">$54.1</td>\n\t\t\t<td class="left2">$162.5</td>\n\t\t\t<td class="left2">$527.3</td>\n\t\t</tr>\n\t</tbody>\n</table>\n</div>\n\n<div class="footnotes">\n<p><a id="fn1" name="fn1"> </a>1. Values are subject to minor changes based on revisions to underlying data. <a href="#f1">Return to table</a></p>\n</div>\n\n                                    \n                                    \n                                </div>\n                                \n        </div>            \n        <a id="back-top" class="icon__backTop icon-backtop" href="#" title="Scroll To Top"><span class="sr-only">Back to Top</span></a>\n\n        \n        <div class="row">\n            <div class=\'lastUpdate\' id="lastUpdate">Last Update:\n              \n                            April 21, 2021\n                          \n            </div>\n        </div>\n        </div><!-- / #content .container -->\n        <footer class="container footer">\n    <div class="row footer__content">\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="footer__heading"><span class="text-uppercase">Board of Governors</span> <em>of the</em> <span class="text-uppercase">Federal Reserve System</span></h6>\n            <ul class="list-unstyled">\n                <li class=\'footer__listItem\'><a class="footer__link" href="/aboutthefed.htm">About the Fed</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/newsevents.htm">News & Events</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/monetarypolicy.htm">Monetary Policy</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/supervisionreg.htm">Supervision & Regulation</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/paymentsystems.htm">Payment Systems</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/econres.htm">Economic Research</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/data.htm">Data</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/consumerscommunities.htm">Consumers & Communities</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/financial-stability.htm">Financial Stability</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/economic-disparities-work.htm">Federal Reserve\'s Work Related to Economic Disparities</a></li> \n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="text-uppercase footer__heading">Tools and Information</h6>\n            <ul class="list-unstyled">\n                 <li class=\'footer__listItem\'><a href="/aboutthefed/contact-us-topics.htm" class="footer__link">Contact</a></li>\n                <li class=\'footer__listItem\'><a href="/publications.htm" class="footer__link">Publications</a></li>\n                <li class=\'footer__listItem\'><a href="/foia/about_foia.htm" class="footer__link">Freedom of Information (FOIA)</a></li>\n                <li class=\'footer__listItem\'><a href="https://oig.federalreserve.gov/" class="footer__link">Office of Inspector General</a></li>\n                <li class=\'footer__listItem\'><a href="/publications/annual-report.htm" class="footer__link">Budget &amp; Performance</a> | <a href="/regreform/audit.htm" class="footer__link">Audit</a></li>\n                <li class=\'footer__listItem\'><a href="/eeo.htm" class="footer__link">No FEAR Act</a></li>\n                <li class=\'footer__listItem\'><a href="/espanol.htm" class="footer__link">Espa\xc3\xb1ol</a></li>\n                <li class=\'footer__listItem\'><a href="/website-linking-policies.htm" class="footer__link">Website Policies</a>  | <a href="/privacy.htm" class="footer__link">Privacy Program</a></li>\n                <li class=\'footer__listItem\'><a href="/accessibility.htm" class="footer__link">Accessibility</a></li>\n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <div class="footer__social">\n                <h6 class="text-uppercase footer__heading footer__heading--social">Stay Connected</h6>\n                \n\n\n    \n                <a class="icon__md icon footer__btn icon-facebook-color" href="https://www.facebook.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-twitter-color" href="https://twitter.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-youtube-color" href="https://www.youtube.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-flickr-color" href="https://www.flickr.com/photos/federalreserve/"><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-linkedin-color" href="https://www.linkedin.com/company/federal-reserve-board"><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-rss-color" href="/feeds/feeds.htm"><span class=\'sr-only\'>Subscribe to RSS</span></a>\n            \n                <a class="icon__md icon footer__btn icon-email-color" href="/subscribe.htm"><span class=\'sr-only\'>Subscribe to Email</span></a>\n                    \n    \n \n \n            </div>\n            <a href="https://www.usa.gov/" target="_blank" title="Link to USA.gov">\n                <img class=\'footer__image\' src="/images/USAGov%402x.png" alt="Link to USA.gov" />\n            </a>\n            <a href="/open/open.htm" target="_blank" title="Link to Open.gov">\n                <img class=\'footer__image\' src="/images/OpenGov%402x.png" alt="Link to Open.gov" />\n            </a>\n        </div>\n    </div>\n    <div class="row">\n        <div class="col-xs-12 footer__footer">\n            <p class="text-uppercase footer__text footer__text--left">Board of Governors <em class="text-lowercase">of the</em> Federal Reserve System</p>\n            <p class="footer__text footer__text--right">20th Street and Constitution Avenue N.W., Washington, DC 20551</p>\n        </div>\n    </div>\n</footer>\n\n\n                <script src="/js/jquery1.11.1.min.js" type="text/javascript"></script>\n                <script src="/js/bootstrap.min.js" type="text/javascript"></script>\n                <script src="/js/scripts.js" type="text/javascript"></script>\n                <script src="/js/modal.js" type="text/javascript"></script>\n                <script src="/js/sticky-tables.js" type="text/javascript"></script>\n                <script type="text/javascript" src="/js/slides/ekko-lightbox.min.js"></script>\n                <script src="/js/t4-nav-controller.js" type="text/javascript"></script>\n              \n<script type="text/javascript">\n    $(document).ready(function($) {\n        // delegate calls to data-toggle="lightbox"\n        $(document).delegate(\'*[data-toggle="lightbox"]:not([data-gallery="navigateTo"])\', \'click\', function(event) {\n            event.preventDefault();\n            $(this).ekkoLightbox();\n        });\n        $(\'a[data-click="false"]\').click(function(e) {\n            e.preventDefault();\n            var target = $(this).data(\'target\');\n            $(target).click();\n        });\n    });\n</script>          \n             \n\n\n        \n        <!--<ul></ul> -->\n    <script type="text/javascript" src="/resources/track_downloads.js"></script>\n</body>\n</html>\n '  # noqa: E501
            result = Currency().payments_get_data(url)
            self.assertIsInstance(result, list)
            self.assertEqual(len(result), 3)

        url = "https://www.federalreserve.gov/paymentsystems/coin_paycurrcircvolume.htm"
        with mock.patch('requests.get') as patched:
            patched.return_value.content = b'\xef\xbb\xbf<!doctype html>\n<html lang="en" class="no-js">\n    <head>\n        <meta charset="utf-8">\n        <meta http-equiv="X-UA-Compatible" content="IE=edge"/>\n        <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0 maximum-scale=1.6, user-scalable=1"/>\n        <meta name="keywords" content="Board of Governors of the Federal Reserve System, Federal Reserve Board of Governors, Federal Reserve Board, Federal Reserve" />\n        <meta name="description" content="The Federal Reserve Board of Governors in Washington DC." />\n        <meta property="og:site_name" content="Board of Governors of the Federal Reserve System"/>\n        <meta property="og:type" content="article" /> \n        <meta property="og:image"  content="" /> \n        <meta name="twitter:card" content="summary" />\n        <meta name="twitter:image" content="" />\n        \n\n\n \n        <title>Federal Reserve Board - Payments of Currency to Circulation: Value</title>\n        \n    <link href="/css/bootstrap.css" rel="stylesheet" type="text/css">\n    <link href="/css/bluesteel-theme.css" rel="stylesheet" type="text/css">\n    <script src="/js/modernizr-latest.js" type="text/javascript"></script>\n    <script src="/js/grunticon.loader.js"></script>\n    <script>\n        grunticon(["/css/icons.data.svg.css", "/css/icons.data.png.css", "/css/icons.fallback.css"]);\n    </script>\n    <noscript>\n        <link href="/css/icons.fallback.css" rel="stylesheet">\n    </noscript>                \n    <script src="/js/angular.min.js" type="text/javascript"></script>\n    <script src="/js/app.js" type="text/javascript"></script>\n        \n    <!-- For Linear Gradients in IE9 or greater -->\n    <!--[if gte IE 9]>\n        <style type="text/css">\n          .gradient {\n             filter: none;\n          }\n        </style>\n      <![endif]-->\n\n\n \n    <script type="text/javascript" >\n  (function(i,s,o,g,r,a,m){i[\'GoogleAnalyticsObject\']=r;i[r]=i[r]||function(){\n  (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),\n  m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)\n  })(window,document,\'script\',\'//www.google-analytics.com/analytics.js\',\'ga\');\n  ga(\'create\', \'UA-35121701-1\', \'federalreserve.gov\', {\'cookieExpires\': 0});\n  ga(\'set\', \'anonymizeIp\', true);\n  ga(\'send\', \'pageview\');\n</script></head>\n    <body data-ng-app="pubwebApp">\n        <a href="#content" class="globalskip">Skip to main content</a>\n<nav class="nav-primary-mobile" id="nav-primary-mobile">\n</nav>\n<div class="t1_nav navbar navbar-default navbar-fixed-top" role="navigation">\n    <div class="t1_container clearfix">\n        <a id="logo" class="btn btn-default icon-FRB_logo-bw visible-xs-table-cell t1__btnlogo" href="/default.htm" role="button"><span class="sr-only">Back to Home</span>\n        </a>\n        <a href="/default.htm" id="banner" class="visible-xs-table-cell t1__banner" title="Link to Home Page" role="button">Board of Governors of the Federal Reserve System</a>\n        <ul class="nav navbar-nav visible-md visible-lg t1_toplinks">\n            <li class="navbar-text navbar-text__social">\n                <span class="navbar-link" tabindex="0">Stay Connected</span>\n                <ul class="nav navbar-nav t1_social">\n                    \n\n\n    \n                <li class="social__item"><a data-icon="facebook" href="https://www.facebook.com/federalreserve" class="noIcon"><div class="icon-facebook-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="twitter" href="https://twitter.com/federalreserve" class="noIcon"><div class="icon-twitter-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="youtube" href="https://www.youtube.com/federalreserve" class="noIcon"><div class="icon-youtube-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="flickr" href="https://www.flickr.com/photos/federalreserve/" class="noIcon"><div class="icon-flickr-color icon icon__sm"></div><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="linkedin" href="https://www.linkedin.com/company/federal-reserve-board" class="noIcon"><div class="icon-linkedin-color icon icon__sm"></div><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a></li>\n            \n                <li class="social__item"><a data-icon="rss" href="/feeds/feeds.htm" class="noIcon"><div class="icon-rss-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to RSS</span></a></li>\n            \n                <li class="social__item"><a data-icon="email" href="/subscribe.htm" class="noIcon"><div class="icon-email-color icon icon__sm"></div><span class=\'sr-only\'>Subscribe to Email</span></a></li>\n                    \n    \n \n \n                </ul>\n            </li>\n           \r\n                                <li class="navbar-text"><a class="navbar-link" href="/recentpostings.htm">Recent Postings</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/newsevents/calendar.htm">Calendar</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/publications.htm">Publications</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/sitemap.htm">Site Map</a></li>\r\n                \r\n                                <li class="navbar-text"><a class="navbar-link" href="/azindex.htm">A-Z index</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/careers.htm">Careers</a></li>\r\n                        \r\n                                <li class="navbar-text"><a class="navbar-link" href="/faqs.htm">FAQs</a></li>\r\n                        \r\n                    <li class="navbar-text"><a class="navbar-link" href="/videos.htm">Videos</a></li>\r\n                \r\n                    <li class="navbar-text"><a class="navbar-link" href="/aboutthefed/contact-us-topics.htm">Contact</a></li>\r\n                \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \r\n \n        </ul>\n        <form class="nav navbar-form hidden-xs nav__search" role="search" action="//www.fedsearch.org/board_public/search" method="get">\n            <div class="form-group input-group input-group-sm">\n                <label for="t1search" class=\'sr-only\'>Search</label>\n                <input id="t1search" class="nav__input form-control" name="text" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                    <button type="submit" class="nav__button btn btn-default" id="headerTopLinksSearchFormSubmit" name="Search">\n                        <span class="sr-only">Submit Search Button</span>\n                        <span class="icon-magnifying-glass icon icon__xs icon--right"></span>\n                    </button>\n                </span>\n            </div>\n            <div class=\'nav__advanced\'><a class="noIcon" href="//www.fedsearch.org/board_public">Advanced</a></div>\n        </form>        \n        <div class="btn-group visible-xs-table-cell visible-sm-table-cell t1__dropdown">\n            <span class="icon icon__md icon--right dropdown-toggle icon-options-nav" data-toggle="dropdown">\n                <span class="sr-only">Toggle Dropdown Menu</span>\n            </span>\n        </div>\n    </div>\n</div>\n<header class="jumbotron hidden-xs">\n    <a class="jumbotron__link" href="/default.htm" title="Link to Home Page">\n        <div class="container-fluid">\n            <h1 class="jumbotron__heading">Board of Governors of the Federal Reserve System</h1>\n            <p class="jumbotron__content"><em>The Federal Reserve, the central bank of the United States, provides the nation with a safe, flexible, and stable monetary and financial system.</em></p>\n        </div>\n    </a>\n</header>\n<div class="t2__offcanvas visible-xs-block">\n    <div class="navbar-header">\n        <button type="button" class="offcanvas__toggle icon-offcanvas-nav" data-toggle="offcanvas" data-target="#offcanvasT2Menu" data-canvas="body">\n            <span class="sr-only">Main Menu Toggle Button</span>\n        </button>\n        <span class="navbar-brand offcanvas__title">Sections</span>\n        <button type="button" class="offcanvas__search icon-search-nav" data-toggle="collapse" data-target="#navbar-collapse">\n            <span class="sr-only">Search Toggle Button</span>\n        </button>\n    </div>\n    <div class="navbar-collapse collapse mobile-form" id="navbar-collapse">\n        <form method="get" class="navbar-form" role="search" action="//www.fedsearch.org/board_public/search">\n            <div class="form-group input-group mobileSearch">\n                <label class="sr-only" for="t2search">Search</label>\n                <input id="t2search" name="text" class="form-control" placeholder="Search" type="text">\n                <span class="input-group-btn">\n                <span class="sr-only">Search Submit Button</span>\n                <button type="submit" class="btn btn-default">Submit</button>\n                </span>\n            </div>\n        </form>\n    </div>\n</div>\n<nav class="nav-primary navbar hidden-xs" id="nav-primary" role="navigation">\n    <ul class="nav navbar-nav" role="menubar">\n        \n \n<li role="menuitem" class="dropdown nav-about  dropdown--3Col">\n    <a href="/aboutthefed.htm" class="sr-only-focusable" aria-haspopup="true" id="aboutMenu">About<br />the Fed</a>\n    <ul aria-labelledby="aboutMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-4 list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/structure-federal-reserve-system.htm">Structure of the Federal Reserve System</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/pf.htm">Purposes &amp; Functions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/bios/board/default.htm">Board Members</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/advisorydefault.htm">Advisory Councils</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/federal-reserve-system.htm">Federal Reserve Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fract.htm">Federal Reserve Act</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/currency.htm">Currency</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardmeetings/meetingdates.htm">Board Meetings</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/boardvotes.htm">Board Votes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/diversityinclusion.htm">Diversity & Inclusion</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/careers.htm">Careers</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/procurement/about.htm">Do Business with the Board</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/k8.htm">Holidays Observed - K.8</a>\n</li>\n\n</ul>\n<ul class="col-sm-4 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/contact-us-topics.htm">Contact </a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/foia/about_foia.htm">Requesting Information (FOIA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/faqs.htm">FAQs</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/educational-tools/default.htm">Economic Education</a>\n</li>\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/fed-financial-statements.htm">Fed Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/innovation.htm">Innovation</a>\n</li>\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-news  dropdown--1Col">\n    <a href="/newsevents.htm" class="sr-only-focusable" aria-haspopup="true" id="newsMenu">News<br />&amp; Events</a>\n    <ul aria-labelledby="newsMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        \n<li>\n    <a class="sr-only-focusable" href="/newsevents/pressreleases.htm">Press Releases</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/speeches.htm">Speeches</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/testimony.htm">Testimony</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/newsevents/calendar.htm">Calendar</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/videos.htm">Videos\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/photogallery.htm">Photo Gallery\n</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n\n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-monetary  dropdown--2Col">\n    <a href="/monetarypolicy.htm" class="sr-only-focusable" aria-haspopup="true" id="monetaryMenu">Monetary<br />Policy</a>\n    <ul aria-labelledby="monetaryMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Federal Open Market Committee</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc.htm">About the FOMC</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomccalendars.htm">Meeting calendars and information</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_historical.htm">Transcripts and other historical materials</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/fomc_projectionsfaqs.htm">FAQs</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Monetary Policy Principles and Practice</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/monetary-policy-principles-and-practice.htm">Notes</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Policy Implementation</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policy-normalization.htm">Policy Normalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/policytools.htm">Policy Tools</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/mpr_default.htm">Monetary Policy Report</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/beige-book-default.htm">Beige Book</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/quarterly-balance-sheet-developments-report.htm">Quarterly Report on Federal Reserve Balance Sheet Developments</a>\n</li>\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Review of Monetary Policy Strategy, Tools, and Communications</strong>\n    </p>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/monetarypolicy/review-of-monetary-policy-strategy-tools-and-communications.htm">Overview</a>\n</li>\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-supervision  dropdown--fw">\n    <a href="/supervisionreg.htm"  class="sr-only-focusable" id="supervisionMenu" aria-haspopup="true">Supervision<br />&amp; Regulation</a>\n    <ul aria-labelledby="supervisionMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Institution Supervision</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/community-and-regional-financial-institutions.htm">Community & Regional Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/large-financial-institutions.htm">Large Financial Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/foreign-banking-organizations.htm">Foreign Banking Organizations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/financial-market-utility-supervision.htm">Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/consumer-compliance.htm">Consumer Compliance</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Reports</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supervision-and-regulation-report.htm">Federal Reserve Supervision and Regulation Report</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reporting Forms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/default.aspx">All Reporting Forms</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reporting-forms-whats-new.htm">Recent Updates</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/review.aspx">Information collections under review</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?xTdTa+FaSlI=">Financial Statements</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?+PhkJwiBjbA=">Applications/structure change</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?1/RIXO/UduI=">Federal Financial Institutions Examination Council (FFIEC)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/categoryindex.aspx?L877VwEj9Ss=">Municipal &amp; Government Securities</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Supervision &amp; Regulation Letters</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/srletters/srletters.htm">By Year</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/topics/topics.htm">By Topic</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Banking Applications &amp; Legal Developments</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/application-process.htm">Application Process</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/board-and-reserve-bank-actions.htm">Board & Reserve Bank Action</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/legal-developments.htm">Enforcement Actions & Legal Developments</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Regulatory Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">Regulations</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/publications/supmanual.htm">Manuals</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/basel/basel-default.htm">Basel Regulatory Framework</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/volcker-rule.htm">Volcker Rule</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/training-and-assistance-landing.htm">Education, Training, and Assistance</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Banking &amp; Data Structure</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/reportforms/insider.aspx">Beneficial Ownership reports</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/">U.S. Offices of Foreign Entities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/fhc.htm">Financial Holding Companies</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/isb-default.htm">Interstate Branching</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/suds.htm">Securities Underwriting & Dealing Subsidiaries</a>\n</li>\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-payment  dropdown--fw">\n    <a href="/paymentsystems.htm"  class="sr-only-focusable" id="paymentMenu" aria-haspopup="true">Payment<br />Systems</a>\n    <ul aria-labelledby="paymentMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations &amp; Statutes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regcc-about.htm">Regulation CC (Availability of Funds and Collection of Checks)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/regii-about.htm">Regulation II (Debit Card Interchange Fees and Routing)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/reghh-about.htm">Regulation HH (Financial Market Utilities)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/other-regulations.htm">Other Regulations and Statutes</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Payment Policies</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pfs_about.htm">Federal Reserve\'s Key Policies for the Provision of Financial Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/oo_about.htm">Overnight Overdrafts</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_about.htm">Payment System Risk</a>\n</li>\n\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/telecomm.htm">Sponsorship for Priority Telecommunication Services</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Reserve Bank Payment Services &amp; Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedach_about.htm">Automated Clearinghouse Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/check_about.htm">Check Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/psr_data.htm">Daylight Overdrafts and Fees</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fednow_about.htm">FedNow\xe2\x84\xa0 Service</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedfunds_about.htm">Fedwire Funds Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fedsecs_about.htm">Fedwire Securities Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fisagy_about.htm">Fiscal Agency Services</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/natl_about.htm">National Settlement Service</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Financial Market Utilities &amp; Infrastructures</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/over_about.htm">Supervision & Oversight of Financial Market Infrastructures</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/designated_fmu_about.htm">Designated Financial Market Utilities</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/int_standards.htm">International Standards for Financial Market Infrastructures</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Research, Committees, and Forums</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/payres_about.htm">Payment Research</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/fr-payments-study.htm">Federal Reserve Payments Study (FRPS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/pspa_committee.htm">Payments System Policy Advisory Committee</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/paymentsystems/forums.htm">Forums</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-econ dropdown--right dropdown--2Col">\n    <a href="/econres.htm" class="sr-only-focusable" aria-haspopup="true" id="econMenu">Economic<br />Research</a>\n    <ul aria-labelledby="econMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n             <div class="row">\n                <ul class="col-sm-6 list-unstyled">\n                \n<li class="nav__header">\n    <p>\n        <strong>Working Papers and Notes</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/feds/index.htm">Finance and Economics Discussion Series (FEDS)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/notes/feds-notes/default.htm">FEDS Notes</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/ifdp/index.htm">International Finance Discussion Papers (IFDP)</a>\n</li>\n\n\n</ul>\n<ul class="col-sm-6 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/econres/theeconomists.htm">Meet the Economists</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Models and Tools</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/us-models-about.htm">FRB/US Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/edo-models-about.htm">Estimated Dynamic Optimization (EDO) Model</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n\n\n\n\n\n                </ul>\n            </div>\n         </li>             \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-data dropdown--right dropdown--fw">\n    <a href="/data.htm"  class="sr-only-focusable" id="dataMenu" aria-haspopup="true">Data</a>\n    <ul aria-labelledby="dataMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 col-nav list-unstyled">\n                    \n<li>\n    <a class="sr-only-focusable" href="/datadownload"><span class="icon icon-ddp icon__sm"></span> Data Download Program</a>\n</li>\n\n\n<li class="nav__header">\n    <p>\n        <strong>Bank Assets and Liabilities</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h3/current/default.htm">Aggregate Reserves of Depository Institutions and the Monetary Base - H.3</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h8/current/default.htm">Assets and Liabilities of Commercial Banks in the U.S. - H.8</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/assetliab/current.htm">Assets and Liabilities of U.S. Branches and Agencies of Foreign Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/chargeoff/">Charge-Off and Delinquency Rates on Loans and Leases at Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sfos/sfos.htm">Senior Financial Officer Survey</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/sloos.htm">Senior Loan Officer Opinion Survey on Bank Lending Practices</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Bank Structure Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/lbr/">Large Commercial Banks</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/minority-depository-institutions.htm">Minority Depository Institutions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for the U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Business Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/cp/">Commercial Paper</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g20/current/g20.htm">Finance Companies - G.20</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/govsecure/current.htm">New Security Issues, State and Local Governments</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/corpsecure/current.htm">New Security Issues, U.S. Corporations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Dealer Financing Terms</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/scoos.htm">Senior Credit Officer Opinion Survey on Dealer Financing Terms</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Exchange Rates and International Data</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h10/current/">Foreign Exchange Rates - H.10/G.5</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/intlsumm/current.htm">International Summary Statistics</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/secholdtrans/current.htm">Securities Holdings and Transactions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/statbanksus/current.htm">Statistics Reported by Banks and Other Financial Firms in the United States</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/iba/default.htm">Structure and Share Data for U.S. Offices of Foreign Banks</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Financial Accounts</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/z1/default.htm">Financial Accounts of the United States - Z.1</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Household Finance</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g19/current/default.htm">Consumer Credit - G.19</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/housedebt/default.htm">Household Debt Service and Financial Obligations Ratios</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mortoutstand/current.htm">Mortgage Debt Outstanding</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/econres/scfindex.htm">Survey of Consumer Finances (SCF)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Industrial Activity</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/g17/current/default.htm">Industrial Production and Capacity Utilization - G.17</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Interest Rates</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h15/">Selected Interest Rates - H.15</a>\n</li>\n</ul>\n<ul class="col-sm-3 list-unstyled col-nav">\n<li class="nav__header">\n    <p>\n        <strong>Micro Data Reference Manual (MDRM)</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/mdrm.htm">Micro and Macro Data Collections</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Money Stock and Reserve Balances</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h41/">Factors Affecting Reserve Balances - H.4.1</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/releases/h6/current/default.htm">Money Stock Measures - H.6</a>\n</li>\n\n\n\n\n\n<li class="nav__header">\n    <p>\n        <strong>Other</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/data/yield-curve-models.htm">Yield Curve Models and Data</a>\n</li>\n\n                </ul>\n            </div>\n        </li>              \n    </ul>\n</li>\n \n<li role="menuitem" class="dropdown nav-consumer dropdown--right dropdown--4Col">\n    <a href="/consumerscommunities.htm" class="sr-only-focusable" aria-haspopup="true" id="consumerMenu">Consumers<br />&amp; Communities</a>\n    <ul aria-labelledby="consumerMenu" role="menu" class="dropdown-menu sub-nav-group navmenu-nav">\n        <li>\n            <div class="row">\n                <ul class="col-sm-3 list-unstyled">\n                    \n<li class="nav__header">\n    <p>\n        <strong>Regulations</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cra_about.htm">Community Reinvestment Act (CRA)</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/reglisting.htm">All Regulations</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Supervision&nbsp;&amp; Enforcement</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/supervisionreg/caletters/caletters.htm">CA Letters</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/apps/enforcementactions/default.aspx">Enforcement Actions</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/independent-foreclosure-review.htm">Independent Foreclosure Review</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li class="nav__header">\n    <p>\n        <strong>Community Development</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/neighborhood-revitalization.htm">Housing and Neighborhood Revitalization</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/small-business-and-entrepreneurship.htm">Small Business and Entrepreneurship</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/workforce.htm">Employment and Workforce Development</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/cdf.htm">Community Development Finance</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/rural-community-economic-development.htm">Rural Community and Economic Development</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/conferences.htm">Conferences</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Research&nbsp;&amp; Analysis</strong>\n    </p>\n</li>\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/shed.htm">Survey of Household Economics and Decisionmaking</a>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/community-development-publications.htm">Research Publications & Data Analysis</a>\n</li>\n\n</ul>\n<ul class="col-sm-3 list-unstyled">\n<li>\n    <a class="sr-only-focusable" href="/aboutthefed/cac.htm">Community Advisory Council</a>\n</li>\n\n<li class="nav__header">\n    <p>\n        <strong>Consumer Resources</strong>\n    </p>\n</li>\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/foreclosure.htm">Mortgage and Foreclosure Resources</a>\n</li>\n\n\n\n\n<li>\n    <a class="sr-only-focusable" href="/consumerscommunities/comm-dev-system-map.htm">Federal Reserve Community Development Resources</a>\n</li>\n\n\n\n                </ul>\n            </div>\n        </li>             \n    </ul>\n</li>\n \n \n \n \n \n \n \n \n \n \n \n \n    </ul>\n</nav>\n        <!-- BODY CONTENT -->\n        <div id="content" class="container container__main" role="main">\n            <div class="row">\n                <div class="page-header">\n                    <ol class="breadcrumb">\n                            <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/default.htm">Home</a></li>\n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems.htm">Payment Systems</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/coin_about.htm">Currency and Coin Services</a></li>\n                                          \n                                          <li class=\'breadcrumb__item\'><a class="breadcrumb__link" href="/paymentsystems/coin_data.htm">Data</a></li>\n                                          \n \n                                          \n \n                                          \n \n                                          \n \n \n                    </ol>\n                    <div class="header-group">\n                        <div id="page-title" class="page-title"> \n                        \n                                \t\t<h2>\n                                \t\t    Currency and Coin Services\n                                \t\t</h2>\n                            \t\t\n                        </div> \n                        \n                  </div>\n                </div><!-- / .page-header -->\n            \n            </div><!-- / .row -->\n  \n\n\n        <div class=\'row\'>\n        \n                                <div class="col-xs-12 col-sm-4 col-md-3">\n                                  <div id="t4_nav" class="t4_nav list-group sticky">\n                                    \n                                    \r\n\r\n\r\n\r\n<!-- <h3 class="text-uppercase">Chapters</h3> -->\r\n\r\n\r\n\r\n\r\n<a href="/paymentsystems/coin_data.htm" class="list-group-item" title="">Data</a><a href="/paymentsystems/coin_currency_orders.htm" class="list-group-item" title="">Currency Print Orders</a><a href="/paymentsystems/coin_addinfo.htm" class="list-group-item" title="">Additional Information</a>\n                                  </div>\n                                </div>\n                        \n                                                    <div id="article" class="col-xs-12 col-sm-8 col-md-9">\n                                                    <h3>Payments of Currency to Circulation: Value</h3>\n                                                \n                                <!-- Article Area -->\n                                    \n                                    <div id="floatRightRail" class="col-xs-12 col-sm-12 col-md-5 pull-right">\n                                        \n                                        \n\n<!-- Text Box Begins -->\n\n<div class="panel panel-related">\n    <div class="panel-heading">\n                   \n                <h5 class="panel-title text-capitalize">\n                    Other Formats  \n                </h5>\n                \n    </div>\n    <div class="panel-body">\n        <ul>\n\t<li><a href="/paymentsystems/files/coin_paycurrcircvalue.pdf">PDF (79&nbsp;KB)</a></li>\n\t<li><a href="/paymentsystems/files/coin_paycurrcircvalue.txt">ASCII</a></li>\n</ul>\n    </div>\n</div>\n\n\n\n\n            \n                                    </div>     \n                                    \n                                    \n                                    <p>Federal Reserve Payments of Currency to Circulation, in billions of dollars<a id="f1" name="f1"></a><a href="#fn1" title="footnote 1"><sup>1</sup></a></p>\n\n<div class="data-table" data-table-popout="" id="data-table-0">\n<table class="pubtables" data-other-sticky-ids="navbar-mobile,nav-primary" data-sticky-columns="1" data-sticky-rows="1">\n\t<thead>\n\t\t<tr class="header" valign="bottom">\n\t\t\t<th align="left" scope="col">Year</th>\n\t\t\t<th scope="col">$1</th>\n\t\t\t<th scope="col">$2</th>\n\t\t\t<th scope="col">$5</th>\n\t\t\t<th scope="col">$10</th>\n\t\t\t<th scope="col">$20</th>\n\t\t\t<th scope="col">$50</th>\n\t\t\t<th scope="col">$100</th>\n\t\t\t<th scope="col">Total</th>\n\t\t</tr>\n\t</thead>\n\t<tbody>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2020</strong></td>\n\t\t\t<td class="left2">$7.8</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$12.4</td>\n\t\t\t<td class="left2">$15.4</td>\n\t\t\t<td class="left2">$308.3</td>\n\t\t\t<td class="left2">$95.9</td>\n\t\t\t<td class="left2">$432.8</td>\n\t\t\t<td class="left2">$872.8</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2019</strong></td>\n\t\t\t<td class="left2">$10.8</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$16.1</td>\n\t\t\t<td class="left2">$17.1</td>\n\t\t\t<td class="left2">$293.2</td>\n\t\t\t<td class="left2">$78.9</td>\n\t\t\t<td class="left2">$367.7</td>\n\t\t\t<td class="left2">$784.1</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2018</strong></td>\n\t\t\t<td class="left2">$11.3</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$16.6</td>\n\t\t\t<td class="left2">$17.0</td>\n\t\t\t<td class="left2">$302.9</td>\n\t\t\t<td class="left2">$75.4</td>\n\t\t\t<td class="left2">$369.2</td>\n\t\t\t<td class="left2">$792.6</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2017</strong></td>\n\t\t\t<td class="left2">$11.6</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$16.5</td>\n\t\t\t<td class="left2">$17.1</td>\n\t\t\t<td class="left2">$303.5</td>\n\t\t\t<td class="left2">$71.9</td>\n\t\t\t<td class="left2">$368.8</td>\n\t\t\t<td class="left2">$789.7</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2016</strong></td>\n\t\t\t<td class="left2">$11.9</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$16.1</td>\n\t\t\t<td class="left2">$17.2</td>\n\t\t\t<td class="left2">$296.3</td>\n\t\t\t<td class="left2">$64.5</td>\n\t\t\t<td class="left2">$323.8</td>\n\t\t\t<td class="left2">$730.0</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2015</strong></td>\n\t\t\t<td class="left2">$12.2</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$15.8</td>\n\t\t\t<td class="left2">$18.6</td>\n\t\t\t<td class="left2">$299.5</td>\n\t\t\t<td class="left2">$61.0</td>\n\t\t\t<td class="left2">$328.6</td>\n\t\t\t<td class="left2">$736.0</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2014</strong></td>\n\t\t\t<td class="left2">$12.3</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$15.3</td>\n\t\t\t<td class="left2">$19.8</td>\n\t\t\t<td class="left2">$304.5</td>\n\t\t\t<td class="left2">$61.0</td>\n\t\t\t<td class="left2">$363.6</td>\n\t\t\t<td class="left2">$776.8</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2013</strong></td>\n\t\t\t<td class="left2">$12.6</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$15.4</td>\n\t\t\t<td class="left2">$19.7</td>\n\t\t\t<td class="left2">$298.9</td>\n\t\t\t<td class="left2">$60.7</td>\n\t\t\t<td class="left2">$340.8</td>\n\t\t\t<td class="left2">$748.3</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2012</strong></td>\n\t\t\t<td class="left2">$12.7</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$15.2</td>\n\t\t\t<td class="left2">$19.5</td>\n\t\t\t<td class="left2">$299.9</td>\n\t\t\t<td class="left2">$60.7</td>\n\t\t\t<td class="left2">$338.9</td>\n\t\t\t<td class="left2">$747.2</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2011</strong></td>\n\t\t\t<td class="left2">$12.6</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$14.8</td>\n\t\t\t<td class="left2">$19.5</td>\n\t\t\t<td class="left2">$296.1</td>\n\t\t\t<td class="left2">$60.9</td>\n\t\t\t<td class="left2">$330.2</td>\n\t\t\t<td class="left2">$734.3</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2010</strong></td>\n\t\t\t<td class="left2">$12.6</td>\n\t\t\t<td class="left2">$0.2</td>\n\t\t\t<td class="left2">$14.4</td>\n\t\t\t<td class="left2">$19.4</td>\n\t\t\t<td class="left2">$295.0</td>\n\t\t\t<td class="left2">$59.3</td>\n\t\t\t<td class="left2">$286.9</td>\n\t\t\t<td class="left2">$687.8</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2009</strong></td>\n\t\t\t<td class="left2">$12.6</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$14.0</td>\n\t\t\t<td class="left2">$19.6</td>\n\t\t\t<td class="left2">$290.8</td>\n\t\t\t<td class="left2">$57.4</td>\n\t\t\t<td class="left2">$270.0</td>\n\t\t\t<td class="left2">$664.6</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2008</strong></td>\n\t\t\t<td class="left2">$13.2</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$14.4</td>\n\t\t\t<td class="left2">$21.1</td>\n\t\t\t<td class="left2">$300.0</td>\n\t\t\t<td class="left2">$64.7</td>\n\t\t\t<td class="left2">$318.6</td>\n\t\t\t<td class="left2">$732.1</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2007</strong></td>\n\t\t\t<td class="left2">$13.5</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$14.4</td>\n\t\t\t<td class="left2">$21.8</td>\n\t\t\t<td class="left2">$318.9</td>\n\t\t\t<td class="left2">$64.2</td>\n\t\t\t<td class="left2">$271.3</td>\n\t\t\t<td class="left2">$704.1</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2006</strong></td>\n\t\t\t<td class="left2">$13.5</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$14.3</td>\n\t\t\t<td class="left2">$23.0</td>\n\t\t\t<td class="left2">$328.3</td>\n\t\t\t<td class="left2">$65.7</td>\n\t\t\t<td class="left2">$266.2</td>\n\t\t\t<td class="left2">$711.2</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2005</strong></td>\n\t\t\t<td class="left2">$13.3</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$14.1</td>\n\t\t\t<td class="left2">$22.7</td>\n\t\t\t<td class="left2">$324.1</td>\n\t\t\t<td class="left2">$66.4</td>\n\t\t\t<td class="left2">$257.7</td>\n\t\t\t<td class="left2">$698.4</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2004</strong></td>\n\t\t\t<td class="left2">$13.0</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$13.5</td>\n\t\t\t<td class="left2">$23.0</td>\n\t\t\t<td class="left2">$320.2</td>\n\t\t\t<td class="left2">$66.8</td>\n\t\t\t<td class="left2">$249.6</td>\n\t\t\t<td class="left2">$686.4</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2003</strong></td>\n\t\t\t<td class="left2">$12.7</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$12.8</td>\n\t\t\t<td class="left2">$22.4</td>\n\t\t\t<td class="left2">$314.4</td>\n\t\t\t<td class="left2">$63.1</td>\n\t\t\t<td class="left2">$207.8</td>\n\t\t\t<td class="left2">$633.4</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2002</strong></td>\n\t\t\t<td class="left2">$12.3</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$12.4</td>\n\t\t\t<td class="left2">$22.5</td>\n\t\t\t<td class="left2">$302.6</td>\n\t\t\t<td class="left2">$62.0</td>\n\t\t\t<td class="left2">$204.3</td>\n\t\t\t<td class="left2">$616.1</td>\n\t\t</tr>\n\t\t<tr class="shade1">\n\t\t\t<td class="left" scope="row"><strong>2001</strong></td>\n\t\t\t<td class="left2">$11.9</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$12.3</td>\n\t\t\t<td class="left2">$23.1</td>\n\t\t\t<td class="left2">$289.8</td>\n\t\t\t<td class="left2">$61.1</td>\n\t\t\t<td class="left2">$201.4</td>\n\t\t\t<td class="left2">$599.6</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td class="left" scope="row"><strong>2000</strong></td>\n\t\t\t<td class="left2">$11.5</td>\n\t\t\t<td class="left2">$0.1</td>\n\t\t\t<td class="left2">$11.8</td>\n\t\t\t<td class="left2">$23.0</td>\n\t\t\t<td class="left2">$264.3</td>\n\t\t\t<td class="left2">$54.1</td>\n\t\t\t<td class="left2">$162.5</td>\n\t\t\t<td class="left2">$527.3</td>\n\t\t</tr>\n\t</tbody>\n</table>\n</div>\n\n<div class="footnotes">\n<p><a id="fn1" name="fn1"> </a>1. Values are subject to minor changes based on revisions to underlying data. <a href="#f1">Return to table</a></p>\n</div>\n\n                                    \n                                    \n                                </div>\n                                \n        </div>            \n        <a id="back-top" class="icon__backTop icon-backtop" href="#" title="Scroll To Top"><span class="sr-only">Back to Top</span></a>\n\n        \n        <div class="row">\n            <div class=\'lastUpdate\' id="lastUpdate">Last Update:\n              \n                            April 21, 2021\n                          \n            </div>\n        </div>\n        </div><!-- / #content .container -->\n        <footer class="container footer">\n    <div class="row footer__content">\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="footer__heading"><span class="text-uppercase">Board of Governors</span> <em>of the</em> <span class="text-uppercase">Federal Reserve System</span></h6>\n            <ul class="list-unstyled">\n                <li class=\'footer__listItem\'><a class="footer__link" href="/aboutthefed.htm">About the Fed</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/newsevents.htm">News & Events</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/monetarypolicy.htm">Monetary Policy</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/supervisionreg.htm">Supervision & Regulation</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/paymentsystems.htm">Payment Systems</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/econres.htm">Economic Research</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/data.htm">Data</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/consumerscommunities.htm">Consumers & Communities</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/financial-stability.htm">Financial Stability</a></li><li class=\'footer__listItem\'><a class="footer__link" href="/economic-disparities-work.htm">Federal Reserve\'s Work Related to Economic Disparities</a></li> \n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <h6 class="text-uppercase footer__heading">Tools and Information</h6>\n            <ul class="list-unstyled">\n                 <li class=\'footer__listItem\'><a href="/aboutthefed/contact-us-topics.htm" class="footer__link">Contact</a></li>\n                <li class=\'footer__listItem\'><a href="/publications.htm" class="footer__link">Publications</a></li>\n                <li class=\'footer__listItem\'><a href="/foia/about_foia.htm" class="footer__link">Freedom of Information (FOIA)</a></li>\n                <li class=\'footer__listItem\'><a href="https://oig.federalreserve.gov/" class="footer__link">Office of Inspector General</a></li>\n                <li class=\'footer__listItem\'><a href="/publications/annual-report.htm" class="footer__link">Budget &amp; Performance</a> | <a href="/regreform/audit.htm" class="footer__link">Audit</a></li>\n                <li class=\'footer__listItem\'><a href="/eeo.htm" class="footer__link">No FEAR Act</a></li>\n                <li class=\'footer__listItem\'><a href="/espanol.htm" class="footer__link">Espa\xc3\xb1ol</a></li>\n                <li class=\'footer__listItem\'><a href="/website-linking-policies.htm" class="footer__link">Website Policies</a>  | <a href="/privacy.htm" class="footer__link">Privacy Program</a></li>\n                <li class=\'footer__listItem\'><a href="/accessibility.htm" class="footer__link">Accessibility</a></li>\n            </ul>\n        </div>\n        <div class="col-xs-12 col-sm-4">\n            <div class="footer__social">\n                <h6 class="text-uppercase footer__heading footer__heading--social">Stay Connected</h6>\n                \n\n\n    \n                <a class="icon__md icon footer__btn icon-facebook-color" href="https://www.facebook.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Facebook Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-twitter-color" href="https://twitter.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve Twitter Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-youtube-color" href="https://www.youtube.com/federalreserve"><span class=\'sr-only\'>Link to Federal Reserve YouTube Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-flickr-color" href="https://www.flickr.com/photos/federalreserve/"><span class=\'sr-only\'>Link to Federal Reserve Flickr Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-linkedin-color" href="https://www.linkedin.com/company/federal-reserve-board"><span class=\'sr-only\'>Federal Reserve LinkedIn Page</span></a>\n            \n                <a class="icon__md icon footer__btn icon-rss-color" href="/feeds/feeds.htm"><span class=\'sr-only\'>Subscribe to RSS</span></a>\n            \n                <a class="icon__md icon footer__btn icon-email-color" href="/subscribe.htm"><span class=\'sr-only\'>Subscribe to Email</span></a>\n                    \n    \n \n \n            </div>\n            <a href="https://www.usa.gov/" target="_blank" title="Link to USA.gov">\n                <img class=\'footer__image\' src="/images/USAGov%402x.png" alt="Link to USA.gov" />\n            </a>\n            <a href="/open/open.htm" target="_blank" title="Link to Open.gov">\n                <img class=\'footer__image\' src="/images/OpenGov%402x.png" alt="Link to Open.gov" />\n            </a>\n        </div>\n    </div>\n    <div class="row">\n        <div class="col-xs-12 footer__footer">\n            <p class="text-uppercase footer__text footer__text--left">Board of Governors <em class="text-lowercase">of the</em> Federal Reserve System</p>\n            <p class="footer__text footer__text--right">20th Street and Constitution Avenue N.W., Washington, DC 20551</p>\n        </div>\n    </div>\n</footer>\n\n\n                <script src="/js/jquery1.11.1.min.js" type="text/javascript"></script>\n                <script src="/js/bootstrap.min.js" type="text/javascript"></script>\n                <script src="/js/scripts.js" type="text/javascript"></script>\n                <script src="/js/modal.js" type="text/javascript"></script>\n                <script src="/js/sticky-tables.js" type="text/javascript"></script>\n                <script type="text/javascript" src="/js/slides/ekko-lightbox.min.js"></script>\n                <script src="/js/t4-nav-controller.js" type="text/javascript"></script>\n              \n<script type="text/javascript">\n    $(document).ready(function($) {\n        // delegate calls to data-toggle="lightbox"\n        $(document).delegate(\'*[data-toggle="lightbox"]:not([data-gallery="navigateTo"])\', \'click\', function(event) {\n            event.preventDefault();\n            $(this).ekkoLightbox();\n        });\n        $(\'a[data-click="false"]\').click(function(e) {\n            e.preventDefault();\n            var target = $(this).data(\'target\');\n            $(target).click();\n        });\n    });\n</script>          \n             \n\n\n        \n        <!--<ul></ul> -->\n    <script type="text/javascript" src="/resources/track_downloads.js"></script>\n</body>\n</html>\n '  # noqa: E501
            result = Currency().payments_get_data(url)
            self.assertIsInstance(result, list)
            self.assertEqual(len(result), 3)

    def test_payments(self) -> None:
        with mock.patch('FedPy.Payment_Systems.Currency.Currency.payments_get_data') as patched:
            patched.return_value = [
                ['Year', '$1', '$2', '$5', '$10', '$20', '$50', '$100', 'Total'],
                ['2020', '2019', '2018'],
                [['7.8', '0.1', '2.5', '1.5', '15.4', '1.9', '4.3', '33.5'],
                 ['10.8', '0.1', '3.2', '1.7', '14.7', '1.6', '3.7', '35.7'],
                 ['11.3', '0.1', '3.3', '1.7', '15.1', '1.5', '3.7', '36.8']],
            ]
            result = Currency().payments("vol")
            patched.assert_called_with('https://www.federalreserve.gov/paymentsystems/coin_paycurrcircvolume.htm')
            self.assertIsInstance(result, pd.DataFrame)

            patched.return_value = [
                ['Year', '$1', '$2', '$5', '$10', '$20', '$50', '$100', 'Total'],
                ['2020', '2019', '2018'],
                [['$78000000000', '$01000000000', '$124000000000', '$154000000000', '$3083000000000', '$959000000000',
                  '$4328000000000', '$8728000000000'],
                 ['$108000000000', '$02000000000', '$161000000000', '$171000000000', '$2932000000000', '$789000000000',
                  '$3677000000000', '$7841000000000'],
                 ['$113000000000', '$02000000000', '$166000000000', '$170000000000', '$3029000000000', '$754000000000',
                  '$3692000000000', '$7926000000000']],
            ]
            result = Currency().payments("val")
            patched.assert_called_with('https://www.federalreserve.gov/paymentsystems/coin_paycurrcircvalue.htm')
            self.assertIsInstance(result, pd.DataFrame)

    def test_reciepts(self) -> None:
        with mock.patch('FedPy.Payment_Systems.Currency.Currency.payments_get_data') as patched:
            patched.return_value = [
                ['Year', '$1', '$2', '$5', '$10', '$20', '$50', '$100', 'Total'],
                ['2020', '2019', '2018'],
                [['$7.8', '$0.1', '$12.4', '$15.4', '$308.3', '$95.9', '$432.8', '$872.8'],
                 ['$10.8', '$0.2', '$16.1', '$17.1', '$293.2', '$78.9', '$367.7', '$784.1'],
                 ['$11.3', '$0.2', '$16.6', '$17.0', '$302.9', '$75.4', '$369.2', '$792.6']],
            ]
            result = Currency().reciepts("val")
            patched.assert_called_with('https://www.federalreserve.gov/paymentsystems/coin_reccurrcircvalue.htm')
            self.assertIsInstance(result, pd.DataFrame)

            patched.return_value = [
                ['Year', '$1', '$2', '$5', '$10', '$20', '$50', '$100', 'Total'],
                ['2020', '2019', '2018'],
                [['7.4', '0.0', '2.4', '1.4', '13.2', '1.5', '2.2', '28.1'],
                 ['10.5', '0.0', '3.1', '1.6', '14.6', '1.5', '2.9', '34.2'],
                 ['11.0', '0.0', '3.2', '1.7', '14.9', '1.5', '2.8', '35.0']],
            ]
            result = Currency().reciepts("vol")
            patched.assert_called_with('https://www.federalreserve.gov/paymentsystems/coin_reccurrcircvolume.htm')
            self.assertIsInstance(result, pd.DataFrame)


if __name__ == '__main__':
    unittest.main()
